/* MAINTAINER: Bernard */

#include "application_client.h"
#include "commander_client.h"
#include "log.h"
#include "settings.h"
#include "translator.h"
#include "conversions.h"
#include "platform.h"
#include "timemanager.h"
#include "params.h"
#include "globals.h"
#include "tank.h"
#include "gfx.h"
#include "sfx.h"
#include "game_client.h"
#include "keybindings.h"
#include "console.h"
#include "networkmanager_client.h"
#include "tracer.h"
#include "mousemotion.h"
#include "gamegui.h"
#include "observer.h"
#include "iortransfer_client.h"
#include "timemanager.h"

using namespace std;

aliasMapType CommanderClient::mAliasMap;

CommanderClient::CommanderClient()
{
	mCommandsHash["quit"]=               command_quit;
	mCommandsHash["help"]=               command_help;
	mCommandsHash["getvar"]=             command_getvar;
	mCommandsHash["setvar"]=             command_setvar;	
	mCommandsHash["sound"]=              command_sound;
	mCommandsHash["music"]=              command_music;
	mCommandsHash["theme"]=              command_soundtheme;
	mCommandsHash["chatmessage"]=        command_chatmessage;
	mCommandsHash["consolemessage"]=     command_consolemessage;
	mCommandsHash["loginconsole"]=       command_log_in_console;
	mCommandsHash["resolution"]=         command_resolution;
	mCommandsHash["fovx"]=               command_fovx;
	mCommandsHash["fs"]=                 command_fs;
	mCommandsHash["clear"]=              command_clear;
	mCommandsHash["players"]=            command_players;
	mCommandsHash["grab"]=               command_grab;
	mCommandsHash["bind"]=               command_bind;
	mCommandsHash["unbindall"]=          command_unbind_all;
	mCommandsHash["key"]=                command_key;
	mCommandsHash["exec"]=               command_exec;
	mCommandsHash["console"]=            command_console;
	mCommandsHash["sleep"]=              command_sleep;
	mCommandsHash["alias"]=              command_alias;
	mCommandsHash["unalias"]=            command_unalias;
	mCommandsHash["pickedbonus"]=        command_picked_bonus;
	mCommandsHash["expiredbonus"]=       command_expired_bonus;
	mCommandsHash["pause"]=              command_pause;
	mCommandsHash["activatechat"]=       command_activate_chat;
	mCommandsHash["coords"]=             command_coords;
	mCommandsHash["targeting"]=          command_targeting;
	mCommandsHash["mousesensitivity"]=   command_mousesensitivity;
	mCommandsHash["resettargeting"]=     command_resettargeting;
	mCommandsHash["showstats"]=          command_showstats;
	mCommandsHash["connect"]=            command_connect;
	mCommandsHash["#"]=                  command_empty;
	mCommandsHash["disconnect"]=         command_disconnect;
	mCommandsHash["camera"]=             command_camera;
//	mCommandsHash["observer"]=           command_observer;
	mCommandsHash["observerspeed"]=      command_observerspeed;
	mCommandsHash["ingamemenu"]=         command_ingamemenu;
	mCommandsHash["soundvolume"]=        command_soundvolume;
	mCommandsHash["musicvolume"]=        command_musicvolume;
	mCommandsHash["reconnect"]=          command_reconnect;
	mCommandsHash["newtank"]=            command_newtank;
	mCommandsHash["setpreferedname"]=    command_setpreferedname;
}

void CommanderClient::processCommand(const std::string &command, int playerID)
{
	// Normal commands processed on client

	stringstream s(command);
	string cmd;
	
	LOGT << "Processing command " << command << endl;

	s >> cmd;
	
	// Check if command is alias
	aliasMapType::iterator alias = mAliasMap.find(cmd);
	if ( alias != mAliasMap.end() )
	{
		// Create substitution of com in string command for alias
		string tmp = alias->second;
		string str;
		getline(s, str);
		if (str.length() > 0)
		{
			tmp.append(str);
		}
	
		// fill new unaliased command to s and read cmd
		s.str(tmp);
		s.seekg (0, ios::beg);
		s.clear();
		s >> cmd;

		//you coudn't make alias for build in commands 
		//processCommand(tmp, playerID);
		//return;
	}	
	
	// Server commands
	if (cmd.size() > 0 && cmd[0] == '/')
	{
		command_to_server(command);
		return;
	}


	// Find command in our table
	commads_hash::iterator it = mCommandsHash.find(cmd);

	if (it == mCommandsHash.end())
	{
		// this is last to catch invalid commands
		getAppClient().getConsole().addMessage( _("Unknown command") );
	}
	else
	{
		(it->second)(s, playerID);
	}
}


void CommanderClient::command_to_server(const std::string &args)
{
	LOGT << "Sending to server: " << args << endl;

	getAppClient().getNetworkManager().sendCommand(args);
}

void CommanderClient::command_quit(std::stringstream &args, int playerID)
{
		getAppClient().quit();	
} 

void CommanderClient::command_help(std::stringstream &args, int playerID)
{
	Console &c = getAppClient().getConsole();
	
	c.addMessage( _("Help for BJS console") );
	c.addMessage( _("List of known commands:") );

/*	commads_hash::iterator it = mCommandsHash.begin();
	for(; it != mCommandsHash.end(); ++it)
	{
		c.addMessage( it->first );
	}
*/
	c.addMessage( _(" see doc/usage/console_commands.txt") );
}
	
void CommanderClient::command_getvar(std::stringstream &args, int playerID)
{
	string varname;
	args >> varname;
	
	if ( !g_settings.exist(varname.c_str()) ) 
	{
		getAppClient().getConsole().addMessage( _("Unknown variable") );
		return;
	}
	
	getAppClient().getConsole().addMessage( g_settings.getString(varname.c_str()) );
}
	
void CommanderClient::command_setvar(std::stringstream &args, int playerID)
{
	string varname, value;
	args >> varname >> value;
	
	if (varname.empty() || value.empty())
	{
		getAppClient().getConsole().addMessage( _("Usage setvar name value") );
		return;
	}

	if ( !g_settings.exist(varname.c_str()) ) 
	{
		getAppClient().getConsole().addMessage( _("Unknown variable") );
		return;
	}

	g_settings.set(varname, value);	
}

void CommanderClient::command_players(std::stringstream &args, int playerID)
{
	const vector<int> &pl = getAppClient().getGame().getValidPlayersID();
	Console &c = getAppClient().getConsole();
	
	stringstream msg;
	msg << _("Number of players: ") << pl.size();
	c.addMessage( msg.str() );

	for (int i = 0; i < (int)pl.size(); ++i)
	{
		stringstream msg;
		Player &p = getAppClient().getGame().getPlayer(pl[i]);
		msg << p.getID() << " " << p.getName();
		c.addMessage(msg.str());
	}
}
void CommanderClient::command_sound(std::stringstream &args, int playerID)
{
	string params;
	args >> params;
	bool enable;
	
	
	if (try_get_bool(params, enable))
	{
		getAppClient().getSFX().enableSamples(enable);
	}
	else
	{
		getAppClient().getConsole().addMessage(_("Usage: sound on|off"));
	}
}

void CommanderClient::command_music(std::stringstream &args, int playerID)
{
	string params;
	args >> params;
	bool enable;
	
	
	if (try_get_bool(params, enable))
	{
		getAppClient().getSFX().enableMusic(enable);
	}
	else
	{
		getAppClient().getConsole().addMessage(_("Usage: music on|off"));
	}
}


void CommanderClient::command_soundtheme(std::stringstream &args, int playerID)
{
	
	string theme;
	args >> theme;

	getAppClient().getSFX().loadTheme(theme);
}

void CommanderClient::command_chatmessage(std::stringstream &args, int playerID)
{
	args.get(); // the space character

	// Test if we will translate
	int c = args.get(); 

	bool translate = false;	
	if (c == NOT_TRANSLATE_CHAR)
	{
		translate = true;
	}
	else
	{
		args.unget();
		translate = false;
	}
			
	
	string msg;
	getline(args, msg);

	if (translate) 	getAppClient().getGame().getGUI().addMessage(_(msg.c_str()));
	else 		    getAppClient().getGame().getGUI().addMessage(msg);
}
	
void CommanderClient::command_consolemessage(std::stringstream &args, int playerID)
{
	args.get(); // the space character

	// Test if we will translate
	int c = args.get(); 

	bool translate = false;	
	if (c == NOT_TRANSLATE_CHAR)
	{
		translate = false;
	}
	else
	{
		args.unget();
		translate = true;
	}
			
	string msg;
	getline(args, msg);

	if (translate) 	getAppClient().getConsole().addMessage(_(msg.c_str()));
	else 	        getAppClient().getConsole().addMessage(msg);
}

void CommanderClient::command_log_in_console(std::stringstream &args, int playerID)
{
	string params;
	args >> params;
	bool enable;
	
	
	if (try_get_bool(params, enable))
	{
		getAppClient().getConsole().setLogInConsole(enable);
	}
	else
	{
		getAppClient().getConsole().addMessage(_("Usage: loginconsole on|off"));
	}
}


void CommanderClient::command_resolution(std::stringstream &args, int playerID)
{
	string resolution;
	args >> resolution;
	g_settings.set(SETTING_RESOLUTION, resolution);
	
	getAppClient().getGFX().reloadGFXSetting();
}


void CommanderClient::command_fovx(std::stringstream &args, int playerID)
{
	string fovx;
	double tmp;
	args >> fovx;

	if (try_get_double(fovx, tmp))
	{
		g_settings.set(SETTING_FOVX, fovx);
	
		getAppClient().getGFX().computePerspective();
	}
	else
	{
		getAppClient().getConsole().addMessage(_("Usage: fovx doublevalue"));
	}
}
	
void CommanderClient::command_fs(std::stringstream &args, int playerID)
{
	string params;
	args >> params;
	bool enable;
	
	
	if (try_get_bool(params, enable))
	{
		g_settings.set(SETTING_FULLSCREEN, params);
		
		getAppClient().getGFX().reloadGFXSetting();
	}
	else
	{
		getAppClient().getConsole().addMessage(_("Usage: fs on|off"));
	}

}

void CommanderClient::command_clear(std::stringstream &args, int playerID)
{
	getAppClient().getConsole().clear();
}

void CommanderClient::command_grab(std::stringstream &args, int playerID)
{
	string params;
	args >> params;
	bool enable;
	
	if (try_get_bool(params, enable))
	{
		if (enable)
			SDL_WM_GrabInput(SDL_GRAB_ON);
		else
			SDL_WM_GrabInput(SDL_GRAB_OFF);
	}
	else
	{
		getAppClient().getConsole().addMessage(_("Usage: grab on|off"));
	}
}


void CommanderClient::command_bind(std::stringstream &args, int playerID)
{

	string key, priority;

	args >> key;

	if (args.fail())
	{
			getAppClient().getConsole().addMessage(_("Usage: bind key down(bool) priority(bool) command"));
			return;
	}

	//TODO: lowercase key

	
	bool down;
	// use mode +key -key
	if ((key[0] == '+' || key[0] == '-') && key.size() != 1)
	{
		down = key[0] == '+';
		key.erase(0,1);
	}
	// use mode with explicit writing
	else
	{
		string downS;
		args >> downS;
		if (!try_get_bool(downS, down))
		{
			getAppClient().getConsole().addMessage(_("Usage: bind key down(bool) priority(bool) command"));
			return;
		}
	}

	args >> priority;
	bool prior;
	if (!try_get_bool(priority, prior))
	{
		getAppClient().getConsole().addMessage(_("Usage: bind key down(bool) priority(bool) command"));
		return;
	}

	// Loading command
	args >> ws; // white skip

	string cmd;
	getline(args, cmd);
	if (cmd[0] == '\0') 
	{
		getAppClient().getConsole().addMessage(_("Usage: bind key down(bool) priority(bool) command"));
		return;
	}


	// Apply :-)
	if (!getAppClient().getKeyBindings().bind(key, down, cmd, prior))
	{
		getAppClient().getConsole().addMessage(_("Usage: bind key down(bool) priority(bool) command"));
	}
}


void CommanderClient::command_key(std::stringstream &args, int playerID)
{

	string button, values;
	args >> button >> values;

	bool value;
	if ( !try_get_bool(values, value) ||
	     !getAppClient().getKeyBindings().setButton(button, value) )
		
	{
		getAppClient().getConsole().addMessage(_("Usage: key button true/false"));
		return;
	}
	
	
}


void CommanderClient::command_unbind_all(std::stringstream &args, int playerID)
{
	getAppClient().getKeyBindings().unbindAll();
}

void CommanderClient::command_exec(std::stringstream &args, int playerID)
{
    string file;
	args >> file;
	ifstream config( locateFile( file ).c_str() );
	if ( !config )
	{
		getAppClient().getConsole().addMessage(_("File not found"));
		return;
	}

	string commandline;
	while ( getline( config, commandline ) )
	{
		getAppClient().getCommander().addCommand(commandline);
	}
}

void CommanderClient::command_console(std::stringstream &args, int playerID)
{
	getAppClient().getConsole().changeActivity();
}

void CommanderClient::command_sleep(std::stringstream &args, int playerID)
{
	int ms;
	args >> ms;
	gTime.sleep(ms);
}

void CommanderClient::command_alias(std::stringstream &args, int playerID)
{
	string name; // name of alias
	
	args >> name;
	
	// Loading command
	args >> ws; // white skip

	string cmd;
	getline(args, cmd);
	if (cmd[0] == '\0') 
	{
		getAppClient().getConsole().addMessage(_("Usage: alias key priority(bool) command"));
		return;
	}

	// put alias into alias table
	mAliasMap[name] = cmd;
}

void CommanderClient::command_unalias(std::stringstream &args, int playerID)
{
	mAliasMap.clear();
}


void CommanderClient::command_picked_bonus(std::stringstream &args, int playerID)
{
	int id;
	args >> id;
	
	if (args.fail()) return;
	if (id < 0 || (unsigned int)id >= gParams.mBonus.size()) return;

	// Amuse user
	getAppClient().getGame().getGUI().pickedBonus(id);
}

void CommanderClient::command_expired_bonus(std::stringstream &args, int playerID)
{
	int id;
	args >> id;
	
	if (args.fail()) return;
	if (id < 0 || (unsigned int)id >= gParams.mBonus.size()) return;

	// Amuse user
	getAppClient().getGame().getGUI().expiredBonus(id);

}

void CommanderClient::command_pause(std::stringstream &args, int playerID)
{
	bool  pause;
	args >> pause;
	
	if (args.fail()) return;
	gTime.pause(pause);

}

void CommanderClient::command_activate_chat(std::stringstream &args, int playerID)
{
	getAppClient().getGame().getGUI().activateChatWindow();
}

void CommanderClient::command_coords(std::stringstream &args, int playerID)
{
	Player *pl = getAppClient().getGame().getClientPlayer();
	if (pl && pl->getTank())
	{
		Tank *t = pl->getTank();

		t->getPosition();
		Console &c = getAppClient().getConsole();
	
		stringstream msg;
		msg << _("Coords: ") << t->getPosition();
		c.addMessage( msg.str() );
	}
}

void CommanderClient::command_targeting(std::stringstream &args, int playerID)
{
	
	string yawS;
	args >> yawS;

	bool yaw;
	
	// apply changes
	MouseMotion &mm = getAppClient().getGame().getMouseMotion();	
	if ( try_get_bool(yawS, yaw))
	{
		mm.setFloatingView(yaw);
	}
	else
	{
		mm.setFloatingView(!mm.getFloatingView());
	}
}


void CommanderClient::command_mousesensitivity(std::stringstream &args, int playerID)
{
	double sensX, sensY;
	args >> sensX >> sensY;

	if (args.fail())
	{
		getAppClient().getConsole().addMessage(
			_("Usage: mousesensitivity x_sensitivity y_sensitivity"));
		return;
	}

	// apply changes
	MouseMotion &mm = getAppClient().getGame().getMouseMotion();
	mm.setYawSensitivity(sensX);
	mm.setPitchSensitivity(sensY);
}


void CommanderClient::command_resettargeting(std::stringstream &args, int playerID)
{
	getAppClient().getGame().getMouseMotion().init();
}


void CommanderClient::command_showstats(std::stringstream &args, int playerID)
{
	string visibleS;
	args >> visibleS;

	bool visible;
	if (!try_get_bool(visibleS, visible))
	{
		getAppClient().getConsole().addMessage(
			_("Usage: showstats visible(bool)"));
		return;
	}

	getAppClient().getGame().getGUI().showStatsWindow(visible);
}

void CommanderClient::command_connect(std::stringstream &args, int playerID)
{
	string address;
	unsigned short port;	

	// Load address
	args >> address;
	if (args.fail())
	{
		getAppClient().getConsole().addMessage(
			_("Usage: connect server_address [server_port]"));
		return;
	}

	// Load port
	args >> port;
	if (args.fail())
	{
		// use "default" port for connect
		port = (unsigned short) g_settings.getInt(SETTING_TMP_NETWORK_CONNECT_PORT);
	}

	// Test if application not connected
	if (getAppClient().getNetworkManager().connected())
	{
		getAppClient().getConsole().addMessage(
			_("Allready connected to the server."));
		return;
	}

	int try_cnt   = g_settings.getInt(SETTING_CONNECT_TRY_CNT);
	int try_sleep = g_settings.getInt(SETTING_CONNECT_TRY_SLEEP);

	// Get IOR from server
	ServerInfo sinfo;

	bool got_ior = false;
	for ( int i = 0; i < try_cnt; i++ )
	{
		LOGI << "Connecting to server " << address << ":" << port << "; try " << (i+1) << " of " << try_cnt << endl;
		if ( getServerIORByAddress( address.c_str(), port, sinfo ) )
		{
			got_ior = true;
			break;
		}

		gTime.sleep(try_sleep);
	}

	if (got_ior)
	{
		getAppClient().runGame(sinfo);
	}
	else
	{
		getAppClient().getConsole().errorMessage( _("Unable to connect to the server.") );
		getAppClient().getConsole().addMessage( _("Unable to connect to the server."));
	}
}

void CommanderClient::command_empty(std::stringstream &args, int playerID)
{

}


void CommanderClient::command_disconnect(std::stringstream &args, int playerID)
{
	if (!getAppClient().getNetworkManager().connected())
	{
		getAppClient().getConsole().addMessage(
			_("You must be connected before disconnecting."));
	}

	getAppClient().getGame().endFun();
}


void CommanderClient::command_camera(std::stringstream &args, int playerID)
{
	double positionZplus, positionScale, refPointZplus, refPointScale;

	args >> positionZplus >> positionScale >> refPointZplus >> refPointScale;
	if (args.fail())
	{
		getAppClient().getConsole().addMessage(
			_("Usage: camera zplus distance ref_zplus ref_distance"));
		return;
	}
	
	getAppClient().getGame().getTankTracer().setParams(positionZplus, positionScale, refPointZplus, refPointScale);
}


void CommanderClient::command_observer(std::stringstream &args, int playerID)
{
	Observer &observer = getAppClient().getGame().getObserver();
	if (observer.isActive())
	{
		observer.deactivate();
	}
	else
	{
		observer.activate();
	}
}

void CommanderClient::command_observerspeed(std::stringstream &args, int playerID)
{
	double speed;

	args >> speed;
	if (args.fail())
	{
		getAppClient().getConsole().addMessage(
			_("Usage: observerspeed speed"));
	}

	Observer &observer = getAppClient().getGame().getObserver();
	observer.setSpeed(speed);
}


void CommanderClient::command_ingamemenu(std::stringstream &args, int playerID)
{
	GameGUI &game_gui = getAppClient().getGame().getGUI();

	if (game_gui.isMenuActive())
	{
		game_gui.deactivateMenu();
	}
	else
	{
		game_gui.activateMenu();
	}
}
	


void CommanderClient::command_reconnect(std::stringstream &args, int playerID)
{
	getAppClient().reconnect();
}

void CommanderClient::command_soundvolume(std::stringstream &args, int playerID)
{	
	int volume;
	args >> volume;
	if (args.fail())
	{
		getAppClient().getConsole().addMessage(_("Usage: soundvolume volume"));	
	}
	
	getAppClient().getSFX().setSamplesVolume(volume);
}

void CommanderClient::command_musicvolume(std::stringstream &args, int playerID)
{
	int volume;
	args >> volume;
	if (args.fail())
	{
		getAppClient().getConsole().addMessage(_("Usage: musicvolume volume"));	
	}
	
	getAppClient().getSFX().setMusicVolume(volume);	
}

void CommanderClient::command_newtank(std::stringstream &args, int playerID)
{
	getAppClient().getCommander().addCommand("/newtank " + g_settings.getString(SETTING_PLAYER_PREFERED_TANK));
}
	
void CommanderClient::command_setpreferedname(std::stringstream &args, int playerID)
{
	getAppClient().getCommander().addCommand("/changename " + g_settings.getString(SETTING_PLAYER_PREFERED_NAME));		
}
		
	
