/* MAINTAINER: Lukas */

#include "events_client.h"
#include "application_client.h"
#include "log.h"
#include "history.h"
#include "globals.h"
#include "world.h"
#include "commander.h"
#include "game_client.h"
#include "tank.h"
#include "projectile.h"
#include "explosion_manager.h"
#include "game.h"
#include "gamegui.h"
#include "timemanager.h"

template<class T>
void EvBasicWorldEntityChanged::loadBasic(const T & data)
{
	assertL(mHistPoint != 0);

	// orientation
	mHistPoint->mOrientation.set(data.orientation.w,
		data.orientation.x, data.orientation.y,
		data.orientation.z);

	// position
	mHistPoint->mPosition.x = data.position.x;
	mHistPoint->mPosition.y = data.position.y;
	mHistPoint->mPosition.z = data.position.z;

	// speed
	mHistPoint->mSpeed.x = data.speed.x;
	mHistPoint->mSpeed.y = data.speed.y;
	mHistPoint->mSpeed.z = data.speed.z;

	// time stamp
	setTimeStamp(data.timeStamp);
}

EvBasicWorldEntityChanged::EvBasicWorldEntityChanged(const NETWEBasicType & data)
{
	mID = data.ID;

	// create history point
	mHistPoint = new HistoryPoint();
	if (mHistPoint == 0) DIE("new failed");

	//transfer data from IDL to HistoryPoint
	loadBasic(data);
}


EvTankWorldEntityChanged::EvTankWorldEntityChanged(const NETWETank & data)
{
	mID = data.basic.ID;

	// create history point
	TanksHistoryPoint *tp = new TanksHistoryPoint();
	if (tp == 0) DIE("new failed");
	mHistPoint = tp;

	//transfer data from IDL to HistoryPoint

	// basic part
	loadBasic(data.basic);

	// tank specific
	tp->mPitch       = data.pitch; // Doesn't have to be in client's main version
	tp->mYaw         = data.yaw;   // Doesn't have to be in client's main version
	//tp->mPitch       = 0
	//tp->mYaw         = 0;
	tp->mWeapon      = data.weapon;
	tp->mProjectiles = data.projectiles;
	tp->mHP          = data.hp;
	tp->mArmour      = data.armour;
}

/*
EvTankWorldEntityChanged::EvTankWorldEntityChanged(const NETWESimpleTankType & data)
{
	mID = data.ID;

	// create history point
	TanksHistoryPoint *tp = new TanksHistoryPoint();
	if (tp == 0) DIE("new failed");
	mHistPoint = tp;

	//transfer data from IDL to HistoryPoint

	// basic part
	loadBasic(data);

	// tank specific
	tp->mPitch       = data.mPitch;
	tp->mYaw         = data.mYaw;
	tp->mWeapon      = data.mWeapon;
	tp->mProjectiles = 1; // data.mProjectiles; - not is simple version
	tp->mHP          = data.mHP;
	tp->mArmour      = 0; // data.mArmour; - not in simple version
}
*/

EvNewTank::EvNewTank(const NETNewTank & data)
{
	// orientation
	mOrientation.set(data.orientation.w,
		data.orientation.x, data.orientation.y,
		data.orientation.z);

	// position
	mPosition.x = data.position.x;
	mPosition.y = data.position.y;
	mPosition.z = data.position.z;

	mID = data.id;
	mPlayer_id = data.player;
	mType_id = data.type;
}

EvNewProjectile::EvNewProjectile(const NETNewProjectile & data)
{
	// orientation
	mOrientation.set(data.orientation.w,
		data.orientation.x, data.orientation.y,
		data.orientation.z);

	// position
	mPosition.x = data.position.x;
	mPosition.y = data.position.y;
	mPosition.z = data.position.z;

	mID = data.id;
	mShooter = data.player;
	mType_id = data.type;
}

EvNewBonusBox::EvNewBonusBox(const NETNewBonusBox & data)
{
	// orientation
	mOrientation.set(data.orientation.w,
		data.orientation.x, data.orientation.y,
		data.orientation.z);

	// position
	mPosition.x = data.position.x;
	mPosition.y = data.position.y;
	mPosition.z = data.position.z;

	mID = data.id;
	mBonus_function_id = data.type;
}

EvNewCollision::EvNewCollision(const NETCollision & data)
{
	mCollision = new Collision(data.entity1ID, data.entity2ID,
		// position
		GLVector3d(data.position.x, data.position.y, data.position.z),
		// normal
		GLVector3d(data.normal.x, data.normal.y, data.normal.z));
}

void EvBasicWorldEntityChanged::process()
{
	World * wrld = &(getAppClient().getGame().getWorld());

	WorldEntity * we = wrld->getWorldEntity( mID );


//if (we && we->getClassType()== cTank)
//	LOGD << "Received update for entity " << mID <<" position: " << mHistPoint->mPosition << endl;

	// This is a "fix" by Bernard for message with not existing ID
	if ( we )
	{
		we->updateHistory( mHistPoint );
	}
	else
	{
		LOGE << "Almost FATAL: processing entity change to non-existing entity!!!" << endl;
	}
}

void EvWorldUpdateEnd::process()
{
	World * wrld = &(getAppClient().getGame().getWorld());
//LOGD << "Commit start" << endl;
	worldent_iterator i		= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	int ntime = gTime.getGameTicks();
	int time = ntime - getAppClient().getGame().getCommitTime();

	while ( i != end )
	{
		(*i)->commitHistoryUpdate( time );
		++i;
	}	

	getAppClient().getGame().setCommitTime( gTime.getGameTicks() );
//LOGD << "Commit end" << endl;
}

void EvCommand::process()
{
	getAppClient().getCommander().addCommand(mCommand);
}

void EvNewCollision::process()
{
	// TODO dodelat
	assertL(mCollision)
	if (mCollision == 0) { return; }
	
	int explosionType = 0;
	
	World &world = getAppClient().getGame().getWorld();
	WorldEntity * we = world.getWorldEntity( mCollision->getWe1ID() );

	// The entity isn't there any more;
	if (!we ) return;
	if (we->getClassType() != cProjectile) 
	{
		LOGE << "Explosion caused by non projectile?" << endl;
		return;
	}
	
	Projectile *p = dynamic_cast<Projectile*>(we);
	assertL(p);
	int projectileType = p->getParamsID();
	if (projectileType == -1) return;
	
	// Hit to entity
	if (mCollision->getWe2ID() != INVALID_ENTITY_ID)
	{
		explosionType = gParams.projectile(projectileType).explosion_hit_entity_id;
	}
	else // Hit to ground
	{
		explosionType = gParams.projectile(projectileType).explosion_hit_ground_id;		
	}

	GLVector3d tmp1( mCollision->getPosition() ), tmp2( mCollision->getNormal() );
	getAppClient().getGame().getExplosionManager().addExplosion(
		explosionType, 
		GLVector3f(GLfloat(tmp1.x), GLfloat(tmp1.y), GLfloat(tmp1.z)), 
		GLVector3f( GLfloat(tmp2.x), GLfloat(tmp2.y), GLfloat(tmp2.z) ) );
}

void EvNewTank::process()
{
	if ( !getAppClient().getGame().getWorld().createTank( mPosition, mOrientation, mType_id, mPlayer_id, mID ) )
		LOGE << "Error creating tank! ( ID: " << mID << " )" << endl;
}

void EvNewProjectile::process()
{
	if ( !getAppClient().getGame().getWorld().createProjectile( mPosition, mOrientation, mType_id, mShooter, mID ) )
		LOGE << "Error creating projectile! ( ID: " << mID << " )" << endl;
}

void EvNewBonusBox::process()
{
	if ( !getAppClient().getGame().getWorld().createBonusBox( mPosition, mOrientation, mBonus_function_id, mID, false ) )
		LOGE << "Error creating bonus box! ( ID: " << mID << " )" << endl;
}

void EvDeletedObj::process()
{
	WorldEntity * we = getAppClient().getGame().getWorld().getWorldEntity( mID );
	if ( !we )
		LOGE << "Error deleting object! No such object! ( ID: " << mID << " )" << endl;

	if ( !getAppClient().getGame().getWorld().deleteObjectGeneric( mID ) )
				LOGE << "Error deleting object! ( ID: " << mID << " )" << endl;
/*
	switch ( we->getClassType() )
	{
		case cTank:
		{
			if ( !getAppClient().getGame().getWorld().deleteTank( mID ) )
				LOGE << "Error deleting tank! ( ID: " << mID << " )" << endl;
			break;
		}
		case cProjectile:
		{
			if ( !getAppClient().getGame().getWorld().deleteProjectile( mID ) )
				LOGE << "Error deleting projectile! ( ID: " << mID << " )" << endl;
			break;
		}
		case cBonusBox:
		{
			if ( !getAppClient().getGame().getWorld().deleteBonusBox( mID, false ) )
				LOGE << "Error deleting bonus box! ( ID: " << mID << " )" << endl;
			break;
		}
		default:
			DIE( "Processing delete event with unknown or bad class type" );
	}
*/
}


/* -----------   PLAYER EVETS  ----------   */

void EvNewPlayer::process()
{
	Player p;
	p.setName(mName);
	getAppClient().getGame().newPlayer(p, mID);
}

EvPlayerChanged::EvPlayerChanged(const NETPlayer & data)
{
	mID     = data.pID;
	mKills  = data.kills;
	mDeaths = data.deaths;
	mScore  = data.score;
	mTeam   = data.team;
	mName   = data.name;
	mReady  = data.ready;
}

void EvPlayerChanged::process()
{
	if (getAppClient().getGame().isPlayerIDValid(mID))
	{
		Player &p = getAppClient().getGame().getPlayer(mID);
		p.setKills(mKills);
		p.setDeaths(mDeaths);
		p.setScore(mScore);
		p.setTeam(mTeam);
		p.setName(mName);
		p.ready(mReady);
	}
	else
	{
		LOGE << "Invalid player ID to process. " << endl;
	}
}

void EvDeletedPlayer::process()
{
#ifdef _DEBUG
	World * wrld = &getApp().getGameCommon().getWorld();
	worldent_iterator it = wrld->getWorldEntitiesBegin();
	worldent_iterator ite = wrld->getWorldEntitiesEnd();
	for ( ; it != ite; ++it )
	{
		WorldEntity * we = wrld->getWorldEntity( it );
		assertL( we );

		switch ( we->getClassType() )
		{
		case cTank:
			{
				if ( dynamic_cast< Tank* >( we )->getPlayerID() == mID )
				{
					DIE( "There exist tank for deleted player!" );
				}
				break;
			}

		case cProjectile:
			{
				if ( dynamic_cast< Projectile* >( we )->getOwner()->getID() == mID )
				{
					DIE( "There exists projectile that fired deleted player!" );
				}
				break;
			}
		default:
			break;
		}
	}
#endif

	getAppClient().getGame().removePlayer(mID);
}

void EvStateOfGame::process()
{
	getAppClient().getGame().setStateOfGame( mSOG );
}

void EvConditionStatusList::process()
{
	getAppClient().getGame().getGUI().setEndConditionStatuses( mConditionStatusList );
}
