/* MAINTAINER: Lukas */

#ifndef _EVENTS_CLIENT_H
#define _EVENTS_CLIENT_H

#include "event.h"
#include "netobjects.h"
#include "GLVector.h"
#include "GLQuaternion.h"
#include <string>
#include "collision_bjs.h"

//enum eStateOfGame;
#include "game.h"

class HistoryPoint;

/**
 * Event indicates that some basic WorldEntity was changed
 */
class EvBasicWorldEntityChanged : public Event
{

public:
	/**
	 * Constructor takes (NETWEBasicType) data and extracts it
	 * into its private mHistPoint
	 *
	 * \param data it contains packed WE changes
	 */
	EvBasicWorldEntityChanged(const NETWEBasicType & data);
	virtual ~EvBasicWorldEntityChanged() { };

	// TODO comment
	virtual void process();

	/**
	 * Returns ID of changed WorldEntity
	 *
	 * \return WorldEntity ID
	 */
	int getObjectID() { return mID; };

	/**
	 * Changed WorldEntity data
	 */
	HistoryPoint * mHistPoint;


protected:
	/**
	 * ID of changed WorldEntity
	 */
	int mID;

	EvBasicWorldEntityChanged() {}

	/**
	 * \brief Loading HistoryPoint from data.
	 */
	template<class T>
	void loadBasic(const T & data);
};


/**
 * Event indicates that some Tank was changed
 */
class EvTankWorldEntityChanged : public EvBasicWorldEntityChanged
{

public:
	/**
	 * Constructor takes (NETWETank) data and extracts it
	 * into its private mTankHistPoint
	 *
	 * \param data it contains packed WE changes
	 */
	EvTankWorldEntityChanged(const NETWETank & data);
//	EvTankWorldEntityChanged(const NETWESimpleTankType & data);
	virtual ~EvTankWorldEntityChanged() { };
};

/**
 * Event indicates end of game state update
 */
class EvWorldUpdateEnd : public Event
{

public:
	
	EvWorldUpdateEnd() { };
	virtual ~EvWorldUpdateEnd() { };

	// TODO comment
	virtual void process();

private:

};

/**
 * Command from server
 */
class EvCommand : public Event
{

public:
	
	EvCommand(std::string command) { mCommand = command; };
	virtual ~EvCommand() { };

	/**
	 * Calls commander that cares about rest
	 */
	virtual void process();

private:

	std::string mCommand;

};

/**
 * Command from server
 */
class EvNewCollision : public Event
{

public:
	
	EvNewCollision(const NETCollision & data);
	virtual ~EvNewCollision() { delete mCollision; };

	// TODO comment
	virtual void process();

private:

	Collision * mCollision;

};

/**
 * New tank is created
 */
class EvNewTank : public Event
{

public:
	
	EvNewTank(const NETNewTank & data);
	virtual ~EvNewTank() { };

	// TODO comment
	virtual void process();

private:

	GLVector3d mPosition;
	GLQuaterniond mOrientation;
	int mType_id;
	int mPlayer_id;
	int mID;

};

/**
 * New projectile is created
 */
class EvNewProjectile : public Event
{

public:
	
	EvNewProjectile(const NETNewProjectile & data);
	virtual ~EvNewProjectile() { };

	// TODO comment
	virtual void process();

private:

	GLVector3d mPosition;
	GLQuaterniond mOrientation;
	int mType_id;
	int mShooter;
	int mID;

};

/**
 * New bonus box is created
 */
class EvNewBonusBox : public Event
{

public:
	
	EvNewBonusBox(const NETNewBonusBox & data);
	virtual ~EvNewBonusBox() { };

	// TODO comment
	virtual void process();

private:

	GLVector3d mPosition;
	GLQuaterniond mOrientation;
	int mBonus_function_id;
	int mID;

};

/**
 * Objects was deleted
 */
class EvDeletedObj : public Event
{

public:
	
	EvDeletedObj(int ID) { mID = ID; };
	virtual ~EvDeletedObj() { };

	// TODO comment
	virtual void process();

private:

	int mID;

};

/**
 * New player is created (enterd the game)
 */
class EvNewPlayer : public Event
{
public:

	EvNewPlayer(const NETNewPlayer & data) { mID = data.pID; mName = data.name; }
	virtual ~EvNewPlayer() { }

	virtual void process();

private:
	int mID;
	std::string mName;
};


/**
 * Player update
 */
class EvPlayerChanged : public Event
{
public:

	EvPlayerChanged(const NETPlayer & data);
	virtual ~EvPlayerChanged() { }

	virtual void process();

private:
	int mID;
	int mKills;
	int mDeaths;
	int mScore;
	int mTeam;
	bool mReady;
	std::string mName;
};

/**
 * Delete player (the player quit the game)
 */
class EvDeletedPlayer : public Event
{
public:

	EvDeletedPlayer(int ID) { mID = ID; }
	virtual ~EvDeletedPlayer() { }

	virtual void process();

private:
	int mID;
};

/**
 * State of game
 */
class EvStateOfGame : public Event
{
public:

	EvStateOfGame(eStateOfGame sog) { mSOG = sog; }
	virtual ~EvStateOfGame() { }

	virtual void process();

private:
	eStateOfGame mSOG;
};

/**
 * Player update
 */
class EvConditionStatusList : public Event
{
public:

	EvConditionStatusList(std::list<ConditionStatus> & csList) { mConditionStatusList = csList; };
	virtual ~EvConditionStatusList() { }

	virtual void process();

private:
	std::list<ConditionStatus> mConditionStatusList;

};

#endif // _EVENTS_CLIENT_H
