/* MAINTAINER: Bernard */

#ifndef _FONT_MANAGER_H
#define _FONT_MANAGER_H


#include "hash_hack.h"
#include "stlextdefs.h"
#include "universal_manager.h"
#include "font.h"
#include "task.h"

// Some font types will be defined

//FONT_MENU_BIG...
/**
 * \brief Singleton that creates and manages fonts.
 *
 * Usage:
 * 
 * FontRef font;  // create
 * font = FontManager::getRef().get("FreeMono.ttf", 20);
 * ..
 * font->RenderUTF8("Hello World!", 30, 30); // usage
 * ..
 * font = 0;   // destroy
 *
 * The manager is derived from Task becasue fonts need to clean
 * cache sometimes so regular call to FontManager is needed.
 * You may run out of memory if you don't call update :-)
 * The manager isn't thread safe.
 */
class FontManager : public UniversalManager<Font, FontManager>, public Task
{
protected:
	friend class Singleton<FontManager>;	///< Singleton if a friend of mine! (stupid solution...)

	FontManager();
	~FontManager();

public:
//	TRef getFont(int type);

	/**
	 * \brief Get some font.
	 * 
	 * Same font (reference) may be returned in mutiple calls.
	 * \param font_name Filename with font. The file is located.
	 * \param size Size of font in some font points.
	 * \return Reference to font.
	 */
	TRef get(const char* font_name, int size);

	/**
	 * \brief Font's cache cleanup.
	 * 
	 * We need this because font's caches must be 
	 * regullary cleaned. Should be called from task.
	 */
	virtual void update(int dtime, int atime);

private:
	int mNextCleanup; ///< \brief Miliseconds until next cleanup.
	
};

typedef FontManager::TUniversalManager::TRef FontRef; ///< \brief can be used in place of FontManagerManager::FontRef


// Used for the explicit template instantiation in template_instantces_*.cpp file
#ifdef INSTANTIATE_TEMPLATE
	template class Singleton<FontManager>;
#endif


#endif // _FONT_MANAGER_H
