/* MAINTAINER: Bernard */

#ifndef _SFX_H
#define _SFX_H

#include <vector>
#include <string>
#include "isound.h"
#include "musicmanager.h"
#include "soundtheme.h"

/**
 * \brief Handle sound and provide access to music subsystem.
 * 
 * 
 */
class SFX
{
public:
	SFX();
	~SFX();

	void init();
	
	void clean();

	/**
	 * \brief get Music manager
	 */
	MusicManager & getMusicManager()
	{
		return mMusicManager;
	}
	
	/**
	 * \brief Provide access to samples manager.
	 */
	ISoundSamplesManager &getSamplesManager();

	
	/**
	 * \brief Return current sound system
	 */
	ISoundSystem& getSoundSystem();


	// play some sound - locateFile will be performed 
	// somewhere inside
	void play(const char *file);
	
	void loadTheme(const std::string &file);
	
	void loadSettingTheme();
	
	void enableSamples(bool enable = true);
	void enableMusic(bool enable = true);
	void setSamplesVolume(int volume);
	void setMusicVolume(int volume);

	bool isSamplesEnabled()
	{
		return mSamplesEnabled;
	}
	
	bool isMusicEnabled()
	{
		return mMusicEnabled;
	}
	
	bool getOpenALDevices(std::vector<std::string>& deviceV);
	
	const std::vector<std::string>& getThemes() { return mThemes; }
	
private:
	ISoundSystem *mSoundSystem; // \brief workink system
	ISoundSystem *mSoundSystemEmpty; // \brief empty for case of disable sound
	MusicManager  mMusicManager;

	bool mSamplesEnabled;
	bool mMusicEnabled;
	
	
	std::vector<std::string> mThemes; // List of all themes (filenames)
	void loadThemesList();
	
};

#endif

