/* MAINTAINER: Martin */

#include "tank.h"

#include "grutils.h"
using namespace grutils;

#include "render_visitor.h"
#include "log.h"
#include "platform.h" // testing
#include "model.h"
#include "animation.h"
#include "ac_transformation.h"
#include "history.h"

#include "application_client.h"
#include "game_client.h"
#include "explosion_manager.h"

bool Tank::init()
{
	if ( !CommonEntity::init() )
		return false;

	Model * m = getModel();
	if ( !m )
		return false;

	const std::string turretName("turret"), gunName("gun");
	
	Animation * anim = m->createAnimation( "tank_controll" );
	{
		// specialni kontroler pro vezku
		class TurretAnimSource : public AnimationDataSource< GLMatrix4d >
		{
		public:
			TurretAnimSource( const Tank * tankInstance ):
			  mTankInstance(tankInstance)
			{}

			virtual void getValue( value_type & value, unsigned int time_ms )
			{
				value.loadRotateZ( mTankInstance ? mTankInstance->getTankYaw() : 0 );
			}

		private:
			const Tank * mTankInstance;
		};

		// specialni kontroler pro gun
		class GunAnimSource : public AnimationDataSource< GLMatrix4d >
		{
		public:
			GunAnimSource( const Tank * tankInstance ):
			  mTankInstance(tankInstance)
			{}

			virtual void getValue( value_type & value, unsigned int time_ms )
			{
				value.loadRotateY( mTankInstance ? -mTankInstance->getTankPitch() : 0 );
			}

		private:
			const Tank * mTankInstance;
		};

		;

		// kontroler transformace veze tanku
		{
			std::auto_ptr<AnimationController> p( new TransformationController( new TurretAnimSource( this ), turretName ) );
			anim->createController( p ); // p predal vlastnictvi dovnitr procedury		
		}
					
		// kontroler transformace kanonu tanku
		{
			std::auto_ptr<AnimationController> p( new TransformationController( new GunAnimSource( this ), gunName ) );
			anim->createController( p );
		}

		// povolime animaci, jeji efekt se projevi v pristim volani update
		anim->enable(true);
		return true;
	}

	return false;
}

void Tank::render() 
{
}

void TanksHistoryPoint::transferToNET(NETGameState & data)
{
	// no use for this function on client

	TIN

	DIE();
}

void TanksHistoryPoint::transferToNETDirect(NETWETank & data)
{
	// no use for this function on client

	TIN

	DIE();
}

void Tank::packCreationData(NETGameState & data)
{
	// no use for this function on client

	TIN

	DIE();
}

void Tank::setWeaponServer(int type_id)
{
	DIE("Cannot call this method on client!" );
}

void Tank::setNextWeaponServer()
{
	DIE("Cannot call this method on client!" );
}

void Tank::setPreviousWeaponServer()
{
	DIE("Cannot call this method on client!" );
}

int Tank::canFire( const double simulation_time, const int sim_timestamp )
{
	DIE("Cannot call this method on client!" );

	return false; // Unreachable code, but without it gcc produce warning
}

void Tank::fallenServer()
{
	DIE("Cannot call this method on client!" );
}

void Tank::checkDestroy( Player * player )
{
	DIE("Cannot call this method on client!" );
}

void Tank::handleCollisionWEntityServer( int ID, const GLVector3d &coll_pos, const GLVector3d &coll_normal )
{
	DIE("Cannot call this method on client!" );
}

void Tank::killedCallbackClient()
{
	//handle tank's death
	GLVector3d tmp( getPosition() );
	GLVector3f position( (float)tmp.x, (float)tmp.y, (float)tmp.z ), direction( 0, 0, 1 );
	
	EntityPublicInfoPtr epi( new EntityPublicInfo() );
	if ( epi.get() )
	{
		setPublicInfo( epi );
		updateClient(0);
		getAppClient().getGame().getExplosionManager().addExplosion(
			mExplosionID,
			epi
		);
	} 
	else 
	{
		LOGD << "Can't create a public info for tank." << std::endl;
	}
}

void Tank::tankControllServer( const double ode_to_real )
{
	DIE("Cannot call this method on client!" );
}

