#include "GL/glew.h"
#include "terrain_geomipmap.h"
#include "log.h"

#include "application_client.h"
#include "globals_client.h"
#include "gfx.h"

// TODO: remove settings #include after removing hacks
#include "settings.h"

template class HierarchyTree<TerrainGeoMipMap::Node>;

void TerrainGeoMipMap::NodeDelegate::nodeCreation(TerrainHierarchyTree::NodeNavigator &navig)
{
	// get node's bounding box
	const int * boxMin = navig->getNodeBoxMin();
	const int * boxMax = navig->getNodeBoxMax();
	
	// TODO: safety
	float box_center[3], box_extent[3];
	for (int i = 0; i < 2; ++i) {
		box_center[i] = 0.5f * (boxMin[i] + boxMax[i]) * (getTerrain()->getBlockSize() - 1);
		box_extent[i] = 0.5f * (boxMax[i] - boxMin[i]) * (getTerrain()->getBlockSize() - 1);
	}

	float h_max(0), h_min(0); // height max, height min

	if (navig->isLeaf()) {
		// compute min/max height value in leaf-node

		TerrainGeoMipMap::TerrainBlock::BlockDescriptor desc;
		getTerrain()->setDescriptor(boxMin[0], boxMin[1], desc);
	
		// TODO: some kind of iterator would be great!
		int h, i_h_min(0), i_h_max(0);
		for (int y = desc.y; y < desc.y + desc.h; ++y)
			for (int x = desc.x; x < desc.x + desc.w; ++x)
			{
				h = mTerrain->getTrueHeight(x, y);
				if (h < i_h_min)
					i_h_min = h;
				if (h > i_h_max)
					i_h_max = h;
			}
		h_min = (float)i_h_min;
		h_max = (float)i_h_max;

	} else {
		// compute min/max height by maximalizing min/max height of children
		
		TerrainHierarchyTree::NodeNavigator child;
		
		// min/max of first child
		navig.getChild(0, child);
		const float 
			*centerFirst(child->getBoundingBoxCenter()), 
			*extentFirst(child->getBoundingBoxExtent());
		h_max = centerFirst[2] + extentFirst[2];
		h_min = centerFirst[2] - extentFirst[2];

		// adjusting min/max with respect to other child
		for (int i = 1; i < TerrainHierarchyTree::ChildrenCount; ++i) {
			
			navig.getChild(i, child);
			const float 
				*center(child->getBoundingBoxCenter()), 
				*extent(child->getBoundingBoxExtent());

			h_max = std::max(h_max, center[2] + extent[2]);
			h_min = std::min(h_min, center[2] - extent[2]);
		}
	}

	// set the computed min/max height into bounding box structure
	box_center[2] = 0.5f * (h_max + h_min);
	box_extent[2] = 0.5f * (h_max - h_min);

	// ... set the data (bounding box structure)
	navig->setBoundingBox(box_center, box_extent);
}

TerrainGeoMipMap::TerrainGeoMipMap(int blockSize):
mBlocks(0), mBlocksW(0), mBlocksH(0), mBlockShift(0)
{
	// TODO: NO HARDCODING the block size
	mBlockShift = 5;

	// TODO: This set-stuffs suxx, find out a better way!
	mNodeDelegate.setTerrain(this);
	mHierarchyTree.setNodeDelegate(&mNodeDelegate);
	
	mFrustumNeedsUpdate = true;
	setFrustumState(true);
}

void TerrainGeoMipMap::drawGeometry()
{
	// Preserve modelview matrix.
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	// Set scale.
	getRealScale().glScale();
	
	if (!mFrustumEnabled || mFrustumNeedsUpdate) {
		TerrainBlock::BlockDescriptor bd;
		TerrainBlock * ptb = mBlocks;
		for (int y = 0; y < mBlocksH; ++y) {
			for (int x = 0; x < mBlocksW; ++x, ++ptb) {
				setDescriptor(x, y, bd);
				ptb->render(bd);
			}
		}
	} else {
		hierarchicRender(mHierarchyTree.getRoot());
		
		// hierarchy tree visualisation - development purposes
		/*
		if (g_settings.isTrue(SETTING_MARTIN_TMP_RENDER_HIERARCHY_TREE))
			mHierarchyTree.render();
		*/
	}

	// Restore modelview matrix.
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

void TerrainGeoMipMap::renderBlockBoundaryIfAny( int x, int y, const TerrainBlock::BlockDescriptor &bd )
{
	// --------------------------
	// --- BOUNDARY RENDERING ---
	// --------------------------
	// Test if block is on a boundary
	// If it is:
	//	- get boundary sides (up to four)
	//  - render faces of this side and connected them to this block
	GLVector3d scaleInv;
	getRealScaleInv( scaleInv );
	const float fXY(50), fZ(100);
	if ( y == 0 )
	{
		// lower boundary
		const int yp( 0 );
		glBegin( GL_TRIANGLE_STRIP );
			for ( int xp = bd.x; xp < bd.x + bd.w; ++xp )
			{
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp, yp - (float)scaleInv.y * fXY, -(float)scaleInv.z * fZ );
			}
		glEnd();
	} 
	if ( y == mBlocksH - 1 )
	{
		const int yp( bd.y + bd.h - 1 );
		// upper boundary
		glBegin( GL_TRIANGLE_STRIP );
			for ( int xp = bd.x + bd.w - 1; xp >= bd.x ; --xp )
			{
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp, yp + (float)scaleInv.y * fXY, -(float)scaleInv.z * fZ );
			}
		glEnd();
	}
	if ( x == 0 )
	{
		// left boundary
		const int xp( 0 );
		glBegin( GL_TRIANGLE_STRIP );
			for ( int yp = bd.y + bd.h - 1; yp >= bd.y; --yp )
			{
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp - (float)scaleInv.x * fXY, (float)yp, -(float)scaleInv.z * fZ );
			}
		glEnd();
	}
	if ( x == mBlocksW - 1 )
	{
		// right boundary
		const int xp( bd.x + bd.w - 1 );
		glBegin( GL_TRIANGLE_STRIP );
			for ( int yp = bd.y; yp < bd.y + bd.h; ++yp )
			{
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp + (float)scaleInv.x * fXY, (float)yp, -(float)scaleInv.z * fZ );
			}
		glEnd();
	}

	if ( y == 0 )
	{
		const int yp( 0 );
		if ( x == 0 )
		{
			const int xp( 0 );
			glBegin( GL_TRIANGLES );
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp - (float)scaleInv.x * fXY, (float)yp, -(float)scaleInv.z * fZ );	
				glVertex3f( (float)xp, yp - (float)scaleInv.y * fXY, -(float)scaleInv.z * fZ );	
			glEnd();
		}
		if ( x == mBlocksW - 1 )
		{
			const int xp( bd.x + bd.w - 1 );
			glBegin( GL_TRIANGLES );
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp, yp - (float)scaleInv.y * fXY, -(float)scaleInv.z * fZ );	
				glVertex3f( (float)xp + (float)scaleInv.x * fXY, (float)yp, -(float)scaleInv.z * fZ );	
			glEnd();
		}
	}
	if ( y == mBlocksH - 1 )
	{
		const int yp( bd.y + bd.h - 1 );
		if ( x == 0 )
		{
			const int xp( 0 );
			glBegin( GL_TRIANGLES );
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp, yp + (float)scaleInv.y * fXY, -(float)scaleInv.z * fZ );	
				glVertex3f( (float)xp - (float)scaleInv.x * fXY, (float)yp, -(float)scaleInv.z * fZ );	
			glEnd();
		}
		if ( x == mBlocksW - 1 )
		{
			const int xp( bd.x + bd.w - 1 );
			glBegin( GL_TRIANGLES );
				glVertex3f( (float)xp, (float)yp, (float)bd.terr->getTrueHeight(xp, yp) );
				glVertex3f( (float)xp + (float)scaleInv.x * fXY, (float)yp, -(float)scaleInv.z * fZ );	
				glVertex3f( (float)xp, yp + (float)scaleInv.y * fXY, -(float)scaleInv.z * fZ );	
			glEnd();
		}
	}

	// ---------------------------------
	// --- end of boundary rendering ---
	// ---------------------------------
}

// TODO: do terrain rendering more structuralized
void TerrainGeoMipMap::render(void)
{
	// which textures will be used
	bool 
		useLightmap = this->mLightmapTexture.isValid(),
		useGlobal = this->mGlobalTexture.isValid(),
		useDetailMix = 
			this->mDetailTextures.size() == 1 && 
			this->mMixMapTextures.size() == 1 && 
			this->mDetailPeriods.size() == 1 &&
			this->mDetailTextures[0].isValid() && 
			this->mMixMapTextures[0].isValid();

	//useDetailMix = false;
	//useLightmap = false;
	//useGlobal = false;

	int sizeX, sizeY;
	getExtents(sizeX, sizeY);
	
	// s, t vectors for generating global texture coordinates
	GLVector4d tv[2] = {
		GLVector4d(1.0/(sizeX-1), 0, 0, 0),
		GLVector4d(0.0, 1.0/(sizeY-1), 0, 0)
	};

	// disable all texture units except TU 0
	getAppClient().getGFX().disableMultitexturing();
	
	// disable lighting
	glDisable(GL_LIGHTING);
	glColor3f(1.0f, 1.0f, 1.0f);

	GLenum 
		detailTU	( GL_TEXTURE0  ), 
		mixTU		( detailTU + 1 ),
		globalTU	( mixTU    + 1 ), 
		lightmapTU	( GL_TEXTURE3  );
	
	GLint maxtus(0);
	glGetIntegerv( GL_MAX_TEXTURE_UNITS, &maxtus );

	// check texture units count and adapt if the count is less than four
	switch ( maxtus )
	{
	case 1:
		globalTU = GL_TEXTURE0;
		useLightmap = false;
		useDetailMix = false;
		break;
	case 2:
	case 3:
		globalTU = GL_TEXTURE0;
		lightmapTU = GL_TEXTURE1;
		useDetailMix = false;
		break;
	default:
		break;
	};

	GLint detailTUMode, mixTUMode, globalTUMode, lightmapTUMode;

	if ( useDetailMix )
	{
		// --- DETAIL TEXTURE ---
		glActiveTexture( detailTU );
		glGetTexEnviv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &detailTUMode );
		mDetailTextures[0]->bind();
		
		GLint repmode = GLEW_VERSION_1_4 ? GL_MIRRORED_REPEAT : (GL_ARB_texture_mirrored_repeat ? GL_MIRRORED_REPEAT_ARB : GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, repmode );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, repmode );
				
		// define texgen coords
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGendv(GL_S, GL_OBJECT_PLANE, (tv[0] * (mDetailTextures[0]->getMaxS() * getScale().x / mDetailPeriods[0].x)).val );
		glTexGendv(GL_T, GL_OBJECT_PLANE, (tv[1] * (mDetailTextures[0]->getMaxT() * getScale().y / mDetailPeriods[0].y)).val );

		// TU mode
		
		// combine
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);

		// RGB - dot3
		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_DOT3_RGB);
		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, mixTU);
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);
		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, detailTU);
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR);

		// ALPHA - modulate
		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, mixTU);
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA);
		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA, detailTU);
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA, GL_SRC_ALPHA);

		// enable texgen coords
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);

		// enable texture
		glEnable(GL_TEXTURE_2D);

		// --- MIXMAP ---
		glActiveTexture( mixTU ); 
		glGetTexEnviv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &mixTUMode );
		mMixMapTextures[0]->bind();
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		
		// define texgen coords
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGendv(GL_S, GL_OBJECT_PLANE, (tv[0] * mMixMapTextures[0]->getMaxS()).val );
		glTexGendv(GL_T, GL_OBJECT_PLANE, (tv[1] * mMixMapTextures[0]->getMaxT()).val );

		// enable texgen coords
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);

		// enable texture
		glEnable(GL_TEXTURE_2D);

		// TU mode

		// combine
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE );

		// add weight part from alpha channel
		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_ADD );
		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PREVIOUS );
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR );
		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS );
		glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_ALPHA );
	}

	// enable TU, bind a texture
	if (useGlobal)
	{
		// --- GLOBAL MAP ---
		glActiveTexture( globalTU ); 
		glGetTexEnviv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &globalTUMode );
		mGlobalTexture->bind();
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		
		// texgen coords (define)
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);

		GLVector4d tcv[2] = 
		{
			tv[0] * mGlobalTexture->getMaxS(),
			tv[1] * mGlobalTexture->getMaxT()
		};
		glTexGendv(GL_S, GL_OBJECT_PLANE, tcv[0].val );
		glTexGendv(GL_T, GL_OBJECT_PLANE, tcv[1].val );
		// texgen coords (enable)
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		// enable texture
		glEnable(GL_TEXTURE_2D);

		// TU mode
		if ( useDetailMix ) 
		{
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
			glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_ADD_SIGNED);
			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, globalTU);
			glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);
			glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS);
			glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_ONE_MINUS_SRC_COLOR);
		} else {
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		}
	}

	// enable TU, bind a texture
	if (useLightmap)
	{
		glActiveTexture( lightmapTU );
		glGetTexEnviv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &lightmapTUMode );
		mLightmapTexture->bind();
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		// texgen coords (define)
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGendv(GL_S, GL_OBJECT_PLANE, (tv[0] * mLightmapTexture->getMaxS()).val );
		glTexGendv(GL_T, GL_OBJECT_PLANE, (tv[1] * mLightmapTexture->getMaxT()).val );
		// texgen coords (enable)
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		// enable texture
		glEnable(GL_TEXTURE_2D);

		// tex environmnent setup
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	}

	drawGeometry();

	if (useGlobal)
	{
		// disable texture and coordinates generation
		glActiveTexture(globalTU);
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, globalTUMode );
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
	}

	if (useLightmap)
	{
		// disable texture and coordinates generation
		glActiveTexture( lightmapTU );
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, lightmapTUMode );
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
	}

	if (useDetailMix)
	{
		// disable texture and coordinates generation
		glActiveTexture(mixTU);
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, mixTUMode );
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);

		glActiveTexture(detailTU);
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, detailTUMode );
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
	}	
}

TerrainGeoMipMap::TerrainBlock::BlockDescriptor& TerrainGeoMipMap::setDescriptor(int bx, int by, TerrainBlock::BlockDescriptor & d)
{
	struct { int w, h; } hm;
	
	// get heightmap extents
	getExtents(hm.w, hm.h);
	
	d.terr = this;
	d.x = std::min(std::max(bx << mBlockShift, 0), hm.w - 1);
	d.y = std::min(std::max(by << mBlockShift, 0), hm.h - 1);
	d.w = (1 << mBlockShift) + 1;
	d.h = (1 << mBlockShift) + 1;	
	if (d.x + d.w > hm.w) 
		d.w = hm.w - d.x;
	if (d.y + d.h > hm.h) 
		d.h = hm.h - d.y;
	
	return d;
}

void TerrainGeoMipMap::TerrainBlock::selectLevel(const TerrainGeoMipMap::TerrainBlock::BlockDescriptor &bd, double blockSqrDistance, double pixelTolerance)
{	
	const int nlevels = getLevelsCount(bd);
	
	int level = 0;
	blockSqrDistance *= pixelTolerance * pixelTolerance;
	for (int i = nlevels - 1; i >= 0; --i)
	{
		if (mLevels[i].getSqrDist() <= blockSqrDistance)
		{
			level = i;
			break;
		}
	}
	
	mActualLevel = level;
}

void TerrainGeoMipMap::update(const class Camera * camera)
{
	assertL( camera && "Camera pointer is null!" );
	GLVector3d pos, inv;

	// get position of camera and transform it to position in heightmap domain
	// this position can be used in LOD-computation when we counts with distance from camera
	getRealScaleInv(inv);
	camera->getPos(pos);
	pos = pos * inv;

	mLocation = pos;	

	if (mFrustumEnabled) {
		// copy frustum of given camera into private frustum variable
		GLMatrix4d m(GLMatrix4d::identity());
		mFrustum = camera->getFrustum();

		// apply heightmap-to-real-terrain scale transformation to this frustum copy
		m.applyScale(getRealScale());
		mFrustum.transform(m, true);
		
		// perform recursive frustum culling
		TerrainHierarchyTree::NodeNavigator nav = mHierarchyTree.getRoot();
		frustumCulling(nav);
		
		// HACK: adjusted location so that LOD will appear under the tank?
		pos = camera->getRef();
		pos = pos * inv;
		mLocation = pos;	

		mFrustumNeedsUpdate = false;
	}else{
		mFrustumNeedsUpdate = true;
	}
}

void TerrainGeoMipMap::init(void)
{
	VisualTerrain::init();

	createBlocks();
	createHierarchyTree();
	
	// HACK: frustum state should not be treated as setting
	//setFrustumState(g_settings.isTrue(SETTING_MARTIN_TMP_FRUSTUM_CULLING));
	setFrustumState(true);

	initBlocks();
}

void TerrainGeoMipMap::createBlocks()
{
	if (mBlocks)
	{
		delete [] mBlocks;
		mBlocks = 0;
	}

	struct 
	{
		int w, h;
	} hm, ba;

	// get heightmap extents
	getExtents(hm.w, hm.h);

	// compute extents of block-array ( rounded up to block size )
	ba.w = ( (hm.w - 2 ) >> mBlockShift ) + 1;
	ba.h = ( (hm.h - 2 ) >> mBlockShift ) + 1;

	// allocate memory for block-array and set member variables width and height 
	mBlocks = new TerrainBlock [ba.h * ba.w];
	if (!mBlocks)
		DIE("Not enought memory for array of terrain blocks!");

	
	// set references to terrain-block-neighbours for each terain-block
	TerrainBlock * neighbours[4];
	for (int y = 0; y < ba.h; ++y) 
	{		
		for (int x = 0; x < ba.w; ++x) 
		{
			neighbours[TerrainBlock::N_TOP] = &(mBlocks[((y == 0) ? y : (y - 1)) * ba.w + x]);
			neighbours[TerrainBlock::N_BOTTOM] = &(mBlocks[((y == ba.h - 1) ? y : (y + 1)) * ba.w + x]);
			neighbours[TerrainBlock::N_LEFT] = &(mBlocks[y * ba.w + ((x == 0) ? x : (x - 1))]);
			neighbours[TerrainBlock::N_RIGHT] = &(mBlocks[y * ba.w + ((x == ba.w - 1) ? x : (x + 1))]);
			mBlocks[y * ba.w + x].setNeighbours(neighbours);
		}
	}

	mBlocksW = ba.w;
	mBlocksH = ba.h;
}

void TerrainGeoMipMap::initBlocks()
{
	TerrainBlock::BlockDescriptor bd;
	TerrainBlock * block = mBlocks;

	for (int y = 0; y < mBlocksH; ++y)
		for (int x = 0; x < mBlocksW; ++x, ++block)
		{
			setDescriptor(x, y, bd);
			block->initLevels(bd);
		}
}

void TerrainGeoMipMap::createHierarchyTree()
{
	int minPoint[TerrainHierarchyTree::D], maxPoint[TerrainHierarchyTree::D];
	
	// TODO: safety (array bounds)
	minPoint[0] = 0;
	minPoint[1] = 0;
	maxPoint[0] = mBlocksW;
	maxPoint[1] = mBlocksH;
	mHierarchyTree.setExtent(minPoint, maxPoint);
	mHierarchyTree.build();
}

void TerrainGeoMipMap::frustumCulling(TerrainHierarchyTree::NodeNavigator & navigator)
{
	// test the node's bounding box against frustum and store the result in this node
	GLVector3f center, extent;
	navigator->getBoundingBox(center.val, extent.val);
	Frustum::ClipState state = mFrustum.testBox(GLVector3d(center.x, center.y, center.z), GLVector3d(extent.x, extent.y, extent.z));
	navigator->setFrustumClipState(state);
	
	// recurse only to intersected inner nodes
	if (state == Frustum::INTERSECT && !navigator->isLeaf()) 
	{
		TerrainHierarchyTree::NodeNavigator child;
		for (int i = 0; i < TerrainHierarchyTree::ChildrenCount; ++i)
			frustumCulling(navigator.getChild(i, child));
	} else {
		TerrainBlock::BlockDescriptor bd;
		
		const int *boxMin = navigator->getNodeBoxMin();
		const int *boxMax = navigator->getNodeBoxMax();

		for (int y = boxMin[1]; y < boxMax[1]; ++y) 
		{
			for (int x = boxMin[0]; x < boxMax[0]; ++x) 
			{
				setDescriptor(x, y, bd);				
				processVisibleBlock(bd, mBlocks[y * mBlocksW + x]);
			}
		}
	}
}

void TerrainGeoMipMap::hierarchicRender(const TerrainHierarchyTree::NodeNavigator & navigator)
{
	bool render_blocks(false); // rendering (true) or recursion (false)

	// decide if node is inside frustum, outside or intersect
	switch(navigator->getFrustumClipState()) {
		
		case Frustum::INSIDE:
			// get blocks' range and render
			render_blocks = true;
			break;

		case Frustum::OUTSIDE:
			// node and entire subtree is not visible => return
			return;

		default:
			if (navigator->isLeaf())
				// a leaf intersecting the frustum => get range and render
				render_blocks = true;
			break;
	}

	if (render_blocks) {
		// get blocks' rectangular range and render each block in that range
		const int 
			*boxMin(navigator->getNodeBoxMin()), 
			*boxMax(navigator->getNodeBoxMax());
		
		// TODO: Do some ChildrenCount safety and better block-iteration, please
		assertL(TerrainHierarchyTree::ChildrenCount == 2 && "Supported only for children count = 2, sorry...");		
		TerrainBlock::BlockDescriptor bd;
		for (int y = boxMin[1]; y < boxMax[1]; ++y) 
		{
			for (int x = boxMin[0]; x < boxMax[0]; ++x) 
			{
				setDescriptor(x, y, bd);
				// TODO: avoid of "y*mBlocksW+x" each iteration
				
				mBlocks[y*mBlocksW+x].render(bd);
				
				renderBlockBoundaryIfAny( x, y, bd );

			}
		}
		return;
	} else {
		assertL(!navigator->isLeaf() && navigator->getFrustumClipState() == Frustum::INTERSECT && "Invariant broken in hierarchic rendering!");
		
		// recurse to node's children
		TerrainHierarchyTree::NodeNavigator child;
		for (int i = 0; i < TerrainHierarchyTree::ChildrenCount; ++i)
			hierarchicRender(navigator.getChild(i, child));
	}
}

TerrainGeoMipMap::TerrainBlock * TerrainGeoMipMap::TerrainBlock::getNeighbour( 
	const TerrainGeoMipMap::TerrainBlock::NEIGHBOUR_ID nid)
{
	return mNeighbours[nid];
}

void TerrainGeoMipMap::TerrainBlock::render(const BlockDescriptor &bd, int x1, int y1, int x2, int y2, int step)
{
	// TODO: IMPORTANT BUGFIX:
	// triangulation should correspond with triangulation on server. Now it is fixed, but seems lame.
	glCullFace(GL_FRONT);
	assertL(y1 >= bd.y && x1 >= bd.x && y2 < bd.y + bd.h && x2 < bd.x + bd.w && bd.terr != 0 && "Coordinates out of block bounds!");
	for (int y = y1; y <= y2 - step; y += step) 
	{			
		glBegin(GL_TRIANGLE_STRIP);
		/*
		glVertex3i(x1, y+step, bd.terr->getTrueHeight(x1, y+step));
		glVertex3i(x1, y, bd.terr->getTrueHeight(x1, y));
		for (int x = x1; x <= x2 - step; x += step) 
		{
			glVertex3i(x+step, y+step, bd.terr->getTrueHeight(x+step, y+step));
			glVertex3i(x+step, y, bd.terr->getTrueHeight(x+step, y));
		}
		*/
		
		glVertex3i(x1, y, bd.terr->getTrueHeight(x1, y));
		glVertex3i(x1, y+step, bd.terr->getTrueHeight(x1, y+step));
		
		for (int x = x1; x <= x2 - step; x += step) 
		{
			glVertex3i(x+step, y, bd.terr->getTrueHeight(x+step, y));
			glVertex3i(x+step, y+step, bd.terr->getTrueHeight(x+step, y+step));
		}
		
		

		glEnd();
	}
	glCullFace(GL_BACK);
}


void TerrainGeoMipMap::TerrainBlock::render(const BlockDescriptor &bd)
{
	int step = getStep();
	if (!(step >= 1 && step <= bd.w && step <= bd.h))
		return;	
	
	int ns[4];		// neighbours' steps
	bool flags[4];	// flags 'ns[i] <= step'
	int x1, y1, x2, y2;

	flags[N_LEFT]	= (ns[N_LEFT]	= getNeighbour(N_LEFT	)->getStep()) <= step;
	flags[N_TOP]	= (ns[N_TOP]	= getNeighbour(N_TOP	)->getStep()) <= step;
	flags[N_RIGHT]	= (ns[N_RIGHT]	= getNeighbour(N_RIGHT	)->getStep()) <= step;
	flags[N_BOTTOM]	= (ns[N_BOTTOM]	= getNeighbour(N_BOTTOM	)->getStep()) <= step;
	
	x1 = bd.x;
	y1 = bd.y;
	x2 = bd.x + bd.w - 1;
	y2 = bd.y + bd.h - 1;
	if (flags[N_LEFT] & flags[N_TOP] & flags[N_RIGHT] & flags[N_BOTTOM])
	{
		// A) neighbours are on the same level or greater -> draw this whole block		
	} else {
		// B) there exist neighbours on lower LOD level -> avoid T vertices				

		// render connections
		if (!flags[N_LEFT]) {			
			blendLeft(bd, ns[N_LEFT], !flags[N_TOP], !flags[N_BOTTOM]);			
			x1 += step;			
		}

		if (!flags[N_TOP]) {
			blendTop(bd, ns[N_TOP], !flags[N_LEFT], !flags[N_RIGHT]);
			y1 += step;
		}

		if (!flags[N_RIGHT]) {
			blendRight(bd, ns[N_RIGHT], !flags[N_TOP], !flags[N_BOTTOM]);
			x2 -= step;
		}

		if (!flags[N_BOTTOM]) {
			blendBottom(bd, ns[N_BOTTOM], !flags[N_LEFT], !flags[N_RIGHT]);
			y2 -= step;
		}
	}
	
	// render full block or inner (without blends) rectangle
	render(bd, x1, y1, x2, y2, step);
}

void TerrainGeoMipMap::TerrainBlock::renderTriFanA(TerrainBase * tb, int points[4][2], int step, int axis)
{
	int pt[2];
	glBegin(GL_TRIANGLE_FAN);
	glVertex3i(points[0][0], points[0][1], tb->getTrueHeight(points[0][0], points[0][1]));
	glVertex3i(points[1][0], points[1][1], tb->getTrueHeight(points[1][0], points[1][1]));
	pt[1 - axis] = points[2][1 - axis];
	for (int j = points[2][axis]; j >= points[3][axis]; j -= step)
	{
		pt[axis] = j;
		glVertex3i(pt[0], pt[1], tb->getTrueHeight(pt[0], pt[1]));
	}

	glEnd();
}

void TerrainGeoMipMap::TerrainBlock::renderTriFanB(TerrainBase * tb, int points[4][2], int step, int axis)
{
	int pt[2];
	glBegin(GL_TRIANGLE_FAN);
	pt[1 - axis] = points[2][1 - axis];
	for (int j = points[3][axis]; j <= points[2][axis]; j += step)
	{
		pt[axis] = j;
		glVertex3i(pt[0], pt[1], tb->getTrueHeight(pt[0], pt[1]));
	}
	glVertex3i(points[1][0], points[1][1], tb->getTrueHeight(points[1][0], points[1][1]));
	glVertex3i(points[0][0], points[0][1], tb->getTrueHeight(points[0][0], points[0][1]));
	glEnd();
}

void TerrainGeoMipMap::TerrainBlock::blendLeft(const BlockDescriptor &bd, int step2, bool b1, bool b2)
{
	// renderTriFanB
	blend(bd, step2, b1, b2, 1, bd.coord[0], bd.coord[0] + getStep(), &TerrainGeoMipMap::TerrainBlock::renderTriFanB);
}

void TerrainGeoMipMap::TerrainBlock::blendTop(const BlockDescriptor &bd, int step2, bool b1, bool b2)
{
	// renderTriFanA
	blend(bd, step2, b1, b2, 0, bd.coord[1], bd.coord[1] + getStep(), &TerrainGeoMipMap::TerrainBlock::renderTriFanA);
}

void TerrainGeoMipMap::TerrainBlock::blendRight(const BlockDescriptor &bd, int step2, bool b1, bool b2)
{
	// renderTriFanA
	blend(bd, step2, b1, b2, 1, bd.coord[0] + bd.extent[0] - 1, bd.coord[0] + bd.extent[0] - 1 - getStep(), &TerrainGeoMipMap::TerrainBlock::renderTriFanA);
}

void TerrainGeoMipMap::TerrainBlock::blendBottom(const BlockDescriptor &bd, int step2, bool b1, bool b2)
{
	// renderTriFanB
	blend(bd, step2, b1, b2, 0, bd.coord[1] + bd.extent[1] - 1, bd.coord[1] + bd.extent[1] - 1 - getStep(), &TerrainGeoMipMap::TerrainBlock::renderTriFanB);
}

void TerrainGeoMipMap::TerrainBlock::blend(const BlockDescriptor &bd, int step2, bool b1, bool b2, int axis, int blendStripCoord1, int blendStripCoord2, TerrainGeoMipMap::TerrainBlock::PRenderTriFanMethod renderTriFan)
{
	int step(getStep());
	int y1(bd.coord[axis]), y1max(bd.coord[axis] + bd.extent[axis] - 1), y2max;
	int points[4][2];
	
	y2max = b2 ? (y1max - step) : y1max;
	
	points[1][1 - axis] = points[0][1 - axis] = blendStripCoord1;//bd.coord[1 - axis];
	points[3][1 - axis] = points[2][1 - axis] = blendStripCoord2;//bd.coord[1 - axis] + step;

	// first step, non-rectangular region
	if (b1)
	{
		// fan((x1, y1), (x1, y1 + step2), (x1 + step, min(y1 + step2, y2max)), ..., (x1 + step, y1 + step));
		points[0][axis] = y1;
		points[1][axis] = y1 + step2;
		points[2][axis] = min(points[1][axis], y2max);
		points[3][axis] = y1 + step;				
		(this->*(PRenderTriFanMethod)renderTriFan)(bd.terr, points, step, axis);

		// update y1
		y1 += step2;
	}		
	
	// last step (except last step = first step), non-rectangular region
	if (b2 & (y1 != y1max))
	{
		// fan((x1, y1max - step2), (x1, y1max), (x1 + step, y2max), ..., (x1 + step, y1max - step2));
		points[0][axis] = points[3][axis] = y1max - step2;
		points[1][axis] = y1max;
		points[2][axis] = y2max;		
		(this->*(PRenderTriFanMethod)renderTriFan)(bd.terr, points, step, axis);
		
		// update y1max
		y1max -= step2;
	}	

	// middle steps, trifan in rectangular regions
	for (;y1 < y1max; y1 += step2)
	{
		// fan((x1, y1), (x1, y1 + step2), (x1 + step, y1 + step2), ..., (x1 + step, y1);
		points[0][axis] = points[3][axis] = y1;
		points[1][axis] = points[2][axis] = y1 + step2;
		(this->*(PRenderTriFanMethod)renderTriFan)(bd.terr, points, step, axis);
	}
	
}

float TerrainGeoMipMap::TerrainBlock::getFovX(const BlockDescriptor &bd)
{ return bd.terr->getFovX(); }

int TerrainGeoMipMap::TerrainBlock::getViewHRes(const BlockDescriptor &bd)
{ return getAppClient().getGFX().getHeight(); }

void TerrainGeoMipMap::TerrainBlock::initLevels(const TerrainGeoMipMap::TerrainBlock::BlockDescriptor &bd)
{
	int nlevels = getLevelsCount(bd);
	if (mLevels)
	{
		delete [] mLevels;
		mLevels = 0;
	}
	
	mLevels = new GeoMipMapLevel[nlevels];
	
	// heightmap scale
	GLVector3d scale;
	bd.terr->getRealScale(scale);
	
	// first level is full detail
	mLevels[0].setSqrDist(0);
	
	// compute distance from level 0 to each level i = 1, ..., nlevels
	for (int i = 1; i < nlevels; ++i)
	{		
		int step = 1 << i;
		double inv_step = 1.0 / step;
		double delta_i = 0.0;
		
		// iteration on i'th level, squares [x; y; x + step; y + step]
		for (int y = bd.y; y < bd.y + bd.h - step; y += step)
		{
			for (int x = bd.x; x < bd.x + bd.w - step; x += step)
			{
				double h1, h2, h3, h4, h, tmp;
				double xpf, ypf;
				h1 = (double)bd.terr->getTrueHeight(x, y);
				h2 = (double)bd.terr->getTrueHeight(x + step, y);
				h3 = (double)bd.terr->getTrueHeight(x + step, y + step);
				h4 = (double)bd.terr->getTrueHeight(x, y + step);
				
				// iterate through this square and compute maximum distance from level 0
				ypf = 0.0;
				for (int yp = y; yp <= y + step; ++yp, ypf += inv_step)
				{
					xpf = 0.0;
					for (int xp = x; xp <= x + step; ++xp, xpf += inv_step)
					{
						// interpolated height
						h = (xp <= x + step - yp + y) ? 
							// triangle A: h1, h2, h4							
							//(h1 * (1.0 - xpf) + h2 * xpf + h1 * (1.0 - ypf) + h4 * ypf - h1)
							(h1 + xpf * (h2 - h1) + ypf * (h4 - h1))
							:
							// triangle B: h3, h2, h4							
							//(h3 * ypf + h2 * (1.0 - ypf) + h3 * xpf + h4 * (1.0 - xpf) - h3);
							(h3 + (1 - ypf) * (h2 - h3) + (1 - xpf) * (h4 - h3));

						tmp = (double)bd.terr->getTrueHeight(xp, yp) - h;
						
						// NOTE: Maybe maximizing (local_positive_delta - local_negative_delta)
						// would be better metric (???)
						if (fabs(tmp) > fabs(delta_i))
							delta_i = tmp;						
					}
				}				
			}
		}
		// now, delta_i is max. height change for i'th level
		// recompute to real terrain scale and compute dist-value for i'th level using delta_i
		float dist = getMinDistance(bd, (float)(delta_i * scale.z));
		mLevels[i].setSqrDist(dist * dist);
	}
	mActualLevel = 0;
}

void TerrainGeoMipMap::processVisibleBlock(TerrainGeoMipMap::TerrainBlock::BlockDescriptor &bd, TerrainGeoMipMap::TerrainBlock &block)
{
	double tau = 8.0;

	// compute squared distance from camera
	double 
		dx = mLocation.x - (float)(bd.x + 0.5 * bd.w),
		dy = mLocation.y - (float)(bd.y + 0.5 * bd.h);

	dx *= dx;
	dy *= dy;

	// select apropriate GeoMipMap Level
	block.selectLevel(bd, dx + dy, tau);
}
