/* MAINTAINER: Bernard */

//#define WANT_TRACE

#include "gamegui.h"
#include "log.h"
#include "params.h"
#include "application_client.h"
#include "translator.h"
#include "sfx.h"

#include "statswindow.h"
#include "infolayer.h"
#include "messages.h"
#include "menuingame.h"


GameGUI::GameGUI()
{
	mStatsWindow = 0;
	mInfoLayer   = 0;
	mMessages    = 0;
	mMenuInGame  = 0;
	mInicialized = false;
}

GameGUI::~GameGUI()
{
	assertL(!mInicialized);

	if (mStatsWindow) delete mStatsWindow;
	if (mInfoLayer)   delete mInfoLayer;
	if (mMessages)    delete mMessages;
	if (mMenuInGame)  delete mMenuInGame;
		
}

void GameGUI::render()
{	
	assertL(mStatsWindow && mInfoLayer && 
	        mMessages && mMenuInGame);
	assertL(mInicialized);

	TIN

	TPOINTS("Info Layer");
	mInfoLayer->render();

	TPOINTS("Messages");
	mMessages->render();

	mStatsWindow->render();

	mMenuInGame->render();
}


void  GameGUI::update(int dtime, int atime)
{
	assertL(mInicialized);

	// todo - call update on other
	assertL(mMenuInGame);
	mMenuInGame->update(dtime, atime);
}
		
void  GameGUI::setPlayer(Player &player)
{
	if (!mInicialized) return;

	assertL(mStatsWindow && mInfoLayer);
	mStatsWindow->setPlayer(&player);
	mInfoLayer->setPlayer(&player);
}
		
void GameGUI::addMessage(const std::string &msg)
{
	if (!mInicialized) return;

	assertL(mMessages);
	if (mMessages == 0) return;
	mMessages->addMessage(msg);
}

void GameGUI::init()
{
	TIN;
	assertL(!mInicialized);

	if (!mStatsWindow) mStatsWindow = new StatsWindow();
	if (!mInfoLayer)   mInfoLayer = new InfoLayer();
	if (!mMessages)    mMessages = new Messages();
	if (!mMenuInGame)  mMenuInGame = new MenuInGame();

	if (mStatsWindow == 0 || mInfoLayer == 0 ||
	    mMessages == 0 || mMenuInGame == 0)
	{
		DIE("new failed");
	}

	mInfoLayer->init();
	mMessages->init();
	mStatsWindow->init();

	mInicialized = true;
	
//	activateMenu();
}

void GameGUI::clean()
{
	if (mInicialized)
	{
		assertL(mStatsWindow && mInfoLayer && 
		        mMessages);

		mInicialized = false;
	
		mInfoLayer->clean();
		mMessages->clean();
		mStatsWindow->clean();
		
		if (mMenuInGame)  delete mMenuInGame;
		mMenuInGame = 0;
	}
}


void GameGUI::pickedBonus(int id)
{
	TIN;
	if (!mInicialized) return;

	const Params::sBonusParams& bonus = gParams.bonus(id);
	
	// Amuse the user
	addMessage(_(bonus.pick_message.c_str()));
	getAppClient().getSFX().play(bonus.pick_sound.c_str());

	assertL(mInfoLayer);
	mInfoLayer->pickedBonus(id);
}

void GameGUI::expiredBonus(int id)
{
	if (!mInicialized) return;

	assertL(mInfoLayer);
	mInfoLayer->expiredBonus(id);
}

void GameGUI::activateChatWindow()
{
	if (!mInicialized) return;

	assertL(mMessages);
	mMessages->activate(true);
}


void GameGUI::showStatsWindow(bool visibility)
{
	if (!mInicialized) return;

	assertL(mStatsWindow);
	mStatsWindow->show(visibility);	
}

void GameGUI::activateMenu()
{
	if (!mInicialized) return;
	
	assertL(mMenuInGame);
	mMenuInGame->activated();	
}

void GameGUI::deactivateMenu()
{
	if (!mInicialized) return;

	assertL(mMenuInGame);
	mMenuInGame->deactivated();	
}

bool GameGUI::isMenuActive()
{
	if (!mInicialized) return false;

	assertL(mMenuInGame);
	return mMenuInGame->isActivated();
}

void GameGUI::setEndConditionStatuses(const std::list<ConditionStatus> &conditions)
{
	if (!mInicialized) return;
	
	assertL(mInfoLayer);
	mInfoLayer->setEndConditionStatuses( conditions );
}

void GameGUI::gameStarted()
{
	if (!mMenuInGame)  mMenuInGame = new MenuInGame();

	if (mMenuInGame == 0)
	{
		DIE("new failed");
	}
	
	/*
	if (isMenuActive())
	{
		deactivateMenu();
//		activateMenu();	
	}
	else
	{
//		activateMenu();
		deactivateMenu();
	}
	
	LOGD << "GAME STARTED!!!!" << endl;
	
	deactivateMenu();
	*/
	
}
		
void GameGUI::gameEnded()
{
	if (mMenuInGame)  delete mMenuInGame;
	mMenuInGame = 0;	
}

