/* MAINTAINER: Bernard */

#include "gui_constants.h"
#include "tinyxml/tinyxml.h"
#include "settings.h"
#include "log.h"
#include "platform.h"
#include "conversions.h"

GUIConstants gc;

GUIConstants::GUIConstants()
{
}


#define GET_VALUE(x) \
	child = handle.FirstChild("guiconf").FirstChild(#x).Element(); \
	value = child->GetText(); \
	if ( value == 0 ) \
	{ \
		LOGE << "Missing element " << #x << " in file " << filename << endl; \
		DIE(); \
	}

#define GET_STRUCT_VALUE( structure, x) \
	child = handle.FirstChild("guiconf").FirstChild(#structure).FirstChild(#x).Element(); \
	value = child->GetText(); \
	if ( value == 0 ) \
	{ \
		LOGE << "Missing element " << #structure << "." << #x << " in file " << filename << endl; \
		DIE(); \
	} 

#define LOAD_INT(x) \
	GET_VALUE(x) \
	x = get_long(value);
		
#define LOAD_STRING(x) \
	GET_VALUE(x) \
	x = value;
	
#define LOAD_DOUBLE(x) \
	GET_VALUE(x) \
	x = get_double(value);

#define LOAD_BOOL(x) \
	GET_VALUE(x) \
	x = get_bool(value);

#define LOAD_FLOAT(x) \
	GET_VALUE(x) \
	x = (float)get_double(value);

#define LOAD_STRUCT_INT( structure, x ) \
	GET_STRUCT_VALUE( structure, x) \
	structure.x = get_long(value);
		
#define LOAD_STRUCT_STRING( structure, x ) \
	GET_STRUCT_VALUE( structure, x) \
	structure.x = value;
	
#define LOAD_STRUCT_DOUBLE( structure, x ) \
	GET_STRUCT_VALUE( structure, x) \
	structure.x = get_double(value);

#define LOAD_STRUCT_BOOL( structure, x ) \
	GET_STRUCT_VALUE( structure, x) \
	structure.x = get_bool(value);

#define LOAD_STRUCT_FLOAT( structure, x ) \
	GET_STRUCT_VALUE( structure, x) \
	structure.x = (float)get_double(value);

void GUIConstants::load()
{
	string filename;
		
	TiXmlDocument doc;
	TiXmlHandle   handle(&doc);
	TiXmlElement* child;
	const char *value;


	filename = locateFile(g_settings.getString(SETTING_GUI_CONFIG_FILE));
	if (!doc.LoadFile(filename)) 
	{
		LOGE << "Unable to load " << filename
		     << " Probably file not exist or isn't a valid XML file" << endl;
		DIE();
	}
	
	child = handle.FirstChild("guiconf").Element();
	if (!child)
	{
		LOGE << "There is no root element guiconf in file " << filename  << endl;
		DIE();
	}
	
/* *********** INFOLAYER ************ */
	LOAD_STRING(info_font);
	LOAD_INT(info_font_size);
	LOAD_STRING(info_icons_font);
	LOAD_INT(info_icons_font_size);
	
	/* text with info about readiness */
	LOAD_FLOAT(info_str_r);
	LOAD_FLOAT(info_str_g);
	LOAD_FLOAT(info_str_b);
	LOAD_FLOAT(info_str_key_r);
	LOAD_FLOAT(info_str_key_g);
	LOAD_FLOAT(info_str_key_b);
	LOAD_INT(info_str_pos_percentage);
	
	LOAD_INT(bonus_wx);
	LOAD_INT(bonus_y);
	LOAD_INT(bonus_dy);

	LOAD_STRING(kills_icon);
	LOAD_INT(kills_wx); // position w-x
	LOAD_INT(kills_y);   // icon y
	LOAD_INT(kills_ty);  // text y
	LOAD_FLOAT(kills_r);
	LOAD_FLOAT(kills_g);
	LOAD_FLOAT(kills_b);

	LOAD_INT(weapon_wx); 
	LOAD_INT(weapon_y);
	LOAD_INT(weapon_ty);
	LOAD_FLOAT(weapon_r);
	LOAD_FLOAT(weapon_g);
	LOAD_FLOAT(weapon_b);
	LOAD_FLOAT(weapon_r_critical);
	LOAD_FLOAT(weapon_g_critical);
	LOAD_FLOAT(weapon_b_critical);
	LOAD_FLOAT(weapon_critical_ammo);
	
	LOAD_STRING(lives_icon);
	LOAD_INT(lives_wx); 
	LOAD_INT(lives_y);
	LOAD_INT(lives_ty);
	LOAD_FLOAT(lives_r_ok);
	LOAD_FLOAT(lives_g_ok);
	LOAD_FLOAT(lives_b_ok);
	LOAD_FLOAT(lives_r_ko);
	LOAD_FLOAT(lives_g_ko);
	LOAD_FLOAT(lives_b_ko);

	LOAD_STRING(armour_icon);
	LOAD_INT(armour_wx);
	LOAD_INT(armour_y);
	LOAD_INT(armour_ty);
	LOAD_FLOAT(armour_min_r);
	LOAD_FLOAT(armour_min_g);
	LOAD_FLOAT(armour_min_b);
	LOAD_FLOAT(armour_max_r);
	LOAD_FLOAT(armour_max_g);
	LOAD_FLOAT(armour_max_b);
	
	
	LOAD_STRING(radar_icon);
	LOAD_INT(radar_x);
	LOAD_INT(radar_y);
	LOAD_INT(radar_icon_radius);
	LOAD_DOUBLE(radar_radius);
	LOAD_FLOAT(radar_point_size);
	LOAD_DOUBLE(radar_tank_r);
	LOAD_DOUBLE(radar_tank_g);
	LOAD_DOUBLE(radar_tank_b);
	LOAD_DOUBLE(radar_bonus_r);
	LOAD_DOUBLE(radar_bonus_g);
	LOAD_DOUBLE(radar_bonus_b);
	LOAD_DOUBLE(radar_projectile_r);
	LOAD_DOUBLE(radar_projectile_g);
	LOAD_DOUBLE(radar_projectile_b);
	LOAD_DOUBLE(radar_other_r);
	LOAD_DOUBLE(radar_other_g);
	LOAD_DOUBLE(radar_other_b);

	LOAD_FLOAT(aim_point_size);
	LOAD_DOUBLE(aim_r);
	LOAD_DOUBLE(aim_g);
	LOAD_DOUBLE(aim_b);
	LOAD_DOUBLE(aim_dtime);
	LOAD_DOUBLE(aim_max_time);
	LOAD_DOUBLE(aim_stime);
	LOAD_STRING(aim_gunsight_icon);
	LOAD_STRING(aim_gunsight_icon_out);
	LOAD_STRING(aim_gunsight_icon_small);
		
	LOAD_INT(pause_msg_dy);
	
	LOAD_STRING(messages_font);
	LOAD_INT(messages_font_size);
	LOAD_INT(messages_lines);
	LOAD_INT(messages_top);
	LOAD_INT(messages_left);
	LOAD_INT(messages_max_time);
	LOAD_INT(messages_fade_time);
	LOAD_FLOAT(messages_r);
	LOAD_FLOAT(messages_g);
	LOAD_FLOAT(messages_b);
	LOAD_FLOAT(messages_edit_r);
	LOAD_FLOAT(messages_edit_g);
	LOAD_FLOAT(messages_edit_b);


	LOAD_FLOAT(tank_names_font_scale);
	LOAD_FLOAT(tank_names_z);


	LOAD_INT(blood_maxhit);
	LOAD_INT(blood_delay);
	LOAD_INT(blood_minhit);
	LOAD_STRING(blood_bighit_sound);
	LOAD_STRING(blood_smallhit_sound);


/* *********** statswindow ************ */
	
	LOAD_STRING(stats_header_font);
	LOAD_STRING(stats_data_font);
	LOAD_STRING(stats_win_image);
	LOAD_STRING(stats_ready_image);
    LOAD_INT(stats_ready_image_size);
	LOAD_INT(stats_header_font_size);
	LOAD_INT(stats_data_font_size);
	LOAD_FLOAT(stats_header_font_r);
	LOAD_FLOAT(stats_header_font_g);
	LOAD_FLOAT(stats_header_font_b);
	LOAD_FLOAT(stats_data_font_r);
	LOAD_FLOAT(stats_data_font_g);
	LOAD_FLOAT(stats_data_font_b);
	LOAD_FLOAT(stats_self_font_r);
	LOAD_FLOAT(stats_self_font_g);
	LOAD_FLOAT(stats_self_font_b);
	LOAD_FLOAT(stats_odd_line_box_r);
	LOAD_FLOAT(stats_odd_line_box_g);
	LOAD_FLOAT(stats_odd_line_box_b);
	LOAD_FLOAT(stats_odd_line_box_a);
	LOAD_FLOAT(stats_even_line_box_r);
	LOAD_FLOAT(stats_even_line_box_g);
	LOAD_FLOAT(stats_even_line_box_b);
	LOAD_FLOAT(stats_even_line_box_a);
	LOAD_INT(stats_win_vertical_percentage);
	LOAD_INT(stats_win_horizontal_percentage);
	LOAD_INT(stats_win_header_divider_percentage);
	LOAD_INT(stats_win_line_indent_percentage);
	LOAD_INT(stats_win_name_col_percentage);
	LOAD_INT(stats_win_score_col_percentage);
	LOAD_INT(stats_win_kills_col_percentage);
	LOAD_INT(stats_win_deaths_col_percentage);
	LOAD_INT(stats_line_separation_width);
	LOAD_INT(stats_line_width);

/* *********** Console ************ */

	LOAD_INT(fps_wx);
	LOAD_INT(fps_hy);
	LOAD_FLOAT(fps_r);
	LOAD_FLOAT(fps_g);
	LOAD_FLOAT(fps_b);

	LOAD_INT(con_box_x);
	LOAD_INT(con_box_dy);
	LOAD_FLOAT(con_box_r);
	LOAD_FLOAT(con_box_g);
	LOAD_FLOAT(con_box_b);
	LOAD_FLOAT(con_box_a);

	LOAD_STRING(con_font);
	LOAD_INT(con_font_size);
	LOAD_INT(con_font_x);
	LOAD_INT(con_font_hy);
	
	LOAD_STRING(con_errmsg_font);
	LOAD_INT(con_errmsg_font_size);
	LOAD_FLOAT(con_errmsg_r);
	LOAD_FLOAT(con_errmsg_g);
	LOAD_FLOAT(con_errmsg_b);
	LOAD_INT(con_errmsg_max_time);
	LOAD_INT(con_errmsg_fade_time);
	
	
/* ************* Loading window *******/
	LOAD_STRING(loading_font);
	LOAD_INT(loading_font_size);
	
/* ************ Ingame menu *********/
    LOAD_INT(ingame_autoshow_delay);


	
/* *********** Main menu **********/
    LOAD_STRING(mmain_background);

/* *********** Condition info *****/
	LOAD_STRUCT_STRING( conditions, position );
	LOAD_STRUCT_STRING( conditions, namefont );
	LOAD_STRUCT_STRING( conditions, valuefont);
	LOAD_STRUCT_INT( conditions, namesize );
	LOAD_STRUCT_INT( conditions, valuesize);
	LOAD_STRUCT_FLOAT( conditions, name_r );
	LOAD_STRUCT_FLOAT( conditions, name_g );
	LOAD_STRUCT_FLOAT( conditions, name_b );
	LOAD_STRUCT_FLOAT( conditions, value_r);
	LOAD_STRUCT_FLOAT( conditions, value_g);
	LOAD_STRUCT_FLOAT( conditions, value_b);
	LOAD_STRUCT_INT( conditions, width );
	LOAD_STRUCT_INT( conditions, border_space );
}
