/* MAINTAINER: Bernard */

//#define WANT_TRACE

#include <sstream>
#include <iomanip>
#include "log.h"
#include "infolayer.h"
#include "tank.h"
#include "player.h"
#include "application_client.h"
#include "conversions.h"
#include "timemanager.h"
#include "translator.h"
#include "params.h"
#include "gui_constants.h"
#include "world.h"
#include "gfx.h"
#include "sfx.h"
#include "game_client.h"
#include "mousemotion.h"
#include "keybindings.h"
#include "globals.h"

using namespace std;

InfoLayer::InfoLayer()
{
	TIN;

	mPlayer = 0;
	mFont = 0;
	mFontIcons = 0;
	
	mLastHP = 0;
	mLastBloodShown = 0;
	mLastHitPower = 0;
}

InfoLayer::~InfoLayer()
{

}

void InfoLayer::render()
{
	TIN;
	
	updateInfoVariables();

	getAppClient().getGFX().disableMultitexturing();
	glColor3f( 1.0, 1.0, 1.0);

	render3DNames();

	// Aiming first since it is 3D and then switch to 2D
	renderAiming();

	// Gamse state info - like pause and message from gameplay coordinator
	renderGameStateInfo();
	
	renderBonuses();
    	
	renderStatusBar();

	renderRadar();

	renderBlood();

	
}

void InfoLayer::updateInfoVariables()
{
	// Update mPlayer and mTank;
	mPlayer = getAppClient().getGame().getClientPlayer();
	if (mPlayer)
	{
		mTank = mPlayer->getTank();
		if (mTank && !mTank->isAlive())
		{
			mTank = 0;
		}
	}
	
	mWidth  = gGL2D.width();
	mHeight = gGL2D.height();	
}

void InfoLayer::init()
{
	TIN;

	mPlayer = getAppClient().getGame().getClientPlayer();

	// Reset HP 
	mLastHP = 0;
	mLastBloodShown = 0;
	mLastHitPower = 0;


	// Loading font
	mFont = FontManager::getRef().get(gc.info_font.c_str(), gc.info_font_size);
	if (!mFont)
	{
		LOGE << "Loading font " << gc.info_font << " failed!" << endl;
		DIE();
	}
	mFontIcons = FontManager::getRef().get(gc.info_icons_font.c_str(), gc.info_icons_font_size);
	if (!mFontIcons)
	{
		LOGE << "Loading font " << gc.info_icons_font << " failed!" << endl;
		DIE();
	}
	mCondMsgFont = FontManager::getRef().get( gc.conditions.namefont.c_str(), gc.conditions.namesize );
	if (!mCondMsgFont)
	{
		LOGE << "Loading font " << gc.conditions.namefont << " failed!" << endl;
		DIE();
	}
	mCondValFont = FontManager::getRef().get( gc.conditions.valuefont.c_str(), gc.conditions.valuesize );
	if (!mCondValFont)
	{
		LOGE << "Loading font " << gc.conditions.valuefont << " failed!" << endl;
		DIE();
	}

	
	// Loading bonus icons
	mBonusTextures.assign(gParams.mBonus.size(), NULL);
	for (int i = 0; (unsigned int)i < gParams.mBonus.size(); i++)
	{
		const string &icon = gParams.bonus(i).icon;

		if (icon.empty()) 
		{	
			mBonusTextures[i] = NULL;
			continue;
		}

		Texture2DGL *tex = new Texture2DGL();
		if (tex == 0) DIE("new");
		tex->load(icon);

		mBonusTextures[i] = tex;		
	}

	
	// Loading weapons icons
	mWeaponsTextures.assign(gParams.mWeapons.size(), NULL);
	for (int i = 0; (unsigned int)i < gParams.mWeapons.size(); i++)
	{
		const string &icon = gParams.weapon(i).icon;

		if (icon.empty()) 
		{
			mWeaponsTextures[i] = NULL;
			continue;
		}

		Texture2DGL *tex = new Texture2DGL();
		if (tex == 0) DIE("new");
		tex->load(icon);

		mWeaponsTextures[i] = tex;		
	}
	
	// Other Gui Bitmaps
	mTexLives.load(gc.lives_icon);
	mTexArmour.load(gc.armour_icon);
	mTexKills.load(gc.kills_icon);
	mTexRadar.load(gc.radar_icon);
	mTexGunSight.load(gc.aim_gunsight_icon);
	mTexGunSightOutOfReach.load(gc.aim_gunsight_icon_out);
}

void InfoLayer::clean()
{
	TIN;

	// assertL(mBonusTextures != 0);

	// clean font 
	mFont = 0;
	mFontIcons = 0;
	mCondMsgFont = 0;
	mCondValFont = 0;
	
	// clean bonus icons
	for (int i = 0; (unsigned int)i < gParams.mBonus.size(); i++)
	{
		if (mBonusTextures[i] != NULL)
		{
			delete mBonusTextures[i];
			mBonusTextures[i] = NULL;
		}	
	}
	mBonusTextures.clear();

	// clean weapon icons
	for (int i = 0; (unsigned int)i < gParams.mWeapons.size(); i++)
	{
		if (mWeaponsTextures[i] != NULL)
		{
			delete mWeaponsTextures[i];
			mWeaponsTextures[i] = NULL;
		}	
	}
	mWeaponsTextures.clear();

}

void InfoLayer::renderBalisticAiming()
{
	TIN;

	if (mPlayer == 0 || mTank == 0) return;

	getAppClient().getGFX().set3DOpenGL();


	// get speed of projectile
	int weaponID = mTank->getWeapon();
	if (weaponID == INVALID_WEAPON_ID) return;
	
	double speed = gParams.projectile(gParams.weapon(weaponID).projectile).speed; 

	// get gravity constant
	double g     = getAppClient().getGame().getMapParams().gravity_constant;

	// Get vector where projectile starts its trip and which direction
	GLVector3d pos;
	GLQuaterniond ori;
	mTank->getShotOriAndStart( ori, pos );
	GLVector3d direction = ori.rotateVector(GLVector3d(speed, 0, 0));

	// Add tank speed to the direction of projectile
	direction += mTank->getSpeed();

	// prepare GL point drawing
	glPointSize(gc.aim_point_size);
//	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	

	glColor3d(gc.aim_r, gc.aim_g, gc.aim_b);
	glBegin(GL_POINTS);
	glVertex3d(pos.x, pos.y, pos.z);
	glEnd();


	GLVector3d current;
	for (double t = gc.aim_stime; t < gc.aim_max_time; t *= gc.aim_dtime)
	{
		// compute position of balistic curve in time
		current = pos + direction * t;
		current.z -= 0.5*t*t*g; // the gravity part

		glBegin(GL_POINTS);
		glVertex3d(current.x, current.y, current.z);
		glEnd();
	}	
}
	
void InfoLayer::render3DAiming()
{
	if (!mTank) return;

	renderBalisticAiming();
	
	glColor3f( 1.0, 1.0, 1.0);
		
	MouseMotion& targeting = getAppClient().getGame().getMouseMotion();
//	const GLVector3d &target = targeting.get3DTarget();
	
	// m3DTarget = neco podle mX a mY
	GLint viewport[4];
/*	GLdouble modelview[16];
	GLdouble projection[16];
	GLdouble winX, winY, winZ;

	glGetDoublev(GL_MODELVIEW_MATRIX, modelview );
	glGetDoublev(GL_PROJECTION_MATRIX, projection);*/
	glGetIntegerv(GL_VIEWPORT, viewport);

     
    //winZ = 1;     - takhle by to strilelo vzdycky az "dozadu"

//	gluProject(target.x, target.y, target.z, modelview, projection, viewport, &winX, &winY, &winZ);

//    winY = viewport[3] - winY - 1; // Flip mouse coordinates to OpenGL coortinages

//	cout << "Unprojected: " << winX << " " << winY << endl;
	
	
	int x, y;
	targeting.getMouseXY(x, y);
	y = viewport[3] - y - 1; // Flip mouse coordinates to OpenGL coortinages
	
	if (targeting.isOutOfReach())
	{
	    getAppClient().getGFX().set2DOpenGL();			
//		gGL2D.drawRect( (int)winX, (int)winY, mTexGunSightOutOfReach); 
		gGL2D.drawRect( x, y, mTexGunSightOutOfReach); 
//		gGL2D.renderToCamera(target, 20, mTexGunSightOutOfReach);
	}
	else
	{
	    getAppClient().getGFX().set2DOpenGL();			
//		gGL2D.drawRect( (int)winX, (int)winY, mTexGunSight);
		gGL2D.drawRect( x, y, mTexGunSight);	

//		gGL2D.renderToCamera(target, 20, mTexGunSight );
	}
	
}

void InfoLayer::renderAimingCross(const GLVector3d &target)
{	
	//TODO: here could be some size limit if targeting to near point
	gGL2D.renderToCamera(target, 20, mTexGunSight);	
//	renderBalisticAiming();
}



void InfoLayer::renderAiming()
{
	
//	if (getAppClient().getGame().getMouseMotion().is3DTargeting())
	{
		render3DAiming();
	}
/*	else
	{
		renderBalisticAiming();	
	}
*/	
	//	renderBalisticAiming();	
}


void InfoLayer::render3DNames()
{
	if (mPlayer == 0) return;
	int   ourteam = mPlayer->getTeam();

	getAppClient().getGFX().set3DOpenGL();

	Game &g = getAppClient().getGame();

	const std::vector<int>& ids = g.getValidPlayersID();
	std::vector<int>::const_iterator it = ids.begin();

	// Go through all players
	for(; it != ids.end(); ++it)
	{
		Player &p = g.getPlayer(*it);

		//if (mPlayer == &p) continue;

		Tank *tank;
		if ((tank = p.getTank()) != 0)
		{
			// Skip name for our tank
			if (tank == mTank) continue;
			
			// skip tank wrecks
			if (!tank->isAlive()) continue;

			const GLVector3d& pos = tank->getPosition();
			
			if (p.getTeam() == ourteam)
				glColor3f( 0.0, 0.0, 1.0);
			else
				glColor3f( 1.0, 0.0, 0.0);

			mFont->RenderUTF83D(p.getName(), (float)pos.x, (float)pos.y, 
				(float)(pos.z+gc.tank_names_z), (float)gc.tank_names_font_scale);
		}
	}
}


void InfoLayer::renderBlood()
{
	int hp;
	if (mTank) hp = mTank->getHP();
	else hp = 0;
	
	// check HP and draw hit
	int max  = gc.blood_maxhit;
	int time = gc.blood_delay;
	int min  = gc.blood_minhit;

	int ticks = gTime.getGameTicks();
	if (hp < mLastHP)
	{
		mLastHitPower = (float)(mLastHP - hp);
		if (mLastHitPower > max) mLastHitPower = (float)max;
		if (mLastHitPower < min) mLastHitPower = (float)min;
		
		// and do the sound
		if ( mLastHitPower > (min+max)/2)
		{   // Big hit
			getAppClient().getSFX().play(gc.blood_bighit_sound.c_str());
		}
		else
		{    // small hit
			getAppClient().getSFX().play(gc.blood_smallhit_sound.c_str());			
		}
		
		mLastHitPower /= 100; // into percents
		mLastBloodShown = ticks;
	}
	mLastHP = hp; // also in case mLastHP < t->getHP()
	
	// And here we draw the red screen
	if (ticks-mLastBloodShown < time)
	{
		getAppClient().getGFX().set2DOpenGL();
	
		float  alpha = mLastHitPower * (1.0f - (float)(ticks-mLastBloodShown)/time);
		gGL2D.drawRect(0, 0, mWidth, mHeight, 1, 0, 0, alpha);
	}
}




void InfoLayer::renderRadar()
{
	TIN;
	
	if (!mTank) return;

	getAppClient().getGFX().set2DOpenGL();

	// Draw the radar background
	glColor3f( 1.0, 1.0, 1.0);
	gGL2D.drawRect( gc.radar_x, gc.radar_y, mTexRadar); 

	// Prepare tank position and rotation - the rotation is used to
	// rotate positions of other entities on radar
	const GLVector3d &pos = mTank->getPosition();
	GLQuaterniond orient;
	getAppClient().getGame().getMouseMotion().getCameraOrientation(orient);
	orient.inverse();

	// We will go through all world ettites	
	worldent_iterator it  = getAppClient().getGame().getWorld().getWorldEntitiesBegin();
	worldent_iterator end = getAppClient().getGame().getWorld().getWorldEntitiesEnd();

	GLVector3d dist;

	glPointSize(gc.radar_point_size);
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_POINTS);

	for (;it != end; ++it)
	{
		// we don't need our tank to be on radar
		if (*it == mTank) continue;

		// Compute distance between entites
		dist = pos - (*it)->getPosition();
		dist.z = 0; // we ignore z coordinate for radar purposes

		double l;   // Draw only those who are near
		if ((l = dist.length()) <= gc.radar_radius)
		{
			// rotate the vector using tank's vector 
			// and scale it to radar scale - this is
			// some magic probably :-)
			dist = orient.rotateVector(dist);
			dist.z = 0;
			if (dist.lengthSqr() == 0) continue;
			dist.normalize();
			dist *= l*(double)gc.radar_icon_radius/gc.radar_radius;

			// pick the color denepding on type of entity
			switch ((*it)->getClassType())
			{
			case cProjectile :
				glColor3d(gc.radar_projectile_r, gc.radar_projectile_g, gc.radar_projectile_b);
			    break;
			case cBonusBox :
				glColor3d(gc.radar_bonus_r, gc.radar_bonus_g, gc.radar_bonus_b);
			    break;
			case cTank :
				glColor3d(gc.radar_tank_r, gc.radar_tank_g, gc.radar_tank_g);
			    break;
			default:
				glColor3d(gc.radar_other_r, gc.radar_other_g, gc.radar_other_b);
			    ;
			}
			
			// Finally draw the one point :-) Correct order
			// was found by many experiments.
			glVertex3d(gc.radar_x+dist.y, gc.radar_y-dist.x, 0.0);

		}
	}

	glEnd();
}


void InfoLayer::pickedBonus(int id)
{
	TIN;

	// Add to queue only if it is interesting :-)
	if (gParams.bonus(id).duration > 0 || gParams.bonus(id).outlive_tank)
		mBonusID.push_back(id);
}

void InfoLayer::expiredBonus(int id)
{
	TIN;

	// Try to find in list and remove it
	std::list<int>::iterator it = mBonusID.begin();
	for (; it != mBonusID.end(); ++it)
	{
		if (*it == id)
		{
			mBonusID.erase(it);
			break;
		}

	}
}


void InfoLayer::renderGameStateInfo()
{
	getAppClient().getGFX().set2DOpenGL();
	
	// Pause sign
	if (gTime.isPaused())
	{
    	mFont->RenderUTF8WithShadow( _("Pause"), mWidth/2, mHeight/2 + gc.pause_msg_dy,
    				   gc.info_str_r, gc.info_str_g, gc.info_str_b);
	}
	
	
		//display "waiting for..." message
	if ( getAppClient().getGame().getStateOfGame() == WaitForPlayers )
	{
		if (mPlayer && !mPlayer->isReady())
		{
			std::string message(_("Waiting for you to get ready."));
			mFont->RenderUTF8WithShadow
			(
				message, mWidth/2, mHeight - (mHeight * gc.info_str_pos_percentage / 100),
	    		gc.info_str_r, gc.info_str_g, gc.info_str_b
			);

			KeyBindings& keybind = getAppClient().getKeyBindings();
			message = string(_("Press") ) + " ";
			message.append( keybind.whatKeyForCommand("/ready") );
			
			mFont->RenderUTF8WithShadow(  message, mWidth/2, 
				mHeight - (mHeight * gc.info_str_pos_percentage / 100 + mFont->Ascent()),
	    		gc.info_str_key_r, gc.info_str_key_g, gc.info_str_key_b
			);		
		}
		else
		{
		   	mFont->RenderUTF8WithShadow(
				_("Waiting for others get ready."), mWidth/2, 
				mHeight - ( mHeight * gc.info_str_pos_percentage / 100 ),
				gc.info_str_r, gc.info_str_g, gc.info_str_b
			);
		}
		glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
	}
		
	if ( getAppClient().getGame().getStateOfGame() > WaitForPlayers )
	{
			//prepare drawing position
		int ascent = max( mCondMsgFont->Ascent(), mCondValFont->Ascent() );
		int rect_height = ascent * (int)mCondStatuses.size();
		int rect_width = gc.conditions.width;

		int pos_x = mWidth - rect_width - gc.conditions.border_space;
		int pos_y = gc.conditions.border_space;

			//depends on position parameter
		if ( gc.conditions.position == "right-top" )
		{
			pos_x = mWidth - rect_width - gc.conditions.border_space;
			pos_y = gc.conditions.border_space;
		}
		if ( gc.conditions.position == "left-top" )
		{
			pos_x = gc.conditions.border_space;
			pos_y = gc.conditions.border_space;
		}
		if ( gc.conditions.position == "left-bottom" )
		{
			pos_x = gc.conditions.border_space;
			pos_y = mHeight - rect_height - gc.conditions.border_space;
		}
		if ( gc.conditions.position == "right-bottom" )
		{
			pos_x = mWidth - rect_width - gc.conditions.border_space;
			pos_y = mHeight - rect_height - gc.conditions.border_space;
		}
		if ( gc.conditions.position == "center-bottom" )
		{
			pos_x = mWidth / 2 - rect_width / 2;
			pos_y = mHeight - rect_height - gc.conditions.border_space;
		}
		if ( gc.conditions.position == "center-top" )
		{
			pos_x = mWidth / 2 - rect_width / 2;
			pos_y = gc.conditions.border_space;
		}

		std::list< ConditionStatus >::iterator it = mCondStatuses.begin();
		std::list< ConditionStatus >::iterator ite = mCondStatuses.end();

		int line = 1;
		for ( ; it != ite; ++it, ++line )
		{
			ConditionStatus &cs = *it;

			assertL( cs.name.length() );
			cs.name[0] = toupper( cs.name[0] );

			// _("Fraglimit")   _("Timelimit")  - to extract strings for translaions
			mCondMsgFont->RenderUTF8WithShadow
			(
				_(cs.name.c_str()), pos_x, mHeight - ( pos_y + ascent * line ),
				gc.conditions.name_r, gc.conditions.name_g, gc.conditions.name_b,
				Font::LEFT_BOTTOM
			);

			string value;
			bool err = true;
			if ( cs.name == "Fraglimit" )
			{
				value = itos( cs.currentvalue ) + string(" ") + _("of") + string( " " ) + itos( cs.targetvalue );
				err = false;
			}
			if ( cs.name == "Timelimit" )
			{
				int timeleft = cs.targetvalue - cs.currentvalue;
				string sign;
				if ( timeleft < 0 )
					sign = "-";
				timeleft = abs( timeleft );
				int mins = timeleft/1000/60;
				int secs = (timeleft % (60 * 1000))/1000;

				stringstream ss;
				ss << sign << mins << ":" << setw(2) << setfill('0') << secs;
				value = ss.str();
				err = false;
			}

			if ( err )
				LOGE << "Unknown end map condition: " << cs.name << endl;

			mCondValFont->RenderUTF8WithShadow(
				value, pos_x + rect_width, mHeight - ( pos_y + ascent * line ),
				gc.conditions.value_r, gc.conditions.value_g, gc.conditions.value_b,
				Font::RIGHT_BOTTOM
			);
		}
	}
}

void InfoLayer::renderBonuses()
{
	if (!mPlayer) return;
	
	getAppClient().getGFX().set2DOpenGL();
	
	// Clear bonuses if the tank is dead
	Tank *t = mPlayer->getTank();
	if (!t || !t->isAlive())
	{
		// No tank means no bonuses. So clear them
		mBonusID.clear();
		return;
	}		
	
	// Bonuses
	int bott = gc.bonus_y;
	int    x = mWidth - gc.bonus_wx;
	
	std::list<int>::iterator it = mBonusID.begin();
	for (; it != mBonusID.end(); ++it)
	{
		//string name = gParams.bonus(*it).name;
		//mFont->RenderUTF8( name, w - 200, bott);
		
		// we draw only icons..
		if (mBonusTextures[*it] != NULL)
		{
			gGL2D.drawRect( x, bott, (*mBonusTextures[*it])); 
			bott += gc.bonus_dy;
		}
	}
}

void InfoLayer::renderStatusBar()
{	
	getAppClient().getGFX().set2DOpenGL();
	
	if (!mPlayer) return;
	
	// Kills	
	glColor3f( 1.0f, 1.0f, 1.0f);
	gGL2D.drawRect( mWidth - gc.kills_wx, gc.kills_y, mTexKills); 
	mFontIcons->RenderUTF8WithShadow(
		 xtos(mPlayer->getKills()), mWidth - gc.kills_wx, gc.kills_ty,
		 gc.kills_r, gc.kills_g, gc.kills_b
	);

	if (!mTank) return;	
	// Tank specific things

	// Weapon and projectiles	
	int weapon = mTank->getWeapon();
	if (weapon !=  INVALID_WEAPON_ID)
	{
		if (mWeaponsTextures[weapon] !=NULL)
		{
			glColor3f( 1.0f, 1.0f, 1.0f);	
			gGL2D.drawRect(mWidth - gc.weapon_wx, gc.weapon_y, *mWeaponsTextures[weapon] );
		}
		
		// create message  ammon/max_ammo
		int projectiles = mTank->getProjectiles();
		int projectileType = gParams.weapon(weapon).projectile;
		int max_projectiles = gParams.projectile(projectileType).carry_max;
		stringstream msg;
		msg << projectiles << "/" << max_projectiles;
		
		// determine if we have only a few projectiles
		if ((int)(max_projectiles * gc.weapon_critical_ammo) < projectiles)
		{
			mFontIcons->RenderUTF8WithShadow(
				msg.str(), mWidth-gc.weapon_wx, gc.weapon_ty,
				gc.weapon_r, gc.weapon_g, gc.weapon_b
			);
		}
		else
		{
			mFontIcons->RenderUTF8WithShadow(
				msg.str(), mWidth-gc.weapon_wx, gc.weapon_ty,
				gc.weapon_r_critical, gc.weapon_g_critical, gc.weapon_b_critical
			);			
		}
	}

	// Tank's HP
	float hp = (float)mTank->getHP() / (float)mTank->getMaxHP();
	float lives_r = hp*gc.lives_r_ok + (1-hp)*gc.lives_r_ko;
	float lives_g = hp*gc.lives_g_ok + (1-hp)*gc.lives_g_ko;
	float lives_b = hp*gc.lives_b_ok + (1-hp)*gc.lives_b_ko;
	stringstream msgHP;
	msgHP << mTank->getHP() << "/" << mTank->getMaxHP();
	
	glColor3f( 1.0f, 1.0f, 1.0f);	
	gGL2D.drawRect( mWidth-gc.lives_wx, gc.lives_y, mTexLives); 
    mFontIcons->RenderUTF8WithShadow(
    	msgHP.str(), mWidth-gc.lives_wx, gc.lives_ty,
    	lives_r, lives_g, lives_b
    );

	// Tank's Armour
	float armour =   (float)mTank->getArmour() /  (float)mTank->getMaxArmour();
	float armour_r = armour*gc.armour_max_r + (1-armour)*gc.armour_min_r;
	float armour_g = armour*gc.armour_max_g + (1-armour)*gc.armour_min_g;
	float armour_b = armour*gc.armour_max_b + (1-armour)*gc.armour_min_b;
	stringstream msgArmour;
	msgArmour << mTank->getArmour() << "/" << mTank->getMaxArmour();
	
	glColor3f( 1.0f, 1.0f, 1.0f);
	gGL2D.drawRect(mWidth-gc.armour_wx, gc.armour_y, mTexArmour); 
    mFontIcons->RenderUTF8WithShadow(
    	msgArmour.str(), mWidth-gc.armour_wx, gc.armour_ty,
        armour_r, armour_g, armour_b 
    );
}
