/* MAINTAINER: Bernard */
#ifndef  _INFO_LAYER_H
#define  _INFO_LAYER_H

#include "font_manager.h" 
#include "opengl2d.h"
#include "game.h"

#include <list>

class Player;
class Tank;

/**
 * \brief Class for rendering all the game info for player.
 * 
 * The class is able to render aiming, Names over enemy players,
 * radar, interesting game icons, gamestate info and also turns the
 * whole screen into red if player get some hit.
 */
class InfoLayer
{
public:
	InfoLayer();
	virtual ~InfoLayer();

	void setPlayer(Player *player) { mPlayer = player; }
	
	/**
	 * \brief Method for dooing all the fun.
	 * 
	 * Call all renderXXX private function.
	 */
	virtual void render();

	/**
	 * \brief Load textures and fonts.
	 */
	virtual void init();

	/**
	 * \brief Free all loaded font and textures lists.
	 */
	virtual void clean();

	/**
	 * \brief Add bonus of ID tho the list of currently render bonuses.
	 * 
	 * \param id Type of bonus.
	 */
	void pickedBonus(int id);

	/**
	 * \brief Remove bonus icon of some type from list of bonuses.
	 * 
	 * \param id Type of bonus.
	 */
	void expiredBonus(int id);
	
	/**	\brief Provides list of statuses of end-map-conditions.
	 *
	 *	\param cstatus List of statuses
	 */
	void setEndConditionStatuses(const std::list< ConditionStatus > & cstatus )
	{
		mCondStatuses = cstatus;
	}
	
private:	

	/**
	 * \brief Render only target of shooting.
	 */
	void render3DAiming();
	
	/**
	 * \brief Render points of projectile route.
	 */
	void renderBalisticAiming();
	
	/**
	 * \brief Decide between balistic and 3D aiming.
	 * 
	 * The decision depends on currently selected mode
	 * int mousemotion - tank aiming
	 */
	void renderAiming();
	
	/**
	 * \brief Render and compute radar.
	 */
	void renderRadar();
	
	/**
	 * \brief Render aiming cross to some specific position.
	 */
	void renderAimingCross(const GLVector3d &target);
	
	/**
	 * \brief Render names over other players tanks
	 * */
	void render3DNames();
	
	/**
	 * \brief Render RED blood over screen if player's tank
	 *        get some hit.
	 */
	void renderBlood();

	/**
	 * \brief Render message from GameArbitter and pause
	 */
	void renderGameStateInfo();


	/**
	 * \brief Render box with bonuses.
	 */
	void renderBonuses();

	/**
	 * \brief Render status bar at the bottom
	 */
	void renderStatusBar();


	/**
	 * \brief update variables like mPlayer, mTank, mWidth and mHeight
	 */
	void updateInfoVariables();

	Player       *mPlayer;  ///< \brief Monitored player. May be NULL.
	Tank           *mTank;  ///< \brief Monitored tank. May be NULL. If tank isn't alive, it is also NULL
	FontRef         mFont;  ///< \brief Font for info layer - for text.
	FontRef    mFontIcons;  ///< \brief Font for info layer - for numbers under icons.
	FontRef	 mCondMsgFont;	///< \brief Font for condition name message.
	FontRef	 mCondValFont;	///< \brief Font for condition value message.
	int           mWidth;   ///< \brief Width of screen.
	int           mHeight;  ///< \brief Height of screen. Updatet every frame.

	typedef std::vector<Texture2DGL*> vectorTexturesType;

	vectorTexturesType mBonusTextures;   ///< \brief Icons with bonuses.
	vectorTexturesType mWeaponsTextures; ///< \brief Icons with weapons.

	Texture2DGL mTexLives;
	Texture2DGL mTexArmour;
	Texture2DGL mTexKills;
	Texture2DGL mTexRadar;
	Texture2DGL mTexBlood;
	Texture2DGL mTexGunSight;
	Texture2DGL mTexGunSightOutOfReach;

	int   mLastHP;         ///< \brief Last number of players HP.
	int   mLastBloodShown; ///< \brief Timestamt when blood was shown for the last time.
	float mLastHitPower;   ///< \brief Power of last hit - more hit, more red screen.

	std::list<int>  mBonusID; ///< \brief Just list with IDs of currently picked bonuses.

	std::list< ConditionStatus > mCondStatuses; ///< \brief List of mesages 

};

#endif // _STATS_WINDOW_H
