/* MAINTAINER: Bernard */

#include "SDL.h"
#include "SDL_opengl.h"

#include "platform.h"
#include "log.h"
#include "messages.h"
#include "conversions.h"
#include "application_client.h"
#include "opengl2d.h"
#include "timemanager.h"
#include "gui_constants.h"
#include "inputmanager.h"
#include "gfx.h"
#include "commander.h"

using namespace std;


Messages::Messages()
{
	mPriority = IA_PRIORITY_CHAT;
	mActive = false;
	
	mFont = 0;
}

Messages::~Messages()
{

}


void Messages::init()
{
	TIN;

	// registration to input manager
	getAppClient().getInputManager().addInputAcceptor(*this);

	// Loading font
	mFont = FontManager::getRef().get(gc.messages_font.c_str(), gc.messages_font_size);
	if (!mFont)
	{
		LOGE << "Loading font failed!" << endl;
		DIE();
	}
}

void Messages::clean()
{
	TIN;

	// unregister from input
	getAppClient().getInputManager().removeInputAcceptor(*this);

	// clean font 
	mFont = 0;
}



void Messages::addMessage(const std::string &msg)
{
	TIN;
	
	//LOGD << "New message " << msg << endl;
	Message m(msg);

	mMessages.push_front(m);
	
	if ((signed int)mMessages.size() > gc.messages_lines)
		mMessages.pop_back();
}

void Messages::render()
{
	const int top  = gc.messages_top; 
	const int left = gc.messages_left;
	
	assertL(mFont);
	

	gGL2D.init();

	// OpenGL init
	getAppClient().getGFX().set2DOpenGL();
	
	// Decorations
	
	int t = gTime.getTicks();

	MessageList::iterator it = mMessages.begin();
	int i = gc.messages_lines;
	while( it != mMessages.end() )
	{
		if ((t - it->timeStamp) > gc.messages_max_time)
		{
			it = mMessages.erase(it);
		}
		else
		{
			// set alpha
			float alpha = 1.0;

			if ((t-it->timeStamp) > gc.messages_fade_time)
			{
				alpha = 1.0f - (float)((t-it->timeStamp)-gc.messages_fade_time)/
				        (float)(gc.messages_max_time-gc.messages_fade_time);
			}

			
			// Shadow under text
			glColor4f(0, 0, 0, alpha);
			mFont->RenderUTF8(
				it->msg,
				left+1,
				getAppClient().getGFX().getHeight() - ( top + i*mFont->Ascent()) -1
			);
			
			// The text itself
			glColor4f(gc.messages_r, gc.messages_g, gc.messages_b, alpha);		
			mFont->RenderUTF8(
				it->msg,
				left,
				getAppClient().getGFX().getHeight() - ( top + i*mFont->Ascent())
			);
			


			++it;
			--i;
		}
	}	
	
	// commandline
	if (mActive)
	{
		// The text	with shadow
		mFont->RenderUTF8WithShadow
		(
			mInputLine.getLineURender(), left,
			getAppClient().getGFX().getHeight() - ( top + (gc.messages_lines+2)*mFont->Ascent()),
			gc.messages_edit_r, gc.messages_edit_g, gc.messages_edit_b, Font::LEFT_BOTTOM
		);
	}

}


bool Messages::wantKey(const SDL_keysym &keysym)
{
	return mActive;
}

bool  Messages::keyDown(const SDL_keysym &keysym)
{
	TIN 

	if (!mActive) return false;

	//LOGT << "Processing new key " << keysym.sym << " :-)\n";
	
	switch (keysym.sym) 
	{

	// process command & erase commmand line
	case SDLK_RETURN:
	case SDLK_KP_ENTER:		
		{
			// avoid sending empty line
			if ( mInputLine.getLineU() != "" ) 
			{
				// create command for server and mass it to commander
				string command("/chat ");	
				command.append(mInputLine.getLineU());
				getAppClient().getCommander().addCommand(command);
			}

			// And deactivate
			mInputLine.clear();
			activate(false);
		}
		break;
	
	// adding one char to commandline
	default:
		mInputLine.processKey(keysym);
		break;
	}
	
	return true;
}


void Messages::activate(bool state)
{
	if (mActive != state)
	{
		mActive = state;
		getAppClient().getInputManager().wantAllInput(mActive);
	}
}
