
#include "statswindow.h"
#include "application_client.h"
#include "log.h"
#include "gui_constants.h"
#include "gfx.h"
#include "opengl2d.h"
#include "translator.h"
#include "game_client.h"
#include "player.h"
#include "conversions.h"
#include "platform.h"

StatsWindow::StatsWindow()
{
	mPlayer = 0;
	mVisible = false;
}

StatsWindow::~StatsWindow()
{
}

void StatsWindow::render()
{
	if ( (! mVisible) && ( getAppClient().getGame().getStateOfGame() != EndOfGame ) ) return;
	//if (mPlayer == NULL) return;

	// Set to 2D
	getAppClient().getGFX().set2DOpenGL();
	
	
	int w = gGL2D.width();
	int h = gGL2D.height();

	int tb_diff = (h - ( h * gc.stats_win_horizontal_percentage / 100 )) / 2;
	int rl_diff = (w - ( w * gc.stats_win_vertical_percentage / 100 )) / 2;

	int win_r, win_l, win_b, win_t;

	win_l = rl_diff;
	win_r = w - rl_diff;
	win_b = h - tb_diff;
	win_t = tb_diff;
		
		//draw basic background image
	gGL2D.drawRect( win_l, win_t, win_r, win_b, mTexture );

		//add indentations to left and right window edges
	int indent_diff = ( win_r - win_l ) * gc.stats_win_line_indent_percentage / 100;
	win_l += indent_diff;
	win_r -= indent_diff;
	
		//calculate position of separator line
	int sep_pos = win_t + ( ( win_b - win_t ) * gc.stats_win_header_divider_percentage / 100 );

		//calculate collumn startings
	int tot_col_percentage = 
		gc.stats_win_name_col_percentage +
		gc.stats_win_score_col_percentage +
		gc.stats_win_kills_col_percentage +
		gc.stats_win_deaths_col_percentage;

	if ( tot_col_percentage > 100 )
	{
		LOGE << "Columns in stats window has more than 100% of line width!!" << endl;
	}

	int space_width = ( win_r - win_l ) * ( 100 - tot_col_percentage ) / 100 / 5 /* 5 collumns */;

	int namecol_start	= win_l + space_width;
	int scorecol_start	= namecol_start + (( win_r - win_l ) * gc.stats_win_name_col_percentage / 100 ) + space_width;
	int killscol_start	= scorecol_start + (( win_r - win_l ) * gc.stats_win_score_col_percentage / 100 ) + space_width;
	int deathscol_start	= killscol_start + (( win_r - win_l ) * gc.stats_win_kills_col_percentage / 100 ) + space_width;


		//draw header
	float hcolor[4] = { gc.stats_header_font_r, gc.stats_header_font_g, gc.stats_header_font_b, 1.0f };

	int header_text_bottom = sep_pos - gc.stats_line_separation_width;

	textWithShadow( mHeaderFont, _("Name"),	  h, namecol_start,	 header_text_bottom, hcolor);
	textWithShadow( mHeaderFont, _("Score"),  h, scorecol_start, header_text_bottom, hcolor);
	textWithShadow( mHeaderFont, _("Kills"),  h, killscol_start, header_text_bottom, hcolor);
	textWithShadow( mHeaderFont, _("Deaths"), h, deathscol_start,header_text_bottom, hcolor);

		//Sort players
	GameClient &game =	getAppClient().getGame();
	game.sortValidPlayers();

		//Render Players
	playerVectorType::iterator it  = game.getValidPlayersVectorBegin();
	playerVectorType::iterator end = game.getValidPlayersVectorEnd();

	float dcolor[4] = { gc.stats_data_font_r, gc.stats_data_font_g, gc.stats_data_font_b, 1.0f };
	float scolor[4] = { gc.stats_self_font_r, gc.stats_self_font_g, gc.stats_self_font_b, 1.0f };

	int linenum = 1;
	for( ; it != end; ++it)
	{
			//draw line rectangle
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		glEnable( GL_BLEND );
		
		int ltop = sep_pos + linenum * gc.stats_line_separation_width + (linenum - 1) * gc.stats_line_width;
		int lbot = sep_pos + linenum * gc.stats_line_separation_width + linenum * gc.stats_line_width;

		if ( linenum % 2 == 1 )	
		{		//odd lines
			glColor4d
			(
				gc.stats_odd_line_box_r,
				gc.stats_odd_line_box_g,
				gc.stats_odd_line_box_b,
				gc.stats_odd_line_box_a
			);
		}
		else
		{		//even lines
			glColor4d
			(
				gc.stats_even_line_box_r,
				gc.stats_even_line_box_g,
				gc.stats_even_line_box_b,
				gc.stats_even_line_box_a
			);
		}

		glBegin( GL_QUADS );

			glVertex3i( win_r, h - lbot, 0 );
			glVertex3i( win_r, h - ltop, 0 );
			glVertex3i( win_l, h - ltop, 0 );
			glVertex3i( win_l, h - lbot, 0 );

		glEnd();

		glDisable( GL_BLEND );

			//and finaly player's data
		Player * pl = *it;
		assertL( pl );

		float (*fcolor)[4];
		if ( game.getClientPlayer() && ( game.getClientPlayer()->getID() == pl->getID() ) )
			fcolor = &scolor;
		else
			fcolor = &dcolor;

		if ( game.getStateOfGame() == WaitForPlayers && pl->isReady() )
		{
			glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
			gGL2D.drawRect( win_l - gc.stats_ready_image_size / 2, h - ( ltop + gc.stats_ready_image_size / 2 ), mTickImage );
			glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
		}

		textWithShadow( mDataFont, pl->getName(),			h, namecol_start,	lbot, *fcolor );
		textWithShadow( mDataFont, xtos( pl->getScore() ),	h, scorecol_start,	lbot, *fcolor );
		textWithShadow( mDataFont, xtos( pl->getKills() ),	h, killscol_start,	lbot, *fcolor );
		textWithShadow( mDataFont, xtos( pl->getDeaths()),	h, deathscol_start,	lbot, *fcolor );

		linenum++;
	}
}

void StatsWindow::init()
{
	TIN;

	mPlayer = getAppClient().getGame().getClientPlayer();

	// Loading fonts
	mHeaderFont = FontManager::getRef().get( gc.stats_header_font.c_str(), gc.stats_header_font_size );
	mDataFont	= FontManager::getRef().get( gc.stats_data_font.c_str(), gc.stats_data_font_size );
	if ( !mHeaderFont || !mDataFont )
	{
		DIE( "Loading font failed!" );
	}

	mTexture = TextureManager::getInstance()->add( locateFile( gc.stats_win_image ) );
	if ( ! mTexture.isValid() )
	{
		DIE( "Cannot load texture for window with score" );
	}

	mTickImage.load( gc.stats_ready_image );
	if ( ! mTickImage.isValid() )
	{
		DIE( "Cannot load texture for window with score" );
	}
}

void StatsWindow::clean()
{
	TIN;

	mHeaderFont = 0;
	mDataFont	= 0;
	mTexture = 0;
}
void StatsWindow::textWithShadow( const FontRef & font, const std::string &text, int win_height, int x, int y, float color[4] )
{
		//render shadow
	glColor3f( 0.0, 0.0, 0.0);
	font->RenderUTF8(text, x+1, win_height - y-1, Font::LEFT_BOTTOM);
	
	glColor4fv( color );
	font->RenderUTF8(text, x, win_height - y, Font::LEFT_BOTTOM);
}

