/* MAINTAINER: Bernard */

#ifndef _GAME_STATE_COPY_CLIENT_H
#define _GAME_STATE_COPY_CLIENT_H


#include "mutex.h"
/**
 * \brief Class for providing access to some pieces of game state in thread safe manner.
 *
 * If you need something from gamestate to be possible to access from different threads
 * try considering this class. The class has method updateCopy() which is called from main 
 * thread and which updates the copy. It is called from TashSimulation after the simulation
 * step. So if you acces copyed variables you don't need to get the most latest values
 * but you get relatively new values.
 */
class GameStateCopyClient
{
public:
	GameStateCopyClient();
	~GameStateCopyClient();

	/**
	 * \brief Create copy of some variables from gamestate
	 */
	void updateCopy();

	/**
	 * \brief Get yaw and pitch from copy.
	 *
	 * Thread safe version. The yaw and pitch are from current's player tanks.
	 * \param yaw copy value of yaw
	 * \param pitch copy value of pitch
	 */
	void getYawPitch(double &yaw, double &pitch);
	
	void setFire(bool fire);
	bool getFire();

private:
	Mutex mMutex;  ///< \brief Lock for all class - it makes access to all copyed values OK.

	double mYaw;  ///< \brief yaw of current player's tank
	double mPitch; ///< \brief pitch of current player's tank
	bool   mFire;  ///< \brief true if tank is shooting
};

#endif
