#include "GL/glew.h"

#include "gobj_rendervisitor.h"
#include "render_visitor.h"
#include "SDL_opengl.h"

#include "gobj_all.h"

// TODO: hna?
#include "hna_material.h"

#include "grutils.h"
using namespace grutils;
using namespace hna;

namespace gobj
{
	void glDrawWiredCube(const gobj::GeometryObject * obj)
	{
		gobj::GeometryObject::BoundingBox bb;
		obj->computeBoundingBox(bb);

		glPushAttrib( GL_POLYGON_BIT | GL_CURRENT_BIT );
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glPushMatrix();
		
		glColor3f(1, 1, 1);
		glTranslated( bb.center[0], bb.center[1], bb.center[2] );
		glScaled( bb.extents[0], bb.extents[1], bb.extents[2] );
		glDrawCube();

		glPopMatrix();
		glPopAttrib();
	}
	
	//RenderVisitor::TGobjDisplayLists RenderVisitor::mDLs;

	RenderVisitor::RenderVisitor()
		:/*mUseDL(true),*/ mMaterial(0)
	{}
	
	RenderVisitor::~RenderVisitor()
	{
	}
	
	void RenderVisitor::setMaterial( hna::Material * mat )
	{ 
		mMaterial = mat;
	}
	
	hna::Material * RenderVisitor::getMaterial()
	{ 
		return mMaterial; 
	}

	void RenderVisitor::visitBox(const class Box * box)
	{
		/*if ( !beginDL( box ) )
			return;*/

		double ex, ey, ez;
		box->getExtents(ex, ey, ez);
		
		glPushMatrix();
		glScaled(ex, ey, ez);
		glTranslated(0, 0, 0.5);
		glScaled(0.5, 0.5, 0.5);
		glDrawCube();
		glPopMatrix();

		//glDrawWiredCube( box );
		
		//endDL();
	}

	void RenderVisitor::visitSphere(const class Sphere * sphere)
	{
		/*if ( !beginDL( sphere ) )
			return;*/

		// render sphere
		glDrawSphere(sphere->getRadius(), 16, 16);

		//glDrawWiredCube( sphere );
		
		//endDL();
	}
	
	void RenderVisitor::visitTriMesh(const class TriMesh * trimesh)
	{		
		const TriMesh::Channel * meshChannel = trimesh->getMeshChannel();
		const TriMesh::Channel * normalsChannel = trimesh->getNormalsChannel();
		hna::Material * material = getMaterial();
		hna::Material::ChannelToTextureUnitIterator it, texChannelsBegin, texChannelsEnd;		
		
		if (material)
			material->getMapChannelInfo(texChannelsBegin, texChannelsEnd);

		/*if ( !beginDL( trimesh ) )
			return;*/
		
		// render trimesh
		glBegin(GL_TRIANGLES);
		
		TriMesh::Channel::TFacesContainer::size_type fi, fn;

		fn = meshChannel->faces.size();

		// loop through faces
		for ( fi = 0; fi < fn; ++fi )
		{
			// get geometry face and normal face
			const TriMesh::Channel::TFace &face = meshChannel->faces[fi];
			const TriMesh::Channel::TFace &nface = normalsChannel->faces[fi];
			
			// for every face index get the vertex coordinate
			for ( int ci = 0; ci < TriMesh::Channel::TFace::D; ++ci )
			{
				
				if ( material )
				{
					// set texture coordinates
					for ( it = texChannelsBegin; it != texChannelsEnd; ++it )
					{
						const gobj::TriMesh::Channel *texChannel( trimesh->getTexMappingChannel( it->first ) );
						if ( texChannel )
						{
							const TriMesh::Channel::TCoord &texCoord( texChannel->coordinates[texChannel->faces[fi][ci]] );
							glMultiTexCoord2f(GL_TEXTURE0_ARB + it->second, texCoord.x, texCoord.y);
						}
					}
				}
								
				normalsChannel->coordinates[nface[ci]].glNormal();
				meshChannel->coordinates[face[ci]].glVertex();
			}
		}
		
		glEnd();
		
		//glDrawWiredCube( trimesh );
				
		/*endDL();*/
	}
/*
	bool RenderVisitor::beginDL( const class GeometryObject * go )
	{		
		if (!mUseDL)
			return true;

		TGobjDisplayLists::const_iterator it = mDLs.find(go);
		mDLCreated = false;
		if ( it == mDLs.end() )
		{
			int list = glGenLists(1);
			mDLs[go] = list;
			if ( glIsList(list) )
			{
				glNewList( list, GL_COMPILE );
				mDLCreated = true;
				mDL = list;
				return true;
			}
		}
		else
		{
			glCallList( it->second );
		}
		return false;
	}

	void RenderVisitor::endDL()
	{
		if (mUseDL && mDLCreated)
		{
			glEndList();
			glCallList(mDL);
		}
	}
*/
}
