/* MAINTAINER: Bernard */

#include "inputacceptor.h"
#include "application_client.h"
#include "log.h"
#include "inputmanager.h"

// Lower number == earlier - like unix
const int IA_PRIORITY_ONEKEY                  =  3;
const int IA_PRIORITY_KEYBINDINGS_HI_PRIORITY =  4;
const int IA_PRIORITY_CONSOLE                 =  5;
const int IA_PRIORITY_MENU                    =  6;
const int IA_PRIORITY_CHAT                    =  7;
const int IA_PRIORITY_OBSERVER                =  8;
const int IA_PRIORITY_MOUSEMOTION             =  9;
const int IA_PRIORITY_KEYBINDINGS             = 10;



InputForward::InputForward()
{

}

void InputForward::setNice(int priority)
{
	mPriority = priority;	
}

void InputForward::setOwner(InputAcceptor *ia)
{
	mIFOwner = ia;
}
	
bool InputForward::keyDown(const SDL_keysym &keysym)
{
	assertL((keysym.sym < SDLK_LAST && keysym.sym >= 0));

	if (mIFOwner) return (mForwardedKeys[keysym.sym] && mIFOwner->keyDown(keysym));
	return false;
}

bool InputForward::keyUp(const SDL_keysym &keysym)
{
	assertL((keysym.sym < SDLK_LAST && keysym.sym >= 0));
	
	if (mIFOwner) return (mForwardedKeys[keysym.sym] && mIFOwner->keyUp(keysym));
	return false;
}

bool InputForward::mouseButton(int button, bool pressed)
{
	if (mIFOwner) return mIFOwner->mouseButton(button, pressed);
	return false;
}

bool InputForward::mouseMotion(int dx, int dy)
{
	if (mIFOwner) return mIFOwner->mouseMotion(dx, dy);
	return false;
}

bool InputForward::wantKey(const SDL_keysym &keysym)
{
	return mForwardedKeys[keysym.sym];
}

void InputForward::forwardKey(int num)
{
	assertL((num < SDLK_LAST && num >= 0));
	mForwardedKeys[num] = true;

	getAppClient().getInputManager().setKeyReceiver(num, this);
}

void InputForward::clearForwardKey(int num)
{
	assertL((num < SDLK_LAST && num >= 0));
	if (mForwardedKeys[num])
	{
		mForwardedKeys[num] = false;
		getAppClient().getInputManager().setKeyReceiver(num, 0);	
	}
}

void InputForward::clearForwardTable()
{
	for (int i = 0; i < SDLK_LAST; i++) mForwardedKeys[i] = false;
	getAppClient().getInputManager().cleanKeyReceiver(this);
}
