/* MAINTAINER: Bernard */

#ifndef _INPUT_ACCEPTOR_H
#define _INPUT_ACCEPTOR_H

/**
 * Basic class for all clases that want any input.
 * Like console, stats window,..
 * Also some mouse handling may apear here
 */

#include "SDL.h"

class InputAcceptor
{
public:
	InputAcceptor() { mPriority = 0;};
	
	virtual ~InputAcceptor() {};
	
	/**
	 * Inform acceptor that key was down.
	 *
	 * \return true if the key was accepted
	 */
	virtual bool keyDown(const SDL_keysym &keysym) { return wantKey(keysym); }

	/**
	 * Only works in network acceptor now
	 */
	virtual bool keyUp(const SDL_keysym &keysym) { return wantKey(keysym); }


	virtual bool keyChange(const SDL_keysym &keysym, bool down)
	{
		if (down) return keyDown(keysym);
		return keyUp(keysym);
	}

	// MUST BE THREAD SAFE RESPOND!
 	virtual bool mouseButton(int button, bool pressed) { return false; }

	virtual bool mouseMotion(int dx, int dy)  { return false; }
 

	int getNice() { return mPriority; }
		
    // return true if component wants the key -
    // must be thread safe
	virtual bool wantKey(const SDL_keysym &keysym) { return false; }

protected:   
   // InputAcceptors are sorted depending on priority
   // each acceptor should have different - some constants probably
   int mPriority;

   
};


class InputForward : public InputAcceptor
{
public:
	InputForward();
	virtual ~InputForward() {}

	void setNice(int priority);
	void setOwner(InputAcceptor *ia);
	
	virtual bool keyDown(const SDL_keysym &keysym);
	virtual bool keyUp(const SDL_keysym &keysym);
 	virtual bool mouseButton(int button, bool pressed);
	virtual bool mouseMotion(int dx, int dy);
	virtual bool wantKey(const SDL_keysym &keysym);

	void forwardKey(int num);
	void clearForwardKey(int num);
	void clearForwardTable();

private:
	InputAcceptor *mIFOwner;
	bool mForwardedKeys[SDLK_LAST];
};

// Priority of acceptors
extern const int IA_PRIORITY_ONEKEY;
extern const int IA_PRIORITY_KEYBINDINGS_HI_PRIORITY;
extern const int IA_PRIORITY_CONSOLE;
extern const int IA_PRIORITY_MENU;
extern const int IA_PRIORITY_CHAT;
extern const int IA_PRIORITY_OBSERVER;
extern const int IA_PRIORITY_MOUSEMOTION;
extern const int IA_PRIORITY_KEYBINDINGS;

#endif // _INPUT_ACCEPTOR_H
