/* MAINTAINER: Bernard */

//#define WANT_TRACE

#include "SDL.h"

#include "log.h"
#include "inputmanager.h"
#include "inputacceptor.h"

#include "application_client.h"
#include "game_client.h"

using namespace std;

InputManager::InputManager()
{
	TIN;

	mNetAcceptor = 0;
		
	for (int i = 0; i < SDLK_LAST; i++)
	{
		mWantedKeys_mainThread[i] = 0;
	}

	mAllInputWanters = 0;
}

InputManager::~InputManager()
{
	
}

InputManager::InputManager(const InputManager &im)
{
	LOGE << "It is not allowed to create copy of InputManager." << endl;
	DIE();
}
	
void InputManager::addInputAcceptor(InputAcceptor &ia)
{
	TIN; 
	LOGT << "ia.getNice() = " << ia.getNice() << endl;	
	mIAVector[ia.getNice()] = &ia;
}
		
void InputManager::removeInputAcceptor(InputAcceptor &ia)
{
	TIN;

	mIAVector.erase(ia.getNice());
}
		
void InputManager::checkSDLEvents()
{
//	TIN;
		
	SDL_Event event;
	
	while ( SDL_PollEvent(&event) ) 
	{
		switch (event.type) 
		{
   
		case SDL_KEYUP :
		case SDL_KEYDOWN :
			TPOINTS("Key event arrived");
			checkKeyEvent(event, event.key.keysym);
			break;

		case SDL_MOUSEBUTTONDOWN:
		case SDL_MOUSEBUTTONUP:
			TPOINTS("Mouse buttom event arrived")		
			mouseButton(event.button.button, event.button.state == SDL_PRESSED);
			break;
			
		default:
			// save event for later processing from main thread
			lock();
			mEvents.push(event);
			unlock();
			// LOGD << "Uhandled event " << (int)event.type << " from SDL" << endl;
		}
	}
}

		
void InputManager::processSDLEvents()
{
//	TIN;
		
	SDL_Event event;
		
	lock();		
	while (!mEvents.empty())
	{
		event = mEvents.front();
		mEvents.pop();
		unlock();

		switch (event.type) 
		{
   
		case SDL_QUIT :  
			LOGD << "SDL_QUIT" << endl;
			getAppClient().quit();
			break;
		
		case SDL_KEYUP :
			keyChange(event.key.keysym, false);
			break;
			
		case SDL_KEYDOWN :
			keyChange(event.key.keysym, true);
			break;

		case SDL_MOUSEMOTION:
			mouseMotion(event.motion.xrel, event.motion.yrel);
			break;
			
		default:
			// LOGD << "Uhandled event " << (int)event.type << " from SDL" << endl;
			;
		}

		lock();
	}
	unlock();
}


void InputManager::checkKeyEvent(SDL_Event &event, SDL_keysym keysym)
{
//	TIN;
	TVARIABLE(keysym.sym);
	
	// validity check (to avid segfault when indexing array)
	if (keysym.sym >= SDLK_LAST || keysym.sym < 0) 
	{
		LOGE << "Received invalid key value " << keysym.sym << endl;
		return;
	}
	
	// classic test
	lock();
	if (mWantedKeys_mainThread[keysym.sym] != 0)
	{
		mEvents.push(event);
		unlock();
		return;
	}
	unlock();
	
	// all input test
	lock();
	if ( mAllInputWanters > 0)
	{
		mEvents.push(event);
		unlock();
		return;
	}
	unlock();
	
	TPOINTS("Nobody wanted the key. Passing to the network.");
			
	// Pass input to "client/network/..."
	if (mNetAcceptor != 0)
	{
		if (event.type == SDL_KEYDOWN)
			mNetAcceptor->keyChange(keysym, true);
		else
			mNetAcceptor->keyChange(keysym, false);
	}
}

/// process keys
void InputManager::keyChange(SDL_keysym keysym, bool down)
{
	TIN;
	TVARIABLE(keysym.sym);
	
	// validity check (to avoid segfault when indexing array)
	if (keysym.sym >= SDLK_LAST || keysym.sym < 0) 
	{
		LOGE << "Received invalid key value " << keysym.sym << endl;
		return;
	}
			
	// test against priority acceptors in list
	lock();
	map<int, InputAcceptor*>::iterator it;
	for ( it = mIAVector.begin(); it !=  mIAVector.end(); ++it )
	{
		LOGT << "Testing againts priority " << it->first << endl;	
		if (it->second == 0) continue;
		if (it->second->keyChange(keysym, down))
		{
			TPOINTS("Key processed.");
			unlock();
			return;
		}
	}
	unlock();
		
	// key forwarding check from table
	lock();
	if (mWantedKeys_mainThread[keysym.sym] != 0)
	{
		if (mWantedKeys_mainThread[keysym.sym]->keyChange(keysym, down)) 
		{
			TPOINTS("Key processed.");
			unlock();
			return;
		}
	}
	unlock();
	
	// nobody wanted this key -> stop mWantedAllInput
	lock();
	mAllInputWanters = 0;
	unlock();
	
	TPOINTS("Nobody wanted the key :(");
}


void InputManager::mouseMotion(int dx, int dy)
{
//	TIN;
	
//	LOGT << dx << " " << dy << endl;
	
	// test against priority acceptors in list
	lock();
	map<int, InputAcceptor*>::iterator it;
	for ( it = mIAVector.begin(); it !=  mIAVector.end(); ++it )
	{
//		LOGT << "Testing againts priority " << it->first << endl;	
		if (it->second == 0) continue;
		if (it->second->mouseMotion(dx, dy))
		{
			unlock();
			return;
		}
	}
	unlock();
}	

void InputManager::wantAllInput(bool wanted)
{	
	lock();
	if (wanted) ++mAllInputWanters;
	else --mAllInputWanters;
	if (mAllInputWanters > 0)
		SDL_EnableUNICODE(1);
	else
		SDL_EnableUNICODE(0);
	unlock();
}

void InputManager::setKeyReceiver(int key, InputAcceptor *receiver)
{
	TVARIABLE(key);
	
	lock();
	mWantedKeys_mainThread[key] = receiver;
	unlock();
}


void InputManager::setNetAcceptor(InputAcceptor *netAcceptor)
{
	lock();
	mNetAcceptor = netAcceptor;
	unlock();
}
	
void InputManager::cleanKeyReceiver(InputAcceptor *receiver)
{
	lock();
	for (int i = 0; i < SDLK_LAST; i++)
	{
		if (mWantedKeys_mainThread[i] == receiver) mWantedKeys_mainThread[i] = NULL;
	}
	unlock();
}


void InputManager::mouseButton(int button, bool pressed)
{

	TVARIABLE(button);
	
	// validity check (to avid segfault when indexing array)
	if (button < 0) 
	{
		LOGE << "Received invalid button value " << button << endl;
		return;
	}
	
	// test against priority acceptors in list
	lock();
	map<int, InputAcceptor*>::iterator it;
	for ( it = mIAVector.begin(); it !=  mIAVector.end(); ++it )
	{
		LOGT << "Testing againts priority " << it->first << endl;	
		if (it->second == 0) continue;
		if (it->second->mouseButton(button, pressed))
		{
			TPOINTS("Mouse button accepted");
			unlock();
			return;
		}
	}
	unlock();
	

	TPOINTS("Nobody wanted mouse. Go to netwok.");

	// Pass input to "client/network/..."
	if (mNetAcceptor)
	{
		mNetAcceptor->mouseButton(button, pressed);
	}
}
