/* MAINTAINER: Bernard */

// #define WANT_TRACE

#include "log.h"
#include "inputstring.h"
#include "conversions.h"

using namespace std;

InputString::InputString()
{
	clear();
}


InputString::~InputString()
{

}

void InputString::clear()
{
	mLineLeft.erase();
	mLineRight.erase();

	updateCachedStrings();
}


void  InputString::processKey(const SDL_keysym &keysym)
{
	TIN 

	LOGT << "Processing new key " << keysym.sym << " :-)\n";
	
	switch (keysym.sym) 
	{
	case SDLK_BACKSPACE:
		if ( mLineLeft.length() > 0 )
		{
			mLineLeft.erase( mLineLeft.length()-1 );
			updateCachedStrings();
		}
		break;
		
	case SDLK_DELETE:
		if ( mLineRight.length() > 0 ) 
		{
			 mLineRight.erase(0,1);
		}
		break;
		
	// moving cursor in current command left
	case SDLK_LEFT:
		if ( mLineLeft.length() > 0 ) 
		{
			// move one character from Left to right
			mLineRight.insert( 0, &mLineLeft.at(mLineLeft.length()-1), 1);
			mLineLeft.erase(mLineLeft.length()-1);
		}
		break;
		
	// moving cursor in command right
	case SDLK_RIGHT:
		if ( mLineRight.length() > 0 )	
		{

			mLineLeft.append(&mLineRight[0], 1);
			mLineRight.erase(0,1);
		}
		break;

     case SDLK_HOME:
     	// erase whole left part
		mLineRight.insert( 0, mLineLeft);
		mLineLeft.erase();
     	break;
     	
     	
     case SDLK_END:
	     // erase right part
     	mLineLeft.append(mLineRight);
     	mLineRight.erase();
     	break;
	

	// Nothong done on return
	case SDLK_RETURN:
    case SDLK_KP_ENTER:	
	case SDLK_ESCAPE:
	case SDLK_TAB:
		break;
	
	// NUMPAD
     case SDLK_KP0:  mLineLeft += '0'; break ;
     case SDLK_KP1:  mLineLeft += '1'; break ;
     case SDLK_KP2:  mLineLeft += '2'; break ;
     case SDLK_KP3:  mLineLeft += '3'; break ;
     case SDLK_KP4:  mLineLeft += '4'; break ;
     case SDLK_KP5:  mLineLeft += '5'; break ;
     case SDLK_KP6:  mLineLeft += '6'; break ;
     case SDLK_KP7:  mLineLeft += '7'; break ;
     case SDLK_KP8:  mLineLeft += '8'; break ;
     case SDLK_KP9:  mLineLeft += '9'; break ;
     case SDLK_KP_DIVIDE:   mLineLeft += '/'; break;
    /* 
     case SDLK_KP_MULTIPLY:  mLineLeft += '*'; break;
     case SDLK_KP_MINUS:     mLineLeft += '-'; break;
     case SDLK_KP_PLUS:      mLineLeft += '+'; break;
     case SDLK_KP_EQUALS:    mLineLeft += '='; break;
	*/
	
	// adding one char to commandline
	default:
		
	//	LOGT << "Adding key (unicode) " << keysym.unicode << endl;
		TVARIABLE(keysym.unicode);
		TVARIABLE(keysym.sym);
				
		if (keysym.unicode != 0) 
		{	
			mLineLeft += keysym.unicode;
		}
		break;
	}
	
	updateCachedStrings();
	
}

void InputString::updateCachedStrings()
{
	mLineW.erase();
	mLineW.append(mLineLeft).append(mLineRight);

	mLineWR.erase();
	mLineWR.append(mLineLeft).append(L"_").append(mLineRight);

	wstoutf8(mLineW, mLineU8);
	wstoutf8(mLineWR, mLineU8R);
}

	
string InputString::getLineU()
{
	return mLineU8;
}


wstring InputString::getLineW()
{
	return mLineW;
}


string InputString::getLineURender()
{
	return mLineU8R;
}


wstring InputString::getLineWRender()
{
	return mLineWR;
}
	
