/* MAINTAINER: Bernard */

#ifndef _INPUT_STRING_H
#define _INPUT_STRING_H

#include <string>
#include "SDL.h"

/**
 * \brief provides function of editbox - the string editing.
 *
 * The class is not supposed to be a base class. It should be
 * used as member variable.
 */
class InputString
{
public:
	InputString();
	virtual ~InputString();

	virtual void processKey(const SDL_keysym &keysym);
	
	/**
	 * \brief Delete all string content.
	 */
	virtual void clear();
	
	/**
	 * \brief return current line.
	 * 
	 * Encoded in utf-8
	 */
	std::string getLineU();


	std::wstring getLineW();


	/**
	 * \brief Return string with cursor position.
	 *
	 * Encoded in utf-8
	 */
	std::string getLineURender();

	/**
	 * \brief return line with cursor position.
	 */
	std::wstring getLineWRender();
	
protected:

	void updateCachedStrings();
	
	std::wstring mLineLeft;  ///< Part of line before cursor
	std::wstring mLineRight; ///< Part of line after editing cursor

	// "chached" strings for line a line with cursor
	std::wstring mLineW;
	std::wstring mLineWR;
	std::string  mLineU8;
	std::string  mLineU8R;
};

#endif
