/* MAINTAINER: Bernard */

//#define WANT_TRACE
#include "log.h"
#include "inputtonetwork.h"
#include "application_client.h"
#include "game_client.h"
#include "networkmanager_client.h"
#include "game_state_copy.h"

using namespace std;

InputToNetwork::InputToNetwork()
{

	// No buttons are pressed
	for (int i = 0; i < BUTTON_MAX; ++i)
		mButtons[i] = false;

	clearActionTables();

}


bool InputToNetwork::wantKey(const SDL_keysym &keysym)
{	

	// validity check (to avid segfault when indexing array)
	assertL((keysym.sym < SDLK_LAST && keysym.sym >= 0));

	lock();
		bool rv = (mAccepted[keysym.sym] != -1);
	unlock();
	
	return rv;
}


bool InputToNetwork::keyDown(const SDL_keysym &keysym)
{
	assertL((keysym.sym < SDLK_LAST && keysym.sym >= 0));

	lock();
	int butt = mAccepted[keysym.sym];
	unlock();

	return setButton(butt, true);
}


bool InputToNetwork::keyUp(const SDL_keysym &keysym)
{
	assertL((keysym.sym < SDLK_LAST && keysym.sym >= 0));

	lock();
	int butt = mAccepted[keysym.sym];
	unlock();

	return setButton(butt, false);
}

bool InputToNetwork::mouseButton(int button, bool pressed)
{
	assertL(button < MAX_SDL_MOUSE_BUTTONS && button >= 0);

	lock();
	int butt = mMouseButt[button];
	unlock();

	return setButton(butt, pressed);
}

bool InputToNetwork::setButton(int button, bool value)
{
	if (button == -1) return false;
	assertL(button < BUTTON_MAX && button >= 0);

	lock();
	if (mButtons[button] != value)
	{
		mButtons[button] = value;
		unlock();
		sendButton(button, value);
		LOGT << "Sended " << button << "=" << value << endl;
	}
	else
	{
		unlock();
	}
	
	return true;
}

void InputToNetwork::sendButton(int button, bool value)
{

	double yaw = 0, pitch = 0;

	getAppClient().getGame().getGameStateCopy().getYawPitch(yaw, pitch);

	getAppClient().getNetworkManager().sendInputButton(button, value, yaw, pitch);
	
	if (BUTTON_FIRE == button)
	{
		getAppClient().getGame().getGameStateCopy().setFire(value);
	}
}


void InputToNetwork::setKeyAction(int key, int action)
{
	if (key >= SDLK_LAST || key < 0) return;
	
	lock();
	mAccepted[key]  = action;
	unlock();
}

void InputToNetwork::setMouseAction(int button, int action)
{
	if (button >= MAX_SDL_MOUSE_BUTTONS || button < 0) return;

	lock();
	mMouseButt[button] = action;
	unlock();
}

void InputToNetwork::clearActionTables()
{
	lock();
	for (int i = 0; i < SDLK_LAST; ++i) mAccepted[i] = -1;
	for (int i = 0; i < MAX_SDL_MOUSE_BUTTONS; ++i)  mMouseButt[i] = -1;
	unlock();
}

