/* MAINTAINER: Bernard */

#ifndef _INPUT_TO_NET_H
#define _INPUT_TO_NET_H

/**
 * Basic class for all clases that want any input.
 * Like console, stats window,..
 * Also some mouse handling may apear here
 */

#include "SDL.h"
#include "inputacceptor.h"
#include "inputconstants.h"
#include "mutex.h"

#define MAX_SDL_MOUSE_BUTTONS 255 // actually only first 5 are used

/**
 * \brief Class for processing controls of tank like movement.
 *
 * 
 */
class InputToNetwork : public InputAcceptor , public Mutex
{
public:
	InputToNetwork();
	
	virtual ~InputToNetwork() {};
	
	/**
	 * Inform acceptor that key was down.
	 *
	 * \return true if the key was accepted
	 */
	virtual bool keyDown(const SDL_keysym &keysym);

	virtual bool keyUp(const SDL_keysym &keysym);
 
 	virtual bool mouseButton(int button, bool pressed);

	// return true if component wants the key -
	// must be thread safe
	virtual bool wantKey(const SDL_keysym &keysym);

	/**
	 * \brief Create mapping from key to logic buttton.
	 *
	 * \param key Keyboard key for mapping.
	 * \param action logical button id
	 */
	void setKeyAction(int key, int action); 

	/**
	 * \brief Create mapping from mouse button to logic button.
	 *
	 * \param button Mouse button. Note that left button is 1 not 0.
	 * \param action logic button id.
	 */
	void setMouseAction(int button, int action); 

	/**
	 * \brief Clear all key to button mappings.
	 */
	void clearActionTables();

	/**
	 * \brief Change state of logical button.
	 *
	 * It sends the button to server if its state is changed.
	 * \param button index of button
	 * \param value new value
	 * \return false iv button == -1 and true else.
	 */
	bool setButton(int button, bool value);
	
protected:   

	/**
	 * \brief Send button to server.
	 *
	 * The yaw and pitch of current tank are also sended since
	 * we want accurate shooting in case of fire button
	 * \param button number of logical button
	 * \param value state of the button
	 */
	void sendButton(int button, bool value);


	int mAccepted[SDLK_LAST]; /// \brief mapping from keys to logic buttons (commands)
	int mMouseButt[MAX_SDL_MOUSE_BUTTONS]; /// \brief mapping from mouse buttons to logic buttons (commands)
   
	bool mButtons[BUTTON_MAX]; /// \brief Current state of logic button (commands)
};

#endif // _INPUT_ACCEPTOR_H
