/* MAINTAINER: Bernard */

//#define WANT_TRACE

#include <cmath>
#include "SDL_opengl.h"
#include "mousemotion.h"
#include "application_client.h"
#include "player.h"
#include "tank.h"
#include "log.h"
#include "settings.h"
#include "gfx.h"
#include "game_client.h"


using namespace std;

MouseMotion::MouseMotion()
{
//	mUseChange = false;
//	mFloatingPitch = true;
	mFloatingView = false;
	mYawSensitivity = 0.02;
	mPitchSensitivity = 0.003;
	mPriority = IA_PRIORITY_MOUSEMOTION;
	
	mX = 0;
	mY = 0;
	mOutOfReach = false;
}
	
bool MouseMotion::mouseMotion(int dx, int dy)
{
	SDL_GetMouseState(&mX, &mY);

	int w2  = getAppClient().getGFX().getWidth()/2;
	int h2  = getAppClient().getGFX().getHeight()/2;

	if ( mFloatingView && (mX != w2 || mY != h2))
	{

		double dx = mX - w2;
//		mViewYaw   -= dx * mYawSensitivity;
		mViewYaw -= pow(fabs(dx),0.25) * dx * mYawSensitivity;	
//		mViewYaw -= log(fabs(dx+2)) * dx * mYawSensitivity;
		mX = w2;	
		
		// Tohle je pro plne rozhlizeni

		double dy = mY - h2;
		if (g_settings.isTrue(SETTING_INVERT_MOUSE))
		{
			mViewPitch += dy * mPitchSensitivity;		
		}
		else
		{
			mViewPitch -= dy * mPitchSensitivity;					
		}
		if (mViewPitch >  80) mViewPitch =  80;
		if (mViewPitch < -80) mViewPitch = -80;
		mY = h2;
		
		// mViewPitch = 0;

		int x, y;
		SDL_GetMouseState(&x, &y);
		if (x != mX || y != mY) SDL_WarpMouse(mX, mY);
		SDL_GetMouseState(&x, &y);
		if (x != mX || y != mY) SDL_WarpMouse(mX, mY);		
	}
	
	return true;
}


void MouseMotion::update(int dtime, int atime)
{
	updateYawPitch();
}

void MouseMotion::init()
{
#ifndef _DEBUG
	SDL_ShowCursor(SDL_DISABLE);
#endif

	SDL_WarpMouse(getAppClient().getGFX().getWidth()/2, 
	              getAppClient().getGFX().getHeight()/2);
//	mWantedPitch = 0;
//	mWantedYaw   = 0;
	mViewYaw     = 0;
	mViewPitch   = 0;
}

void MouseMotion::updateYawPitch()
{
	Player *pl = getAppClient().getGame().getClientPlayer();
	if (!pl) return;
	
	Tank *tank = pl->getTank();
	if (!tank) return;
	
	// Finally set yaw and pitch
	mOutOfReach = !tank->setYawPitchFrom3DTarget(m3DTarget);

	
}

void MouseMotion::addedWorldObject( WorldEntity * we )
{
	if ( we->getClassType() != cTank )
		return;

	Player * pl = gAppClient->getGame().getClientPlayer();
	if ( !pl )
		return;

	Tank * pl_tnk = pl->getTank();
	if ( !pl_tnk )
		return;

	if ( pl_tnk->getID() == we->getID() )
	{
		// LOOK like we have a new tank
		init();
	}

}

void MouseMotion::setFloatingView(bool fy)
{ 
	if ( mFloatingView  == fy ) return;

	mFloatingView = fy;

	if (mFloatingView)
	{
		// Place mouse to x center
		int w2  = getAppClient().getGFX().getWidth()/2;
		int h2  = getAppClient().getGFX().getWidth()/2;
		
		int x,y;
		SDL_GetMouseState(&x, &y);
		if (x != w2) SDL_WarpMouse(w2, h2);
		SDL_GetMouseState(&x, &y);
		if (x != w2) SDL_WarpMouse(w2, h2);
		
		Player *pl = getAppClient().getGame().getClientPlayer();
		if (pl)
		{
			Tank *t = pl->getTank();
			if (t)
			{
				GLVector3d direction = t->getOrientation().rotateVector(GLVector3d(1,0,0));
				float  tank_yaw = atan2f((float)direction.y, (float)direction.x)*180.0f/(float)M_PI;
				
				mViewYaw   = tank_yaw;
				mViewPitch = 0;
			}
		}	
	}
}


void MouseMotion::read3DTargetFromZBuffer()
{

	// Make sure that all is rendered
	glFlush();
		
	// m3DTarget = neco podle mX a mY
	GLint viewport[4];
	GLdouble modelview[16];
	GLdouble projection[16];
	GLfloat winX, winY, winZ;
   	GLdouble posX, posY, posZ;

    glGetDoublev(GL_MODELVIEW_MATRIX, modelview );
    glGetDoublev(GL_PROJECTION_MATRIX, projection);
    glGetIntegerv(GL_VIEWPORT, viewport);

    winX = (float)mX;
    winY = (float)viewport[3] - (float)mY - 1; // Flip mouse coordinates to OpenGL coortinages
    glReadPixels(mX, (int)winY, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ);  
    //winZ = 1;     - takhle by to strilelo vzdycky az "dozadu"

	gluUnProject(winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);
	
	m3DTarget.x = posX;
	m3DTarget.y = posY;
	m3DTarget.z = posZ;	
}

void MouseMotion::render()
{
	read3DTargetFromZBuffer();
	
	updateYawPitch();
}	


void MouseMotion::computeCameraOrientation(GLVector3d &vect_to_orient)
{
	GLQuaterniond rotation;
	getCameraOrientation(rotation);
	
	vect_to_orient = rotation.rotateVector(GLVector3d(1,0,0));
	// In new physics the tank is too "jumpy" and does more rotations
	// so the camera doesn;t look good.
	vect_to_orient.z = 0; // to avoid "skuby" when fast game
}

void MouseMotion::getCameraOrientation(GLQuaterniond &orientation)
{
	if (mFloatingView)
	{
		orientation = GLQuaterniond(GLVector3d(0,0,1), mViewYaw);
	}
	else
	{
		Player *pl = getAppClient().getGame().getClientPlayer();
		if (!pl) return;
	
		Tank *tank = pl->getTank();
		if (!tank) return;
		
		orientation = tank->getOrientation();
	}	
}
