/* MAINTAINER: Bernard */

#ifndef _MOUSE_MOTION_H
#define _MOUSE_MOTION_H

#include "inputacceptor.h"
#include "task.h"
#include "layer.h"
#include "GLVector.h"
#include "GLQuaternion.h"

/**
 * \brief Targeting by the mouse.
 *
 * This class works as input acceptor for mouse motions and
 * compute yaw and pitch for current tank from it.
 * It gets current mouse position in mouseMotion()
 * and recount yaw and pitch in update()
 * You may also force yaw and pitch recount in updateYawPitch()
 * 
 * Class is also as Layer because it may want to take 3D coordinates
 * of redered world.
 */
class MouseMotion : public InputAcceptor, public Task, public Layer
{
public:
	MouseMotion();
	
	virtual ~MouseMotion() {};
	

	/**
	 * \brief respond for mouse motion event from inputManger
	 *
	 * Store mouse position for later yaw pitch compitation
	 * \param dx differenc of x coordinate since last event
	 * \param dy differenc of y coordinate since last event
	 * \return true if current player exists and has some tank for
	 *              which we can change yaw and pitch
	 */
	virtual bool mouseMotion(int dx, int dy);

	virtual void update(int dtime, int atime);

	/**
	 * \brief Does no rendering at all but read 3D coordinates from
	 *        Z-buffer.
	 */
	virtual void render();	

	/**
	 * \brief Compute new yaw and pitch of current players
	 *        tank.
	 */
	void updateYawPitch();

	/**
	 * \brief Do some mouse inicialization
	 */
	void init();

	virtual void addedWorldObject( WorldEntity * we );

	void setFloatingView(bool fy);
	bool getFloatingView() const { return  mFloatingView; }
//	void setUseChange(bool uc);
	void setYawSensitivity(double ysen)  { mYawSensitivity = ysen; }
	void setPitchSensitivity(double psen){ mPitchSensitivity = psen; }
//	void set3DTarget(bool use3D);

//	bool is3DTargeting() { return mUse3DTarget; }
	const GLVector3d& get3DTarget() { return m3DTarget; }

	void computeCameraOrientation(GLVector3d &vect_to_orient);
	void getCameraOrientation(GLQuaterniond &orientation);

	double getViewPitch() const { return mViewPitch; }

	/**
	 * \brief Return true if it is probable that target is out of reach.
	 */
	bool isOutOfReach() { return mOutOfReach; }

	void getMouseXY(int &x, int &y)
	{
		x = mX;
		y = mY;	
	}	

protected:

	/**
	 * \brief Read point from Z-buffer if needed for targeting.
	 */	
	void read3DTargetFromZBuffer();
		
	double mWantedPitch;
	double mWantedYaw;
	
	double mViewYaw;
	double mViewPitch;

	double mYawSensitivity;
	double mPitchSensitivity;
	

//	bool  mUseChange;
//	bool  mFloatingPitch;
	bool  mFloatingView;
//	bool  mUse3DTarget;

	bool  mOutOfReach; // reset at the beginning of every updateYawPitch()

	int  mX;  // X of mouse
	int  mY;  // Y of mouse
	
	GLVector3d m3DTarget; // place where we want to target in 3D
};

#endif // _MOUSE_MOTION_H
