#ifndef _LAYER_H
#define _LAYER_H

/**
*	\brief interface of visual layers
*	\note Visual layers should be placed in TaskRender.
*/
class Layer
{
protected:	
	Layer() {}

public:
	virtual ~Layer() {}

	/**
	*	\brief	Rendering code.
	*	\note	It is a pure virtual method, must be implemented in subclasses.
	*/
	virtual void render() = 0;
	
	/**
	*	\brief	Frame-based update of structures in this layer.
	*	\note	Does nothing by default.
	*/
	virtual void update() {}
	
	/*
	*	\brief Sets the projection and model-view matrix for concrete layer.
	*	\note Sets Ortho2D projection by default.
	*/
	virtual void setView();
};

#endif //_LAYER_H
