/* MAINTAINER: Bernard */
#include <CEGUI.h>
#include "menuingame.h"
#include "application_client.h"
#include "commander.h"
#include "log.h"
#include "game_client.h"
#include "player.h"
#include "params.h"
#include "settings.h"
#include "conversions.h"
#include "translator.h"
#include "tank.h"
#include "timemanager.h"
#include "gui_constants.h"
#include "gamegui.h"
#include "gfx.h"

using namespace CEGUI;
using namespace std;

// Sample sub-class for ListboxTextItem that auto-sets the selection brush
// image.  This saves doing it manually every time in the code.

/**
 * \brief Class for adding tankID to ListboxTextItem
 */
class TankListItem: public CEGUI::ListboxTextItem
{
public:
 	TankListItem(const CEGUI::String& text, int tankID) : ListboxTextItem(text)
    {
    	// SleekSpace selection isn't very good - the TaharezLook is better
        setSelectionBrushImage("TaharezLook", "MultiListSelectionBrush");
        mTankID = tankID;
    }

	int mTankID; ///< \brief Tank ID associated with this text item.
};


MenuInGame::MenuInGame()
{
	try
	{
		WindowManager &wm = WindowManager::getSingleton();

		// Main menu
		mMInMain = wm.loadWindowLayout("menu.ingame.layout");
		translateWindowTextRecursive(mMInMain);

		FrameWindow *frame = static_cast<FrameWindow*>(mMInMain->getChild("ingame.frame"));
		frame->setRollupEnabled(false);

		Window *w;
		w = frame->getChild("ingame.frame.exitgame");
		w->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&MenuInGame::handleExitGame, this));

		w = frame->getChild("ingame.frame.togame");
		w->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&MenuInGame::handleToGame, this));

		w = frame->getChild("ingame.frame.quit");
		w->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&MenuInGame::handleExitToOS, this));
		
// Player
		w = frame->getChild("ingame.frame.name"); 
		w->subscribeEvent(Editbox::EventTextAccepted, CEGUI::Event::Subscriber(&MenuInGame::handleApplyPlayerName, this));

// Tank
		MultiColumnList* mclbox = static_cast<MultiColumnList*>(frame->getChild("ingame.frame.tanklist"));
        mclbox->subscribeEvent(Editbox::EventMouseDoubleClick, CEGUI::Event::Subscriber(&MenuInGame::handleApplyTank, this));
	 	mclbox->addColumn( (const CEGUI::utf8*)_("Name"),     0, UDim(0.25f,0));
	 	mclbox->addColumn( (const CEGUI::utf8*)_("HP"),       1, UDim(0.15f,0));
	 	mclbox->addColumn( (const CEGUI::utf8*)_("Armour"),   2, UDim(0.15f,0));
	 	mclbox->addColumn( (const CEGUI::utf8*)_("Speed"),    3, UDim(0.2f,0));
	 	mclbox->addColumn( (const CEGUI::utf8*)_("Turn"),     4, UDim(0.2f,0));
		mclbox->resetList();				
		for(int row = 0, tank_id = 0; (unsigned int)tank_id < gParams.mTanks.size(); ++tank_id)
		{
			const Params::sTankParams &tankparam = gParams.mTanks[tank_id];
			if (!tankparam.valid) continue;
			
			int     id   = tankparam.id;
			mclbox->addRow();
			mclbox->setItem(new TankListItem((const CEGUI::utf8*) _(tankparam.name.c_str()),            id), 0, row);
			mclbox->setItem(new TankListItem(itos(tankparam.hp),        id), 1, row);
			mclbox->setItem(new TankListItem(itos(tankparam.armour),    id), 2, row);
			mclbox->setItem(new TankListItem(xtos(tankparam.max_speed), id), 3, row);
			mclbox->setItem(new TankListItem( (const CEGUI::utf8*) _(tankparam.turning_user_info.c_str()), id), 4, row);
			
			++row;
		}


		LOGD << "In Game Menu constructed" << endl;
	} catch ( CEGUI::Exception &e)
	{
		LOGE << "CEGUI exception: " << e.getMessage() << endl;
		DIE("CEGUI ERROR");

	}

	mTankWasAlive = false;
	mTimeOfDead   = 0;
	mShownForLastDead = false;


	mWindow = mMInMain;
	
//	activated();
}



void MenuInGame::activated()
{
	MenuBasic::activated();
	
	try
	{
		Window *frame = mMInMain->getChild("ingame.frame");
		
// Tank
		MultiColumnList* mclbox = static_cast<MultiColumnList*>(frame->getChild("ingame.frame.tanklist"));

		long tankID = -1;
		try_get_long(g_settings.getString(SETTING_PLAYER_PREFERED_TANK), tankID);
		
		mclbox->clearAllSelections();
		for(int row = 0; (unsigned int)row < mclbox->getRowCount(); ++row)
		{
			for(int col = 0; (unsigned int)col <mclbox->getColumnCount(); ++col)
			{
				TankListItem* p_selected = static_cast<TankListItem*>(mclbox->getItemAtGridReference(MCLGridRef(row, col)));
				if (!p_selected) continue;
				bool select = p_selected->mTankID == tankID;
				
				mclbox->setItemSelectState(MCLGridRef(row, col), select);
			}
		}
				
		
// Player
		Window *w = frame->getChild("ingame.frame.name"); 
		Player *pl = getAppClient().getGame().getClientPlayer();
		if (pl != 0)
		{
			LOGD << "Setting name " << pl->getName();
			w->setText((const CEGUI::utf8*)(pl->getName().c_str()));
		}

		LOGD << "In Game Menu constructed" << endl;
	} catch ( CEGUI::Exception &e)
	{
		LOGE << "CEGUI exception: " << e.getMessage() << endl;
		DIE("CEGUI ERROR");

	}
		
}

MenuInGame::~MenuInGame()
{
	WindowManager &wm = WindowManager::getSingleton();
	if (mMInMain) wm.destroyWindow(mMInMain);
}

bool MenuInGame::handleExitGame(const CEGUI::EventArgs& e)
{
	deactivated();
	getAppClient().getCommander().addCommand("disconnect");
	return true;
}

void MenuInGame::deactivated()
{
	// Tank
	Window *frame = mMInMain->getChild("ingame.frame");
	MultiColumnList* mclbox = static_cast<MultiColumnList*>(frame->getChild("ingame.frame.tanklist"));
	
	TankListItem* p_selected = static_cast<TankListItem*>(mclbox->getFirstSelectedItem());
	if (p_selected != 0)
	{
		g_settings.set(SETTING_PLAYER_PREFERED_TANK, xtos(p_selected->mTankID));
	}

	// Player
	Window *w = frame->getChild("ingame.frame.name"); 
	g_settings.set(SETTING_PLAYER_PREFERED_NAME, xtos(w->getText()));
	
	MenuBasic::deactivated();
	
	// We also deactivate stats window. It may happen that the stats
	// window was turned on and we overtook it's input...
	getAppClient().getGame().getGUI().showStatsWindow(false);
	
	// center the mouse when leaving the stats menu
	int w2  = getAppClient().getGFX().getWidth()/2;
	int h2 = getAppClient().getGFX().getHeight()/2;
	SDL_WarpMouse(w2, h2);
	SDL_WarpMouse(w2, h2);
		
}


bool MenuInGame::handleToGame(const CEGUI::EventArgs& e)
{
	deactivated();
	getAppClient().getCommander().addCommand("setpreferedname");
	getAppClient().getCommander().addCommand("newtank");
		
	return true;
}

bool MenuInGame::handleExitToOS(const CEGUI::EventArgs& e)
{
	getAppClient().getCommander().addCommand("quit");
	return true;
}


bool MenuInGame::handleApplyPlayerName(const CEGUI::EventArgs& e)
{
		
// Player
	deactivated();
	getAppClient().getCommander().addCommand("setpreferedname");
		
	return true;
}


bool MenuInGame::handleApplyTank(const CEGUI::EventArgs& e)
{
	if (getAppClient().getGame().getClientPlayer() &&
		getAppClient().getGame().getClientPlayer()->getTank() == 0)
	{
		deactivated();
		getAppClient().getCommander().addCommand("newtank");
	}
	
	return true;
}

void MenuInGame::update(int dtime, int atime)
{	
	MenuBasic::update(dtime, atime);
	
	// This is code which automaticky show the menu.
	// It measures when tank die and if so and some delay the
	// menu appears
	Tank *tank = 0;
	if (getAppClient().getGame().getClientPlayer())
	{
		tank = getAppClient().getGame().getClientPlayer()->getTank();
	}
		
	if (tank && tank->isAlive())
	{
		mTankWasAlive = true;
	}
	else
	{
		if (mTankWasAlive)
		{
			mTankWasAlive = false;
			mShownForLastDead = false;	
			mTimeOfDead = gTime.getTicks();
		}
		
		if (!mShownForLastDead && 
		     (gTime.getTicks()-mTimeOfDead > gc.ingame_autoshow_delay || tank == 0) &&
		     g_settings.isTrue(SETTING_AUTO_INGAME_MENU))
		{
			activated();
			mShownForLastDead = true;
		}
	}
	
	
	if (!mActivated) return;
	
	// Check for (back to game) button
	// - it is hiddent if player allready has a tank
	if (tank == 0)
	{	
		Window *frame = mMInMain->getChild("ingame.frame");
		frame->getChild("ingame.frame.togame")->setVisible(true);
	}
	else
	{
		Window *frame = mMInMain->getChild("ingame.frame");
		frame->getChild("ingame.frame.togame")->setVisible(false);		
	}
}
