/* MAINTAINER: Bernard */
#ifndef _MENU_IN_GAME_H
#define _MENU_IN_GAME_H

#include "menubasic.h"

namespace CEGUI
{
	class EventArgs;
};

/**
 * \brief Class for handling little ingame menu dialog.
 * 
 * Player may select prefered tank and name in this dialog. Also
 * may quit game or just disconnect from server. It is possible that
 * the dialog is checking when player drops dead and appears automatic.
 * The behaviour is controled by SETTING_AUTO_INGAME_MENU.
 */
class MenuInGame : public MenuBasic
{
public:

	MenuInGame();
	~MenuInGame();

	/**
	 * \brief Beside the activation of menu also refresh data in dialog.
	 * 
	 * The player may fange it's name from console so we need to reflect this change.
	 */
	virtual void activated();
	
	/**
	 * \brief Beside deactivation also read data from menu and put them to settigs.
	 * 
	 * Note that values are stored in menu but they are not applyed.
	 * Menas the name isn't send to server or no command for new tank is called.
	 */
	virtual void deactivated();

	/**
	 * \brief Control automatic activation.
	 * 
	 * The menu has ability to activate automaticly if player's tank
	 * is destroyed. This function is checking state of tank and show
	 * the menu if needed.
	 * Also control visibility of button "Play". The button is enabled
	 * only if client has no tank.
	 */
	virtual void update(int dtime, int atime);

	/**
	 * \brief Add 'disconnect' command to commander.
	 */
	bool handleExitGame(const CEGUI::EventArgs& e);

	/**
	 * \brief Add commant 'quit' to commander.
	 */
	bool handleExitToOS(const CEGUI::EventArgs& e);

	/**
	 * \brief Apply options and deactivate the menu
	 */
	bool handleToGame(const CEGUI::EventArgs& e);
		
	/**
	 * \brief Change player's name and deactivate the menu.
	 */
	bool handleApplyPlayerName(const CEGUI::EventArgs& e);

	/**
	 * \brief Send command for new tank and deactivate the menu.
	 */
	bool handleApplyTank(const CEGUI::EventArgs& e);
	
protected:
	
	CEGUI::Window*  mMInMain; ///< \brief Window with ingame menu

// data for automatic actiovation	
	bool mTankWasAlive;     ///< \brief True if tank was alive in last update call.
	int  mTimeOfDead;       ///< \brief Timestamp when destruction of tank was detected.
	bool mShownForLastDead; ///< \brief True if menu was allready shown for last dead	
};

#endif
