#ifndef _AC_TRANSFORMATION_H_
#define _AC_TRANSFORMATION_H_

#include "animation.h"

#include <string>
#include "GLMatrix.h"

namespace hna
{
	class Transformation;
};

class TransformationController : public AnimationController
{
public:
	typedef AnimationDataSource<GLMatrix4d> anim_source_type;
	typedef hna::Transformation transformation_type;
	typedef transformation_type * transformation_ptr_type;

	/**
	*	\brief ctor
	*
	*	TransformationController takes ownership over animationDataSource and is responsible for
	*	deleting it.
	*
	*	\param animationDataSource
	*	\param nodeName
	*/
	TransformationController( anim_source_type * animationDataSource, const std::string & nodeName );
	
	virtual ~TransformationController()
	{
		delete mAnimationDataSource;
	}
	
	void update( class Model * model, unsigned int time_ms );
private:

	transformation_ptr_type getTransformation( class Model * model );
	class HierarchyNode * getNodeByName(class Model * model, const std::string & nodeName );
	
	void saveOriginalTransformation( class Model * model );
	bool isOriginalTransformationSaved() { return mOriginalTransformationSaved; }

	anim_source_type * mAnimationDataSource;
	std::string mNodeName;
	class HierarchyNode * mNode;
	
	bool mOriginalTransformationSaved;
	GLMatrix4d mOriginalNodeTM;
};

#endif
