#ifndef _ANIMATION_H_
#define _ANIMATION_H_

#include <list>
#include <memory>

// interface
class AnimationController
{
public:
	virtual ~AnimationController();

	virtual void update( class Model * model, unsigned int time_ms ){};
};

// interface template
template <class T>
class AnimationDataSource
{
public:	
	typedef T value_type;
	virtual ~AnimationDataSource(){}
	virtual void getValue( value_type & value, unsigned int time_ms ) = 0;
	
};

class Animation
{
public:
	Animation()
		: mEnabled(false)
	{}

	~Animation();
	
	void createController( std::auto_ptr< AnimationController > animController );
	
	void enable( bool enableAnim )
	{
		mEnabled = enableAnim;
	}

	bool isEnabled() 
	{ 
		return mEnabled; 
	}

	/**
	*	\brief updates each animation controller
	*	\param time_ms time in milliseconds
	*/
	void update( class Model * model, unsigned int time_ms );

private:
	
	typedef std::list< AnimationController * > controller_container_type;

	void getAllControllers( controller_container_type::iterator & first, controller_container_type::iterator & end )
	{
		first = mControllerList.begin();
		end = mControllerList.end();
	}

	std::list< AnimationController * > mControllerList;
	bool mEnabled;
};

#endif
