#include "hna_material.h"
#include "texturemanager.h"
#include <string>

namespace hna 
{
	/**
	*	\brief simple material - color texture
	*
	*	\note available only on client
	*/
	class ColorTextureMaterial : public Material
	{
	public:
		struct InfoExt
		{
			/** \brief texture coordinate id */
			enum CoordID
			{
				COORD_S = 0,
				COORD_T = 1,
				COORDINATES_COUNT = 2
			};

			/** \brief texture repeat mode */
			enum RepeatMode 
			{
				TILE, 
				MIRROR, 
				CLAMP
			};

			struct Coordinate
			{
				RepeatMode repeatMode;
				double 
					tiling,
					offset;
			};

			Coordinate s, t;
			bool generateMipmaps;
			bool translucent;

			void setRepeat( CoordID c, RepeatMode mode, double tiling );
			void setOffset( CoordID c, double offset );

			static const Coordinate DEFAULT_COORDINATE_VALUE;
			static const InfoExt DEFAULT_INFO_EXT_VALUE;
		};

		ColorTextureMaterial( std::string materialName, std::string texturefileName, int explicitMapChannel, const InfoExt &ie = InfoExt::DEFAULT_INFO_EXT_VALUE );
		ColorTextureMaterial( const ColorTextureMaterial & mat );
		virtual ~ColorTextureMaterial(void);

		virtual Attribute * clone() { return ( this ) ? new ColorTextureMaterial(*this) : 0; }
		
		/** \brief setup texturing environment, bind mapping channel to texture unit */
		virtual void applyEffect();
		virtual void removeEffect();

		bool isTranslucent();
	
	private:
		TextureRef mTexture;
		int mExplicitMapChannel;

		InfoExt mInfoExt;
	};
};
