// MAINTAINER: Martin
/**
*	\file
*	\brief graphical representation of a geometry object
*/
#ifndef _GRAPHICAL_REP_H_
#define _GRAPHICAL_REP_H_

namespace hna
{
	class Material;
};

namespace gobj
{
	class GeometryObject;

	/**
	*	\brief graphical representation of a geometry object
	*
	*	Geometry object could be represented for example by display list, VBO, or some custom representation.
	*	GraRep is the base class for such a representation.
	*/
	class GraphicalRepresentation
	{
	public:
		virtual ~GraphicalRepresentation(){}

		/**
		*	\brief creation of a graphical representation
		*	\param gobj geometry object for which the representation should be created
		*	\param mat material with mapping channels. Texture coordinates will be taken from mat.
		*	\return true on success, false on failure
		*/
		virtual bool build( const class GeometryObject * gobj, hna::Material * mat ) = 0;

		virtual bool isValid() const = 0;

		/**
		*	\brief release graphical representation
		*/
		virtual void release() = 0;

		/**
		*	\brief graphical representation rendering code
		*/
		virtual void render() const = 0;
	};
};

#endif //_GRAPHICAL_REP_H_
