#include "grep_dl.h"
#include "graphical_representation_creator.h"
#include "hierarchy_node.h"
#include "model.h"
#include "hna_material.h"
#include <memory>

void GraRepCreator::visitHierarchyNode(class HierarchyNode * node) 
{
	using namespace gobj;

	MaterialObtainer mo;

	// get material associted with this node
	node->acceptAttributeVisitor( &mo );
	
	// get geometry object associated with this node
	gobj::GeometryObject * gobj = node->getGeometryObject();
	if ( gobj != 0 )
	{
		// create new graphical representation
		std::auto_ptr< gobj::GraphicalRepresentation > rep( new GRepDisplayList() );

		if ( rep.get() )
		{
			// build graphical representation of gobj, use node's material
			hna::Material * mat ( mo.getMaterial() );
			if ( mat != 0 )
				mat->applyEffect();

			bool success = rep->build( gobj, mat );

			if ( mat != 0 )
				mat->removeEffect();

			// give this graphical representation to geometry object if creation succeeded
			gobj->setGraphicalRepresentation( success ? rep.release() : 0 );
		}
	}

	visitHierarchyChildren( node );
}

bool GraRepCreator::createGraphicalRepresentation( class Model * model )
{
	HierarchyNode * node( model ? model->getRoot() : 0 );
	if ( node )
		node->accept( this );

	return true;
}
