#include "grep_dl.h"
#include "gobj_rendervisitor.h"
#include "gobj.h"

#include "log.h"

using namespace gobj;

GRepDisplayList::GRepDisplayList()
	: mDisplayList(0)
{
}

GRepDisplayList::~GRepDisplayList()
{
	release();
}

bool GRepDisplayList::build( const class GeometryObject * gobj, hna::Material * mat )
{
	if ( gobj == 0)
		return false;

	gobj::RenderVisitor grv;
	grv.setMaterial( mat );

	// alocate new display list
	mDisplayList = glGenLists( 1 );

	if ( mDisplayList != 0 ) {
		// build a display list
		glNewList( mDisplayList, GL_COMPILE );
			gobj->accept( &grv );
		glEndList();
		return true;
	}
	return false;
}

bool GRepDisplayList::isValid() const
{
	return (mDisplayList != 0) && glIsList( mDisplayList );
}

void GRepDisplayList::release()
{
	if ( isValid() ) 
	{
		glDeleteLists( mDisplayList, 1 );
		mDisplayList = 0;
	}
}

void GRepDisplayList::render() const
{
	assertL( isValid() && "Trying to call invalid display list!" );

	glCallList( mDisplayList );
}
