// MAINTAINER: Martin
/**
*	\file
*	\brief graphical representation of a geometry object
*/
#ifndef _GRAPHICAL_REPRESENTATION_DL_H_
#define _GRAPHICAL_REPRESENTATION_DL_H_

#include "GL/glew.h"
#include "graphical_rep.h"

namespace hna
{
	class Material;
};

namespace gobj
{
	/**
	*	\brief display list representation of a geometry object
	*/
	class GRepDisplayList: public GraphicalRepresentation
	{
	public:
		/** \brief initializes display list to invalid list (handle 0) */
		GRepDisplayList();

		/** \brief releases the display list */
		virtual ~GRepDisplayList();

		/**
		*	\brief creation of a graphical representation
		*	\param gobj geometry object for which the representation should be created
		*	\param mat material with mapping channels. Texture coordinates will be taken from mat.
		*	\return true on success, false on failure
		*/
		virtual bool build( const class GeometryObject * gobj, hna::Material * mat );

		/** \brief display list validity check */
		virtual bool isValid() const;

		/** \brief delete display list */
		virtual void release();

		/** \brief calls the display list */
		virtual void render() const;

	private:
		
		/** \brief OpenGL display list handle */
		GLuint mDisplayList;
	};
};

#endif //_GOBJ_GRAPHICAL_REPRESENTATION_H_
