/* MAINTAINER: Martin */
#ifndef _ATTR_MATERIAL_H_
#define _ATTR_MATERIAL_H_

#include "hna.h"

#include "hash_hack.h"
#include "stlextdefs.h"

#include <string>

// TODO: hna namespace or not?

namespace hna 
{
	/**
	*	\brief Material base class
	*
	*/
	class Material: public Attribute
	{		
	public:
		enum TransparencyType
		{
			TYPE_OPAQUE, 
			TYPE_MASKED, 
			TYPE_TRANSLUCENT
		};

		/** \brief  */
		Material( std::string name ): mName( name ) { setTransparencyType( TYPE_OPAQUE ); }
		
		/** \brief copy ctor */
		Material( const Material &t ): mName( t.mName ) { setTransparencyType( t.getTransparencyType() ); }

		virtual ~Material(){}
		
		/** \brief clones this attribute */
		virtual Attribute* clone(void)
		{ 
			return new Material(*this); 
		}

		/** 
		*	\brief accepts attribute visitor 
		*	\param vis visitor to accept
		*/
		virtual void accept(class AttrVisitor * vis);

		/** \brief applies material */
		virtual void applyEffect(){}
		
		/** \brief removes material */
		virtual void removeEffect(){}

		typedef STL_EXT_NS::hash_map< int, int > ChannelToTextureUnitMap;
		typedef ChannelToTextureUnitMap::const_iterator ChannelToTextureUnitIterator;

		void getMapChannelInfo( ChannelToTextureUnitIterator & begin, ChannelToTextureUnitIterator & end )
		{
			begin = mChannelToTextureUnitMap.begin();
			end = mChannelToTextureUnitMap.end();			
		}

		void translateMapChannelInfo( const ChannelToTextureUnitIterator & it, int & channelID,  int & textureUnitID )
		{
			channelID = it->first;
			textureUnitID = it->second;
		}

		virtual bool isShared() { return true; }

		TransparencyType getTransparencyType() const { return mTransparencyType; }
		void setTransparencyType(TransparencyType type) { mTransparencyType = type; }

	protected:
		
		void assignMapChannelInfo( int channelID, int textureUnitID )
		{
			mChannelToTextureUnitMap[channelID] = textureUnitID;
		}

		void clearMapChannelInfo()
		{
			mChannelToTextureUnitMap.clear();
		}

	private:
		std::string mName;
		ChannelToTextureUnitMap mChannelToTextureUnitMap;
		TransparencyType mTransparencyType;
	};
}


#endif //_ATTR_MATERIAL_H_
