/* MAINTAINER: Lukas */

#ifndef _ASYNCCONNECT_H
#define _ASYNCCONNECT_H

#include <string>
#include "asyncfunction.h"
#include "netobjects.h"

/**
 * Asynchronous connect call result
 */
class ConnectResult
{
public:
	ConnectResult(int playerID, std::string msg)
	{
		mPlayerID = playerID;
		mMsg = msg;
	}

	/**
	 * Gets player id on server
	 *
	 * \return player id on server
	 */
	int getPlayerID()
	{
		return mPlayerID;
	}

	/**
	 * Gets connect message
	 *
	 * \return connect message
	 */
	std::string getMsg()
	{
		return mMsg;
	}
	
private:
	int mPlayerID;
	std::string mMsg;
};

/**
 * Asynchronous connect call
 */
class AsyncConnect : public AsyncFunction<ConnectResult>
{
friend class NetworkReceiverClient;

public:
	/**
	 * Invoke call
	 *
	 * \param connector connector corba object
	 * \param playerName player name
	 * \param client client corba object
	 */
	AsyncConnect::resultCPtr invoke(const Connector_var connector, const string playerName,
									const Client_var client);
};

#endif // _ASYNCCONNECT_H
