/* Maintainer: Petko */
#ifndef _IORTRANSFER_CLIENT_H_
#define _IORTRANSFER_CLIENT_H_

#include <string>
#include <vector>

/**	\brief Holds information about one server
 *
 */
struct ServerInfo
{
	std::string IOR;				///< \brief IOR of the server
	std::string HelloMessage;		///< \brief server's welcome message
	std::string MapDescription;		///< \brief description of the map
	std::string IP;					///< \brief IP/DNS name of server
	int PlayerCount;				///< \brief number of players connected to the server
};


/**	\brief Communicates with the server to get its IOR
 *
 *	\param address IP address or DNS name where the server should be.
 *	\param port Port on which the server should listen
 *	\param sinfo [out] Here are stored information about server, if successfully negotiated
 *	\return true if no error, false if error (or timeout) occurred
 */
bool getServerIORByAddress( const char * address, unsigned short port, ServerInfo & sinfo );

/**	\brief Broadcasts to LAN to get information about all LAN servers running
 *
 *	\param port Port where servers should listen for IOR requests
 *	\return Vector of all server that responded correctly
 */
std::vector< ServerInfo > getLANServers( unsigned short port );


#endif	//_IORTRANSFER_CLIENT_H_

