/* MAINTAINER: Lukas */

#include "networkreceiver_client.h"
#include "networkmanager_client.h"
#include "application_client.h"
#include "game_client.h"
#include "corbaexceptionmacros.h"

// TODO LUKAS debug smazat
#ifdef TESTING_ODE
std::list< GLVector3d > contacts_dbg;
std::list< struct _info > objs_dbg;
#endif

#include "events_client.h"
#include <fstream>

void NetworkReceiverClient::init(NetworkManagerClient * owner)
{
	mOwner = owner;
	mClient = 0;
}

void NetworkReceiverClient::clean()
{
	mOwner = 0;

	if(mClient)
	{
		delete mClient;
		mClient = 0;
	}
}

void NetworkReceiverClient::addGameState(const NETGameState & networkGameState)
{
	// !!! Called from corba !!!

	// this means no objets are set for receiver
	assertL(mClient);

	if(networkGameState.gsNumber == mLastGSNumber + 1)
	{
		// process actual
		processGameState(networkGameState);

		mLastGSNumber = networkGameState.gsNumber;
		
		// process stored
		while(! mUnprocessedGS.empty() &&
			mUnprocessedGS.begin()->gsNumber == mLastGSNumber + 1 )
		{
			TGameStateList::iterator nextGS = mUnprocessedGS.begin();

			processGameState(*nextGS);

			mLastGSNumber = nextGS->gsNumber;

			mUnprocessedGS.pop_front();
		}
	}
	else // we miss some gamestate / we must wait
	{
		// store game state

		TGameStateList::iterator nextGS = mUnprocessedGS.begin();
			
		while( mUnprocessedGS.end() != nextGS &&
			nextGS->gsNumber < networkGameState.gsNumber)
		{
			++nextGS;
		}

		mUnprocessedGS.insert(nextGS, networkGameState);
	}
}

void NetworkReceiverClient::processGameState(const NETGameState & networkGameState)
{
	// !!! Called from corba !!!

	assertL(mOwner);

	// lock queue for whole game state processing
	getAppClient().getGame().getEventQueue().lockQueue();

	// process all incoming gamestates
	mOwner->mUnpackerManager.unpack(networkGameState);

	// TODO LUKAS debug smazat ----

	class EvDebugData : public Event
	{
	public:

		EvDebugData(char * data, int length, int id)
		{
			mData = data;
			mLength = length;
			mID = id;
		}

		virtual void process()
		{
			// TODO PETER tady si to zpracovat
			// pozor potom si ty data uvolnit!!!!
			switch (mID)
			{
			case 0:
				break;
#ifdef TESTING_ODE
			case 1:
				{
					GLVector3d * cpos = (GLVector3d*)mData;
					contacts_dbg.push_back( *cpos );
					break;
				}
			case 256:
				{
					struct _info * in = (struct _info*)mData;
					objs_dbg.push_back( *in );
					break;
				}
			case 1024:
				{
					contacts_dbg.clear();
					objs_dbg.clear();
					break;
				}
#endif
			default:
				{
					//nic
				}
			}
		}
		
		~EvDebugData()
		{
			delete mData;
			mData = 0;
		}

		char * mData;
		int mLength;
		int mID;
	};

	for(unsigned int j = 0; j < networkGameState.debug.length(); ++j)
	{
		char * data = new char[1024];
		for(int k = 0; k < 1024; ++k)
			data[k] = networkGameState.debug[j].data[k];

		EvDebugData * ev = 
			new EvDebugData(data, 
				networkGameState.debug[j].length,
				networkGameState.debug[j].param2);

		// push to EventQueue (threadsafe)
		getAppClient().getGame().getEventQueue().push(ev);
	}

	// TODO LUKAS debug smazat ----

	// add special end event
	EvWorldUpdateEnd * ev = new EvWorldUpdateEnd();

	getAppClient().getGame().getEventQueue().push(ev);

	// unlock queue after whole game state processing
	getAppClient().getGame().getEventQueue().unlockQueue();
}

void NetworkReceiverClient::connectedCallback(Server_ptr se, int playerID, string msg)
{
	// !!! Called from corba !!!

	assertL(mOwner);

	// set network manager
	mOwner->sharedVarsLock();

	mOwner->setServerRef(se);

	mOwner->setPlayerID(playerID);

	mOwner->sharedVarsUnlock();

	// set async result
	mOwner->mAsyncConnect.setResult(new ConnectResult(playerID, msg));

	// TODO LUKAS callback manager
	mOwner->connectedCallback(msg);
}

void NetworkReceiverClient::disconnectedCallback(int reason, std::string message)
{
	// TODO LUKAS callback manager
	mOwner->disconnectedCallback(reason, message);
}

void NetworkReceiverClient::gameInfoCallback(GameInfo * gi)
{
	// !!! Called from corba !!!

	// set async result
	mOwner->mAsyncGameInfo.setResult(gi);
}

void NetworkReceiverClient::setNetworkObjects(Client_i * cl, PortableServer::POA_var bidirPoa)
{
	// We must set it one less then is first that commes over network
	mLastGSNumber = GS_START_NUMBER - 1;

	if(mClient)
	{
		delete mClient;
		mClient = 0;
	}

	mClient = cl;

	mBidirPoa = bidirPoa;
}

void NetworkReceiverClient::deactivate()
{
	if(mClient)
	{

		// disable client's object
		try
		{
			mClient->deactivate();

			Client_var clientRef = mClient->_this();
			
			if(! CORBA::is_nil(clientRef))
				mBidirPoa->deactivate_object(*(mBidirPoa->reference_to_id(clientRef)));
		}
		catch(CORBA::SystemException& e)
		{
			SYSEXCEPTION_FATALERROR("Can't disable client's object")
		}
		catch(CORBA::Exception& e)
		{
			EXCEPTION_FATALERROR("Can't disable client's object")
		}
		catch(omniORB::fatalException& e)
		{
			OMNIORB_FATAL_EXCEPTION("Can't disable client's object")
		}
		catch(...)
		{
			UNKNOWN_ERROR("Can't disable client's object")
		}

		mClient->_remove_ref();
		mClient = 0;
	}

	mBidirPoa = PortableServer::POA::_nil();
}
