/* MAINTAINER: Lukas */

#ifndef _NETWORRECEIVER_CLIENT_H
#define _NETWORRECEIVER_CLIENT_H

#include <string>
#include <list>
#include "GLQuaternion.h"
#include "netobjects_client.h"
#include "globals.h"

struct GameInfo;

class NetworkManagerClient;

#ifdef TESTING_ODE
// TODO LUKAS debug smazat
struct _info
{
	GLVector3d pos;
	GLQuaterniond ori;
};
#endif

/**
 * Receives data from network
 */
class NetworkReceiverClient
{

public:
	/**
	 * init NetworkReceiver and set NetworkReceiver owner
	 *
	 * \param owner NetworkReceiver owner
	 */
	void init(NetworkManagerClient * owner);

	void clean();

	/**
	 * Recieves packed game state and unpack it into the EventQueue
	 *
	 * \param networkGameState packed game state
	 */
	void addGameState(const NETGameState & networkGameState);

	/**
	 * Process packed game state
	 *
	 * \param networkGameState game state to process
	 */
	void processGameState(const NETGameState & networkGameState);

	/**
	 * Sets network manager according to how server processed connect
	 *
	 * \param playerID new player's id
	 * \param additional msg from server
	 */
	void connectedCallback(Server_ptr se, int playerID, std::string msg);

	/**
	 * Calls disconnect handler
	 *
	 * \param reason disconect reason
	 * \param additional msg from server
	 */
	void disconnectedCallback(int reason, std::string message);

	/**
	 * Sets game info async holder
	 *
	 * \param gi game info
	 */
	void gameInfoCallback(GameInfo * gi);

	/**
	 * Set object for receiver. Register corba objects for receiver.
	 *
	 * \param cl pointer to corba object
	 * \param bidirPoa poa that owns cl
	 */
	void setNetworkObjects(Client_i * cl, PortableServer::POA_var bidirPoa);

	/**
	 * No more inputs from network(). Deactivate corba objects.
	 */
	void deactivate();

private:

	// class owner
	NetworkManagerClient * mOwner;

	//typedef list<const NETGameState> TGameStateList; // doesn't work with const under MacOSX :-(
	typedef std::list<NETGameState> TGameStateList;

	/**
	 * Unprocessed game state list
	 */
	TGameStateList mUnprocessedGS;

	/**
	 * Last processed game state
	 */
	int mLastGSNumber;

	/**
	 * Pointer to Client_i (reference is stored in mClientRef)
	 */
	Client_i * mClient;

	/**
	 * Bidir Poa
	 */
	PortableServer::POA_var mBidirPoa;
};

#endif // _NETWORRECEIVER_CLIENT_H
