/* MAINTAINER: Lukas */

#include "unpackers.h"
#include "application_client.h"
#include "game.h"
#include "game_client.h"

template <class TEvent> template <class TData>
void HelperUnpackerWorker<TEvent>::helperUnpack(TData & data)
{
	for(unsigned int j = 0; j < data.length(); ++j)
	{
		TEvent * ev = 
			new TEvent(data[j]);

		// push to EventQueue (threadsafe)
		getAppClient().getGame().getEventQueue().push(ev);
	}
}

/* std method without template
void BasicHistoryUnpacker::unpack(const NETGameState &gameState)
{
	// EvBasicWorldEntityChanged event creator

	const NETWEBasicTypeSeq & basicTs = gameState.objects.basicTypes;

	for(unsigned int j = 0; j < basicTs.length(); ++j)
	{
		EvBasicWorldEntityChanged * ev = 
			new EvBasicWorldEntityChanged(basicTs[j]);

		// push to EventQueue (threadsafe)
		getAppClient().getGame().getEventQueue().push(ev);
	}
}
*/

void BasicHistoryUnpacker::unpack(const NETGameState &gameState)
{
	// EvBasicWorldEntityChanged event creator

	helperUnpack(gameState.objects.basicTypes);
}

void TankHistoryUnpacker::unpack(const NETGameState &gameState)
{
	// EvTankWorldEntityChanged event creator

	helperUnpack(gameState.objects.tanks);
}

void CommandUnpacker::unpack(const NETGameState &gameState)
{
	// EvCommand event creator

	for(unsigned int j = 0; j < gameState.commands.length(); ++j)
	{
		// this creates new event from string stored in IDL
		// (const char *) is there because IDL is const
		// new string is created
		EvCommand * ev = 
			new EvCommand((const char *)gameState.commands[j]);

		// push to EventQueue (threadsafe)
		getAppClient().getGame().getEventQueue().push(ev);
	}
}

void CollisionUnpacker::unpack(const NETGameState &gameState)
{
	// EvTankWorldEntityChanged event creator

	helperUnpack(gameState.collisions);
}

void NewTankUnpacker::unpack(const NETGameState &gameState)
{
	// EvNewTank event creator

	helperUnpack(gameState.newObj.newTanks);
}

void NewProjectileUnpacker::unpack(const NETGameState &gameState)
{
	// EvNewProjectile event creator

	helperUnpack(gameState.newObj.newProjectiles);
}

void NewBonusBoxUnpacker::unpack(const NETGameState &gameState)
{
	// EvNewBonusBox event creator

	helperUnpack(gameState.newObj.newBonusBoxes);
}

void DeletedObjUnpacker::unpack(const NETGameState &gameState)
{
	// EvDeletedObj event creator

	helperUnpack(gameState.deletedObjs);
}

void PlayerUnpacker::unpack(const NETGameState &gameState)
{
	// EvPlayerChanged event creator

	helperUnpack(gameState.players);
}

void NewPlayerUnpacker::unpack(const NETGameState &gameState)
{
	// EvNewPlayer event creator

	helperUnpack(gameState.newPlayers);
}

void DeletedPlayerUnpacker::unpack(const NETGameState &gameState)
{
	// EvDeletedPlayer event creator

	helperUnpack(gameState.deletedPlayers);
}

void StateOfGameUnpacker::unpack(const NETGameState &gameState)
{
	// EvStateOfGame event creator

	if(gameState.stateOfGame != NETNoChange)
	{
		// this creates new event
		EvStateOfGame * ev = 
			// Over network is enum send as int(CORBA::long)
			new EvStateOfGame(static_cast<eStateOfGame>(gameState.stateOfGame));

		// push to EventQueue (threadsafe)
		getAppClient().getGame().getEventQueue().push(ev);
	}
}

void ConditionStatusUnpacker::unpack(const NETGameState &gameState)
{
	// Ev event creator

	const NETConditionStatusSeq & csSeq = gameState.conditionStatuses;

	std::list<ConditionStatus> csList;

	for(unsigned int j = 0; j < csSeq.length(); ++j)
	{
		ConditionStatus cs;

		cs.name = csSeq[j].name;
		cs.currentvalue = csSeq[j].currentValue;
		cs.targetvalue = csSeq[j].targetValue;

		csList.push_back(cs);
	}

	if(! csList.empty() 
		// TODO LUKAS hack - unpacker should look only in it's struct and constant should be const
		|| (gameState.gsNumber % 20 == 0))
	{
		EvConditionStatusList * ev = 
			new EvConditionStatusList(csList);

		// push to EventQueue (threadsafe)
		getAppClient().getGame().getEventQueue().push(ev);
	}
}

// HERE PLACE YOU UNPACKER CLASS IMPLEMENTATION(PROCESS METHOD)
