/* MAINTAINER: Lukas */

#ifndef _UNPACKERWORKER_H
#define _UNPACKERWORKER_H

#include "serializemanager.h"
#include "log.h"

/**
 * This is basic class for all unpackers
 */
class UnpackerWorker : public SerializeWorker
{
public:
	/**
	 * Register unpacker at UnpackerManager with default priority
	 */
	UnpackerWorker();

	/**
	 * Register unpacker at UnpackerManager with set priority
	 *
	 * \priority unpacker priority
	 */
	UnpackerWorker(int priority);
	
	/**
	 * Unregister unpacker at UnpackerManager
	 */
	virtual ~UnpackerWorker();

	/**
	 * Unpacks given gamestate
	 *
	 * What is unpacked depends on concrete unpacker.
	 * This method is called from another thread(not main) and must be threadsafe.
	 *
	 * \param gameState net data that will be unpacked
	 */
	virtual void unpack(const NETGameState & gameState) = 0;

private:

	static const int defaultPriority = 200;
};

#endif // _UNPACKERWORKER_H
