/* MANTAINER: Bernard */

#ifndef _OBSERVER_H
#define _OBSERVER_H

#include "camera.h"
#include "task.h"
#include "inputacceptor.h"

/**
 * \brief Class for moving around the world without tank.
 *
 * Implementation of simple observer. The observer is
 * silently sleeping and when it is actvated it steal mouse
 * input and game gives him camera.
 */ 
class Observer : public Camera, public Task, public InputAcceptor
{
	public:
		Observer();

		/**
		 * \brief Update the position of observer.
		 *
		 * The moving and computing camera related variables
		 * is done here.
		 */
		virtual void tick();	

		/**
		 * \brief Response to mouse motion.
		 *
		 * Proces mouse motion as chage of viewing directon. Works
		 * only if observer is activated.
		 */
		virtual bool mouseMotion(int dx, int dy);

		/**
		 * \brief Activate the observer.
		 */
		void activate();

		/**
		 * \brief Deactivate the observer.
		 */
		void deactivate();

		/**
		 * \brief Get current status of observer
		 */
		bool isActive();

		/**
		 * \brief Set movement speed of observer.
		 *
		 * \param speed New speed. May be zero for stop or negative for moving backward
		 */
		void setSpeed(double speed);

	protected:
		
		double mYaw;     // \brief yaw of observer - changed directly by mouse
		double mPitch;   // \brief yaw of observer - changed directly by mouse
		bool   mActive;  // \brief activity state of observer
		double mSpeed;   // \brief current speed of movement
};

#endif
