/* MAINTAINER: Bernard */

#include "SDL.h"
#include "GL/glew.h"
#include "SDL_opengl.h"
#include "opengl2d.h"
#include "gfx.h"
#include "application_client.h"
#include "log.h"
#include "SDL_image.h"
#include "platform.h"
#include "camera.h"
#include "game_client.h"


OpenGL2D gGL2D;

void Texture2DGL::load(const std::string &file)
{

	SDL_Surface *surf = IMG_Load(locateFile(file).c_str());
	if (surf == 0)
	{
		LOGE << "Unable to load file " << file << endl;
		DIE("File locating failed");
	}

	mXOriginal = surf->w;
	mYOriginal = surf->h;

	SurfDefault sd( surf );

	this->upload( &sd, GL_RGBA, 0 );

	// texture doesn't hold SDL_Surface so we need to free it
	SDL_FreeSurface(surf);

	mValid = true;
}

void OpenGL2D::drawLine(int x1,   int y1,  int x2,    int y2,     int r, int g, int b, int a)
{
	assertL(mGFX);
	mGFX->set2DOpenGL();
	LOGW << "drawLine not implemented" << endl;
	
}

void OpenGL2D::drawRect(int left, int bottom, int w, int h, float r, float g, float b, float a)
{
	assertL(mGFX);

	mGFX->set2DOpenGL();	

//	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

//	glColor4i(255, 255, 255, 128);
	//glEnable(GL_COLOR_MATERIAL);	
	//glColor3f(r, g, b);
	//glColor4f(1.0, 1.0, 0.5, 0.5);
	glColor4f(r, g, b, a);

	glBegin(GL_QUADS);
	glVertex3i(left,   bottom,   0);
	glVertex3i(left+w, bottom,   0); 
	glVertex3i(left+w, bottom+h, 0);
	glVertex3i(left,   bottom+h, 0);
	glEnd();
}

void OpenGL2D::drawRect(int left, int top, int right, int bottom, Texture2DGL &tex)
{
	assertL(mGFX);

}

void OpenGL2D::drawRect(int left, int top, int right, int bottom, TextureRef &tex)
{
	
	if ( tex.isValid() )
	{
		GLfloat maxs( tex->getMaxS() ), maxt( tex->getMaxT() );
		tex->bind();

		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
		glEnable( GL_TEXTURE_2D );

		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		glEnable( GL_BLEND );
		
		glBegin( GL_QUADS );

		// TODO: NPOT textures...	
		// Priklad: 768 / 1024.0 pro T souradnici funguje...
		//	- ale nefunguje Texture::getMaxS() a getMaxT() pri nastaveni texturovaci matice (spatne pocitam?)
			glTexCoord2f( maxs, 0.0f );
			glVertex3i( right, top, 0 );

			glTexCoord2f( maxs, maxt );
			glVertex3i( right, bottom, 0 );

			glTexCoord2f( 0.0f, maxt );
			glVertex3i( left, bottom, 0 );
			
			glTexCoord2f( 0.0f, 0.0f );
			glVertex3i( left, top, 0 );

		glEnd();

		glDisable( GL_BLEND );
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
		glDisable( GL_TEXTURE_2D );
	}
	else
	{
		DIE( "Requested texture is not valid!" );
	}
}

void OpenGL2D::drawRect(int centerX, int centerY, Texture2DGL &tex)
{
	assertL(mGFX);

	//tex.bind(GL_TEXTURE0);
	tex.bind();

	int x = centerX - tex.mXOriginal/2;
	int y = centerY - tex.mYOriginal/2; 
	float maxs = tex.getMaxS();
	float maxt = tex.getMaxT();

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
 
	glBegin(GL_QUADS);
//	glMultiTexCoord2f(GL_TEXTURE0, 0, 0);
	glTexCoord2f(0, 0);
	glVertex3i(x, y, 0);

//	glMultiTexCoord2f(GL_TEXTURE0, maxs, 0);
	glTexCoord2f(maxs, 0);
	glVertex3i(x + tex.mXOriginal, y, 0);

//	glMultiTexCoord2f(GL_TEXTURE0, maxs, maxt);
	glTexCoord2f(maxs, maxt);
	glVertex3i(x + tex.mXOriginal, y + tex.mYOriginal, 0);

//	glMultiTexCoord2f(GL_TEXTURE0, 0, maxt);
	glTexCoord2f(0, maxt);
	glVertex3i(x, y + tex.mYOriginal, 0);
	glEnd();

	glDisable(GL_BLEND);

	glDisable(GL_TEXTURE_2D);

}

void OpenGL2D::init()
{
	mGFX = &getAppClient().getGFX();
}

int  OpenGL2D::width()
{ 
	assertL(mGFX);
	
	return mGFX->getWidth(); 
}

int  OpenGL2D::height() 
{ 
	assertL(mGFX);

	return mGFX->getHeight(); 
}


void  OpenGL2D::renderToCamera(const GLVector3d &position, float size_x, Texture2DGL &tex)
{
	Camera* camera =  getAppClient().getGame().getCamera();
	
	assertL(camera);
#ifndef _DEBUG
	if (camera == 0) 
	{ 
		return;
	}
#endif

	GLVector3d camv;
	camera->getPos(camv);

	GLVector2f dirtocam((float)(position.x-camv.x), (float)(position.y-camv.y));

	if (dirtocam.lengthSqr() == 0) return;
	dirtocam.normalize();



	float size_x2 = size_x / 2;
	float size_z2 = tex.mYOriginal * size_x2 / tex.mXOriginal;

	float dx = -dirtocam.y * size_x2;
	float dy =  dirtocam.x * size_x2;
	float x  = (float)position.x;
	float y  = (float)position.y;
	float z  = (float)position.z;

	getAppClient().getGFX().set3DOpenGL();
	tex.bind();

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
	glBegin(GL_QUADS);
		glTexCoord2f(0,   1.0); glVertex3f(x - dx, y - dy, z + size_z2);
		glTexCoord2f(1.0, 1.0); glVertex3f(x + dx, y + dy, z + size_z2);
		glTexCoord2f(1.0,   0); glVertex3f(x + dx, y + dy, z - size_z2);
		glTexCoord2f(0,     0); glVertex3f(x - dx, y - dy, z - size_z2);
	glEnd();

	//glDisable(GL_BLEND);

	glDisable(GL_TEXTURE_2D);


}
