/* MAINTAINER: Bernard */
#ifndef _OPEN_GL_2D_H
#define _OPEN_GL_2D_H

#include "texturemanager.h"
#include <string>

class GFX;
class OpenGL2D;

/**
 * \brief Upgraded texture for OpenGL2D.
 *
 * The texture holds dimensions of original picture as
 * bonus. It is needed if you draw some icon as 2D and
 * want to have it in original size.
 * You may also inicialize the texture without TextureManager.
 */
class Texture2DGL : public Texture
{
public:

	Texture2DGL()
	{
		mValid = false;
	}

	/**
	 * \brief Load texture from file.
	 * 
	 * If any arror encountered, the applicatin is halted.
	 * \param file File with the texture to load. Location of file will
	 *             be done before loading.
	 */
	void load(const std::string &file);

	bool isValid()
	{
		return mValid;
	}

private:
	friend class OpenGL2D;

	int mXOriginal; ///< \brief Dimension of original picture.
	int mYOriginal; ///< \brief Dimension of original picture.

	bool mValid; ///< \brief true after loading som texture
};

/**
 * \brief Class for helping 2D OpenGL drawing.
 *
 * It should be able to draw basic things like copy texture
 * or some rect. So far only some functions are implemented.
 * The other will be added as needed.
 */
class OpenGL2D
{
public:

	/**
	 * \brief Currently not implemented (Nobody wanted it)
	 */
	void drawLine(int x1,   int y1,  int x2,    int y2,     int r, int g, int b, int a);
	
	/**
	 * \brief Draw a rect with a specific color
	 */
	void drawRect(int left, int bottom, int w, int h, float r, float g, float b, float a);
	
	/**
	 * \brief Draw rect filled with a texture - Currently not implemented
	 */
	void drawRect(int left, int top, int right, int bottom, Texture2DGL &tex);
	void drawRect(int left, int top, int right, int bottom, TextureRef &tex);
	
	/**
	 * \brief Draw rect such that it's dimensions are same as dimension
	 *        of original picture for text.
	 */
	void drawRect(int centerX, int centerY, Texture2DGL &tex);

	void renderToCamera(const GLVector3d &position, float size_x, Texture2DGL &tex);

	/**
	 * \brief Inicialization of internal structures.
	 *
	 * This function must be calleb before any usage. It should be called
	 * once at the beginning of the game. No more calls.
	 */
	void init();

	/**
	 * \brief return Width of a screen
	 * Serves like a shortcut to GFX::getWidth()
	 */
	int  width();
	
	/**
	 * \brief return height of a screen
	 * Serves like a shortcut to GFX::getHeight()
	 */
	int  height();

protected:
	GFX  *mGFX;
};

extern OpenGL2D gGL2D;

#endif
