#include "GL/glew.h"
#include "outer_fog_renderer.h"
#include "terrain_base.h"

OuterFogRenderer::OuterFogRenderer( TerrainBase * tb, int surfacesCount, float surfacesSpacing )
:	mTerrainBase( tb ),
	mSurfacesCount( surfacesCount >= 0 ? surfacesCount : 0 ),
	mSurfacesSpacing( surfacesSpacing )
{
}

TerrainBase * OuterFogRenderer::getTerrain()
{
	return mTerrainBase;
}

void OuterFogRenderer::render()
{
	TerrainBase * tb = getTerrain();
	if ( !tb )
		return;

	// draw outer fog
	GLVector3d scale;
	tb->getScale(scale);
	scale.z /= 255;

	glPushMatrix();
	// x, y normalized terrain ( [0, 1] ), z meters
	scale.glScale();
	
	glEnable(GL_BLEND);
	glBlendFunc( GL_ONE_MINUS_CONSTANT_ALPHA, GL_CONSTANT_ALPHA );
	
	float zp( -mSurfacesCount * mSurfacesSpacing );
	for (int z = -mSurfacesCount, n = 1; z <= 0; ++z, ++n, zp += mSurfacesSpacing )
	{
		glBlendColor(0, 0, 0, float( n ) / ( n + 1 ) );
		glBegin( GL_QUADS );
			
			// south
			glVertex3f( -1,  0, zp );
			glVertex3f(	-1, -1, zp );
			glVertex3f(	 2, -1, zp );
			glVertex3f(	 2,  0, zp );

			// north
			glVertex3f(	-1, 1, zp );
			glVertex3f(	 2, 1, zp );
			glVertex3f(	 2, 2, zp );
			glVertex3f( -1, 2, zp );

			// west
			glVertex3f( 0,  0, zp );
			glVertex3f(	0, 1, zp );
			glVertex3f(	-1, 1, zp );
			glVertex3f(	-1,  0, zp );

			// east
			glVertex3f( 1,  0, zp );
			glVertex3f(	2,  0, zp );
			glVertex3f(	2,  1, zp );
			glVertex3f(	1,  1, zp );

		glEnd();
	}

	glPopMatrix();

	glDisable( GL_BLEND );
}

