#include "queue_inserter.h"
#include "hna_transform.h"
#include "model.h"

QueueInserter::QueueInserter( counted_ptr<RenderQueue> renderQueue )
	: mRenderQueue( renderQueue ), mCurrentFrontFaceOrientation( GL_CCW )
{}


#include "transform_obtainer.h"
hna::Transformation * QueueInserter::getTransformation()
{
	// TODO: get transformation
	return 0;
}

void QueueInserter::setParentTM( const GLMatrix4d & tm )
{
	mParentTM = tm;
}

void QueueInserter::getParentTM( GLMatrix4d & tm )
{
	tm = mParentTM;
}

void QueueInserter::setFrontFaceOrientation( GLenum orientation )
{
	mCurrentFrontFaceOrientation = orientation;
}

GLenum QueueInserter::getCurrentFrontFaceOrientation() const
{
	return mCurrentFrontFaceOrientation;
}

void QueueInserter::switchFrontFaceOrientation()
{
	mCurrentFrontFaceOrientation = (mCurrentFrontFaceOrientation == GL_CCW) ? GL_CW : GL_CCW;
}

void QueueInserter::insertModel( Model * m, const GLVector3d & position, const GLQuaterniond & orientation )
{
	HierarchyNode * root( m->getRoot() );
	if ( root )
	{
		// initialize parent TM from position and orientation in world space
		setFrontFaceOrientation( GL_CCW );
		GLQuaterniond tmp( orientation );
		GLMatrix4d posrot( tmp.matrix() );
		posrot.m12 = position.x;
		posrot.m13 = position.y;
		posrot.m14 = position.z;
		setParentTM( posrot );
		
		root->accept( this );
	}
}

void QueueInserter::visitHierarchyNode(class HierarchyNode * node)
{
	// store current parent transformation matrix ( TM ) as ptm
	GLMatrix4d ptm;
	getParentTM( ptm );

	// calculate this-to-world TM
	
	//const hna::Transformation * t = getTransformation();

	// TODO: better transformation obtaining
	const hna::Transformation * t( dynamic_cast<hna::Transformation*>( node->getTransformation().get() ) );


	GLMatrix4d tmp( t ? ( ptm * t->getNodeTM() ) : ptm );

	bool switchBefore(false), switchAfter(false), restoreSavedOrientation( false );
	GLenum savedFrontFaceOrientation( GL_CCW );

	if ( t != 0 )
	{
		const bool mirrorNTM( t->isMirrorNodeTM() );
		const bool mirrorOTM( t->isMirrorObjectOffsetTM() );
		
		switchBefore = ( mirrorNTM && !mirrorOTM ) || ( !mirrorNTM && mirrorOTM );
		switchAfter =  ( mirrorNTM &&  mirrorOTM ) || ( !mirrorNTM && mirrorOTM );
		restoreSavedOrientation = ( switchBefore || switchAfter ) && !( switchBefore && switchAfter );
		savedFrontFaceOrientation = restoreSavedOrientation ? getCurrentFrontFaceOrientation() : GL_CCW;
	}

	if ( switchBefore )
		switchFrontFaceOrientation();
	
	// set that matrix as the parent TM for child nodes
	setParentTM( tmp );

	gobj::GeometryObjectRef gobj = node->getGeometryObject();
	if ( ( node->isRenderable() & gobj.isValid() ) && gobj->getGraphicalRepresentation() != 0 )
	{
		// calculate to-world TM by concatenating the local offset transformation
		if ( t )
			tmp *= t->getObjectOffsetTM();

		RenderPacket packet;

		// TODO: avoid matrices copies that are not necessary
		packet.toWorld = tmp;		
		
		// TODO: avoid dynamic cast
		packet.materialRef = node->getMaterial();
		packet.mat = dynamic_cast<hna::Material*>( packet.materialRef.get() );

		packet.frontFaceOrientation = getCurrentFrontFaceOrientation();
		packet.gobj = gobj;
		
		
		assertL( (mRenderQueue.get() != 0) && "Invalid pointer to render queue!" );
		mRenderQueue->insertPacket( packet );
	}

	if ( switchAfter ) 
		switchFrontFaceOrientation();
	
	visitHierarchyChildren( node );

	if ( restoreSavedOrientation )
		setFrontFaceOrientation( savedFrontFaceOrientation );

	// restore parent TM for neighbours
	setParentTM( ptm );
}
