/* MAINTAINER: Martin */

#ifndef _RENDER_QUEUE_H_
#define _RENDER_QUEUE_H_

#include "GLMatrix.h"
#include "gobj.h"
#include "hna_material.h"
#include "counted_ptr.h"

#include <vector>
// TODO: speed: try to locally disable security in std::vector<T>

struct RenderPacket
{
	RenderPacket()
	{}

	RenderPacket( const RenderPacket & rp )
		:	toWorld( rp.toWorld ), 
			mat( rp.mat ), 
			frontFaceOrientation( rp.frontFaceOrientation ),
			gobj( rp.gobj ), materialRef( rp.materialRef )
	{}

	GLMatrix4d toWorld;
	hna::Material * mat;
	GLenum frontFaceOrientation;
	gobj::GeometryObjectRef gobj;	

	/**
	*	This only holds counted pointer to material attribute.
	*/
	counted_ptr<hna::Attribute> materialRef;
};

class RenderQueue
{
private:

	typedef std::vector< RenderPacket > container_type;

public:	
	
	typedef container_type::size_type size_type;

	static const size_type INITIAL_SIZE = 256;
	
	RenderQueue( size_type initialSize = INITIAL_SIZE );
	
	bool insertPacket( const RenderPacket & p );
	void render();
	void sort();
	void clear();

private:
	
	enum TransparencyHint
	{
		HINT_OPAQUE,
		HINT_MASKED,
		HINT_TRANSLUCENT,
	};

	void renderOpaque();
	void renderMasked();
	void renderTranslucent();

	/**
	*	\brief compares two render packets (by their materials)
	*
	*	\return true, if material address of 'left' is greater than 'right', false otherwise
	*/
	static bool isGreater( const RenderPacket & left, const RenderPacket & right );

	//inline PacketHolder * allocatePacketHolder( const TransparencyHint hint );
	inline TransparencyHint getTransparencyHint( const RenderPacket & p );
	inline container_type & getBufferByHint( const TransparencyHint hint );

	void setPacketSorting( bool enable ) { mPacketSortingEnabled = enable; }
	
	/**
	*	\brief default is enabled
	*/
	bool isPackedSortingEnabled() const { return mPacketSortingEnabled; }
	
	container_type
		mOpaque,
		mMasked,
		mTranslucent;

	bool mPacketSortingEnabled;
};

#endif //_RENDER_QUEUE_H_
