#include "render_visitor.h"
#include "gobj_rendervisitor.h"
#include "graphical_rep.h"
#include "hna_transform.h"

RenderVisitor::RenderVisitor()
	: mCurrentFrontFaceOrientation( GL_CCW )
{}

void RenderVisitor::setFrontFaceOrientation( GLenum orientation )
{
	mCurrentFrontFaceOrientation = orientation;
	glFrontFace( mCurrentFrontFaceOrientation );
}

GLenum RenderVisitor::getCurrentFrontFaceOrientation() const
{
	return mCurrentFrontFaceOrientation;
}

void RenderVisitor::switchFrontFaceOrientation()
{
	mCurrentFrontFaceOrientation = (mCurrentFrontFaceOrientation == GL_CCW) ? GL_CW : GL_CCW;
	glFrontFace( mCurrentFrontFaceOrientation );
}

void RenderVisitor::visitHierarchyNode(HierarchyNode * node)
{
	mAAV.clearReferencesToVisitedObjects();	
	mAAV.setState( mAAV.applyEffect );
	node->acceptAttributeVisitor(&mAAV);

	// Test both NTM and OTM for mirror transformation:	
	// 
	// - NTM and OTM both positive => switch polygon orientation after rendering graphical representation of this node
	// - only NTM positive => switch orientation only before rendering graphical rep.
	// - only OTM positive => switch orientation before rendering graphical rep. and turn it back right after that.
	bool switchBefore(false), switchAfter(false), restoreSavedOrientation( false );
	GLenum savedFrontFaceOrientation( GL_CCW );

	hna::Transformation * t = mAAV.getVisitedTransformation();
	if ( t != 0 )
	{
		const bool mirrorNTM( t->isMirrorNodeTM() );
		const bool mirrorOTM( t->isMirrorObjectOffsetTM() );
		
		switchBefore = ( mirrorNTM && !mirrorOTM ) || ( !mirrorNTM && mirrorOTM );
		switchAfter =  ( mirrorNTM &&  mirrorOTM ) || ( !mirrorNTM && mirrorOTM );
		restoreSavedOrientation = ( switchBefore || switchAfter ) && !( switchBefore && switchAfter );
		savedFrontFaceOrientation = restoreSavedOrientation ? getCurrentFrontFaceOrientation() : GL_CCW;
	}

	//mRV.setMaterial( mAAV.getVisitedMaterial() );
	
		
	// accept gobj::RenderVisitor on associated geometry object ( = render asociated geometry object )
	if ( switchBefore )
		switchFrontFaceOrientation();

	if ( node->isRenderable() )
	{
		const gobj::GeometryObject *o = node->getGeometryObject();
		if ( o != 0 ) 
		{
			const gobj::GraphicalRepresentation * rep( o->getGraphicalRepresentation() );
			if ( rep != 0 )
			{
				rep->render();
			}
		}
	}

	if ( switchAfter ) 
		switchFrontFaceOrientation();

	// remove temporary attribute's effects
	mAAV.clearReferencesToVisitedObjects();
	mAAV.setState( mAAV.removeNodeEffect );
	node->acceptAttributeVisitor(&mAAV);

	visitHierarchyChildren(node);

	if ( restoreSavedOrientation )
	{
		setFrontFaceOrientation( savedFrontFaceOrientation );
	}

	// remove hierarchic-persistent attribute's effects
	mAAV.setState( mAAV.removeSubtreeEffect );
	node->acceptAttributeVisitor(&mAAV);
}

