/* MAINTAINER: Martin */
#ifndef _SKYBOX_RENDERER_H_
#define _SKYBOX_RENDERER_H_

#include "texturemanager.h"
#include "GLMatrix.h"

// TODO: comment: explanation of sides in BJS and texture mapping coordinates (some picture would be great)
/**
*	\brief skybox - graphical representation and rendering
*
*	Skybox is a box that has the same orientation as the world (terrain). When skybox has to be rendered, 
*	current camera is placed into skybox center, skybox is rendered and camera 
*	(MODELVIEW matrix) is restored. Every frame the world is rendered, skybox has to be rendered first (it 
*	draws the background with z-testing disabled).
*/
class SkyboxRenderer
{
public:
	/** \brief sides of a box */
	enum BoxSide 
		{
		LEFT = 0,
		FRONT = 1,
		RIGHT = 2,
		BACK = 3,
		TOP = 4,
		BOTTOM = 5,
		};

	enum { 
		SIDES_COUNT = 6, ///< \brief number of skybox sides (should be 6 for a box)
	};

	SkyboxRenderer();

	/**
	*	\brief assign a texture to skybox side
	*	\param side skybox side
	*	\param texture reference to a texture
	*/
	void assignTexture( const BoxSide side, TextureRef texture );

	/**
	*	\brief renders skybox
	*	Skybox should be rendered before anything else, because it doesn't use z-testing, nor alpha testing.
	*/
	void render();

private:

	/**
	*	\brief translates skybox side identificator to index that indexes mSides
	*	\param side skybox side identificator
	*	\return skybox side index
	*/
	int getSideIndex( const BoxSide side ) const;

	/**
	*	\brief get texture associated with skybox side
	*	\param side skybox side identificator
	*	\return texture reference
	*/
	TextureRef getTexture( const BoxSide side );
	
	/**
	*	\brief get camera transformation and stores it to \c m
	*	Current implementation anly reads the OpenGL MODELVIEW matrix.
	*	\param m where to store camera transformation matrix
	*/
	void getCameraTransformation( GLMatrix4d & m );

	/** \brief array of skybox sides - contains textures */
	vector< TextureRef > mSides;
};


#endif //_SKYBOX_RENDERER_H_
