/* MAINTAINER: Bernard */

#include <cstdlib>
#include "musicmanager.h"	
#include "log.h"
#include "platform.h"

#include "tinyxml/tinyxml.h"

#include "soundtheme.h"
#include "settings.h"

using namespace std;

MusicManager::MusicManager()
{
	mISMusic = 0;
	mPlay = false;
	mListName = "music.xml";
}

MusicManager::~MusicManager()
{

}
	
void MusicManager::init(ISoundMusic *ism)
{
	mISMusic = ism;
}
	
void MusicManager::loadList(const std::string &file)
{

	// look into theme and then try to locate the file
	TiXmlDocument doc(locateFile(SoundTheme::getRef().fromTheme(file)));
	TiXmlHandle   handle(&doc);


	if (!doc.LoadFile()) // Here is some Valgrind problem :(
	{
		LOGE << "Unable to load " << file
		     <<" Probably file not exist or isn't a valid XML file" << endl;
		return;
	}	

	// clear current list of songs
	mFiles.clear();

	// parse file with song list
	TiXmlElement* child = handle.FirstChild( "music" ).FirstChild( "song" ).Element();
	for (; child; child=child->NextSiblingElement()) 
	{
		const char *file   = child->Attribute("file");
		if (file != 0) 
		{
			mFiles.push_back(file);
			
			LOGT << "Loaded music file " << file << endl;
		}
		else
		{
			LOGE << "In file " << file << " in music/song mising attribute file" << endl;
		}
	}

	// stop playing old music
	mISMusic->stop();
	// and start new if wanted
	update(0);

	mListName = file;
}
	
void MusicManager::reloadList()
{
	loadList(mListName);
}

void MusicManager::stop()
{
	mPlay = false;
	mISMusic->stop();
}

void MusicManager::play()
{
	mPlay = true;
	update(0);
}

// check if still playing and start playing if necessary
void MusicManager::update(int atime)
{
	if (mPlay && mISMusic && mISMusic->isPlaying() == false && mFiles.size() > 0)
	{
		// randomly select song to play
		mISMusic->play(locateFile(mFiles[rand()%mFiles.size()]), false);
	}
}


