/* MAINTAINER: Bernard */

#ifndef _MUSIC_MANAGER_H
#define _MUSIC_MANAGER_H

#include "isound.h"
#include <string>
#include <vector>
#include "task.h"

/**
 * \brief Handle music file lists.
 *
 * Music manager is usefull if you have list if music files and
 * want randomly play from the list.
 * The music manager is also task - it should be registered as
 * main application task. It needs to be task since it check
 * if the music stop playing and in such case select a new music
 * to play.
 */
class MusicManager : public Task
{
public:

	MusicManager();
	~MusicManager();
	
	/**
	 * \brief Init system for music playing.
	 * 
	 * \param ism Isn't deleted
	 */
	void init(ISoundMusic *ism);

	void setISM(ISoundMusic *ism)
	{
		mISMusic = ism;
	}
	
	/**
	 * \brief Load music file list.
	 *
	 * Load XML file containing list of music files. The currently playing music
	 * (if any) is stopped and new music is started (if music should be playing).
	 * \param file theme name / basic filename - theme and location will be
	 *        perfomed on it.
	 */
	void loadList(const std::string &file);
	
	/**
	 * \brief
	 *
	 * Again load last list - good if theme changed
	 */
	void reloadList();
	
	/**
	 * \brief Stop playing currently played music (if any)
	 */
	void stop();
	
	/**
	 * \brief Start playing music.
	 */
	void play();

	/**
	 * \brief Check if music is still playing.
	 *
	 * If the music stoped playing and should be playing, new
	 * file from music file list is loaded and is played.
	 */
	virtual void update(int atime);

	
//	void enable(bool really); // ?
	
private:

	std::vector<std::string> mFiles; ///\brief files to play
	std::string              mListName; /// name of list
	
	ISoundMusic             *mISMusic;
	
	
	bool                     mPlay; /// \brief true if this should play
	
};

#endif // _MUSIC_MANAGER_H
