/* MAINTAINER: Bernard */


//#define WANT_TRACE

#include "SDL.h"
#include "SDL_mixer.h"
#include "sound_sdl_mixer.h"
#include "log.h"
#include "worldentity.h"
#include "sound_empty.h"
#include "platform.h"
#include "soundtheme.h"


// ---------------------------------------------------------------------------
// ------------------------------------------- SoundSystem_SDL ---------------
// ---------------------------------------------------------------------------



// This also handle inicialization of music
SoundSystem_SDL::SoundSystem_SDL()
{
	// Init SDL_AUDIO
	if(SDL_WasInit(SDL_INIT_AUDIO) == 0)
		if (SDL_InitSubSystem(SDL_INIT_AUDIO) < 0 ) 
		{
			LOGE << "Unable to inicialize audio: " <<  SDL_GetError() << endl;

			// CREATE OBEJCTS AS EPMTY!
			createEmpty();
			
			return;
		}


	// 44.1KHz, signed 16bit, system byte order, 
	// stereo audio, using 1024 byte chunks
	if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 2048) == -1) 
	{
		LOGE << "Unable to inicialize audio: " << Mix_GetError() << endl;
		
		// CREATE OBEJCTS AS EPMTY!
		createEmpty();
		
		return;
	}
		
	// Create music object
	mMusic =  new SoundMusic_SDL();
	if (mMusic == 0)
	{
		LOGE << "Unable to create music object!" << endl;
		DIE();
	}

	// Create samples object
	mSamplesManager = SoundSamplesManager_SDL::getInstance();
	if (mSamplesManager == 0)
	{
		LOGE << "Unable to create samples manager object!" << endl;
		DIE();
	}
	
	LOGI << "SDL_Mixer audio created" << endl;
	
}

void SoundSystem_SDL::createEmpty()
{
	mMusic = new SoundMusic_Empty();
	if (mMusic == 0)
	{
		DIE("Unable to allcoate object SoundMusic_Empty");
	}

	mSamplesManager = new SoundSamplesManager_Empty();
	if (mSamplesManager == 0)
	{
		DIE("Unable to allcoate object SoundSamplesManager_Empty");
	}
}

SoundSystem_SDL::~SoundSystem_SDL()
{
	deleteObjects();
	
	Mix_CloseAudio();
	
	// shit down AUDIO subsystem
	SDL_QuitSubSystem(SDL_INIT_AUDIO);
}

// ---------------------------------------------------------------------------
// ------------------------------------------- SoundMusic_SDL ----------------
// ---------------------------------------------------------------------------

//  Should do some logging
SoundMusic_SDL::SoundMusic_SDL()
{
	mMusic = 0;
}

SoundMusic_SDL::~SoundMusic_SDL()
{
	stop();
	if (mMusic != 0) Mix_FreeMusic(mMusic);
	mMusic = 0;
}

void SoundMusic_SDL::play(const char* file, bool loop)
{

	assertL(file != 0);
	
	if (mMusic != 0) Mix_FreeMusic(mMusic);

	mMusic = Mix_LoadMUS(file);
	if (!mMusic)
	{
		LOGE << "Error when opening audio file " << file << ":" <<  Mix_GetError() << endl;
		return;
	}
	
	int rv;
	if (loop)
		rv = Mix_PlayMusic(mMusic, -1);
	else
		rv = Mix_PlayMusic(mMusic, 1);
	
	if (rv == -1)
	{
		LOGE << "Unable to play musc. Mix_PlayMusic: " << Mix_GetError() << endl;
		// well, there's no music, but most games don't break without music...
	}
}

void SoundMusic_SDL::stop()
{
	Mix_HaltMusic();
}
	
bool SoundMusic_SDL::isPlaying()
{
	return (Mix_PlayingMusic() == 1);
}
int SoundMusic_SDL::getVolume()
{
	return mVolume;
}

int SoundMusic_SDL::getMaxVolume()
{
	return 	MIX_MAX_VOLUME;
}
	
void SoundMusic_SDL::setVolume(int volume)
{
	if (volume < 0) volume = 0;
	if (volume > getMaxVolume()) volume = getMaxVolume();
	mVolume = volume;
	
	Mix_VolumeMusic(mVolume);
}


// ---------------------------------------------------------------------------
// ------------------------------------------- SoundSample_SDL ---------------
// ---------------------------------------------------------------------------

SoundSample_SDL::SoundSample_SDL(Mix_ChunkRef chunk, SoundSamplesManager_SDL *manager)
{
	mManager = manager;
	mChanell = -1;
	mChunk   = chunk;
	mSuicide = false;
	mRegisteredAsPositionSound = false;
}

SoundSample_SDL::~SoundSample_SDL()
{
	TIN;

	// remove from updaters and remove from callbackers
	if (mChanell != -1)
	{
		assertL(mManager);
		mManager->setStopCallback(mChanell, 0);
		Mix_HaltChannel(mChanell);
		mChanell = -1;
	}
	
	if (mRegisteredAsPositionSound)
	{
		assertL(mManager);
	}
	
	// substract some references...
	mChunk = 0;
	
	//TODO: Mix_Chunk ? What to do with it?
	//if (mChunk != 0)
	//	Mix_FreeChunk(mChunk);
}

void SoundSample_SDL::stopped()
{
	mChanell = -1;
	suicideOpportunity();
}

void SoundSample_SDL::play(bool loop)
{
	if (mChunk == 0) 
	{
		suicideOpportunity();
		return;
	}
	
	if (mChanell != -1)
	{
		Mix_HaltChannel(mChanell);
	}

	if (loop) mChanell = Mix_PlayChannel(-1, mChunk, -1);
	else      mChanell = Mix_PlayChannel(-1, mChunk, 0);

	TVARIABLE(mChanell);
	// No sound produced - probably all chanells full
	if (mChanell < 0)
	{
		LOGD << "Consider enlarging number of chanells" << endl;
		suicideOpportunity();
		return;
	}

	assertL(mManager);

	mManager->setStopCallback(mChanell, this);
}




// ---------------------------------------------------------------------------
// ------------------------------------------- SoundSamplesManager_SDL -------
// ---------------------------------------------------------------------------

SoundSamplesManager_SDL *SoundSamplesManager_SDL::sThisManager = 0;

SoundSamplesManager_SDL::SoundSamplesManager_SDL()
{
	TIN;
	
	assertL(sThisManager == 0); // Only one instance of manager is possible
	sThisManager = this;

	Mix_ChannelFinished(SoundSamplesManager_SDL::stopCallback);
	mAllocatedChanells = Mix_AllocateChannels(16);
}

SoundSamplesManager_SDL::~SoundSamplesManager_SDL()
{
	TIN;

	Mix_ChannelFinished(0);
	sThisManager = 0;
}


ISoundSample* SoundSamplesManager_SDL::newSample(const char* file)
{
	LOGT << "Loading wav: " << file << endl;
	
	ISoundSample* sample;
	
	// Load the file/wav
	string key = locateFile(SoundTheme::getRef().fromTheme(file));
	
	// I. FIND
	ManagedItem * pmi;
	if ((pmi = find(key)) == 0)
	{
	    // II. INIT
		Mix_Chunk* chunk = Mix_LoadWAV(key.c_str());
		if (chunk == 0)
		{
			LOGE << "Unable to load file " << file << " themed as " << key <<  endl;
			sample = new SoundSample_SDL(0,0);
			if (sample == 0)
			{
				DIE("Unable to allocate sample object. Shit.");
			}
			return sample;
		}
		else 
		{
			// III. REGISTER
			pmi = createAndRegister(key, chunk);
		}
	}

	sample = new SoundSample_SDL(pmi, this);
	if (sample == 0)
	{
		DIE("Unable to allocate sample object. Shit.");
	}

	return sample;
}


void SoundSamplesManager_SDL::stopCallback(int chanell)
{
	TIN;

	TVARIABLE(chanell);
	
	assertL(sThisManager); // this should NOT be == 0 when this is called
	assertL(chanell >= 0);

	if ((signed int)sThisManager->mPlayingChannels.size() > chanell && chanell >= 0)
	{
		if (sThisManager->mPlayingChannels[chanell])
		{
			if (sThisManager->mPlayingChannels[chanell]->mChanell == chanell)
				sThisManager->mPlayingChannels[chanell]->stopped();
			sThisManager->mPlayingChannels[chanell] = 0;	
		}
	}
}

void SoundSamplesManager_SDL::setStopCallback(int channel, SoundSample_SDL* sample)
{
	// write to vector sample
	assertL(channel >= 0);
	if ((unsigned int) channel >= mPlayingChannels.size()) 
	{
		TPOINTS("Resizing");
		mPlayingChannels.resize(channel + 4);
	}
	mPlayingChannels[channel] = sample;
}	


void SoundSamplesManager_SDL::stopAll()
{
	//for (int i = 0; i < mAllocatedChanells; ++i)
	for (int i = 0; i < (signed int)mPlayingChannels.size(); ++i)
		Mix_HaltChannel(i);
}

/*
void SoundSamplesManager_SDL::destroyContent(Mix_Chunk* data)
{
	assertL(data != 0);
	
	Mix_FreeChunk(data);

	// LOGE << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl;
	
}
*/


// Partial specialization instead of overloading...
template<>
void UniversalManager<Mix_Chunk, SoundSamplesManager_SDL>::destroyContent(Mix_Chunk* data)
{
	assertL(data != 0);

	LOGT << "Mix_FreeChunk" << endl;
	Mix_FreeChunk(data);
}

int SoundSamplesManager_SDL::getVolume()
{
	return mVolume;
}

int SoundSamplesManager_SDL::getMaxVolume()
{
	return 	MIX_MAX_VOLUME;
}
	
void SoundSamplesManager_SDL::setVolume(int volume)
{
	mVolume = volume;
	Mix_Volume(-1, mVolume);
}
