/* MAINTAINER: Bernard */

#ifndef _I_SOUND_SDL_H
#define _I_SOUND_SDL_H

#include <list>
#include <vector>
#include "isound.h"
#include "SDL_mixer.h"
#include "universal_manager.h"

class SoundSystem_SDL;
class SoundSamplesManager_SDL;
class SoundMusic_SDL;
class SoundSample_SDL;


class SoundSystem_SDL : public ISoundSystem
{
public:	
	/**
	 * \brief Create objects of SoundMusic_SDL and SoundSample_SDL
	 */
	SoundSystem_SDL();

	virtual ~SoundSystem_SDL();
	
protected:
	/**
	 * \brief Create emtpy implementation
	 */
	void createEmpty();
};


/**
 * \brief Implements ISoundMusic using SDL_Mixer
 */
class SoundMusic_SDL : public ISoundMusic
{
public:	

	SoundMusic_SDL();
	~SoundMusic_SDL();
	
	virtual void play(const char* file, bool loop);

	virtual void stop();
	
	virtual bool isPlaying();
	
	/**
	 * \brief Get current volume of music.
	 * 
	 * \return volume is in range 0..getMaxVolume()
	 */
	virtual int getVolume();
	
	/**
	 * \brief Get maximum possible volume.
	 * 
	 * \return maximum possible volume.
	 */
	virtual int getMaxVolume();
	
	/**
	 * \brief Set music volume to some variable.
	 * 
	 * \param volume new volume
	 */
	virtual void setVolume(int volume);	
protected:

	Mix_Music *mMusic; // currently playing music
	
	int mVolume; ///< /brief Current volume of all samples	
};	



/**
 * \brief Virtual class for holding sound samples.
 *
 *  Nowhere implemented yet.
 */
class SoundSamplesManager_SDL : public ISoundSamplesManager,
								public UniversalManager<Mix_Chunk, SoundSamplesManager_SDL>
{
public:
	// In constructor the sound buffer will load a sample or will
	// be given a loaded sample. We don't care right now

	SoundSamplesManager_SDL();
	virtual ~SoundSamplesManager_SDL();
	
	/**
	 * \brief Return object allocated by new.
	 *
	 * Allways valid return.
	 */
	virtual ISoundSample* newSample(const char* file);
	
	virtual void stopAll();

	void setStopCallback(int channel, SoundSample_SDL* sample);
	
	/**
	 * \brief Get current volume of samples.
	 * 
	 * \return volume is in range 0..getMaxVolume()
	 */
	virtual int getVolume();
	
	/**
	 * \brief Get maximum possible volume.
	 * 
	 * \return maximum possible volume.
	 */
	virtual int getMaxVolume();
	
	/**
	 * \brief Set samples volume to some variable.
	 * 
	 * \param volume new volume
	 */
	virtual void setVolume(int volume);	
	
//protected:	
public:	
//	virtual void destroyContent(Mix_Chunk* data);
	
private:

	
	std::list<SoundSample_SDL*>  mPositionList; // List of playing samples with position
	std::list<SoundSample_SDL*>  mAllList; // List of all loaded samples - no need probably
	std::list<WorldEntity*>  mUpdatingEntites; // List of entites that want to have update callback - registered in task addWorldObject iterface.
	
	std::vector<SoundSample_SDL*> mPlayingChannels; // For callback of playing

	static SoundSamplesManager_SDL *sThisManager;  // because of callback stopCallback
	static void stopCallback(int chanell);

	int mAllocatedChanells; 
	
	int mVolume; ///< /brief Current volume of all samples
};


typedef SoundSamplesManager_SDL::TUniversalManager::TRef Mix_ChunkRef; 
	

class SoundSample_SDL : public ISoundSample
{
public:
	SoundSample_SDL(Mix_ChunkRef chunk, SoundSamplesManager_SDL *manager);
	~SoundSample_SDL();

	virtual void play(bool loop = false);

	// Callback from SoundSamplesManager_SDL when sound is stopped
	void stopped();

protected:

	
	Mix_ChunkRef             mChunk;
	SoundSamplesManager_SDL *mManager;

	friend class SoundSamplesManager_SDL;

	int        mChanell;
	bool       mRegisteredAsPositionSound;
	double     mX;
	double     mY;
	double     mZ;
};

// Used fot the explicit template instantiation in template_instantces_*.cpp file
#ifdef INSTANTIATE_TEMPLATE
	template class Singleton< SoundSamplesManager_SDL >;
#endif


#endif
