/* MAINTAINER: Bernard */

//#define WANT_TRACE

#include "tinyxml/tinyxml.h"
#include "soundtheme.h"
#include "log.h"

using namespace std;
using namespace STL_EXT_NS;


void SoundTheme::load(const std::string &filename)
{

	TIN;

	TVARIABLE(filename);
	
	TiXmlDocument doc(filename);
	TiXmlHandle   handle(&doc);


	if (!doc.LoadFile()) 
	{
		LOGE << "Unable to load " << filename 
		     <<" Probably file not exist or isn't a valid XML file" << endl;
		
		return;
	}	
	

	// Forget previous theme
	mThemeTable.clear();

	// Search whole documents for translation pairs and add them to 
	// translation table
	TiXmlElement* child = handle.FirstChild( "theme" ).FirstChild( "item" ).Element();
	for (; child; child=child->NextSiblingElement()) 
	{
		const char *key   = child->Attribute("original");
		const char *value = child->Attribute("in_theme");
		if (key != 0 && value != 0) 
		{
			string keyS;   // This is here because of valgrind didn't like
			string valueS; // mThemeTable.insert(make_pair(key, value));

			keyS   = key;
			valueS = value;
			
//			mThemeTable.insert(make_pair(keyS, valueS));
			mThemeTable[keyS] = valueS;

			LOGT << "Loaded item of theme '" << key << "' =  '" << value << "'" << endl;
		}
	}
}
	
std::string SoundTheme::fromTheme(const std::string& original)
{
	TIN;
	
	hash_map<string, string>::iterator it = mThemeTable.find(original);
	if (it == mThemeTable.end()) 
	{
		LOGT << "theme: " << original  << " -> " << original << endl;
		return original;
	}
	
	LOGT << "theme: " << original  << " -> " << it->second << endl;
	return it->second;

}
