/* MAINTAINER: Martin */

#ifndef _TASK_RENDER_H_
#define _TASK_RENDER_H_

#include "task.h"
#include "layer.h"
#include <list>

class TaskRender: public Task
{
public:	
	TaskRender();

	virtual void init();
	virtual void clean();
	
	/**
	*	\brief Rendering. 
	*	
	*	This method is called directly TaskManager
	*	Note that atime will overflow in 20 days and will start from
	*	zero.
	*
	*	\param atime absolute time in ms
	*/
	virtual void update(int atime);

	void pushLayer(Layer& layer);

private:
	typedef std::list<Layer*> TLayerContainer;

	/**
	*	\brief called at the start of each frame
	*
	*	It sets the OpenGL state and current camera.
	*	This means that MODELVIEW_MATRIX is set according to camera and frustum is updated
	*	using the current MODELVIEW_MATRIX and PROJECTION_MATRIX
	*/
	void updateBegin(void);

	/**
	*	\brief called at the end of each frame
	*
	*	It waits for OpenGL to finish its job and swaps buffers.
	*/
	void updateEnd(void);
	
	/**
	*	\brief renders terrain, tanks, etc.
	*	
	*	It is called in TaskRender::update(int atime).
	*/
	//void renderGameLayer(void);

	/**
	*	\brief renders console, gui.
	*	
	*	It is called in TaskRender::update(int atime) after TaskRender::renderGameLayer().
	*/
	//void renderGUILayer(void);
	
	TLayerContainer mLayers;
};

#endif
