/* MAINTAINER: Bernard */

#ifndef _TASK_SOUND_H_
#define _TASK_SOUND_H_

#ifdef NO_OPENAL

#include "task.h"

class TaskSound: public Task
{
public:

	TaskSound() {}
	virtual ~TaskSound() {}

	void newExplosion(int id, const GLVector3d &pos) { }
	
	static TaskSound* getInstance()
	{
		return 0;	
	}
};

#else

/*Petko: na windows sa inkluduje al bez AL/, 
 */
#if defined(_MSC_VER)
#include <al.h>
#include <alc.h>
#elif defined(__APPLE__)
#include <OpenAL/al.h>
#include <OpenAL/alc.h>
#else
#include <AL/al.h>
#include <AL/alc.h>
#endif

// We have our own ALut
#include "alut.h"

#include <vector>
#include <list>
#include "task.h"
#include "GLVector.h"

class WorldEntity;

struct sound3Ddescription
{
	sound3Ddescription()
	{
		buffer = AL_NONE;
	}
	
	ALuint      buffer;
	float       distance_silence;
	float       distance_sound;
	ALfloat		rolloff_factor;
	ALfloat		reference_distance;
	ALfloat     pitch;
	ALfloat     gain;
};

typedef struct
{
	WorldEntity *we;
	ALuint      source;
	float       distance_silence;
	float       distance_sound;
	bool        playing;
} list3Dmember;

typedef std::list<list3Dmember> list3DOpenAL;
typedef std::list<ALuint>       list3DSources;
typedef std::vector<sound3Ddescription> list3DBuffers;

/**
 * \brief Task that takes care of 3D sound.
 *
 * The basic idea is that task is listening on 
 * addedWorldObject and create sounds as entities
 * are created/destroyed.
 * In regular update it check positions of sound
 * sources and update them.
 */
class TaskSound: public Task
{
public:

	TaskSound();
	virtual ~TaskSound();

	virtual void addedWorldObject(WorldEntity* we);
	virtual void deletedWorldObject(WorldEntity* we);

	virtual void init();

	virtual void clean();
	
	virtual void update(int atime);

	void newExplosion(int id, const GLVector3d &pos);
	
	
	/**
	 * \brief
	 * 
	 * May return 0.
	 */
	static TaskSound* getInstance()
	{
		return mThis;	
	}
	
protected:

	void printOpenALInfo();

	void initOpenAL();
	void cleanOpenAL();

	ALuint loadBuffer(const std::string &file);

	void updateSource(list3Dmember &lm);

	void playBufferForEntity(WorldEntity* we, const sound3Ddescription &dsc);
	void playBufferForPosition(const GLVector3d &pos, const sound3Ddescription &dsc);

	sound3Ddescription loadDescription(const std::string &file);


	//TODO: mBuffer should be removed as it is only for testing
//	ALuint  mBuffer; // only for testing - should be deleted later

    ALCdevice *mDevice;
    ALCcontext *mContext;


	list3DOpenAL  mEntites; // List of entites that produce some sound (like tank)
	list3DSources mOnlySources; // List of sound that don't change positions - like explosion.

	bool mInited;

	list3DBuffers mProjectileBuffers; // preloaded sounds
	list3DBuffers mTankEngineBuffers; // preloaded sounds
	list3DBuffers mExplosionBuffers;  // preloaded sounds

	GLVector3d    mListener;
/*
	double mDistSilent;
	double mDistSound;
	double mRollOffFactor;
	double mReferenceDistance;
	*/
	bool mOpenALInicilized; ///< \brief true if OpenAL was inicialized
	bool mInicialized;  ///< \brief true if task is inicialized and ready to use
	ALfloat mGlobalGain; ///< \brief Multiplication for gain.
	
	
static 	TaskSound* mThis;
};


#endif // NO_OPENAL

#endif
