/* MAINTAINER: Martin CURRENT_EDITOR: Bernard */

// #define WANT_TRACE

#include "tracer.h"
#include "log.h"
#include "application_client.h"
#include "tank.h"
#include "player.h"
#include "game_client.h"
#include "terrain.h"
#include "world.h"
#include "observer.h"
#include "mousemotion.h"
#include <cmath>

#include <iostream>

using namespace std;

Tracer::Tracer(void):
	mEntity(0)
{
	mUpVector = GLVector3d(0,0,1);
	mPosition = GLVector3d(30, 30, 120);
	mReferencePoint = GLVector3d(128, 128, 60);

	mTickCoeficient = 1.0/32.0;
	//mTickCoeficient = 1.0;
	mTickDelta = 10;

	mFollowTurret = false;
	setParams(20, -80, -30, 300);
}

Tracer::~Tracer(void)
{
}


template <typename T>
inline std::ostream& operator<< (std::ostream& ostream, GLVector3<T> vector)
{
	ostream << "[" << vector.x << ", " << vector.y << ", " << vector.z << "]";
	return ostream;
}

bool Tracer::computePosition(GLVector3d &position, GLVector3d &referencePoint)
{

	if (!mEntity) return false;
/*
	GLVector3d orientation = mEntity->getOrientation().rotateVector(GLVector3d(1,0,0));

	orientation.z = 0;

	if (mEntity->getClassType() == cTank)
	{
		Tank *t = dynamic_cast<Tank*>(mEntity);
		if (t)
		{
			GLQuaterniond q(GLVector3d(0,0,1), t->getYaw());
			if (mFollowTurret)
			{
				orientation = q.rotateVector(orientation);
			}
		}
	}
*/
	MouseMotion &targeting = getAppClient().getGame().getMouseMotion();
	
	GLVector3d orientation;
	targeting.computeCameraOrientation(orientation);

	if ( orientation.lengthSqr() == 0 ) return false;
	orientation.normalize();

	position    = mEntity->getPosition() + orientation*mPositionScale;
	position.z += mPositionZplus;
	
	// Test if position is above terrain
	Terrain &terrain = getAppClient().getGame().getWorld().getLandscape().getTerrain();

	// Round position to the map - else we have problems with height map
	terrain.cropToValidXY(position);

	// lift camera higher than map is
	double h = terrain.getHeight(position.x, position.y);
	if (position.z < h + 5) position.z = h + 5;
		
	if (targeting.getFloatingView())
	{
		float pitchRAD = (float)targeting.getViewPitch()/180.0f*(float)M_PI;
	
		orientation *= cosf(pitchRAD);
		orientation.z = sinf(pitchRAD);
	
		// compute the reference point
		referencePoint    = position + orientation;
	}
	else
	{	
		// compute the reference point
		referencePoint    = mEntity->getPosition() + orientation*mRefPointScale;
		referencePoint.z += mRefPointZplus;			
	}

	return true;
}

void Tracer::associate(WorldEntity * entity)
{
	mEntity = entity;	
}

void Tracer::jumpToPosition()
{
	computePosition(mPosition, mReferencePoint);
	

	mPosition = GLVector3d(128,128,100);
	mReferencePoint =  GLVector3d(768, 768, 0);

	TVARIABLE(mPosition);
	TVARIABLE(mReferencePoint);
}

void Tracer::update(int dtime, int atime)
{
	
	GLVector3d pos, refPoint;
	
	if ( computePosition(pos, refPoint) )
	{
		//mPosition = pos*mTickCoeficient + mPosition*(1-mTickCoeficient);
		//mReferencePoint = refPoint*mTickCoeficient + mReferencePoint*(1-mTickCoeficient);
		mPosition = pos;
		//double z = mPosition.z;
		//mPosition.z = pos.z*mTickCoeficient + z*(1-mTickCoeficient);
		
		mReferencePoint = refPoint;
	}
	else
	{
		// Activate observer
		getAppClient().getGame().getObserver().activate();
		/*
		mUpVector = GLVector3d(0,0,1);
		mPosition = GLVector3d(0, 0, 60);
		mReferencePoint = GLVector3d(128, 128, 60);
		*/
	}

	Task::update(dtime, atime);
}

void Tracer::tick()
{
/*
	GLVector3d pos, refPoint;
	
	if ( computePosition(pos, refPoint) )
	{
		mPosition = pos*mTickCoeficient + mPosition*(1-mTickCoeficient);
	//	mReferencePoint = refPoint*mTickCoeficient + mReferencePoint*(1-mTickCoeficient);
	//	mPosition = pos;
	//	mReferencePoint = refPoint;
	}
	*/
}


void Tracer::addedWorldObject( WorldEntity * we )
{
	if ( we->getClassType() != cTank )
		return;

	Player * pl = getAppClient().getGame().getClientPlayer();
	if ( !pl )
		return;

	Tank * pl_tnk = pl->getTank();
	if ( !pl_tnk )
		return;

	if ( pl_tnk->getID() == we->getID() )
	{
		pl_tnk->setClientsMain();
		associate( we );
	}
}

void Tracer::deletedWorldObject(WorldEntity *we)
{
	if ( we == mEntity )
		associate( 0 );
}

