/* MAINTAINER: Bernard */

#ifndef _TRACER_H
#define _TRACER_H

#include "camera.h"
#include "worldentity.h"
#include "task.h"

/**
 *	\brief Camera that traces some world entity. 
 *	
 *	This camera can be associated with player's tank and use some preconfigured 
 *	navigation to follow the tank (keep distance, angle, ...). 
 *
 */  
class Tracer: public Camera, public Task
{
public:
	Tracer(void);
	~Tracer();

	/**
	*	\brief Associates tracer with entity.
	*
	*	\param entity target to follow
	*/
	void associate(WorldEntity * entity);
	
	/**
	 * \brief Update position of tracer.
	 *
	 * The position is computed from current position and position that the
	 * tracer should have. It could do smoother movement of the tracer.
	 */
	void tick();
	
	/**
	 * \brief Set position of camera to computePositions.
	 *
	 * It is because tick have only some procentual move.
	 */
	void jumpToPosition();
	
	/**
	 * \brief Compute position that the camera should have.
	 *
	 * \return false if some error - like to associated entity...
	 */
	bool computePosition(GLVector3d &position, GLVector3d &referencePoint);
	
	/**
	 * \brief Set parametrs of view of tracer.
	 */
	void setParams(double positionZplus, double positionScale, double refPointZplus, double refPointScale)
	{
		mPositionZplus = positionZplus;
		mPositionScale = positionScale;
		mRefPointZplus = refPointZplus;
		mRefPointScale = refPointScale;
	}
	
	virtual void update(int dtime, int atime);

	/**	\brief If the added object is tank that is controlled by current player, it associates camera with it.
	 *
	 */
	virtual void addedWorldObject( WorldEntity * we );

	/**	\brief If the deleted object id the one associated with camera, it's deleted
	 *
	 */
	virtual void deletedWorldObject( WorldEntity * we );
		
	virtual WorldEntity* monitoredEntity() { return mEntity;  }

	void setFollowTurret(bool ft) { mFollowTurret = ft; }
	
	/**
	 * \brief stop following the Entity
	 */
	virtual void clean() { mEntity = 0; }
	
	
private:
	WorldEntity * mEntity;	///< \brief world entity that this tracer follows
	
	double  mPositionZplus;
	double  mPositionScale;
	double  mRefPointScale;
	double  mRefPointZplus;

	double  mTickCoeficient; ///< \brief Percents of how fast to convergent in (0-1]

	bool    mFollowTurret;
};

#endif // _TRACER_H
