/* MAINTAINER: Bernard */

// #define WANT_TRACE

#include "SDL.h"
#include "log.h"
#include "settings.h"
#include "platform.h"
#include "globals.h"

#include "application.h"
#include "translator.h"
#include "params.h"
#include "taskmanager.h"

#include <cstdlib>
#include <ctime>


using namespace std;


Application::Application(int argc, char* argv[])
{	
	mArgc = argc;
	mArgv = argv;
	
	mTaskManager = new TaskManager();
	if (mTaskManager == 0) DIE("new failed");

	initBasicSubsystems();
}

Application::~Application()
{
	cleanBasicSubsystems();

	delete mTaskManager;
	mTaskManager = 0;
}

void Application::initBasicSubsystems()
{
	TIN;

	initPlatform();	
	
	// Start random generator
	srand((unsigned int)time(NULL));
		
	// Global setting
	// init config forposiibility of modifying
	// MAIN_CONFIG_FILE
	g_settings.parseCommandLine(mArgc, mArgv);
	
	if ( !g_settings.loadSettings(locateFile(g_settings.getString(SETTING_MAIN_CONFIG_FILE)).c_str()) )
		LOGE << "Unable to load setttings file " << g_settings.getString(SETTING_MAIN_CONFIG_FILE)
			 << " using defaults." << endl;

	// Load command line again because we want to use commandline
	// options and no config file options
	g_settings.parseCommandLine(mArgc, mArgv);

	
	LOGI << "Global setting loaded." << endl;
	
	// SDL
	if ( SDL_Init(SDL_INIT_TIMER) < 0 ) 
	{
		LOGE << "SDL_Init failed: " <<  SDL_GetError() << endl;
		DIE();
	}

	LOGI << "SDL inicialized." << endl;

	// loading translations
	// TODO: There should be some value default and then some algorithm
	// to decide using locales probably
	g_translator.loadFile( locateFile(g_settings.getString(SETTING_TRANSLATION_FILE)).c_str() );

	// loading parameters of all game objects
	gParams.load();
}


void Application::cleanBasicSubsystems()
{
	TIN;

	// SDL
	SDL_Quit();

	LOGI << "SDL exited." << endl;
}
	
void Application::loadUserConfigFile(const char *file, const char *file_template)
{
	// "create" config file
	checkConfigFile(
			g_settings.getString(file),
			g_settings.getString(file_template)
			);

	// load it
	string conffile =  locateFile(g_settings.getString(file));
	if ( !g_settings.loadSettings(conffile.c_str()) ) 
	{
		LOGE << "Unable to load config file " << conffile
			<< " using defaults." << endl;
	}
}


void Application::saveUserConfigFile(const char *file, const char *file_template)
{
	// "create" config file
	checkConfigFile(
			g_settings.getString(file),
			g_settings.getString(file_template)
			);

	// save it
	string conffile =  locateFile(g_settings.getString(file));
	if ( !g_settings.saveSettings(conffile.c_str()) ) 
	{
		LOGE << "Unable to save config file " << conffile << endl;
	}
}


TaskManager& Application::getTaskManager()
{
	assertL(mTaskManager);

	return *mTaskManager;
}

void Application::logBasicInfo()
{
	LOGI << "Version: " << BJS_VERSION_STRING << endl;
	
#ifdef _DEBUG
	LOGI << "Debug compilation." << endl;
#else
	LOGI << "Release compilation." << endl;
#endif	

	if (g_settings.isTrue(SETTING_DEVELOPER)) 
	{
		LOGI << "Started in developer mode." << endl;
	}	
}
