/* MAINTAINER: Bernard */

#ifndef _CONVERSIONS_H_
#define _CONVERSIONS_H_

//#include <unistd.h>
#include <string>
#include <sstream>
#include <iostream>
#include "GLVector.h"
#include "GLQuaternion.h"


//---- Conversions anything to string 



/**
 * \brief Conver anything that has operator << to string.
 *
 * If you want to convert int consider using itos since it may
 * be faster.
 * \return converted value
 */
template <class T> std::string xtos(const T& t)
{
	std::stringstream s;
	s << t;
	return s.str();
}
/**
 * \brief Convert integer to string.
 * 
 * \param i integer you want to convert.
 * \return string containing the integer
 */
inline std::string itos(int i)
{
	std::stringstream s;
	s << i;
	return s.str();	
}


//---- Conversions for extractibg data from string ----
/**
 * \brief Try to extraxt double value from string.
 * 
 * \param value String with value.
 * \param f     Destination of extraction.
 * \return true if extraction with no problem.
 */
bool try_get_double(const std::string& value, double &f);

/**
 * \brief Try to extract bool value from string.
 * 
 * \param value String containing bool value.
 * \param b Destination of extraction.
 * \return true if conversion was without any problems.
 */
bool try_get_bool  (const std::string& value, bool   &b);


bool try_get_long  (const std::string& value, long   &i);
bool try_get_char  (const std::string& value, unsigned char &c);


/// die on error
long   get_long  (const std::string& value);
double get_double(const std::string& value);
bool   get_bool  (const std::string& value);
unsigned char  get_char(const std::string& value);


//----- Conversion UTF-8 <=> wchar_t

/**
 * \brief Convert UTF-8 string to wchar_t string
 *
 * \param utf8 input string encoded in utf-8
 * \param ws   output string
 */
bool utf8tows(const char *utf8, std::wstring &ws);


/**
 * \brief Convert wchar_t strinf to UTF-8 string
 *
 * \param ws   input string containing wchar_t string
 * \param utf8 output string encoded in UTF-8
 */
bool wstoutf8(const std::wstring &ws, std::string &utf8);

inline std::ostream & operator<<( std::ostream & ost, const GLVector3d & vec )
{
	ost << "( " << vec.x << ", " << vec.y << ", " << vec.z << " )";
	
	return ost;
}

inline std::ostream & operator<<( std::ostream & ost, const GLQuaterniond & quat )
{
	ost << "( ";
	ost << (quat.getW()); 
	ost << ", ";
	ost << (quat.getV()); 
	ost << " )";
	
	return ost;
}
#endif
