/* MAINTAINER: Petko */

#ifndef _EVENTQUEUE_H
#define _EVENTQUEUE_H

#include "event.h"
#include "log.h"
#include <queue>

extern int run_queue;

/**
 * 
 */
class EventQueue
{

public:
	
	EventQueue(){};
	~EventQueue()
	{
		cleanUp();
	}

	/**	\brief Pushes new event to the back of the queue
	 * 
	 *	\param newevent Event to add.
	 */
	void push( Event * newevent )
	{
		//TESTING:
		pushLock( newevent ); return;
		assertL( newevent != 0 );
		mQueue.push( newevent );
	}

	/**	\brief Pushes new event to the back of the queue, threadsafe version
	 * 
	 *	\param newevent Event to add.
	 */
	void pushLock( Event * newevent )
	{
		assertL( newevent != 0 );
		mMutex.lock();
		mQueue.push( newevent );
		mMutex.unlock();
	}
	
	/**	\brief Return first event in queue or 0, if queue is empty
	 *
	 *	\return On empty queue 0, first event in queue otherwise.
	 */
	Event * pop()
	{
		//TODO: testik:
		return  popLock();

		if ( mQueue.empty() )
			return 0;
		else
		{
			run_queue++;
			Event * retval = mQueue.front();
			mQueue.pop();
			return retval;
		}
	}

	/**	\brief Return first event in queue or 0, if queue is empty. Threadsafe version.
	 *
	 *	\return On empty queue 0, first event in queue otherwise.
	 */
	Event * popLock()
	{
		mMutex.lock();
		if ( mQueue.empty() )
		{
			mMutex.unlock();
			return 0;
		}
		else
		{
			Event * retval = mQueue.front();
			mQueue.pop();
			mMutex.unlock();
			return retval;
		}

		//unaccessible: -- In that case the code should not be here sinc it confuses the compiler
		// mMutex.unlock(); 
	}

	/**	\brief Cleans queue and delets events in it.
	 */
	void cleanUp()
	{
		//TESTING:
		cleanUpLock(); return;
		while ( !mQueue.empty() )
		{
			delete pop();
		}
	}

	/**	\brief Cleans queue and delets events in it. Thread-safe version.
	 */
	void cleanUpLock()
	{
		mMutex.lock();
		while ( !mQueue.empty() )
		{
			delete pop();
		}
		mMutex.unlock();
	}

	/**	\brief Lock queue mutex
	 */
	void lockQueue()
	{
		mMutex.lock();
	}

	/**	\brief release mutex on queue
	 */
	void unlockQueue()
	{
		mMutex.unlock();
	}

private:
	
	std::queue< Event* > mQueue;

	/**	\brief Private copy c-tor to avoid copying of queue.
	 */
	EventQueue( const EventQueue & eq )
	{
		DIE( "Attempt to copy event queue!" );
	}

	Mutex mMutex;

};

#endif // _EVENTQUEUE_H
