// MAINTAINER: Martin
#ifndef _BASIC_LOADER_H_
#define _BASIC_LOADER_H_

#include <string>

#include "conversions.h"
#include "tinyxml/tinyxml.h"

/**
*	\brief class for basic XML parsing using TinyXML
*	
*/
class BasicLoader
{
public:
	
	template <typename T>
	bool stringTo( const std::string &str, T & value );

//	bool stringTo<int>( const std::string &str, int & value );
/*
	{
		long tmp(0);
		if ( try_get_long(str, tmp) )
		{
			value = static_cast<int>( tmp );
			return true;
		}
		return false;
	}
	*/
/*
	template< float> bool stringTo( const std::string &str, float & value )
	{
		double tmp(0);
		if ( try_get_double(str, tmp) )
		{
			value = static_cast<float>( tmp );
			return true;
		}
		return false;
	}
*/	
	template <typename T>
	bool loadTextAs( TiXmlHandle textHandle, T & value )
	{
		TiXmlText * text( textHandle.ToText() );
		if (!text)
		{
			// TODO: error log
			return false;
		}
		stringTo( text->ValueStr(), value );
		return true;
	}

	template <typename T>
	bool loadAttributeAs( TiXmlHandle handle, const std::string & name, T & value )
	{
		TiXmlElement * element( handle.ToElement() );
		if (!element)
		{
			// TODO: error log
			return false;
		}
		return element->QueryValueAttribute( name, &value ) == TIXML_SUCCESS;
	}
};


template<> inline bool BasicLoader::stringTo<int>( const std::string &str, int & value )
	{
		long tmp(0);
		if ( try_get_long(str, tmp) )
		{
			value = static_cast<int>( tmp );
			return true;
		}
		return false;
	}

template<> inline bool BasicLoader::stringTo<float>( const std::string &str, float & value )
	{
		double tmp(0);
		if ( try_get_double(str, tmp) )
		{
			value = static_cast<float>( tmp );
			return true;
		}
		return false;
	}

#endif // _BASIC_LOADER_H_

