/* MAINTAINER: Petko, CURRENT_EDITOR: Bernard */
#ifndef _BONUS_H
#define _BONUS_H

#include "params.h"
#include <list>

class Tank;

/** \addtogroup game_data */ /*@{ */

/**	\brief Basic class for all bonuses. It implements "empty bonus" - doesn't change anything.
 *
 *	This class is probably not needed on client. 
 *	When created, the class is bound to BonusBox. When some tank takes the Bonus (collides with
 *	bonusbox) - the bound is cancelled and BonusBox destroyed. Then the bonus is bound to the 
 *	tank. If it is permanent bonus, its function is applyied and then destroyed. Otherwise it
 *	destroys after the bonus expiration.
 */
class Bonus
{
public:
	Bonus();
	virtual ~Bonus();

	/**	\brief Initializes Bonus according to its ID in params
	 *
	 *	\param params_ID ID from Params to initialize by.
	 */
	void initByID( int params_ID );

	// Some callbacks

	/**
	 * \brief Called if tank picks this bonus.
	 *
	 * The bonus will apply it's things to tank. May add to tank's
	 * bonus list or may delete itself.
	 */
	virtual void picked(Tank &t);

	/**
	 * \brief The tank should call this if bonus is too old.
	 *
	 * Thenk tank should remove it from its lists and never touch it again.
	 * Since the bonus delete itself in this function.
	 */
	virtual bool testEspired(int time);
	
	/**
	 * \brief This is callback that tank is destroyed.
	 *
	 * The bonus may create new bonus bow fo itself or it may
	 * delete itself.
	 * \param killed If the tank was destroyed in the field on just fall
	 *               off map or player disconeceted...
	 */
	virtual void tankDestroyed(bool killed);

	int getID()
	{
		return mID;
	}

	const Params::sBonusParams& getParams();

	/**
	 * \brief The bonus should apply relaod time change to tank.
	 *
	 * This method is called when tank change weapon and so it
	 * needs to update reload time.
	 */
	void applyReloadTime();

protected:

	/// \brief create bonus box with same bonus ID
	void dropBonusBox();

	/// \brief Tells whether the bonus is permanent or time-limited
	bool	mPermanent;

	/// \brief Tank that took this bonus. MAY BE NULL!
	Tank*	mInfluencedTank;

	int  mID;        /// \brief index to gParams.mBonus
	int  mDuration;  /// \brief Time how long the bonus is active

	/// \brief Icon that will be displayed on player's screen when he has this bonus
	int		mIconIndex;
		
};

/*@}*/ 
#endif // _BONUS_H
