/* MAINTAINER: Petko */
#ifndef _BONUSBOX_H
#define _BONUSBOX_H

#include "worldentity.h"
#include "bonus.h"

/** \addtogroup game_data */ /*@{ */

/**	\brief Representation of "box with bonus" - that means what player can see
 *
 */
class BonusBox : public WorldEntity
{
public:
	BonusBox();
	~BonusBox();

	/**
	 * Pack new created BonusBox
	 *
	 * Packs data that is needed for creating object on client.
	 *
	 * \param data net game state structure where data will be packed
	 */
	virtual void packCreationData(NETGameState & data);

	/**	\brief Makes class-specific initialization - binds object and its bonus
	 *
	 *	\param bonus_function What should this bonusbox mean
	 *	\return True if successfull
	 */
	bool initSpecific( Bonus * bonus_function );

	/**	\brief Initializes BonuxBox parameter according to ID of bonus function
	 *
	 *	\param params_ID ID from Params to initialize by.
	 */
	void initByID( int bonus_function_id );

	/**	\brief Returns mBonus
	 *
	 *	\return Value of mBonus field, may be NULL!
	 */
	Bonus * getBonus()
	{
		return mBonus;
	}

	/**	\brief Resets the mBonus field to NULL, indicating there's no Bonus
	 */
	void setNoBonus()
	{
		mBonus = 0;
	}

	virtual void render();

	virtual void handleCollisionWEntityServer( int id, const GLVector3d & coll_pos, const GLVector3d & coll_normal );

	/**
	 * \brief Called if tank picks this BonusBox
	 *
	 * It just call picked from current bonus
	 */
	virtual void picked(Tank &t)
	{
		if (mBonus) mBonus->picked(t);
		mBonus = 0;
		mDeleteMe = true;
	}


private:
	Bonus * mBonus; ///< \brief bonus represented

};

/*@}*/ 
#endif // _BONUSBOX_H

