/* MAINTAINER: Lukas */

#ifndef _COLLISION_BJS_H
#define _COLLISION_BJS_H

class Collision
{

public:
	/**
	 * New collision
	 *
	 * \param we1ID first worldentity id in collision
	 * \param we2ID second worldentity id in collision, INVALID_PLAYER_ID means
	 *  that we1ID is in collision with some static object
	 * \param pos collision position
	 * \param normal collision normal
	 */
	Collision(int we1ID, int we2ID, const GLVector3d & position, 
		const GLVector3d & normal)
	{
		mWe1ID = we1ID;
		mWe2ID = we2ID;
		mPosition = position;
		mNormal = normal;

		// invalid
		mCreationTime = 0;
	};

	/**
	 * New collision
	 *
	 * \param we1ID first worldentity id in collision
	 * \param we2ID second worldentity id in collision, INVALID_PLAYER_ID means
	 *  that we1ID is in collision with some static object
	 * \param pos collision position
	 * \param normal collision normal
	 * \
	 */
	Collision(int we1ID, int we2ID, const GLVector3d & position, 
		const GLVector3d & normal, int creationTime)
	{
		mWe1ID = we1ID;
		mWe2ID = we2ID;
		mPosition = position;
		mNormal = normal;
		mCreationTime = creationTime;
	};

	~Collision() { }

	/**
	 * Get first collision entity ID
	 *
	 * \return first collision entity ID
	 */
	int getWe1ID() const
	{
		return mWe1ID;
	}
	
	/**
	 * Get second collision entity ID
	 *
	 * \return second collision entity ID or INVALID_PLAYER_ID if first entity 
	 * is in collision with some static object
	 */
	int getWe2ID() const
	{
		return mWe2ID;
	}

	/**
	 * Get collision position
	 *
	 * \return collision position
	 */
	const GLVector3d & getPosition() const
	{
		return mPosition;
	}

	/**
	 * Get collision normal
	 *
	 * \return collision vector
	 */
	const GLVector3d & getNormal() const
	{
		return mNormal;
	}

	/**
	 * Get time when was collision created
	 *
	 * \return creation time if valis, 0 otherwise
	 */
	int getCreationTime() const
	{
		return mCreationTime;
	}

private:
	int mWe1ID;
	int mWe2ID;
	GLVector3d mPosition;
	GLVector3d mNormal;

	int mCreationTime;
};

#endif // _COLLISION_BJS_H
