/* MAINTAINER: Petko */
#include "commonentity.h"
#include "log.h"
#include "modelmanager.h"

CommonEntity::CommonEntity()
{
	mType = cInvalid;
	mID = 0;
	mPhysicsData = 0;
}

CommonEntity::~CommonEntity()
{
#ifdef _DEBUG
	if ( mModel.get() != 0 )
		LOGD << "CommonEntity was not destroyed before calling its dtor!" << std::endl;
#endif
}

void CommonEntity::basicInit( int ID, GLVector3d position, GLQuaterniond orientation )
{
	setID( ID );
	mPosition = position;
	mOrientation = orientation;
}

bool CommonEntity::init()
{
	mModel.reset( ModelManager::getInstance()->createModelInstance( getModelName() ) );
	return (mModel.get() != 0);
}

void CommonEntity::destroy()
{
	mModel.reset( 0 );
}

void CommonEntity::render()
{
}


void CommonEntity::setID(int ID)
{
	mID = ID;
}

int  CommonEntity::getID()
{
	return mID;
}

ClassType CommonEntity::getClassType()
{
#ifdef _DEBUG
	if ( mType == cInvalid )
	{
		LOGE << "Class has an invalid value of mType ( cInvalid )!\n";
		DIE();
	}
#endif
	return mType;
}

CommonEntity::CommonEntity( const CommonEntity & )
{
	DIE( "Attempt to copy an instance of CommonEntity!" );
}

double CommonEntity::getXYangle()
{
	GLVector3d orient = mOrientation.rotateVector(GLVector3d(1,0,0));
	orient.z = 0;
	if (orient.x == 0 && orient.y == 0) return 0;
	orient.normalize();
	
	double rv = atan2(orient.y, orient.x)*180.0/M_PI;
	if (rv < 0) rv += 360;
	return rv;
}
