// MAINTAINER: Martin

#include "explosion_loader.h"

template <typename T>
bool ExplosionLoader::stringTo( const std::string &str, T & value )
{
	return BasicLoader::stringTo( str, value );
}

template<> bool ExplosionLoader::stringTo< std::vector<float> >( const std::string &str, std::vector<float> & value )
{
	std::vector<float> tmp;
	
	std::stringstream data( str );
//	unsigned int i = 0;
	while ( !data.eof() )
	{
		float f;
		data >> f;
		tmp.push_back( f );
	}
	value = tmp;
	return true;
}

template <typename T>
bool ExplosionLoader::loadTextAs( TiXmlHandle textHandle, T & value )
{
	TiXmlText * text( textHandle.ToText() );
	if (!text)
	{
		// TODO: error log
		return false;
	}
	stringTo( text->ValueStr(), value );
	return true;
}

bool ExplosionLoader::loadRandomVector( TiXmlHandle handle, type::RandomVector & result )
{
	return 
		loadAttributeAs( handle, "deviance", result.deviance ) &&
		loadTextAs( handle.FirstChild(), result.vec );
}

bool ExplosionLoader::loadRandom( TiXmlHandle handle, type::RandomNumber & result )
{
	return 
		loadAttributeAs( handle, "from", result.from ) &&
		loadAttributeAs( handle, "to", result.to );
}

bool ExplosionLoader::loadColorRGBA( TiXmlHandle handle, type::Color & result )
{
	return loadTextAs( handle.FirstChild(), result );
}

bool ExplosionLoader::loadExplosion(TiXmlHandle handle, Params::sExplosionParams & result )
{
	bool b(true);
	
	if (!loadAttributeAs( handle.FirstChildElement("sound").FirstChildElement("file"), "src", result.sound ))
		result.sound = "";

	b &= loadAttributeAs( handle, "id", result.id );
	b &= loadTextAs( handle.FirstChildElement("duration").FirstChild(), result.duration );
	b &= loadAttributeAs( handle.FirstChildElement("next_explosion"), "id", result.next_id );
	b &= loadTextAs( handle.FirstChildElement("next_explosion").FirstChild(), result.time_of_next );
	b &= loadEmitter( handle.FirstChildElement("emitter"), result.emitter );
	b &= loadParticle( handle.FirstChildElement("particle"), result.particle );
	return b;
}

bool ExplosionLoader::loadEmitter(TiXmlHandle handle, type::Emitter & emitter )
{
	TiXmlHandle tmp = handle.FirstChildElement("particles_count");
	bool b(true);
	b &= loadTextAs( tmp.FirstChildElement("initial").FirstChild(), emitter.initialParticlesCount );
	b &= loadTextAs( tmp.FirstChildElement("max").FirstChild(), emitter.maxParticlesCount );
	b &= loadTextAs( handle.FirstChildElement("speed").FirstChild(), emitter.emitSpeed );
	b &= loadTextAs( handle.FirstChildElement("stop_emit").FirstChild(), emitter.stopEmit );
	return b;
}

bool ExplosionLoader::loadParticle(TiXmlHandle handle, type::Particle & particle )
{
	TiXmlHandle tmp(0);
	bool b(true);

	tmp = handle.FirstChildElement("position");
	b &= loadTextAs( tmp.FirstChildElement("radius").FirstChild(), particle.position.radius );
	b &= loadRandom( tmp.FirstChildElement("height").FirstChildElement("random"), particle.position.height );
	
	tmp = handle.FirstChildElement("velocity");
	b &= loadRandomVector( tmp.FirstChildElement("direction").FirstChildElement("random_vector"), particle.direction );
	b &= loadRandom( tmp.FirstChildElement("magnitude").FirstChildElement("random"), particle.speed );
	b &= loadRandom( tmp.FirstChildElement("acceleration").FirstChildElement("random"), particle.acceleration );
	b &= loadRandomVector( tmp.FirstChildElement("global_acceleration").FirstChildElement("random_vector"), particle.globalAcceleration );

	tmp = handle.FirstChildElement("time");
	b &= loadRandom( tmp.FirstChildElement("birth").FirstChildElement("random"), particle.birth );
	b &= loadRandom( tmp.FirstChildElement("spanlife").FirstChildElement("random"), particle.spanlife );
	
	b &= loadColorRGBA(handle.FirstChildElement("color").FirstChildElement("rgba"), particle.color );
	b &= loadAttributeAs( handle.FirstChildElement("sprite").FirstChildElement("file"), "src", particle.sprite.image );
	b &= loadAttributeAs( handle.FirstChildElement("sprite").FirstChildElement("file"), "explicit_alpha", particle.sprite.explicitAlpha );
	{
		std::string s;
		b &= loadAttributeAs( handle.FirstChildElement("blending"), "type", s );
		if (s == "interpolation")
			particle.blendingType = type::BLENDING_INTERPOLATION;
		else if (s == "additive")
			particle.blendingType = type::BLENDING_ADDITIVE;
		else
			particle.blendingType = type::BLENDING_NONE;
		

	}
	b &= loadTextAs( handle.FirstChildElement("size").FirstChild(), particle.size );

	return b;
}
