// MAINTAINER: Martin
#ifndef _EXPLOSION_LOADER_H_
#define _EXPLOSION_LOADER_H_

#include "basic_loader.h"
#include "params.h"

#include <vector>
#include <sstream>
#include "tinyxml/tinyxml.h"

/**
*	\brief loads explosion effect from Tiny XML handle
*/
class ExplosionLoader: private BasicLoader
{
public:

	/**
	*	\brief performs loading
	*	\param handle Tiny XML handle to node with explosion description
	*	\param result where to store loaded values
	*	\return true if all components were successfuly loaded (parsed)
	*/
	bool loadExplosion( TiXmlHandle handle, Params::sExplosionParams & result );

private:
	typedef Params::sExplosionParams type;

	template <typename T>
	bool stringTo( const std::string &str, T & value );

	//template< std::vector<float> > bool stringTo( const std::string &str, std::vector<float> & value );

	template <typename T>
	bool loadTextAs( TiXmlHandle textHandle, T & value );

	bool loadEmitter( TiXmlHandle handle, type::Emitter & emitter );
	bool loadParticle( TiXmlHandle handle, type::Particle & particle );
	
	bool loadRandomVector( TiXmlHandle handle, type::RandomVector & result );
	bool loadRandom( TiXmlHandle handle, type::RandomNumber & result );
	bool loadColorRGBA( TiXmlHandle handle, type::Color & result );
};

template<> bool ExplosionLoader::stringTo< std::vector<float> >( const std::string &str, std::vector<float> & value );

#endif //_EXPLOSION_LOADER_H_
