/* MAINTAINER: Petko */
#ifndef _HISTORY_H
#define _HISTORY_H

#include "GLVector.h"
#include "GLQuaternion.h"

struct NETGameState;
struct NETWEBasicType;
struct NETWETank;

/** \addtogroup game_data */ /*@{ */

/**	\brief Class that contains network- and interpolation-relevant data of object 
 *
 */
class HistoryPoint
{

public:
	HistoryPoint() { };
	virtual ~HistoryPoint()
	{ };
	
	//Data needed for interpolation - mostly it's only position, orientation, speed and timestamp
	GLVector3d		mPosition;
	GLQuaterniond	mOrientation;
	GLVector3d		mSpeed;
	GLVector3d		mAngVelocity;

	int mTimeStamp;
	int mID;

	//nie je const kvoli poondiatemu kvaternionu, kde tie poondiate metody nie su const...
	bool operator==( HistoryPoint & hp );

	/** 
	 * Compare two history points
	 *
	 * \param hp second history point
	 * \return true data equivalent
	 * \return false otherwise
	 */
	virtual bool equals(HistoryPoint & hp)
	{
		// use operator
		return operator==(hp);
	}

	/**	Transfer data from HistoryPoint to given data
	 *
	 *	\param data net structure that will be filled
	 */
	virtual void transferToNET(NETGameState & data);

	/**	Copies this history point, but returns it allways as type HistoryPoint
	 *
	 *	\note Needs redefinition in EVERY descendant class!!
	 *
	 *	\return Copy of history point
	 */
	virtual HistoryPoint * copy()
	{
		HistoryPoint * ret = new HistoryPoint( *this );
		return ret;
	}

protected:
	/**	Transfer data from HistoryPoint to given data
	 *
	 * helper function that is used by virtual void transferToNET(NETGameState & data)
	 *
	 *	\param data net structure that will be filled
	 */
	void transferToNETDirect(NETWEBasicType & data);
};

/**	\brief Tank-specific history point
 */
class TanksHistoryPoint:public HistoryPoint
{
public:
	TanksHistoryPoint(){};
	~TanksHistoryPoint(){};

	double mPitch;  // this is ignored by client's main tank
	double mYaw;    // this is ignored by client's main tank
	int    mWeapon;
	int    mProjectiles;  // This is needed only for client's main tank
	int    mHP;
	int    mArmour;       // This is needed only for client's main tank

	/**	Compulsory redefinition of copy() method
	 *
	 *	\return Copy of history point
	 */
	virtual HistoryPoint * copy()
	{
		HistoryPoint * ret = new TanksHistoryPoint( *this );
		return ret;
	}

	/** 
	 * Compare two history points
	 *
	 * \param hp second history point
	 * \return true data equivalent
	 * \return false otherwise
	 */
	virtual bool equals(HistoryPoint & hp);

	/**	Transfer data from HistoryPoint to given data
	 *
	 *	\param data net structure that will be filled
	 */
	virtual void transferToNET(NETGameState & data);

protected:
	
	/**	Transfer data from HistoryPoint to given data
	 *
	 * helper function that is used by virtual void transferToNET(NETGameState & data)
	 *
	 *	\param data net structure that will be filled
	 */
	void transferToNETDirect(NETWETank & data);

};


/*@}*/ 
#endif // _HISTORY_H
