/* MAINTAINER: Petko  CURRENT_EDITOR: Bernard */
#ifndef _LANDSCAPE_H
#define _LANDSCAPE_H

#include "stlextdefs.h"
#include "hash_hack.h"
#include <list>
#include <string>

#include "skybox.h"
#include "terrain.h"

#include "landscapeentity.h"

#include "terrain.h"
#include "globals.h"

/** \addtogroup game_data */ /*@{ */


	/// \brief Defines type for sequential list of entities
typedef std::list< LandscapeEntity* >			landent_list;	

	/// \brief Defines iterator for sequential pass of components
typedef std::list< LandscapeEntity* >::iterator	landent_iterator;

	/// \brief Type for hash table of objects
typedef STL_EXT_NS::hash_map< int, LandscapeEntity* >	landent_hash;
	

class Landscape
{
public:

	Landscape(void);
	~Landscape(void);

	Terrain& getTerrain(void) 
	{ 
		return mTerrain; 
	}

	Skybox*	 getSkybox()
	{
		return &mSkybox;
	}

	void render(void);

	/** \brief Gets the start of the list of objects.	*/
	landent_iterator	getLandEntitiesBegin();

	/** \brief Gets the end of the list of objects		*/
	landent_iterator	getLandEntitiesEnd();


		// Acces methods to the objects

	/**	
	 * \brief Finds ant returns entity with given ID
	 *
	 *	\param id ID of wanted object
	 *	\return Pointer to the found object, or NULL if doesn't exists object with that ID
	 */
	LandscapeEntity*		getLandEntity(int id);

	/**	
	 * \brief Finds ant returns entity with given iterator
	 *
	 *	\param iterator Iterator to list of objects.
	 *	\return Pointer to the found object, or NULL if the iterator was equal to end().
	 */
	LandscapeEntity*		getLandEntity(landent_iterator iterator) const;
		

	void loadEntities(std::string mapPath);

	/**
	 * \brief Clean all Landscape Entites.
	 */
	void clean();

private:

	/**	
	 * \brief Creates new generic object
	 *
	 *	\param position Position on the map where should new object appear.
	 *	\param orientation Orientation of the object on the map, in which it should appear.
	 *	\param param_id ID of object type (mSubType)
	 *	\param id Required ID of object - if zero, it's assigned some value automatically
	 *	\return Pointer to newly created object, or NULL, if not succesfull.
	 */
	LandscapeEntity* createLandEntity( GLVector3d position, GLQuaterniond orientation,
	                                   int param_id, int id = INVALID_ENTITY_ID );

	LandscapeEntity* createLandEntity( GLVector3d position, GLQuaterniond orientation,
                                              std::string modelName, int id = INVALID_ENTITY_ID);
	/**	
	 * \brief Deletes Land entity
	 *
	 *	\param id ID of object to delete.
	 *	\return True if successfull.
	 */
	bool deleteLandEntity( int id );


	Skybox  mSkybox;
	Terrain mTerrain;

	/**
	 * Return next unique ID in world.
	 * Allways return negative number
	 */
	int getNextID();

	void addLandscapeEntity(LandscapeEntity* le);
	
	void delLandscapeEntity(LandscapeEntity* le);
	
	// container of world objects
	landent_list	mLandList;		///< \brief List of all landscape objects
	landent_hash	mLandHash;		///< \brief Hash table of all landscape objects objects
		
	// int mMaxID; // the variable is shared with World::mMaxID
	int mMinID;	///< \brief Minimal unique ID reached so far

	Landscape( const Landscape & );			///< \brief To avoid copy Landsacpe object
};

/*@}*/ 
#endif // _LANDSCAPE_H




