/* MAINTAINER: Bernard */

#include "tinyxml/tinyxml.h"
#include "log.h"
#include "platform.h"
#include "settings.h"
#include "params.h"
#include "conversions.h"

#include "explosion_loader.h"

using namespace std;


Params gParams; ///< The unique variable that holds all params


Params::Params()
{

}

Params::~Params()
{

}


#define OPEN_FILE(setting, main_tag, element_tag) \
	filename = locateFile(g_settings.getString(setting)); \
	sz = 0; \
	if (!doc.LoadFile(filename)) \
	{ \
		LOGE << "Unable to load " << filename \
		     << " Probably file not exist or isn't a valid XML file" << endl; \
		DIE(); \
	} \
	child = handle.FirstChild( main_tag ).FirstChild( element_tag ).Element(); \
	if (!child) \
	{ \
		LOGE << "There is no data in file " << filename \
		     << "Need tags " << main_tag << " and " << element_tag << endl; \
		DIE(); \
	}

#define OPEN_FILE_NOLOCATE(filename, main_tag, element_tag) \
	if (!doc.LoadFile(filename)) \
	{ \
		LOGE << "Unable to load " << filename \
		     << " Probably file not exist or isn't a valid XML file" << endl; \
		DIE(); \
	} \
	child = handle.FirstChild( main_tag ).FirstChild( element_tag ).Element(); \
	if (!child) \
	{ \
		LOGE << "There is no data in file " << filename \
		     << "Need tags " << main_tag << " and " << element_tag << endl; \
		DIE(); \
	}

#define CHILD_TO_ELEMENT2(tag1, tag2) \
	child = handle.FirstChild( tag1 ).FirstChild( tag2 ).Element(); \
	if (!child) \
	{ \
		LOGE << "There is no data in file " << filename \
		     << "Need tags " << tag1 << " and " << tag2 << endl; \
		DIE(); \
	}

#define CHILD_TO_ELEMENT3(tag1, tag2, tag3) \
	child = handle.FirstChild( tag1 ).FirstChild( tag2 ).FirstChild( tag3 ).Element(); \
	if (!child) \
	{ \
		LOGE << "There is no data in file " << filename \
		     << "Need tags " << tag1 << ", " << tag2 << " and " << tag3 << endl; \
		DIE(); \
	}


#define GET_VALUE(x) \
	value = child->Attribute( #x ); \
	if ( value == 0 ) \
	{ \
		LOGE << "Missing attribute " << #x << " in file " << filename << endl; \
		DIE(); \
	} 

#define LOAD_INT(x) \
	GET_VALUE(x) \
	tmp.x = get_long(value);
		
#define LOAD_STRING(x) \
	GET_VALUE(x) \
	tmp.x = value;

#define LOAD_DOUBLE(x) \
	GET_VALUE(x) \
	tmp.x = get_double(value);
	
#define LOAD_FLOAT(x) \
	GET_VALUE(x) \
	tmp.x = (float)get_double(value);
	
#define LOAD_BOOL(x) \
	GET_VALUE(x) \
	tmp.x = get_bool(value);
	
#define INSERT_TMP_TO(vector) \
	if (sz <= tmp.id) \
	{ \
		sz = tmp.id + 1; \
		vector.resize(sz); \
	} \
	vector[tmp.id] = tmp;

void Params::load()
{
	TIN
		
	
	string filename;
		
	TiXmlDocument doc;
	TiXmlHandle   handle(&doc);
	TiXmlElement* child;
	int sz;
	const char *value;
	
	m3DModels.clear();

	// ----- tanks -----
	OPEN_FILE(SETTING_PARAMS_TANKS_FILE, "tanks", "tank")

	for (; child; child=child->NextSiblingElement()) 
	{
		sTankParams tmp(true);
		
		LOAD_INT(id)
		LOAD_STRING(name)
		LOAD_DOUBLE(max_speed)
		LOAD_INT(engine_power)
		LOAD_INT(hp)
		LOAD_INT(armour)
		LOAD_INT(turning)
		LOAD_DOUBLE(turning_delta_speed)
		LOAD_STRING(turning_user_info)
		LOAD_DOUBLE(weight)
		LOAD_STRING(model)
		LOAD_STRING(engine_sound)
		LOAD_INT(explosion_id)

		INSERT_TMP_TO(mTanks)
		m3DModels.push_back(tmp.model);
	}

	// ----- projectiles -----
	OPEN_FILE(SETTING_PARAMS_PROJECTILES_FILE, "projectiles", "projectile");

	for (; child; child=child->NextSiblingElement()) 
	{
		sProjectileParams tmp(true);
		
		LOAD_INT(id)
		LOAD_STRING(name)
		LOAD_STRING(model)
		LOAD_STRING(shoot_sound)
		LOAD_DOUBLE(speed)
		LOAD_DOUBLE(weight)
		LOAD_INT(damage_min)
		LOAD_INT(damage_max)
		LOAD_BOOL(piercing)
		LOAD_DOUBLE(radius)
		LOAD_INT(carry_max)
		
		LOAD_INT(explosion_hit_ground_id)
		LOAD_INT(explosion_hit_entity_id)

		INSERT_TMP_TO(mProjectiles)
		m3DModels.push_back(tmp.model);
	}

	// ----- weapons -----
	OPEN_FILE(SETTING_PARAMS_WEAPONS_FILE, "weapons", "weapon");

	for (; child; child=child->NextSiblingElement()) 
	{
		sWeaponParams tmp(true);
		
		LOAD_INT(id)
		LOAD_STRING(name)
		LOAD_STRING(model)
		LOAD_STRING(icon)
		LOAD_DOUBLE(reload_time)
		LOAD_INT(projectile)
		LOAD_INT(change_in_time)
		LOAD_INT(change_out_time)
		LOAD_DOUBLE(pitch_min)
		LOAD_DOUBLE(pitch_max)

		INSERT_TMP_TO(mWeapons)
		// m3DModels.push_back(tmp.model); // Model is not currently used
	}

	// ----- bonus -----
	OPEN_FILE(SETTING_PARAMS_BONUS_FILE, "bonuses", "bonus");

	for (; child; child=child->NextSiblingElement()) 
	{
		sBonusParams tmp(true);
		
		LOAD_INT(id)
		LOAD_STRING(name)
		LOAD_STRING(model)
		LOAD_STRING(pick_message)
		LOAD_STRING(pick_sound)
		LOAD_STRING(icon)

		LOAD_INT(duration)
		LOAD_BOOL(outlive_tank)
		LOAD_BOOL(drop_on_expire)

		// types
		LOAD_INT(HP_change)
		LOAD_INT(ammo_addition)
		LOAD_INT(ammo_type)
		LOAD_DOUBLE(max_speed)
		LOAD_DOUBLE(reload_time)
		LOAD_DOUBLE(damage)
		LOAD_INT(armour)
		LOAD_BOOL(invulnerability)
		LOAD_BOOL(invisibility)
		LOAD_INT(weapon)
		
		INSERT_TMP_TO(mBonus)
		m3DModels.push_back(tmp.model);
	}

	// ----- Landscape Entities -----
	OPEN_FILE(SETTING_PARAMS_ENTITIES_FILE, "land_entities", "entity");

	for (; child; child=child->NextSiblingElement()) 
	{
		sLandscapeEntityParams tmp(true);
		
		LOAD_INT(id)
		LOAD_STRING(name)
		LOAD_STRING(model)

		INSERT_TMP_TO(mLandscapeEntities)
		m3DModels.push_back(tmp.model);
	}
	
	// ---- explosions -----
	OPEN_FILE(SETTING_PARAMS_EXPLOSIONS_FILE, "explosions", "explosion");

	for (; child; child=child->NextSiblingElement()) 
	{
		sExplosionParams tmp(true);
		ExplosionLoader loader;
		if ( loader.loadExplosion( child, tmp ) )
		{
			INSERT_TMP_TO(mExplosions)
		}
		else
		{
			LOGE << "Can't load explosion!" << std::endl;
		}
	}

}


const Params::sTankParams& Params::tank(int id)
{
	if (id < 0 || id >= (int)mTanks.size() || mTanks[id].valid == false)
	{
		LOGE << "Wanted invalid ID for tank parameters " << id << endl;
		DIE();
	}

	return mTanks[id];
}


const Params::sProjectileParams& Params::projectile(int id)
{
	if (id < 0 || id >= (int)mProjectiles.size() || mProjectiles[id].valid == false)
	{
		LOGE << "Wanted invalid ID for projectile parameters " << id << endl;
		DIE();
	}

	return mProjectiles[id];
}


const Params::sWeaponParams& Params::weapon(int id)
{
	if (id < 0 || id >= (int)mWeapons.size() || mWeapons[id].valid == false)
	{
		LOGE << "Wanted invalid ID for weapon parameters " << id << endl;
		DIE();
	}

	return mWeapons[id];

}

const Params::sBonusParams& Params::bonus(int id)
{
	if (id < 0 || id >= (int)mBonus.size() || mBonus[id].valid == false)
	{
		LOGE << "Wanted invalid ID for bonus parameters " << id << endl;
		DIE();
	}

	return mBonus[id];
}


const Params::sLandscapeEntityParams& Params::landscapeEntity(int id)
{
	if (id < 0 || id >= (int)mLandscapeEntities.size() || mLandscapeEntities[id].valid == false)
	{
		LOGE << "Wanted invalid ID for landsacpe entity " << id << endl;
		DIE();
	}

	return mLandscapeEntities[id];
}

const Params::sExplosionParams& Params::explosion(int id)
{
	if (id < 0 || id >= (int)mExplosions.size() || mExplosions[id].valid == false)
	{
		LOGE << "Wanted invalid ID for explosion " << id << endl;
		DIE();
	}

	return mExplosions[id];
}



#define GET_VALUE_NAME( name ) \
	value = child->Attribute( name ); \
	if ( value == 0 ) \
	{ \
		LOGE << "Missing attribute " << name << " in file " << filename << endl; \
		DIE(); \
	} 

#define LOAD_STRING_NAME(x, name) \
	GET_VALUE_NAME(name) \
	x = value;

#define LOAD_DOUBLE_NAME(x, name) \
	GET_VALUE_NAME(name) \
	x = get_double(value);

#define LOAD_INT_NAME(x, name )\
	GET_VALUE_NAME(name) \
	x = get_long(value);
		
#define LOAD_BOOL_NAME(x, name) \
	GET_VALUE_NAME(name) \
	x = get_bool(value);
	
#define LOAD_UCHAR_NAME(x, name )\
	GET_VALUE_NAME(name) \
	x = get_char(value);
	

void Params::loadMapBasicInfo(const std::string path, sMapParams &map)
{
	TIN
		
	const string &filename = path;	
	TiXmlDocument doc;
	TiXmlHandle   handle(&doc);
	TiXmlElement* child;
	const char *value;

	sMapParams &tmp = map; // because macros works with tmp :-)

	// Open file
	OPEN_FILE_NOLOCATE(path, "map", "params")


	// Read the parameters from map/params

	LOAD_STRING(name)
	LOAD_STRING(loading_image)

	LOAD_STRING(game_type)
	LOAD_STRING(victory_settings)


	LOAD_INT(max_players)
	LOAD_INT(max_bonus_boxes)

	// from map/terrain
	CHILD_TO_ELEMENT2("map", "terrain")
	 LOAD_DOUBLE(gravity_constant)


	// from map/terrain/heightmap
	CHILD_TO_ELEMENT3("map", "terrain", "heightmap")
	 LOAD_STRING_NAME(map.heightmap,        "file"   )
	 LOAD_DOUBLE_NAME(map.heightmap_scalex, "scalex" )
	 LOAD_DOUBLE_NAME(map.heightmap_scaley, "scaley" )
	 LOAD_DOUBLE_NAME(map.heightmap_scalez, "scalez" )


	// from map/terrain/boundaries
	CHILD_TO_ELEMENT3("map", "terrain", "boundaries")
	 LOAD_DOUBLE_NAME(map.boundaries_min_x, "min_x")
	 LOAD_DOUBLE_NAME(map.boundaries_min_y, "min_y")
	 LOAD_DOUBLE_NAME(map.boundaries_min_z, "min_z")
	 LOAD_DOUBLE_NAME(map.boundaries_max_x, "max_x")
	 LOAD_DOUBLE_NAME(map.boundaries_max_y, "max_y")
	 LOAD_DOUBLE_NAME(map.boundaries_max_z, "max_z")

	// Loading map atributes
	loadMapTerrain(path, map.terrain);

	map.valid = true;
	
}

void Params::loadMapTerrain(const std::string path, TerrainAttributes &terrain)
{
	const string &filename = path;	
	TiXmlDocument doc;
	TiXmlHandle   handle(&doc);
	TiXmlElement* child;
	const char *value;


	// Open file
	OPEN_FILE_NOLOCATE(path, "map", "terrain")

	// lightmap
	CHILD_TO_ELEMENT3("map", "terrain", "lightmap")
	 LOAD_STRING_NAME(terrain.surface.shadowTextureFileName, "file")
	 LOAD_INT_NAME(   terrain.surface.shadowTextureExtents.width, "width")
	 LOAD_INT_NAME(   terrain.surface.shadowTextureExtents.height, "height")
 
	// globalmap
	CHILD_TO_ELEMENT3("map", "terrain", "globalmap")
	 LOAD_INT_NAME(   terrain.surface.globalTextureExtents.width,  "width")
	 LOAD_INT_NAME(   terrain.surface.globalTextureExtents.height, "height")

	// mix_map
	CHILD_TO_ELEMENT3("map", "terrain", "mix_map")
	 LOAD_INT_NAME( terrain.surface.mixmapExtents.width,  "width")
	 LOAD_INT_NAME( terrain.surface.mixmapExtents.height, "height")

	// skybox
	CHILD_TO_ELEMENT3("map", "terrain", "skybox")
	 LOAD_STRING_NAME(terrain.skybox.leftTextureFileName,   "left")
	 LOAD_STRING_NAME(terrain.skybox.rightTextureFileName,  "right")
	 LOAD_STRING_NAME(terrain.skybox.topTextureFileName,    "top")
	 LOAD_STRING_NAME(terrain.skybox.bottomTextureFileName, "bottom")
	 LOAD_STRING_NAME(terrain.skybox.frontTextureFileName,  "front")
	 LOAD_STRING_NAME(terrain.skybox.backTextureFileName,   "back")

	// layers
	// TODO: this should be elsewhere, but params destruction is obviously missing
	terrain.surface.surfaceLayers.clear();
	CHILD_TO_ELEMENT3("map", "terrain", "layers")

	 TiXmlElement *layer = child->FirstChildElement("layer");
	 for(;layer; layer = layer->NextSiblingElement())
	 {
	 	SurfaceLayerAttributes surf_layer;

		TiXmlElement *coverage = layer->FirstChildElement("coverage");
		if (!coverage) DIE("Missing map/terrain/layers/layer/coverage in map");

		// map/terrain/layers/layer/coverage/domain
		child = coverage->FirstChildElement("domain");
		if (!child) DIE("Missing map/terrain/layers/layer/coverage/domain in map");
		 LOAD_DOUBLE_NAME(surf_layer.coverage.domain.height1, "height1")
		 LOAD_DOUBLE_NAME(surf_layer.coverage.domain.height2, "height2")
		 LOAD_DOUBLE_NAME(surf_layer.coverage.domain.slope1,  "slope1")
		 LOAD_DOUBLE_NAME(surf_layer.coverage.domain.slope2,  "slope2")

		// map/terrain/layers/layer/coverage/full
		child = coverage->FirstChildElement("full");
		if (!child) DIE("Missing map/terrain/layers/layer/coverage/full in map");
		 LOAD_DOUBLE_NAME(surf_layer.coverage.full.height1, "height1")
		 LOAD_DOUBLE_NAME(surf_layer.coverage.full.height2, "height2")
		 LOAD_DOUBLE_NAME(surf_layer.coverage.full.slope1,  "slope1")
		 LOAD_DOUBLE_NAME(surf_layer.coverage.full.slope2,  "slope2")

		// map/terrain/layers/layer/noise
		child = layer->FirstChildElement("noise");
		if (!child) DIE("Missing map/terrain/layers/layer/noise in map");
		 LOAD_DOUBLE_NAME(surf_layer.noiseAmount,     "amount")
		 LOAD_STRING_NAME(surf_layer.noise.fileName,  "file")

		// map/terrain/layers/layer/color
		child = layer->FirstChildElement("color");
		if (!child) DIE("Missing map/terrain/layers/layer/color in map");
		 LOAD_UCHAR_NAME(surf_layer.color.constantColor.r,  "r")
		 LOAD_UCHAR_NAME(surf_layer.color.constantColor.g,  "g")
		 LOAD_UCHAR_NAME(surf_layer.color.constantColor.b,  "b")

		// map/terrain/layers/layer/detail
		child = layer->FirstChildElement("detail");
		if (!child) DIE("Missing map/terrain/layers/layer/detail in map");
		 LOAD_INT_NAME(surf_layer.detail.extentsInPixels.width,   "pixel_width")
		 LOAD_INT_NAME(surf_layer.detail.extentsInPixels.height,  "pixel_height")
		 LOAD_INT_NAME(surf_layer.detail.extentsInMeters.width,   "meters_width")
		 LOAD_INT_NAME(surf_layer.detail.extentsInMeters.height,  "meters_height")
		 LOAD_STRING_NAME(surf_layer.detail.fileName,             "file")

		terrain.surface.surfaceLayers.push_back(surf_layer);

	 }
}

bool Params::isProbablyMapFile(const std::string path)
{
	TiXmlDocument doc;
	TiXmlHandle   handle(&doc);

	if (!doc.LoadFile(path)) 
	{
		return false;		
	}
	
	if (!handle.FirstChild("map").Element())
	{
		return false;
	}
	
	return true;
}


