/* MAINTAINER: Petko */
#include "tank.h"
#include "log.h"
#include "globals.h"
#include "application.h"
#include "game.h"
#include "params.h"
#include "player.h"
#include "timemanager.h"
#include "bonus.h"
#include "history.h"
#include "settings.h"
#include <cmath>
#include "platform.h"
#include "globals.h"

Tank::Tank()
{
	mType	= cTank;
	mYaw	= 0;
	mPitch	= 0;
	mLastFireTime= 0;
	mDestroyTime = -1;
	mSuicide = false;
	mExplosionID = INVALID_EXPLOSION_ID;
	mDamageMultiplication = 1.0;


	mClientsMain = false;

	mWeapons     = gApp->getGameCommon().getDefaultWeapons();
	mProjectiles = gApp->getGameCommon().getDefaultProjectiles();
	mDestroyDelay= g_settings.getInt( SETTING_SIMULTAION_TANK_WRECK_LAST ) * 1000;
	mNearNoAim   = 8; //TODO: Where to get? From tank or global?
	
	mWeapon     =  INVALID_WEAPON_ID;		
	mReloadTime =  1;
	mProjectile =  INVALID_PROJECTILE_ID;

	if ( isServer() )
	{
		setNextWeaponServer();
	}
}


Tank::~Tank()
{
	// clear all bonuses
	bonusListType::iterator it = mBonuses.begin();
	while (it != mBonuses.end())
	{
		(*it)->tankDestroyed(false);
		it = mBonuses.erase(it);
	}
}

void Tank::initByID( int params_ID )
{
	
	const Params::sTankParams & vals = gParams.tank( params_ID );

	mHP		= mMaxHP	= vals.hp;
	mMaxArmour= vals.armour;
	mArmour = 0;
	assertL( mArmour <= 100 );

	mMaxSpeed	= vals.max_speed;
	mWeight		= (int)vals.weight;		//The weight of tank is huge, so it is enough to have weight as integer

	mEnginePower= vals.engine_power;
	mEngineForce= (double)mEnginePower / mMaxSpeed;

	mTurnSpeedAdd = vals.turning_delta_speed;

	mSubType	 = vals.id;

	mExplosionID = vals.explosion_id;
	
	setModelName( vals.model );
}

bool Tank::initSpecific( int player_id )
{
	mPlayer = &(gApp->getGameCommon().getPlayer( player_id ));
	mPitch	= 0;
	mYaw	= 0;
	
	// search for wep rotpoint and direction vector
	// note: This is placed here, because it is "costly" operation and it needs to get model
	//	transformations before model animation are updated.
	mWepObtainer.search( this->getModel() );

	return  true;
}

void Tank::createPresentPoint( int time_stamp )
{
	TanksHistoryPoint * present_point = new TanksHistoryPoint;

	present_point->mOrientation = mOrientation;
	present_point->mPosition	= mPosition;
	present_point->mSpeed		= mSpeed;
	present_point->mAngVelocity = mAngVelocity;

	present_point->mTimeStamp	= time_stamp;
	present_point->mID			= mID;

	present_point->mPitch		= mPitch;
	present_point->mYaw			= mYaw;
	present_point->mWeapon      = mWeapon;
	present_point->mHP          = mHP;
	present_point->mArmour      = mArmour;
	present_point->mProjectiles = getProjectiles();

	mPresentPoint = present_point;
}

void Tank::setYawPitch(double yaw, double pitch)
{
	// Don;t change yaw and pitch in destroyed tank
	if (!isAlive()) return;
	
#ifndef _DEBUG
	assertL(isnan(yaw) || isnan(pitch));
#endif
	if (isnan(yaw) || isnan(pitch)) return;
//	if (mWeapon == INVALID_WEAPON_ID) return;

	mPitch = pitch;
	mYaw = yaw;

	// check mPitch limit using weapon params
	if (mWeapon != INVALID_WEAPON_ID)
	{
		if (mPitch > gParams.weapon(mWeapon).pitch_max)
			mPitch = gParams.weapon(mWeapon).pitch_max;
		if (mPitch <  gParams.weapon(mWeapon).pitch_min)
			mPitch = gParams.weapon(mWeapon).pitch_min;
	}
}

void Tank::getShotOriAndStart(GLQuaterniond &orientation, GLVector3d &start)
{
	GLVector3d wep_displacement; // = GLVector3d( 4, 0, 8); //TODO: Tuna toto odvodit z modelu zbrane!!!
														//		ide o relativnu polohu ustia hlavne oproti (rotacnej) zakladni 
														//		zbrane

	GLVector3d wep_rotpoint; //		= GLVector3d( 0, 0, 1.5 );	//TODO: toto odvodit z modelu tanku
															//		ide o miesto, okolo ktoreho budu (okolo osi Z)
															//		rotovat zbrane

	double yaw, pitch;
	computeRelativeYawPitch(yaw, pitch);

	
	// get weapon point rotation point and displacement (direction) wector
	// note: This is "cheap" operation, mWepObtainer keeps results from last search.
	mWepObtainer.get(wep_rotpoint, wep_displacement );

	GLQuaterniond yaw_rot( GLVector3d( 0, 0, 1 ), yaw );
	yaw_rot.normalize();

	// check mPitch limit using weapon params
	// maybe this doesn't have to be here but for sure...
	if (mWeapon != INVALID_WEAPON_ID)
	{
		if (pitch > gParams.weapon(mWeapon).pitch_max)
			pitch = gParams.weapon(mWeapon).pitch_max;
		if (pitch < gParams.weapon(mWeapon).pitch_min)
			pitch = gParams.weapon(mWeapon).pitch_min;
	}
	
	GLQuaterniond pitch_rot( /*yaw_rot.rotateVector*/(ZERO_ROTATION_VECTOR).getCross( GLVector3d( 0, 0, 1 ) ) , pitch );
	pitch_rot.normalize();



	//TODO: skontrolovat poradie!!!
	//orientation = yaw_rot*pitch_rot*mOrientation;
	orientation = /*pitch_rot * */mOrientation * yaw_rot * pitch_rot;
	orientation.normalize();

	// start = orientation.rotateVector( wep_displacement + wep_rotpoint );
	// start+= mPosition;

	// tank_translation * tank_orientation * rotpoint_translation * yaw_rot * pitch_rot * displacement
	start = getOrientation().rotateVector( (yaw_rot * pitch_rot).rotateVector( wep_displacement ) + wep_rotpoint );
	start += getPosition();

	
}

void Tank::getWeaponRotationPoint(GLVector3d & rot_point)
{
	// get it from weapon
	mWepObtainer.getRotPoint(rot_point);
	
	// move it to world coortinates
	rot_point = getOrientation().rotateVector(rot_point) + getPosition();
}


int Tank::getPlayerID() const
{
	return mPlayer->getID();
}


void Tank::enableWeapon(int id)
{
	if (id == INVALID_WEAPON_ID) return;
	if (id < 0 || (unsigned int)id >= mWeapons.size()) return;
	mWeapons[id] = true;
}

void Tank::addProjectiles(int id, int number)
{
	if (id == INVALID_PROJECTILE_ID) return;
	if (id < 0 || (unsigned int)id >= mProjectiles.size()) return;
	mProjectiles[id] += number;

	if (mProjectiles[id] < 0)  mProjectiles[id] = 0;
	if (mProjectiles[id] > gParams.mProjectiles[id].carry_max) 
	{
		mProjectiles[id] = gParams.mProjectiles[id].carry_max;
	}
}

void Tank::doDamage(int damage, bool piercing)
{
	if ( !piercing )
		damage = ( damage * ( 100 - mArmour ) ) / 100 ;

	mHP-= damage;
}

void Tank::updateServer( const int timestamp )
{
	if ( getHP() > 0 )
	{
		bonusListType::iterator it = mBonuses.begin();
		while (it != mBonuses.end())
		{
			if ((*it)->testEspired(timestamp))
			{
				it = mBonuses.erase(it);
			}
			else
			{
				++it;
			}
		}
	}
	else
	{
		if ( mDestroyTime + mDestroyDelay <= timestamp )
			mDeleteMe = true;
	}
}

void Tank::interpolateHistory(int timestamp)
{
	WorldEntity::interpolateHistory( timestamp );

	if ( mPresentPoint == 0 )
	{
		return;
	}

		//try to make also TanksHistoryPoint
	TanksHistoryPoint * tp =  dynamic_cast< TanksHistoryPoint* >( mPresentPoint );
	if (tp == 0)
	{
		LOGE << "Invalid history point - invalid type" << endl;
		return; //TODO: DIE should be here
	}

		//if tank was just destroyed call destruction callback
	if ( mHP > 0 && tp->mHP <= 0 )
	{
		killedCallbackClient();
	}

	mHP     = tp->mHP;
	mArmour = tp->mArmour;
	mWeapon = tp->mWeapon;


		//we shouldn't get yaw and pitch from server if it is the main tank
	if (!mClientsMain)
	{
		mYaw    = tp->mYaw;
		mPitch  = tp->mPitch;
	}
	
	if (mWeapon >= 0 && (unsigned int)mWeapon < mWeapons.size())
	{
		mProjectiles[gParams.weapon(mWeapon).projectile] = tp->mProjectiles;
	}
}


int Tank::getProjectiles()
{
	// be more carefull about the mPresentPoint->mProjectiles
	if (mWeapon >= 0 && (unsigned int)mWeapon < mWeapons.size() && mWeapon != INVALID_WEAPON_ID )
	{
		return mProjectiles[gParams.weapon(mWeapon).projectile];
	}
	return 0;
}

double Tank::getTankYaw() const
{
	double yaw, pitch;
	computeRelativeYawPitch(yaw, pitch);

	return yaw;		
}

double Tank::getTankPitch() const
{
	double yaw, pitch;
	computeRelativeYawPitch(yaw, pitch);
	
	return pitch;
}

void Tank::computeRelativeYawPitch(double &yaw, double &pitch) const
{
		// create vector from yaw and pitch
	yaw   = mYaw *   M_PI/180.0; // go to radians
	pitch = mPitch * M_PI/180.0;
	
	GLVector3d yp(cosf((float)yaw), sinf((float)yaw), sinf((float)pitch));

	// inversly move the vector to coordinates of tank
	yp = getOrientation().getInverse().rotateVector(yp);
	yp.normalize();

	// Finally compute yaw and pitch from vector for tank
	yaw   = atan2f((float)yp.y, (float)yp.x)*180.0f/M_PI;
	pitch = asinf((float)yp.z) * 180.0f / M_PI;
}


inline float computeZinDistance(float angle, float distance, float gravity, float speed)
{
	// time when projestile reaches "distance"
	float time = distance/(speed*cosf((float)(angle*M_PI/180.0f)));
		
	// z coordinate at that time
	return sinf((float)(angle*M_PI/180.0f))*speed*time - 0.5f*gravity*time*time;
}

// returning time
// -1 if out of range
inline float tryComputeYawPitchFrom3DTarget(float dx, float dy, float dz, float speed, 
                                            float gravity, double &yaw_ret, double &pitch_ret, 
                                            float no_near_aim)
{

	
//	LOGD << dx << " " << dy << " " << dz << endl;
	
	// Yaw is easy to compute since we use only x,y coordinates
	yaw_ret   = atan2f(dy, dx)*180.0f/M_PI;
	
	// We want to compute pitch if we want to shoot in some
	// distance and make also dz
	float distanceXY = sqrtf(dx*dx + dy*dy);
	//LOGD << "distance: " << distance << endl;
	
	// Iterative cycle for finding good angle 
	float angle_max = 45;// M_PI/4; // we shoot maximally this angle - 45 degreees
	float angle_min = -80;//-M_PI/2; // we shoot minimally this angle
	
	if (computeZinDistance(45, distanceXY, gravity, speed) < dz)
	{
		angle_min = 45;
		angle_max = 85;
		
		// no way how to hit the target - it is too far and high
		if (computeZinDistance(45, distanceXY, gravity, speed) < 0)
		{
			pitch_ret = 45;
			return -1; 
		}
	}
		
	// Find the angle using binary division
	for (int i = 0; i < 30; ++i)
	{
		float angle = (angle_max+angle_min)/2.0f;	
		
		// compute Z coordinate in distance when shooting under this angle
		float dist_z = computeZinDistance(angle, distanceXY, gravity, speed);
		
		// change min/max angle
		if (dist_z > dz)
		{
			angle_max = angle;
		}
		else
		{
			angle_min = angle;	
		}
	}
	
//	mWantedPitch = angle_max*180.0f/M_PI;
	pitch_ret = angle_max;

	if (distanceXY < no_near_aim)
	{
		return -1;	
	}

	// Finally compute time when projectile reach the final target
	return distanceXY/(speed*cosf((float)(pitch_ret*M_PI/180.0f)));
}

bool Tank::setYawPitchFrom3DTarget(const GLVector3d &target, int iterations)
{
	// Don;t change yaw and pitch in destroyed tank
	if (!isAlive()) return false;	
	
	if (mWeapon == INVALID_WEAPON_ID)
	{
		mYaw = 0;
		mPitch = 0;	
		return false;
	}
	
	float speed   = (float)gParams.projectile(gParams.weapon(mWeapon).projectile).speed; 
	float gravity = (float)getApp().getGameCommon().getMapParams().gravity_constant;
						
	GLVector3d projectile_start;
	getWeaponRotationPoint(projectile_start);
		
	// let's stat with weapon rotation point
	float dx = (float)(target.x - mPosition.x);
	float dy = (float)(target.y - mPosition.y);
	float dz = (float)(target.z - mPosition.z);

	float time = 0;
	for (int i = 0; i < iterations; i++)
	{
		// Lets do first try withou taking into account tank speed
		time = tryComputeYawPitchFrom3DTarget(dx,
		                                      dy,
		                                      dz,
		                                      speed, gravity, mYaw, mPitch, mNearNoAim);
		if (time < 0)
		{
			return false;					
		}
		
		// Update to end of cannon
		GLQuaterniond orientation;
		GLVector3d start;
		getShotOriAndStart(orientation, start);
		
		dx = (float)(target.x - start.x - mSpeed.x * time);
		dy = (float)(target.y - start.y - mSpeed.y * time);
		dz = (float)(target.z - start.z - mSpeed.z * time);
	}
	
	// and let's say that second computation is accurate enough
	return true;
}


