/* MAINATINER: Martin */

/* COMMON */

#include "terrain.h"
#include "terrain_base.h"
#include "platform.h"
#include "application.h"
#include "globals.h"
#include "game.h"

Terrain::Terrain(void)
	:	mTerrainBase(0)
{}

bool Terrain::initCommon()
{
	// get access to map parameters
	Params::sMapParams &map = getApp().getGameCommon().getMapParams();

	// load heightmap
	if ( !loadHeightmap( locateFile( map.heightmap ) ) )
		return false;

	// set terrain scale ( virtual terrain extents )
	setScale(GLVector3d(map.heightmap_scalex, map.heightmap_scaley, map.heightmap_scalez));
	
	return true;
}

//------------------------------------
// WRAPPING methods
//------------------------------------

bool Terrain::loadHeightmap(const std::string fileName)
{
	return mTerrainBase.get() && mTerrainBase->loadHeightmap( fileName );
}

void Terrain::getExtents(int &x, int &y)
{
	if ( mTerrainBase.get() ) 
		mTerrainBase->getExtents(x, y);
}

double Terrain::getHeight(double x, double y)
{
	return mTerrainBase.get() ? mTerrainBase->getHeight(x, y) : 0;
}

unsigned char Terrain::getTrueHeight(int x, int y)
{
	return mTerrainBase.get() ? mTerrainBase->getTrueHeight(x, y) : 0;
}

double Terrain::getScaledHeight(int x, int y)
{
	return mTerrainBase.get() ? mTerrainBase->getScaledHeight(x, y) : 0;
}

double Terrain::getScaledMaxHeight(void)
{
	return mTerrainBase.get() ? mTerrainBase->getScaledMaxHeight() : 0;
}

GLVector3d& Terrain::getScale(GLVector3d &scale)
{
	return mTerrainBase.get() ? mTerrainBase->getScale(scale) : scale;
}

GLVector3d  Terrain::getScale(void)
{
	return mTerrainBase.get() ? mTerrainBase->getScale() : GLVector3d();
}

void Terrain::setScale(const GLVector3d &scale)
{
	if ( mTerrainBase.get() )
		mTerrainBase->setScale(scale);
}

bool Terrain::isXYValid(const GLVector3d &vtotest)
{
    GLVector3d scale;
    getScale(scale);

    return !(vtotest.x < 0 || vtotest.y < 0 || vtotest.x > scale.x || vtotest.y > scale.y);
}

void Terrain::cropToValidXY(GLVector3d &vtocrop)
{
    GLVector3d scale;
    getScale(scale);

	if (vtocrop.x < 0) vtocrop.x = 0;
	if (vtocrop.y < 0) vtocrop.y = 0;
	if (vtocrop.x > scale.x) vtocrop.x = scale.x;
	if (vtocrop.y > scale.y) vtocrop.y = scale.y;

	assertL(isXYValid(vtocrop));
}

