/* MAINTAINER: Petko */
#include "log.h"
#include "application.h"
#include "game.h"
#include "world.h"
#include "worldentity.h"
#include "globals.h"
#include "params.h"
#include "conversions.h"
#include "bonus.h"
#include "bonusbox.h"
#include "projectile.h"
#include "tank.h"

using namespace std;
using namespace STL_EXT_NS;

///TODO: delete this!!!
int run_queue = 0;


World::World()
{
	mMaxID = 1;
	mLandscape = 0;
	LOGD << "new world created" << endl;
}

World::~World()
{
	cleanUp();

	if (mLandscape)
	{
		delete mLandscape;
		mLandscape = 0;
	}

	LOGD << "World deleted" << endl;
}

void World::cleanUp()
{
	std::list< int > IDs;
	worldent_iterator n1it = getWorldEntitiesBegin();
	worldent_iterator n1ite = getWorldEntitiesEnd();
	for (; n1it != n1ite; ++n1it)
		IDs.push_back( (*n1it)->getID() );

	std::list< int >::iterator it = IDs.begin(), ite = IDs.end();
	for (; it != ite; ++it )
		deleteObjectGeneric( *it );

	mWorldList.clear();
	mWorldHash.clear();

//	mMaxID = 1; // don;t do this - it causes problems when reloading map
	mLandscape->clean();	

	LOGD << "World cleaned up" << endl;
}

void World::loadedMap()
{
	if (mLandscape) delete mLandscape;
	mLandscape = new Landscape;
	if (!mLandscape)
	{
		DIE("Load map failed");	
	}
}


BonusBox* World::createBonusBox( GLVector3d position, GLQuaterniond orientation, int bonus_function_id, int id, bool create_bonus )
{
	BonusBox * object = new BonusBox;
	assertL( object != NULL );

	Bonus * bonus_function = 0;
	if (create_bonus)
	{
		bonus_function = new Bonus;
		assertL( bonus_function != NULL );

		bonus_function->initByID( bonus_function_id );
	}

	object->initByID( bonus_function_id );

	object->basicInit( (id==0)?getNextID():id, position, orientation );

	if ( ! ( object->init() && object->initSpecific( bonus_function ) ) )
	{
		LOGW << "Failed one of object initializations!" << endl;
		
		delete object;
		if ( bonus_function )
			delete bonus_function;

		return 0;
	}

	addWorldEntity( object );
	return object;
}

BonusBox* World::createBonusBox( GLVector3d position, GLQuaterniond orientation, Bonus* bonus, int id)
{
	BonusBox * object = new BonusBox;
	assertL( object != NULL );

	object->basicInit( (id==0)?getNextID():id, position, orientation );

	if ( ! ( object->init() && object->initSpecific( bonus ) ) )
	{
		LOGW << "Failed one of object initializations!" << endl;
		delete object;
		delete bonus;

		return 0;
	}

	addWorldEntity( object );
	return object;
}


Projectile* World::createProjectile( GLVector3d position, GLQuaterniond orientation, int type_id, int shooter, int id )
{
	// TODO LUKAS log vymazat
	if(! getApp().getGameCommon().getPlayer( shooter ).getTank())
	{
		LOGD << "Projectile assert plID: " << shooter << " projID: " << id << endl;
	}

	assertL( getApp().getGameCommon().isPlayerIDValid( shooter ) );
	assertL( getApp().getGameCommon().getPlayer( shooter ).getTank() );

	Projectile * object = new Projectile;
	assertL( object != NULL );

	object->initByID( type_id );

	object->basicInit( (id==0)?getNextID():id, position, orientation );

	if ( ! ( object->init() && object->initSpecific( shooter ) ) )
	{
		LOGW << "Failed one of object initializations!" << endl;
		delete object;
		
		return 0;
	}

	addWorldEntity( object );

	object->setSpeed
	( 
		(object->getOrientation().rotateVector( ZERO_ROTATION_VECTOR ).normalize() * 
		gParams.projectile( type_id ).speed ) +
		getApp().getGameCommon().getPlayer( shooter ).getTank()->getSpeed()
	);

	return object;
}

Tank* World::createTank( GLVector3d position, GLQuaterniond orientation, int type_id, int player_id, int id )
{
	assertL( getApp().getGameCommon().isPlayerIDValid( player_id ) );

	LOGD << "Creating tank for player " << player_id << endl;

	Tank * object = new Tank;
	assertL( object != NULL );

	object->initByID( type_id );

	object->basicInit( (id==0)?getNextID():id, position, orientation );

	if ( ! ( object->init() && object->initSpecific( player_id ) ) )
	{
		LOGW << "Failed one of object initializations!" << endl;
		delete object;
		
		return 0;
	}

	getApp().getGameCommon().getPlayer(player_id).setTank( object );

	addWorldEntity( object );
	LOGD << "Creating tank for player " << player_id << " created " << endl;

	return object;
}

bool World::deleteBonusBox( int id, bool delete_bonus )
{
	WorldEntity * object = getWorldEntity( id );
	if ( object == NULL )
	{
		LOGW << "Bonus Box lookup NOT successfull!" << endl;
		return false;
	}

	if ( object->getClassType() != cBonusBox )
	{
		LOGW << "Bonus Box lookup - NOT Bonus Box!" << endl;
		return false;
	}

	BonusBox * bbox = dynamic_cast< BonusBox* >( object );

	if ( delete_bonus )
	{
		if ( bbox->getBonus() )
		{
//TODO: maybe some bonus cleanup routines here
			delete bbox->getBonus();
			bbox->setNoBonus();
		}
	}

	object->destroy();

	delWorldEntity( object );
	return true;
}

bool World::deleteProjectile( int id )
{
	WorldEntity * object = getWorldEntity( id );
	if ( object == NULL )
	{
		LOGW << "Projectile lookup NOT successfull!" << endl;
		return false;
	}

	if ( object->getClassType() != cProjectile )
	{
		LOGW << "Projectile lookup - NOT Projectile!" << endl;
		return false;
	}

	//Projectile * proj = dynamic_cast< Projectile* >( object );
	//some other stuhh here...

	
	object->destroy();

	delWorldEntity( object );
	return true;
}

bool World::deleteTank( int id )
{
	WorldEntity * object = getWorldEntity( id );
	if ( object == NULL )
	{
		LOGW << "Tank lookup NOT successfull!" << endl;
		return false;
	}

	if ( object->getClassType() != cTank )
	{
		LOGW << "Tank lookup - NOT Tank!" << endl;
		return false;
	}

	Tank * tnk = dynamic_cast< Tank* >( object );
	int pl_id = tnk->getPlayerID();

	// TODO LUKAS log vymazat
	LOGD << "Deleting tank for player "  << pl_id << " tank id " << id << endl;

	assertL( getApp().getGameCommon().isPlayerIDValid( pl_id ) );
	
		//TODO: This os only hack that needs network that do whatever it wants
	if( getApp().getGameCommon().getPlayer( pl_id ).getTank()
		&& getApp().getGameCommon().getPlayer( pl_id ).getTank()->getID() == id )
	{
		getApp().getGameCommon().getPlayer( pl_id ).setTank( 0 );
	}
	
	object->destroy();

	delWorldEntity( object );
	return true;
}

bool World::deleteObjectGeneric(int id)
{
	WorldEntity * object = getWorldEntity( id );
	if ( object == NULL )
	{
		LOGW << "Object lookup NOT successfull!" << endl;
		return false;
	}

	switch ( object->getClassType() )
	{
		case cTank:
			return deleteTank( id );
		case cProjectile:
			return deleteProjectile( id );
		case cBonusBox:
			if ( isServer() )
				return deleteBonusBox( id, true );
			else
				return deleteBonusBox( id );
		default:
			LOGW << "Object has unknown or bad type!" << endl;
			return false;
	}

	DIE("WTF??");
}

worldent_iterator World::getWorldEntitiesBegin()
{
	return mWorldList.begin();
}

worldent_iterator World::getWorldEntitiesEnd()
{
	return mWorldList.end();
}

WorldEntity* World::getWorldEntity(int id)
{
	worldent_hash::iterator it = mWorldHash.find( id );

	if ( it == mWorldHash.end() )
	{
		LOGT << "Entity with id = " << id << " not found." << endl;
		return 0;
	}
	
	if ( it->second == 0 ) 
	{
		LOGE << "Critical internal error. NULL MUST NOT BE here.\n";
		DIE();
	}
	return it->second;
}

WorldEntity* World::getWorldEntity(worldent_iterator it) const
{
	if ( it == mWorldList.end() )
	{
		LOGT << "Invalid world entity iterator found." << endl;
		return 0;
	}

	if ( *it == 0 ) 
	{
		LOGE << "Critical internal error. NULL MUST NOT BE here.\n";
		DIE();
	}
	return *it;
}

Landscape& World::getLandscape()
{
	assertL(mLandscape)
	return *mLandscape;
}

void World::addWorldEntity( WorldEntity* we )
{
#ifdef _DEBUG
	if ( mWorldHash.find( we->getID() ) != mWorldHash.end() )
	{
		LOGE << "Trying to add an object with ID already used\n";
		DIE();
	}
	if ( we->getID() <= 0 ) 
	{
		LOGE << "Try to add an object with invalid ID\n";
		DIE();
	}
#endif

	//adding to list:
	mWorldList.push_back( we );

	//adding to hashtable:
	mWorldHash[ we->getID() ] = we;

	//notifying tasks that new entity was created
	getApp().getGameCommon().getTaskManager().addedWorldObject( we );
}

void World::delWorldEntity( WorldEntity* we )
{
#ifdef _DEBUG
	if ( mWorldHash.find( we->getID() ) == mWorldHash.end() )
	{
		LOGE << "Trying to delete object with invalid ID!\n";
		DIE();
	}
#endif

	//notifying tasks that entity is going to be deleted
	getApp().getGameCommon().getTaskManager().deletedWorldObject( we );

	//delete from World containers:
	
	//delete from list:
	mWorldList.remove( we );

	//delete from hashtable:
	mWorldHash.erase( we->getID() );

	delete we;
}

void World::deletePlayersEntities(int ID)
{
	worldent_iterator it = getWorldEntitiesBegin();
	worldent_iterator ite = getWorldEntitiesEnd();
	for ( ; it != ite; ++it )
	{
		WorldEntity * we = getWorldEntity( it );
		assertL( we );

		switch ( we->getClassType() )
		{
			case cTank:
			{
				if ( dynamic_cast< Tank* >( we )->getPlayerID() == ID )
				{
					we->setDeleteFlag();
				}
				break;
			}

			case cProjectile:
			{
				if ( dynamic_cast< Projectile* >( we )->getOwner()->getID() == ID )
				{
					we->setDeleteFlag();
				}
				break;
			}
			default:
				break;
		}
	}

}

int World::getNextID()
{
	return mMaxID++;
}

World::World( const World & )
{
	DIE( "Attempt to copy the instance of World!" );
}
