#ifndef _GR_UTILS_H
#define _GR_UTILS_H

#include "SDL.h"
#include "SDL_opengl.h"
#include <algorithm>

namespace grutils
{
	class SDL_Surface_ReadOnlyAccessor 
	{
	public:
		struct SurfaceWrapper {
			SurfaceWrapper(const SDL_Surface * surf)
				: surface( surf )
			{}
			const SDL_Surface * surface;
		};
		
		SDL_Surface_ReadOnlyAccessor(const struct SDL_Surface * surface);
		SDL_Surface_ReadOnlyAccessor();

		int getWidth();
		int getHeight();
		bool isValid();
		void getColorRGBA(int x, int y, Uint8 &r, Uint8 &g, Uint8 &b, Uint8 &a) const;
		void setSurface(const SDL_Surface * surf);

	private:
		SurfaceWrapper mSurfaceWrapper;
	};

	class SDL_Surface_Accessor : public SDL_Surface_ReadOnlyAccessor
	{
	public:
		SDL_Surface_Accessor(struct SDL_Surface * surface);
		
		void lock();
		void unlock();
		void free();
		SDL_Surface * get() 
		{
			return mSurface;
		}

		void setColorRGBA(int x, int y, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
	private:
		SDL_Surface * mSurface;		
	};

Uint32 getpixel(const Uint8 * pixels, const SDL_PixelFormat * pf);
void putpixel(Uint8 * pixels, const Uint32 pixel_value, const SDL_PixelFormat * pf);
GLvoid glDrawCube();
GLvoid glDrawSphere(double radius, int slices, int stacks);

/**
*	\brief returns intensity in interval [0, 255]
*/
inline int calculateIntensity(int r, int g, int b)
{ return static_cast<Uint8>(0.299 * r + 0.587 * g + 0.114 * b + 0.5); }

template <typename T>
inline T clamp(const T low, const T high, const T value)
{
	return std::max(low, std::min(high, value));
}

} // namespace grutils

#endif // _GR_UTILS_H
