/* MAINTAINER: Bernard */

/** 
 * \file log.cpp
 *
 *
 * Implementation of class for loging.
 */

#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <iostream>
#include <ctime>
#include <sstream>
#include <fstream>
#include <iomanip>
#include "SDL.h"

#include "log.h"
#include "conversions.h"
#include "settings.h"
#include "platform.h"

using namespace std;

Log g_log;


LogAcceptor& LogAcceptor::operator<< (std::ostream& (*op)(std::ostream&))
{
	
	logLine( mLine.str() );
	mLine.str("");
	/*	
		if ( op == std::endl )
		{
			logLine( mLine.str() );
			mLine.clear();
		}
		else
		{
			op(mLine);
		}
	*/	
		return *this;
}


/*****************************************************************************/
Log::Log()
{
#ifdef _DEBUG
    m_tostderr = true;
    m_min_level = LOG_LEVEL_TRACE;
#else
    m_tostderr = false;
    m_min_level = LOG_LEVEL_INFO;
#endif

	mLogAcceptor = 0;
}


/*****************************************************************************/
/**
 * Writes "die" to syslog and exits program with EXIT_FAILURE.
 */
void Log::die()
{
	*this << "die :-(" << endl;

#ifdef _DEBUG
	assert(0);
#else
	SDL_Quit(); // maybe at least restore the resolution
	exit(EXIT_FAILURE);
#endif
}

/*****************************************************************************/
/**
 * Writes s to syslog and exits.
 */
void Log::die(const char *s)
{
	if (s == NULL) die();

	error() << s << endl;
	die();
}


/*****************************************************************************/
//  Prints message and exits
void Log::exit_failure(const char *s)
{
	exit_failure(s, errno);
}

/*****************************************************************************/
void Log::exit_failure(const char *s, int e)
{
//	error() << s << ": " << strerror(e) << endl;
	
	char buff[60];
// WHY MICROSOFT DOESN'T USE POSIX!!!	
#ifdef  _MSC_VER
	strerror_s(buff, 59, e);
#else
	strerror_r(e, buff, 59);
#endif
	
	error() << s << ": " << buff << endl;
	
	die();
}

/*****************************************************************************/
void Log::init(const char* file, bool tostderr)
{	
	// clear previous errors from << 
	m_logfile.clear();
	
	string logpath = getLogDir();
	logpath.append(file);

	m_logfile.open(logpath.c_str(), ios::out);

	if (!m_logfile) 
	{
		LOGE << "Unable to open file " << file << endl;
//		die();
	}

	m_tostderr = tostderr;
}


/*****************************************************************************/
void Log::message(int level, const char *s)
{
	string tmp = s;
	message(level, tmp);
}

/*****************************************************************************/
void Log::message(int level, std::string &s)
{
	m_level = level;
	*this << s << endl;
}


/*****************************************************************************/
void Log::perror(const char *s)
{
	int e = errno;
	//error() << s << ": " << strerror(e) << endl;
	
	char buff[60];
// WHY MICROSOFT DOESN'T USE POSIX!!!	
#ifdef  _MSC_VER
	strerror_s(buff, 59, e);
#else
	strerror_r(e, buff, 59);
#endif
	
	error() << s << ": " << buff << endl;
}


/*****************************************************************************/
Log& Log::time()
{
	struct tm   mtm;
	time_t t = std::time(NULL);   
	
	// PROC KURVA NEMUZE MIT MICROSOFT TU FUNKCI JAKO V POSIXU!!!
#ifndef _MSC_VER	
	if (localtime_r(&t, &mtm) == NULL) 
	{
#else	
	if (localtime_s(&mtm, &t) != 0) 
	{
#endif	
		// cant call die since we may be in recursion :-(
		m_level = LOG_LEVEL_ERROR;
		*this << "E localtime_r failure" << endl;
	}
	
	// writing date
	*this << (mtm.tm_year+1900) << "-" 
		  << setw(2) << setfill('0') << mtm.tm_mon + 1 << "-" 
		  << setw(2) << setfill('0') << mtm.tm_mday    << " "
		  << setw(2) << setfill('0') << mtm.tm_hour    << ":" 
		  << setw(2) << setfill('0') << mtm.tm_min     <<  ":" 
		  << setw(2) << setfill('0') << mtm.tm_sec     << " : ";
	
	return *this;
}
