#ifndef _MATRIX_TO_QUATERNION_H_
#define _MATRIX_TO_QUATERNION_H_

#include "GLQuaternion.h"
#include "GLMatrix.h"

template <class T>
GLQuaternion<T> matrixToQuaternion( const GLMatrix<T> & mat )
{
	GLQuaternion<T> quat;
	T x, y, z, w;
	T s, trace = 1 + mat[0] + mat[5] + mat[10];
	
	if ( trace > 0.00000001 )
	{
		T s = sqrt(trace) * 2;
		
		x = ( mat[6] - mat[9] ) / s;
		y = ( mat[8] - mat[2] ) / s;
		z = ( mat[1] - mat[4] ) / s;
		w = 0.25 * s;

	} else {
		if ( mat[0] > mat[5] && mat[0] > mat[10] )  
		{	// Column 0: 
			s  = sqrt( 1.0 + mat[0] - mat[5] - mat[10] ) * 2;
			x = 0.25 * s;
			y = (mat[1] + mat[4] ) / s;
			z = (mat[8] + mat[2] ) / s;
			w = (mat[9] - mat[6] ) / s;
		} 
		else if ( mat[5] > mat[10] ) 
		{	// Column 1: 
			s  = sqrt( 1.0 + mat[5] - mat[0] - mat[10] ) * 2;
			x = (mat[1] + mat[4] ) / s;
			y = 0.25 * s;
			z = (mat[6] + mat[9] ) / s;
			w = (mat[2] - mat[8] ) / s;
		} 
		else 
		{	// Column 2:
			s  = sqrt( 1.0 + mat[10] - mat[0] - mat[5] ) * 2;
			x = (mat[8] + mat[2] ) / s;
			y = (mat[6] + mat[9] ) / s;
			z = 0.25 * s;
			w = (mat[4] - mat[1] ) / s;
		}
	}
	
	quat.set(w, x, y, z);
	return quat;
}

template GLQuaternion<double> matrixToQuaternion<double>(const GLMatrix<double> & mat);


#endif
