/* MAINTAINER: Martin */
#ifndef _GOBJ_TRIMESH_H_
#define _GOBJ_TRIMESH_H_

#include "gobj.h"
#include "gobj_visitor.h"
#include "log.h"

#include "GLVector.h"
#include <vector>
#include <list>
#include "counted_ptr.h"

namespace gobj
{
	template <class C, class F>
	class CFPair
	{
	public:
		typedef C TCoord;
		typedef F TFace;
		typedef std::vector<TCoord> TCoordsContainer;
		typedef std::vector<TFace> TFacesContainer;

		TCoordsContainer coordinates;
		TFacesContainer faces;
	
		void setCoordinate(typename TCoordsContainer::size_type index, const TCoord &c)
		{
			if (index >= coordinates.size())
				coordinates.resize(index + 1);
			coordinates[index] = c;
		}
		
		void setFace(typename TFacesContainer::size_type index, const TFace &f)
		{
			if (index >= faces.size())
				faces.resize(index + 1);
			faces[index] = f;
		}

		const TCoord * getCoordinate(typename TCoordsContainer::size_type index)
		{
			return &(coordinates[index]);
		}

		const TFace * getFace(typename TCoordsContainer::size_type index)
		{
			return &(faces[index]); 
		}

	};

	class TriMesh: public GeometryObject
	{
	public:		

		TriMesh()
			//:
		{}
		
		virtual ~TriMesh(){}

		virtual void accept(Visitor * vis) const
		{ 
			vis->visitTriMesh(this);
		}

		void computeBoundingBox( BoundingBox &bb ) const
		{
			const Channel * vf = &mMeshChannel;

			Channel::TCoordsContainer::const_iterator it = vf->coordinates.begin(), ite = vf->coordinates.end();
			Channel::TCoord min, max;
			if ( it != ite )
			{
				min = *it;
				max = min;
				++it;
				for ( ; it != ite; ++it )
				{
					const Channel::TCoord &c = *it;
					for ( int i = 0; i < Channel::TCoord::D; ++i )
					{
						if ( c[i] < min[i] )
							min[i] = c[i];
						else if ( c[i] > max[i] )
							max[i] = c[i];
					}
				}
			}
			// HACK: vector dimension of 3 is supposed
			for ( int i = 0; i < 3; ++i )
			{
				bb.extents[i] = ( max[i] - min[i] ) * 0.5;
				bb.center[i] = ( max[i] + min[i] ) * 0.5;
			}
		}
		
		typedef CFPair< GLVector3f, GLVector3i > Channel;
		/*
		typedef CFPair< GLVector3f, GLVector3i > TVF;
		typedef CFPair< GLVector3f, GLVector3i > TNF;
		typedef CFPair< GLVector3f, GLVector3i > TTF;
		
		// data
		TVF vf;	///< \brief ( vertices, faces )
		TNF nf;	///< \brief ( normals, faces )
		
		// TODO: in model, there can exist more map channel with tex. coordinates than only one
		TTF tf; ///< \brief ( texcoords, faces )
		*/

		
	// -------------------- new interface -------
	public:

		Channel * getMeshChannel()
		{ return &mMeshChannel; }

		const Channel * getMeshChannel() const
		{ return &mMeshChannel; }

		Channel * getNormalsChannel()
		{ return &mNormals; }

		const Channel * getNormalsChannel() const
		{ return &mNormals; }

		Channel * getTexMappingChannel( unsigned int index )
		{ 
			if ( index >= mTexMappingChannels.size() )
				return 0;
			return mTexMappingChannels[index].get(); 
		}

		const Channel * getTexMappingChannel( unsigned int index ) const
		{ 
			if ( index >= mTexMappingChannels.size() )
				return 0;
			return mTexMappingChannels[index].get();
		}
		
		Channel * createTexMappingChannel( int index )
		{ 
			if (index < 0)
				return 0;

			if ( (unsigned int)index >= mTexMappingChannels.size() )
			{
				mTexMappingChannels.resize((unsigned int)index + 1);
			}

			return ( mTexMappingChannels[(unsigned int)index] = counted_ptr<Channel>(new Channel) ).get();
		}
	private:
		
		/** \brief vertices */
		Channel mMeshChannel;

		/** \brief normal vectors */
		Channel mNormals;

		/** \brief texture coordinates */

		//std::list< TexMapping > mTexMappingsList;
		std::vector< counted_ptr<Channel> > mTexMappingChannels;
	};
}

#endif //_GOBJ_TRIMESH_H_
