// MAINTAINER: Martin
#ifndef _MODEL_CACHE_H_
#define _MODEL_CACHE_H_

#include <vector>
#include <string>

#include "modelmanager.h"
#include "singleton.h"

/**
*	\brief Keeps model prototypes alive.
*
*	This prevents model prototypes to be released when their refcount would reache zero. This class
*	keeps one reference per cached item. Multiple caching one item has the same effect as caching
*	only once.
*/
class ModelCache: public Singleton< ModelCache >
{
	_singletonDP( ModelCache );
public:

	/**
	*	\brief "cache it, you motherfa!"
	*	Caches model of given name.
	*
	*	\param name model name
	*/
	void cacheIt( const std::string & name );

	/**
	*	\brief clear the cache
	*/
	void clear();

	/**
	*	\brief number of cached items
	*	\return number of cached items
	*/
	int count()
	{
		return static_cast<int>( mCache.size() );
	}

private:

	/** \brief type of cached items - a refcounted pointer */
	typedef ModelManager::TRef ref_type;

	/** \brief cache entries */
	std::vector< ref_type > mCache;
};

// Used fot the explicit template instantiation in template_instantces_*.cpp file
#ifdef INSTANTIATE_TEMPLATE
	template class Singleton< ModelCache >;
#endif


#endif //_MODEL_CACHE_H_
