#include "tank_debugger.h"
#include "hierarchy_node.h"
#include "model.h"
#include "hna_transform.h"

TankDebugger::TankDebugger()
{
	setSuccess( false );
}

TankDebugger::~TankDebugger()
{}

bool TankDebugger::checkModel( class Model * model )
{
	LOGI << "Model dump: " << std::endl;

	if (!model)
	{
		LOGE << " Null model pointer!" << std::endl;
	}
	else 
	{
		HierarchyNode * root( model->getRoot() );

		if (!root)
		// warning only. Null root is unusual, but maybe not a mistake
		{
			LOGW << " Null root node pointer!" << std::endl;
		}
		else
		{
			// dump hierarchy
			setTransformation( GLMatrix4d::identity() );
			root->accept( this );
		}
	}
	LOGI << "Model dump finished.";
	return true;
}

void TankDebugger::visitHierarchyNode( HierarchyNode * node )
{
	LOGI << " Node name: " << node->getName() << std::endl;
	gobj::GeometryObjectRef g = node->getGeometryObject();
	LOGI << " Geometry object: " << std::endl;
	if ( g.isValid() ) 
	{
		LOGI << "  reference count: " << g->getRefCount() << std::endl;
	}
	else
	{
		LOGI << "  none." << std::endl;
	}
}
