#include "modelmanager.h"
#include "hna_transform.h"

#include "hierarchy_node.h"
#include "gobj_all.h"
#include "model_loader.h"

#include "platform.h"

// TODO: remove this line
//#include "tank_debugger.h"


ModelManager::~ModelManager()
{
	delete mModelLoader;
	mModelLoader = 0;
}

ModelManager::TUniversalManager::TRef ModelManager::getModelPrototype(std::string name)
{
	// I. FIND
	ManagedItem * pmi;
	std::string &hashString = name;
	if ((pmi = find(hashString)) != 0)
		return pmi;

    // II. INIT
	// load the model
	
	ModelLoader ml;
	
	Model * pmodel = ml.loadModel( locateFile( name ) );

	if ( !pmodel )
		return 0;

	// Node: BJS coordinate system: z = up, x = foreward, y = left
	// Preprocessing for BJS:
	// - discard nodeTM in the root (this operation transforms the model into its object space which
	//	is defined by pivot point in the root)
	{
		class TransformationAdjuster: public hna::AttrVisitor 
		{
		public:
			virtual void visitTransformation(class hna::Transformation * t)
			{ t->setNodeTM( GLMatrix4d::identity() ); }
		} ta;
	
		pmodel->getRoot()->acceptAttributeVisitor(&ta);
	}

	// III. REGISTER
	return createAndRegister(hashString, pmodel);
}

ModelInstance::ModelInstance(ModelRef prototypeRef):
	Model(*prototypeRef)
{
	setPrototype(mPrototypeRef = prototypeRef);
}

Model * ModelManager::createModelInstance(std::string name)
{
	ModelRef mref(getModelPrototype(name));
	if (!mref.isValid())
		return 0;
	ModelInstance * inst = new ModelInstance(getModelPrototype(name));

	// TODO: remove this block
	/*
	{
		// test tank model
		TankDebugger d;
		if ( !d.checkModel( inst ) )
		{
			LOGI << "Found some error in model " << name << "." << std::endl;
			//DIE("Found some error in model!");
		}
		
	}
	*/

	return inst;
}
