/* MAINTAINER: Lukas */

#ifndef _ASYNCFUNCTION_H
#define _ASYNCFUNCTION_H

#include "asyncresult.h"
#include "syncountedptr.h"

class Client_i;

/**
 * Asynchronous call class
 *
 * T is return value
 */
template<class T>
class AsyncFunction
{
typedef AsyncResult<T> result;

public:
	typedef SynCountedPtr<result> resultCPtr;

protected:

	/**
	 * Prepares and returns AsyncResult class.
	 * If last call wasn't processed yet then blocks. Only one processing call is allowed.
	 *
	 * \return counted pointer on AsyncResult class
	 */
	resultCPtr prepareResult()
	{
		// only one call in progress allowed
		if(mAsyncResult.get())
			mAsyncResult->waitForResult();

		// new asyncResult
		mAsyncResult = resultCPtr(new result());

		return mAsyncResult;
	}

	/**
	 * Sets operation result
	 *
	 * \param res operation result
	 */
	void setResult(T * res)
	{
		mAsyncResult->setResult(res);
	}

private:
	resultCPtr mAsyncResult;
};

#endif // _ASYNCFUNCTION_H
