/* MAINTAINER: Lukas */

#ifndef _SERIALIZEMANAGER_H
#define _SERIALIZEMANAGER_H

#include "netobjects.h"
#include <list>

/**
 * Basic class for all workers
 *
 * Child of this class must register themselves with some instance of SerializeManager.
 * This is necessary for SerializeWorker(and derived calsses) proper work.
 */
class SerializeWorker
{
public:
	
	/**
	 * Constructor with priority set
	 *
	 * Lower priority -> sooner execution. Workers with same priority
	 * are executed in undefined order.
	 */
	SerializeWorker(int priority):mPriority(priority) { };
	virtual ~SerializeWorker() { };

	/**
	 * Get unpacker priority
	 *
	 * \return unpacker priority
	 */
	int getPriority() { return mPriority; };

private:

	/**
	 * Worker priority
	 */
	int mPriority;
};

/**
 * Every serialize manager has his own container where are all registred workers.
 *
 * Container is only carrier class that helps workers with easy registration.
 *
 * For more info see documentation.
 */
class SerializeContainer
{
public:
	SerializeContainer() { };
	~SerializeContainer() { };

};

class SerializeManager
{
public:
	SerializeManager() { };
	virtual ~SerializeManager() { };

	/**
	 * Sons create proper container here. Container where are all workers as attributes.
	 */
	virtual void init() = 0;

	/**
	 * Deletes container
	 */
	void clean() { delete mSContainer; };

	/**
	 * Register new worker
	 *
	 * \param newSW pointer to worker
	 */
	void registerSWorker(SerializeWorker * newSW);

	/**
	 * Unregister registred worker
	 *
	 * \param registredSW pointer to worker
	 */
	void unregisterSWorker(SerializeWorker * registredSW);

protected:
	/**
	 * Pointer to container that holds all workers. This pointer will be set in child.
	 */
	SerializeContainer * mSContainer;

	typedef std::list<SerializeWorker *> tSWorkerList;

	/**
	 * List of registred workers
	 */
	tSWorkerList mSWorkerList;

private:
	// no copy
	SerializeManager(SerializeManager &) { };
};

#endif // _SERIALIZEMANAGER_H
