/* MAINTAINER: Petko */

#include "tcpiptools.h"
#include "log.h"

#ifndef _WINDOWS
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif

namespace TcpIpTools
{

	BJS_SOCKET_TYPE socket( int type )
	{
		return ::socket( AF_INET, type, 0 );
	}

	int bind( BJS_SOCKET_TYPE sock, unsigned short port )
	{
		struct sockaddr_in sa;
		sa.sin_addr.s_addr= INADDR_ANY;
		sa.sin_port= htons( port );
		sa.sin_family= AF_INET;
		LOGD << "Going to bind socket for port " << port << endl;
		return ::bind( sock, (struct sockaddr *)&sa, sizeof sa );
	}

	int recvfrom( BJS_SOCKET_TYPE sock, char * data, int datalen, struct sockaddr_in * from )
	{
#ifdef _WINDOWS
		int sa_size = sizeof ( struct sockaddr_in );
#else
		socklen_t sa_size = sizeof ( struct sockaddr_in );
#endif
		return ::recvfrom( sock, data, datalen, 0, (struct sockaddr *)from, &sa_size );
	}

	int sendto( BJS_SOCKET_TYPE sock, const char * data, int datalen, const struct sockaddr_in * to )
	{
		return ::sendto( sock, data, datalen, 0, (struct sockaddr *)to, sizeof ( struct sockaddr_in ) );
	}

	int sendto( BJS_SOCKET_TYPE sock, const char * data, int datalen, const char * addressee, unsigned short port  )
	{
		struct hostent *ha;
		ha= gethostbyname( addressee );

		if ( !ha )
			return BJS_SOCKET_ERROR;

		struct sockaddr_in addr;
		addr.sin_family= AF_INET;
		addr.sin_port=htons(port);

		memcpy( &(addr.sin_addr.s_addr), ha->h_addr_list[0], ha->h_length );

		return TcpIpTools::sendto( sock, data, datalen, &addr );
	}

	int sendto( BJS_SOCKET_TYPE sock, const char * data, int datalen, unsigned long addressee, unsigned short port  )
	{
		struct sockaddr_in addr;
		addr.sin_family= AF_INET;
		addr.sin_port=htons(port);
		addr.sin_addr.s_addr = addressee;

		return TcpIpTools::sendto( sock, data, datalen, &addr );
	}

	int selectWrite( BJS_SOCKET_TYPE sock, int timeout )
	{
		fd_set wrfds;
		struct timeval tv;
		int retval;

		FD_ZERO(&wrfds);
		FD_SET( sock, &wrfds);

		tv.tv_sec = timeout / 1000;				//seconds
		tv.tv_usec = (timeout % 1000) * 1000;	//microseconds

		retval = select( (int)sock + 1, 0, &wrfds, 0, &tv);
		
		if ( retval == 0 )						//timeout
			return 0;		
		if ( retval == BJS_SOCKET_ERROR )		//error
			return BJS_SOCKET_ERROR;
		
		return 1;								//OK, can write to socket
	}

	int selectRead( BJS_SOCKET_TYPE sock, int timeout )
	{
		fd_set rfds;
		struct timeval tv;
		int retval;

		FD_ZERO( &rfds );
		FD_SET( sock, &rfds);

		tv.tv_sec = timeout / 1000;				//seconds
		tv.tv_usec = (timeout % 1000) * 1000;	//microseconds

		retval = select( (int)sock + 1, &rfds, 0, 0, &tv);

		if ( retval == 0 )						//timeout
			return 0;		
		if ( retval == BJS_SOCKET_ERROR )		//error
			return BJS_SOCKET_ERROR;

		return 1;								//OK, can read from socket
	}

	int setSockBroadcast( BJS_SOCKET_TYPE sock )
	{
		int val = 1;
		return ::setsockopt( sock, SOL_SOCKET, SO_BROADCAST, (char*)&val , sizeof ( int ) );
	}

	void closesocket( BJS_SOCKET_TYPE sock )
	{
#ifdef _WINDOWS
		::closesocket( sock );
#else
		::close( sock );
#endif

	}
}
