#include "noise.h"
#include "SDL.h"
#include "SDL_image.h"
#include "grutils.h"
#include <cmath>
#include <string>

ExternalNoise::ExternalNoise()
	: mImage( 0 )
{}

ExternalNoise::ExternalNoise( const std::string & imagePath )
	: mImage( 0 )
{
	load( imagePath );
}

ExternalNoise::~ExternalNoise()
{
	if ( mImage ) 
	{
		SDL_FreeSurface( mImage );
		mImage = 0;
	}
}

bool ExternalNoise::load( const std::string & imagePath )
{
	grutils::SDL_Surface_Accessor s( IMG_Load( imagePath.c_str() ) );
	if ( !s.isValid() )
		return false;

	if ( mImage ) 
	{
		SDL_FreeSurface( mImage );
		mImage = 0;
	}
	
	mImage = s.get();
	return true;
}

void ExternalNoise::getValue(double x, double y, value_type & value ) const
{
	using namespace std;

	double 
		maxx = this->getMaxXAsDouble(), // maximal x pixel coordinate of noise image
		maxy = this->getMaxYAsDouble(); // maximal y pixel coordinate of noise image

	// HACK: dirty hack
	double tmp1, tmp2;
	x *= 15;
	y *= 15;
	x = modf(x, &tmp1);
	y = modf(y, &tmp2);

	double 
		ix = maxx * x, // calculate x position in pixels, keep as a double
		iy = maxy * y; // calculate y position in pixels, keep as a double

	int 
		x1 = static_cast<int>( floor( ix ) ), // integral part on x axis
		y1 = static_cast<int>( floor( iy ) ); // integral part on y axis

	double dx = ix - floor( ix ); // floating part on x axis
	double dy = iy - floor( iy ); // floating part on y axis

	// calculate intensities for all four integral corners ( neighbours of [ix, iy] )
	double i1 = getIntensityAsDouble(x1, y1);
	double i2 = (x1 < 1) ? getIntensityAsDouble(x1 + 1, y1) : i1;
	double i3 = (y1 < 1) ? getIntensityAsDouble(x1, y1 + 1) : i1;
	double i4 = (x1 < 1 ) ? (y1 < 1) ? getIntensityAsDouble(x1 + 1, y1 + 1) : i2 : (y1 < 1) ? i3 : i1;

	// interpolate result from corner intensities ( bilinear interpolation )
	double res = 
		( ( 1 - dx ) * i1 + dx * i2 ) * ( 1 - dy ) +
		( ( 1 - dx ) * i3 + dx * i4 ) * dy;

	value = value_type( res );
}

double ExternalNoise::getMaxXAsDouble() const
{
	return mImage ? static_cast<double>( mImage->w ) : 0;
}

double ExternalNoise::getMaxYAsDouble() const
{
	return mImage ? static_cast<double>( mImage->h ) : 0;
}

double ExternalNoise::getIntensityAsDouble(int x, int y) const
{
	grutils::SDL_Surface_Accessor sac( mImage );
	if ( !sac.isValid() || ( ( x < 0 ) | ( x >= sac.getWidth() ) | ( y < 0 ) | ( y >= sac.getHeight() ) ) )
		return 0.0;

	Uint8 r, g, b, a;
	sac.getColorRGBA(x, y, r, g, b, a);
	return grutils::calculateIntensity( r, g, b ) * ( 1.0 / 255.0 );
}

