/* MAINTAINER: Bernard */

// Header wit functions that has platform specific implementation

#ifndef _PLATFORM_H
#define _PLATFORM_H

#include <string>
#include <vector>

/**
 * \brief Return path for log file.
 *
 * It should be sure that the directory exist. The path mus be returned
 * such that log file name could be appended. Like
 * "/tmp/log/"
 * \return Path for log.
 */
std::string getLogDir();

/**
 * \brief Make copy of template file to config file if doesn't exist
 *
 * If the file doesn't exist, makes a copy of default config file
 * On unix it may mean also creating a directory ~/.bjs
 * \return true always
 */
bool checkConfigFile(const std::string &file, const std::string &file_template);

/**
 * \brief Try to find file in "game directories"
 *
 * Search data and config directories for a filename. This
 * is useful if you want to open an existing file. Not to
 * create a new file.
 * \return found filename or original file if not found
 */
std::string locateFile(const std::string& file);


/**
 * \brief Start server. 
 * 
 * It is called from client/menu. All required info/data are in setting.
 * If true, second process was started. (Not sure if server was running withou problems but...)
 */
bool startServer();

/**
 * \brief List files in direcotry.
 * 
 * \return false on error and in that case file list is filled with partial results.
 */
bool listDirectory(const std::string &directory, std::vector<std::string> &filesV);


// File utils common for all platforms

#include <fstream>

/**
 * \brief test if file may be opened for reading
 *
 * \param file file to try open
 * \return true if file may be opened
 */
inline bool exist(const char* file)
{
	std::ifstream ifs(file);

	if (!ifs) return false;

	return true;
}

/**	\brief Test if specified file system object exists. Searched from current directory.
 *
 *	\param obj Object to check
 *	\return True if object exists
 */
bool fsObjectExists( const char* obj );

/**
 * \brief Some platforms are unable to do reloading of GFX setting.
 * 
 * But some are able. This can be set during compile time.
 */
extern bool canReloadGFXSetting;

/**	\brief Initializations that are platform-specific
 */
void initPlatform();


#ifdef _MSC_VER
#include <float.h>
inline int isnan(double d)
{
	return _isnan(d);
}
#endif


#endif // _PLATFORM_H
