#include "quadtree.h"

bool Quadtree::Alloc(const int width)
{
	bool *p = new bool[width*width];
	if (!p)
		return false;
	Free();
	m_matrix = p;
	m_root.width = width;
	m_root.x = m_root.y = width >> 1;
	return true;
}

void Quadtree::Free(void)
{
	if (m_matrix){
		delete [] m_matrix;
		m_matrix = 0;
		m_root.width = 0;
	}
}

bool Quadtree::get_subnodes(const QuadtreeNode & node, QuadtreeNode *nodes)
{
	int w2 = node.width >> 1;
	if (!is_subdivided(node))
		return false;
	nodes[0].width = nodes[1].width = nodes[2].width = nodes[3].width = w2 + 1;
	w2 >>= 1;
	nodes[0].x = node.x - w2; nodes[0].y = node.y + w2;
	nodes[1].x = node.x + w2; nodes[1].y = node.y + w2;
	nodes[2].x = node.x + w2; nodes[2].y = node.y - w2;
	nodes[3].x = node.x - w2; nodes[3].y = node.y - w2;
	return true;
}

bool Quadtree::get_neighbor(const neighbor_t type, const QuadtreeNode &node, QuadtreeNode &neighbor)
{
	neighbor.width = node.width;
	switch(type)
	{
	case n_left:
		neighbor.x = node.x - (node.width - 1);
		neighbor.y = node.y;
		break;
	case n_right:
		neighbor.x = node.x + (node.width - 1);
		neighbor.y = node.y;
		break;
	case n_up:
		neighbor.x = node.x;
		neighbor.y = node.y + (node.width - 1);
		break;
	case n_down:
		neighbor.x = node.x;
		neighbor.y = node.y - (node.width - 1);
		break;
	default:
		return false;
	};
	if ((neighbor.x < (neighbor.width >> 1))||
		(neighbor.y < (neighbor.width >> 1))||
		(neighbor.x >= m_root.width - (neighbor.width >> 1))||
		(neighbor.y >= m_root.width - (neighbor.width >> 1)))
		return false;
	return true;
}
