/* MANTAINER: Lukas */

#include "log.h"
#include "mutex.h"
#include "condvar.h"
#include "SDL.h"
#include "SDL_thread.h"


CondVar::CondVar() 
{
	createCondVar();
}

CondVar::CondVar(const CondVar & m) 
{
	createCondVar();
}

CondVar::~CondVar()
{
	assertL(mSDLCondVar != 0);
	SDL_DestroyCond(mSDLCondVar);
	mSDLCondVar = 0;
}

void CondVar::signal()
{
	assertL(mSDLCondVar != 0);
	if(SDL_CondSignal(mSDLCondVar) == -1)
	{
		LOGE <<  "Couldn't signal condition variable" << endl;
		DIE();
	}
}

void CondVar::broadcast()
{
	assertL(mSDLCondVar != 0);
	if(SDL_CondBroadcast(mSDLCondVar) == -1)
	{
		LOGE <<  "Couldn't broadcast condition variable" << endl;
		DIE();
	}
}

void CondVar::wait(Mutex & m)
{
	assertL(mSDLCondVar != 0);
	if(SDL_CondWait(mSDLCondVar, m.mSDLMutex) == -1)
	{
		LOGE <<  "Couldn't wait on condition variable" << endl;
		DIE();
	}
}

void CondVar::createCondVar()
{
	mSDLCondVar = SDL_CreateCond();
	if (mSDLCondVar == 0)
	{
		// NOTE that this may corrup program if called before costruction
		// of log mechanism. :)
		LOGE << "Unable to create condition variable" << endl;
		DIE();
	}
}
