/* MANTAINER: Lukas */

/*
 *  Class that adds condition variable
 */


#ifndef _CONDVAR_H
#define _CONDVAR_H

struct SDL_cond;
class Mutex;

/**
 * \brief Our implementation of condition variable.
 *
 * Implementation using SDL condition variable. If creation of condition variable
 * fail the whole program may fall badly, because the error
 * is logged. This is case of sigleton.s
 * \note If you want to reimplement condition variable be sure that it use
 *       locking from SDL since our Thread class is using SDL threads.
 *       Or change both.
 */
class CondVar
{
public:
	CondVar();
	~CondVar();
	
	/**
	 * It is usually wrong to copy the condition variable - there must be created new
	 * one.
	 *
	 * \param m is ignored
	 */
	CondVar(const CondVar &m); 

	/**
	 * It is not possible to copy condition variable.
	 *
	 * It is here because we want to use default = operators
	 * for classes that use condition variable.
	 *
	 * \param m is ignored
	 */
	CondVar& operator= (const CondVar &m)
	{
		return *this;
	}

	/**
	 * Restart one of the threads that are waiting on the condition variable.
	 */
	void signal();

	/**
	 * Restart all threads that are waiting on the condition variable.
	 */
	void broadcast();

	/**
	 * Wait on the condition variable, unlocking the provided mutex.
	 * The mutex must be locked before entering this function!
	 */
	void wait(Mutex & m);

protected:

	/**
	 * \brief Really create the condition variable.
	 * 
	 * Used internally by constructors.
	 */
	void createCondVar();

	SDL_cond * mSDLCondVar; ///< SDL condition variable. It is created in constructor and destoryed in destructor.
};


#endif // _CONDVAR_H
