/* MAINTAINER: Bernard */

/*
*  Class that adds mutex and locking
*/


#ifndef _MUTEX_H
#define _MUTEX_H

struct SDL_mutex;
class CondVar;

/**
 * \brief Our implementation of mutex.
 *
 * Implementation using SDL mutex. If creation of mutex
 * fail the whole program may fall badly, because the error
 * is logged. This is case of sigleton.s
 * Usage is esay. Just use methods lock and unlock.
 * \note If you want to reimplement Mutex be sure that it use
 *       locking from SDL since our Thread class is using SDL threads.
 *       Or change both.
 */
class Mutex
{
friend class CondVar;
public:
	Mutex(); ///< inits m_mutex and m_rwlock
	~Mutex();
	
	/**
	 * \brief m if ignored bud used for default copy contructors.
	 * 
	 * It is usually wrong to copy the mutex - there must be created new
	 * one.
	 */
	Mutex(const Mutex &m); 

	/**
	 * \brief It is not possible to copy mutex. 
	 *
	 * It is here because we want to use default = operators
	 * for classes that use mutex.
	 */
	Mutex& operator= (const Mutex &m)
	{
		return *this;
	}

	/**
	 * \brief Lock mutex
	 *
	 * Die on error.
	 */
	void lock();   
	/**
	 * \brief Unlock mutex
	 *
	 * Die on error.
	 */
	void unlock();

protected:

	/**
	 * \brief Really create the mutex.
	 * 
	 * Used internally by constructors.
	 */
	void createMutex();

	SDL_mutex *mSDLMutex; ///< SDL mutex. It is created in constructor and destoryed in destructor.
};


#endif
