/* MAINTAINER: Lukas */

#ifndef _SYNCHRONIZER_H
#define _SYNCHRONIZER_H

#include "mutex.h"
#include "condvar.h"

/**
 * Class that synchronize two threads(wait for some action)
 */
class Synchronizer
{
public:
	
	/**
	 * Init completion as you need
	 */
	Synchronizer(bool initCompletion)
	{
		mCommpleted = initCompletion;
	}

	/**
	 * Set completion to false(default)
	 */
	Synchronizer()
	{
		mCommpleted = false;
	}

	/**
	 * Indicates that action is done
	 *
	 * \return true if action is done, false otherwise
	 */
	bool isCompleted()
	{
		mMutex.lock();
		bool done = mCommpleted;
		mMutex.unlock();

		return done;
	}
	
	/**
	 * This function waits for action
	 */
	void waitForCompleting()
	{
		mMutex.lock();
		if(! mCommpleted)
			mCondVar.wait(mMutex);
		else
			mMutex.unlock();
	}
	
	/**
	 * Sets action completion
	 */
	void setCompletion()
	{
		mMutex.lock();
		mCommpleted = true;
		mMutex.unlock();
		
		mCondVar.broadcast();
	}

	/**
	 * Resets synchronizer - can wait for new action
	 */
	void reset()
	{
		mMutex.lock();
		mCommpleted = false;
		mMutex.unlock();
	}

protected:

	bool mCommpleted;
	
	Mutex mMutex;
	
	CondVar mCondVar;
};

#endif // _SYNCHRONIZER_H
