/* MAINTAINER: Bernard */

// #define WANT_TRACE

#include "SDL.h"

#include "log.h"
#include "settings.h"

#include "taskmanager.h"
#include "timemanager.h"


using namespace std;


TaskManager::TaskManager()
{
	mTaskListStamp = 0;
}

TaskManager::~TaskManager()
{
}


void TaskManager::add(Task& task)
{
	mTaskList.push_back(&task);
	mTaskListStamp++;
}

void TaskManager::remove(Task& task)
{	
	list<Task*>::iterator it;
	for ( it = mTaskList.begin(); it != mTaskList.end(); ++it)
	{
		assertL(*it != 0);
		if ( (*it) == &task)
		{
			mTaskList.erase(it);
			mTaskListStamp--;
			return;
		}
	}
}

void TaskManager::removeAll()
{
	mTaskList.clear();
}


void TaskManager::initTasks()
{
	TIN;


	// Init tasks
	list<Task*>::iterator it;
	for ( it = mTaskList.begin(); it != mTaskList.end(); ++it)
	{
		assertL(*it != 0);
		(*it)->init();
	}

	// read time. There may be some time until execution reach the main loop
	// and regular looping but it shouldn't be too long. In worst case it
	// makes first cycles faster than mindelay. But setting for "invalid" -1
	// does even faster first cycles. But there will be some statistics in future
	// depending on how much time was delayed - it may give bad time in first
	// cycles.
	mLastUpdate = gTime.getTicks();
}


void TaskManager::update(bool savecpu)
{
	TIN;

	int taskstamp = mTaskListStamp;
		
	// Call all tasks
	list<Task*>::iterator it;
	for ( it = mTaskList.begin(); it != mTaskList.end(); ++it)
	{
		assertL(*it != 0);
		(*it)->update(gTime.getTicks());
		if (taskstamp != mTaskListStamp) return; // some more things happen meanwhile
	}
		
	// This is here not to waste CPU too much :-)
	if (savecpu)
	{
		int mindelay = g_settings.getInt(SETTING_TASK_MANAGER_LOOP_MINDELAY);
		if (mindelay > 0) 
		{
			int dtime = gTime.getTicks() - mLastUpdate;
			if ( dtime < mindelay ) gTime.sleep(mindelay-dtime);
			mLastUpdate = gTime.getTicks();
		}
	}
	else
	{
			mLastUpdate = gTime.getTicks();
	}
}


void TaskManager::update(int atime)
{
	update(false);
}

void TaskManager::cleanTasks()
{
	TIN
		
	// Clean tasks
	list<Task*>::iterator it;
	for ( it = mTaskList.begin(); it != mTaskList.end(); it = mTaskList.erase(it))
	{
		assertL(*it != 0);
		(*it)->clean();
	}
}


void TaskManager::addedWorldObject(WorldEntity* we)
{
	TIN;
		
	// call added to all tasks
	list<Task*>::iterator it;
	for ( it = mTaskList.begin(); it != mTaskList.end(); ++it)
	{
		assertL(*it != 0);
		(*it)->addedWorldObject(we);
	}
}

void TaskManager::deletedWorldObject(WorldEntity* we)
{
	TIN;
		
	// call added to all tasks
	list<Task*>::iterator it;
	for ( it = mTaskList.begin(); it != mTaskList.end(); ++it)
	{
		assertL(*it != 0);
		(*it)->deletedWorldObject(we);
	}

}


