/* MAINTAINER: Bernard */

#ifndef _TASK_MANAGER_H
#define _TASK_MANAGER_H

#include "task.h"
#include <list>

class WorldEntity;

/**
 * \brief Holder of all runing tasks.
 *
 * Every runing task should be added to this task manager. The task manager
 * call update to tasks and notify them about new/deleted objects in game.
 *
 * The task manager is also Task so you may include more manager into one
 * tree. (don't do cycles :-) 
 */
class TaskManager : public Task
{
public:	
	
	TaskManager();
	~TaskManager();

	/**
	 * \brief Add task to list of tasks - as last.
	 *
	 * You should add all tasks before using anything ither from
	 * TaskManager. Multiple calls with will add task mutiple times!
	 * \param task reference to task that will be added
	 */
	void add(Task& task);

	/**
	 * \brief Remove task from task list.
	 * 
	 * If the task was added mutiple times only one is removed.
	 * \param task task to remove
	 */
	void remove(Task& task);

	/**
	 * \brief Remove all tasks from list.
	 */
	void removeAll();

	/**
	 * \brief Inicialize all tasks.
	 *
	 * Die if any failure. Must be called before any
	 * calls to update.
	 */
	void initTasks();
	
	/**
	 * \brief Clean all tasks.
	 *
	 * Die if any failure. Maybe we could be a less strict in final 
	 * version and accept errors in cleaning :-)
	 */
	void cleanTasks();
	
	/**
	 * \brief Update all tasks.
	 *
	 * This shoul be often called from Engine. Since most of all game
	 * is done here.
	 * \param savecpu if true the task manager will sleep a little
	 *        if called to often not to waste cpu
	 */
	void update(bool savecpu = true);

	/**
	 * \brief Upate callback it taskmanager is part of other task manager.
	 *
	 * Equal to update(fale)
	 * \param atime only for interface. Not used.
	 */
	virtual void update(int atime);
	
	/**
	 * \brief Notify all tasks, that entity was added.
	 *
	 * \param we pointer to newly added entity
	 */
	virtual void addedWorldObject(WorldEntity* we);

	/**
	 * \brief Notify all tastks, that entity will be deleted.
	 *
	 * \param we pointer to entity that will be deleted
	 */
	virtual void deletedWorldObject(WorldEntity* we);
	

private:	

	int mLastUpdate; /// \brief Time of last call to update().
	
	std::list<Task*>    mTaskList; ///< \brief List to hold all tasks.
	int mTaskListStamp;  ///< \brief Int which is changed every time you add/remove task.
};

#endif
