#ifndef _TERRAIN_MAP_ATTRIBUTES_H_
#define _TERRAIN_MAP_ATTRIBUTES_H_

#include <string>

template <typename T>
struct Extents
{
	T width, height;
};

struct SurfaceLayerAttributes
{
	struct CoverageFunction
	{
		struct HeightSlopeRectangle 
		{
			double height1, height2, slope1, slope2;
		};
		HeightSlopeRectangle domain;
		HeightSlopeRectangle full;
	};

	struct NoiseFunction
	{
		std::string fileName;
	};

	struct ColorFunction
	{
		struct ColorRGB
		{
			unsigned char r, g, b;
		};
		ColorRGB constantColor;
	};

	struct DetailTile
	{
		Extents<int> extentsInPixels;
		Extents<double> extentsInMeters;
		std::string fileName;
	};

	CoverageFunction	coverage;
	double				noiseAmount;
	NoiseFunction		noise;
	ColorFunction		color;
	DetailTile			detail;
};

struct TerrainAttributes 
{
	struct SurfaceAttributes
	{
		typedef std::vector< SurfaceLayerAttributes > layers_container_type;

		std::string shadowTextureFileName;
		
		Extents<int> shadowTextureExtents;
		Extents<int> globalTextureExtents;
		Extents<int> mixmapExtents;
		layers_container_type surfaceLayers;
	};

	struct SkyboxAttributes
	{
		std::string 
			leftTextureFileName,
			frontTextureFileName,
			topTextureFileName,
			bottomTextureFileName,
			rightTextureFileName,
			backTextureFileName;
	};

	SurfaceAttributes surface;
	SkyboxAttributes skybox;
};

#endif //_TERRAIN_MAP_ATTRIBUTES_H_

