/* MAINTAINER: Bernard */

// #define WANT_TRACE

#include "SDL.h"
#include "SDL_thread.h"

#include "thread.h"
#include "log.h"


Thread::Thread()
{
	mThread = 0;
	mRunning = false;
	mStop = false;
}

Thread::~Thread()
{

}


void Thread::start()
{
	mThreadMutex.lock();
	
	if (needWait()) wait();
	
	if (mThread != 0)
	{
		LOGE << "Invalid call - thread is allready running" << endl;
		DIE();
	}
	
	setStop(false);
	setRunning(true);
	mThread = SDL_CreateThread(run, (void*)this);

	if (mThread == 0)
	{
		LOGE << "Unable to create new thread" << endl;
		DIE();
	}
	
	mThreadMutex.unlock();
}


int Thread::wait()
{
	mThreadMutex.lock();
	
	if ( mThread == 0)
	{
		LOGE << "Invalid call" << endl;
		DIE();
	}
	
	setStop(true);
	
	// waiting for thread
	int rval;
	SDL_WaitThread(mThread, &rval);

	mThread = 0;
	setStop(false);
	
	mThreadMutex.unlock();

	return rval;
}
	
int Thread::run(void *threadClass)
{
	TIN;
	
	Thread* th = (Thread*)threadClass;
	int rv = th->run();

	LOGD << "Stopping thread." << endl;
//	th->setRunning(false);
	th->mRunning = false;	
	LOGD << "Thread stopped" << endl;
	
	return rv;
}
	
void Thread::setStop(bool stop /*= true*/)
{
	mStopMutex.lock();
	mStop = stop;
	mStopMutex.unlock();
}
	 
bool Thread::getStop()
{
	mStopMutex.lock();
	bool tmp = mStop;
	mStopMutex.unlock();

	return tmp;
}

bool Thread::isRunning()
{
	mRunningMutex.lock();
	bool tmp = mRunning;
	mRunningMutex.unlock();

	return tmp;
}


void Thread::setRunning(bool run)
{
	mRunningMutex.lock();
	mRunning = run;
	mRunningMutex.unlock();
}

bool Thread::needWait()
{
	bool rv;

	if (isRunning()) 
	{
		rv = false;
	}
	else
	{
		mThreadMutex.lock();
		rv = (mThread != 0);
		mThreadMutex.unlock();
	}
	return rv;
}


void Thread::reshedule()
{
	SDL_Delay(1);
}
