/* MAINTAINER: Bernard */

#include <climits>
#include "SDL.h"

#include "timemanager.h"
#include "mutex.h"
#include "log.h"

using namespace std;


TimeManager gTime;

TimeManager::TimeManager()
{
	mGameSpeed = 1.0;
	mGameTime = 0;
	mLastTick = -1;
	mPause = false;
	mPauseEnabled = false;
}

TimeManager::~TimeManager()
{
}

int TimeManager::getTicks()
{
	return (int) (SDL_GetTicks() % (unsigned int)INT_MAX);
}

void TimeManager::sleep(int ms)
{
	SDL_Delay(ms);
}

int TimeManager::getGameTicks()
{
	lock();
	if (mLastTick == -1)
	{
		mLastTick = getTicks();
	}
	else
	{
		int ticks = getTicks();
		if (ticks > mLastTick)
		{
			if (!mPause || !mPauseEnabled)
				mGameTime += (double)(ticks-mLastTick)*mGameSpeed;
			mLastTick = ticks;
		}
	}
	int gticks = (int)mGameTime;
	unlock();
	return gticks;
}

void TimeManager::setGameTicks(int new_game_ticks)
{
 	lock();
 		mGameTime = new_game_ticks;
 	unlock();
}

/*
void TimeManager::sleepGame(int game_ticks)
{
	while (game_ticks > 0)
	{

		mPauseMutex.lock(); // it passes if game isn't paused
		mPauseMutex.unlock();

		int sub=(game_ticks>100)?100:game_ticks;
		game_ticks -= sub;

		lock();
		int sleep_ticks = (int)((double)game_ticks*mGameSpeed);
		unlock();
		
			
		sleep(sleep_ticks);
	}


	mPauseMutex.lock();
	mPauseMutex.unlock();
}
*/
void TimeManager::setGameSpeed(double speed)
{
	lock();
	mGameSpeed = speed;
	unlock();
}

double TimeManager::getGameSpeed()
{
	lock();
	double rv = (mPause && mPauseEnabled)?0:mGameSpeed;
	unlock();
	return rv;
}

bool TimeManager::isPaused()
{
	lock();
	bool rv = mPause && mPauseEnabled;
	unlock();
	return rv;
}

void TimeManager::pause(bool state)
{
	lock();
	if (mPause == state)
	{
		unlock();
		return;
	}

	// to sync the time
	getGameTicks();

	if (state)
	{
		mPauseMutex.lock();
		mPause = true;
	}
	else
	{
		mPauseMutex.unlock();
		mPause = false;
	}
	unlock();
}

void TimeManager::pauseSwitch()
{
	lock();
	pause(!mPause);
	unlock();
}

void TimeManager::enablePause(bool enabled)
{
	lock();
	mPauseEnabled = enabled;
	unlock();	
}
