#include "weightfun.h"
#include "noise.h"
#include "log.h"
//#include "platform.h"

TerrainWeightFunction::TerrainWeightFunction( TerrainBase * tb )
	: heightFunction(tb), slopeFunction(tb), noiseFunction( 0 )
{
	this->addSubFunction( &heightFunction, &heightBuffer );
	this->addSubFunction( &slopeFunction, &slopeBuffer );
}

TerrainWeightFunction::value_type TerrainWeightFunction::sample(double x, double y, int sampleColumn)
{
	assertL(p.noise >= 0 && p.noise <= 1 && "Invalid parameters - have to be in [0, 1]-range!");
	
	value_type result(0);
	
	const value_type slope( slopeBuffer[sampleColumn] ), height( heightBuffer[sampleColumn] );
	
	if ( slope >= p.orect.slope1 && slope <= p.orect.slope2 && height >= p.orect.height1 && height <= p.orect.height2 )
	{
		double vslope(1), vheight(1), vnoise(1);

		if ( slope < p.irect.slope1 )
			vslope = ( slope - p.orect.slope1 ) / ( p.irect.slope1 - p.orect.slope1 );
		else if ( slope > p.irect.slope2 )
			vslope = ( p.orect.slope2 - slope ) / ( p.orect.slope2 - p.irect.slope2 );

		if ( height < p.irect.height1 )
			vheight = ( height - p.orect.height1 ) / ( p.irect.height1 - p.orect.height1 );
		else if ( height > p.irect.height2 )
			vheight = ( p.orect.height2 - height ) / ( p.orect.height2 - p.irect.height2 );

		// get global noise
		if ( noiseFunction.get() != 0 && p.noise != 0 )
			noiseFunction->getValue( x, y, vnoise);

		// blend noise with weight function - interpolate by noise amount
		double value = ((1 - p.noise) * vslope * vheight + p.noise * vnoise);
		
		assertL(value >= 0 && value <= 1 && "Invalid result - has to be in [0,1]-range!");

		result = value;
	}

	return result;
}

void TerrainWeightFunction::sampleValues(double x1, double x2, double y, double sampleX, double sampleY, std::vector<value_type>::iterator first)
{
	// sample subfunctions to buffers
	
	int nsamples = int(sampleX > 0 && x2 > x1 ? (x2 - x1) / sampleX + 1.5 : 0);
	
	ensureBuffersSize( static_cast<buffer_type::size_type>( nsamples ) );

	parent_class_type::sampleValues(x1, x2, y, sampleX, sampleY, first);

	// go through all samples and evaluate results from buffers
	double x = x1;
	for (int i = 0; i < nsamples; ++i, ++first)
	{
		*first = sample( x, y, i );
		x += sampleX;
	}
}
