#ifndef _WEIGHTFUN_H_
#define _WEIGHTFUN_H_

#include "terrain_surface.h"


class TerrainWeightFunction: public CompositeFunction2D< TerrainSurface::SurfaceLayer::WeightFunction::value_type >
{
public:

	typedef Function2D<double> NoiseFunction;
	
	struct Rectangle 
	{
		Rectangle()
			: height1(0), height2(0), slope1(0), slope2(0) {}

		Rectangle(double _height1, double _height2, double _slope1, double _slope2)
			: height1(_height1), height2(_height2), slope1(_slope1), slope2(_slope2) {}

		double height1, height2, slope1, slope2;
	};

	TerrainWeightFunction( TerrainBase * tb );
	
	void sampleValues(double x1, double x2, double y, double sampleX, double sampleY, buffer_type::iterator first);
	
	value_type sample( double x, double y, int sampleColumn );
	
	void setNoise( counted_ptr<NoiseFunction> noiseFunc )
	{
		noiseFunction = noiseFunc;
	}

	void setParameters( const Rectangle & outer, const Rectangle & inner, double noise = 0 )
	{
		p.orect = outer;
		p.irect = inner;
		p.noise  = noise;
	}

protected:

	struct Parameters 
	{
		/** \brief outer rectangle in slope-height space */
		Rectangle orect;
		
		/** \brief inner rectangle in slope-height space */
		Rectangle irect;

		/** \brief noise amount, has to be in the [0,1] range */
		double noise;
	} p;

	typedef CompositeFunction2D< TerrainSurface::SurfaceLayer::WeightFunction::value_type > parent_class_type;
	typedef TerrainSurface::SurfaceLayer::HeightFunction HeightFunction;
	typedef TerrainSurface::SurfaceLayer::SlopeFunction SlopeFunction;

	HeightFunction	heightFunction;
	SlopeFunction	slopeFunction;
	
	counted_ptr< NoiseFunction > noiseFunction;

	buffer_type heightBuffer, slopeBuffer;


	// -- parametrs --
};

#endif //_WEIGHTFUN_H_

