#include "wep_obtainer.h"
#include "hierarchy_node.h"
#include "transform_obtainer.h"
#include "model.h"
#include "GLMatrix.h"
//#include "GLVector.h"

#include <string>

const GLVector3d 
	WepObtainer::DEFAULT_ROTPOINT	 ( 0.0, 0.0, 1.5 ), 
	WepObtainer::DEFAULT_DISPLACEMENT( 8.0, 0.0, 0.0 );

WepObtainer::WepObtainer()
:	mWepNode(0), mRotpointNode(0), mRotpoint( DEFAULT_ROTPOINT ), mDisplacement( DEFAULT_DISPLACEMENT )
{}

void WepObtainer::visitHierarchyNode(class HierarchyNode * node)
{
	using namespace std;
	
	if (node == 0)
		return;

	static const string ROTPOINT_NODE_NAME("turret"), WEP_NODE_NAME("wep");
	
	// Search for node with name "cannon". Its pivot will be the rotpoint.
	// Search for node with name "wep". The ( wep - cannon ) will be the displacement vector.

	const string name = node->getName();
	if ( mRotpointNode == 0 && name == ROTPOINT_NODE_NAME )
		mRotpointNode = node;
	else if ( mWepNode == 0 && name == WEP_NODE_NAME )
		mWepNode = node;
	
	if ( !( mRotpointNode && mWepNode ) )
		visitHierarchyChildren( node );
}

bool WepObtainer::search( class Model * model )
{
	HierarchyNode * root( model ? model->getRoot() : 0 );
	if ( !root )
		return false;
	root->accept( this );

	//if ( mRotpointNode == 0 || mWepNode == 0 )
	//	return false;
	
	// trace back for transformation
	GLMatrix4d rotpointTM( GLMatrix4d::identity() ), wepTM( GLMatrix4d::identity() );
	
	// loop twice - first is the rotpoint, second wep displacement
	HierarchyNode * tmp( mRotpointNode ); 
	GLVector3d * pos( &mRotpoint );
	for (int i = 0; i < 2; ++i)
	{
		// skip invalid (probably not found) data
		if ( tmp == 0 )
			continue;

		// initial transformation ( from parent to node object ), store to 4x4 matrix 'mat'
		TransformObtainer to;
		tmp->acceptAttributeVisitor( &to );
		hna::Transformation * trans = to.get();
		GLMatrix4d mat( trans ? GLMatrix4d( trans->getNodeTM() )/* * trans->getObjectOffsetTM() */: GLMatrix4d::identity() );
		
		// advance to parent
		tmp = tmp->getParent();

		while ( tmp != 0 )
		// root wasn't still exceded
		{
			// get transformation attribute
			TransformObtainer to;
			tmp->acceptAttributeVisitor( &to );
			trans = to.get();
			
			// premultiply mat
			if ( trans )
				mat = GLMatrix4d( trans->getNodeTM() ) * mat;

			// advance to parent
			tmp = tmp->getParent();
		}

		// get the translation part of transform matrix to position (we are not interested in rotation)
		assertL( pos && "Null position pointer!" );
		pos->x = mat.m12;
		pos->y = mat.m13;
		pos->z = mat.m14;
		
		// advance to WepDisplacement
		tmp = mWepNode;
		pos = &mDisplacement;
	}

	if ( isDisplacementValid() )
		mDisplacement -= mRotpoint;

	return true;
}

void WepObtainer::get( GLVector3d & rotpoint, GLVector3d & displacement )
{
	rotpoint = mRotpoint;
	displacement = mDisplacement;
}
