#include "ai.h"
#include "log.h"
#include "application_server.h"
#include "game_server.h"
#include "commander.h"
#include "tank.h"
#include "world.h"
#include "aimanager.h"
#include "gameplaysettings.h"
#include "terrain.h"
#include "landscape.h"

AI::AI(int playerID, AIManager* manager)
{ 
	assertL(manager);
	
 	mPlayerID = playerID; 
 	mAIManager = manager;
 	mTank = 0;
}

void AI::update()
{
	Player &player = getAppServer().getGame().getPlayer(mPlayerID);
	
	//TODO: Here should be something like stop updating AI
	// if game isn't running or something like that
	
	if (getAppServer().getGame().getPlaySettings()->getStateOfGame() == WaitForPlayers)
	{
		player.ready();
	}
	
	if (player.getTank())
	{
		updateTank(player.getTank());
	}
	else
	{	
			//pick random tank
		int tank_num = rand() % (int)gParams.mTanks.size();
		getAppServer().getCommander().addCommand("/newtank " + itos( tank_num ), mPlayerID);
	}
}
	

inline bool isisNan(double x)
{
	return (x != x);
}

void AI::updateTank(Tank *tank)
{
	assertL(tank);

	mTank = tank;

// Random movement
	Tank::TankCommands &cmds = mTank->getCmds();
	switch (getMapLimits())
	{
	case 0: // extra close to border
		cmds.forward  = false;
		cmds.backward = false;
		if (!cmds.left && !cmds.right)
		{
			if (rand()%2 == 1) cmds.left = true;
			else cmds.right = true;	
		}
		cmds.left = !cmds.right;
		break;
	
	case 1:
		cmds.forward  = true;
		cmds.backward = false;
		cmds.left     = false;
		cmds.right    = false;	
		break;
		
	case 2:
		if (rand()%100 == 0) mTank->getCmds().forward = true;
		if (rand()%500 == 0) mTank->getCmds().forward = false;
	
		if (rand()%500 == 0) mTank->getCmds().left = true;
		if (rand()%500 == 0) mTank->getCmds().left = false;
		
		if (rand()%500 == 0) mTank->getCmds().right = true;
		if (rand()%500 == 0) mTank->getCmds().right = false;
		break;
	}

// sometimes chage weapon	
	if (rand()%3000 == 0)
	{
		getAppServer().getCommander().addCommand("/nextweapon", mPlayerID);
	}

// Shooting
	World &world = getAppServer().getGame().getWorld();


	int enemyEntityID = mAIManager->getNearestEnemy(mPlayerID,  tank->getPosition());

	if (enemyEntityID != INVALID_ENTITY_ID)
	{

		WorldEntity *enemyEntity = world.getWorldEntity(enemyEntityID);
		assertL(enemyEntity);

		GLVector3d target = enemyEntity->getPosition();

		//modify target coordinates:
		target += mShakeHand;

		tank->setYawPitchFrom3DTarget(target, 1);
		
		if (rand()%10 == 0) tank->getCmds().fire = false;
		if (rand()%10 == 0)  tank->getCmds().fire = true;
	}
	else
	{
		// don't waste projectiles if no enemy
		tank->getCmds().fire = false;
	}

// modify the shaking hand
	mShakeHand.x += 2.0 - (rand()%1000)/250.0;
	mShakeHand.y += 2.0 - (rand()%1000)/250.0;
	mShakeHand.z += 1.0 - (rand()%1000)/500.0;
	
	if (mShakeHand.x >  10.0) mShakeHand.x =  10;
	if (mShakeHand.x < -10.0) mShakeHand.x = -10;
	if (mShakeHand.y >  10.0) mShakeHand.y =  10;
	if (mShakeHand.y < -10.0) mShakeHand.y = -10;
	if (mShakeHand.z >  5.0) mShakeHand.z =  5;
	if (mShakeHand.z < -5.0) mShakeHand.z = -5;

	if ( isisNan(mShakeHand.x) ) mShakeHand.x = 0;
	if ( isisNan(mShakeHand.y) ) mShakeHand.y = 0;
	if ( isisNan(mShakeHand.z) ) mShakeHand.z = 0;
	
	mTank = 0;
}


int AI::getMapLimits()
{
	assertL(mTank);
	
	GLVector3d position =  mTank->getPosition();
	GLVector3d scale;
	double     angle = mTank->getXYangle(); 
	
	Terrain &terrain = getAppServer().getGame().getWorld().getLandscape().getTerrain();
//	terrain.cropToValidXY(position);
	terrain.getScale(scale);
	
	int rv = 2; // free move for tank
	double border_limit = 100;
	
	
	if (position.y < border_limit)
	{
		if (20 < angle && angle < 160) rv = 1;
		else return 0;	
	}

	if (position.x < border_limit)
	{
		if (70 <= angle && angle <= 290) return 0;
		else rv = 1;	
	}
	
	if (scale.y - position.y < border_limit)
	{
		if (200 < angle && angle < 340) return 1;
		else rv = 0;	
	}


	if (scale.x - position.x < border_limit)
	{
		if (110 <= angle && angle <= 250) rv = 1;
		else return 0;	
	}

	return rv;
}
