#include "ai2.h"


// ------------------------------------- HEAP ----------------------------------------

int AI_heap::insert(PathNode * newitem, PathNode * new_mPathPredecessor)
{
	newitem->mClosed = false;
	newitem->mPathPredecessor = new_mPathPredecessor;
	int which = mHeapArray.size();
	// LOGE << "AI " << " Inserting " << mHeapArray.size()+1 << ". item to heap [" << newitem->mX << "," << newitem->mY << "]" << endl;
	mHeapArray.push_back(newitem);
	
	// buble it up in right place

	while ((which > 0) && (mHeapArray[which]->mFullDistance < mHeapArray[(which-1) / 2]->mFullDistance))
	{
		// swap them
		std::swap(mHeapArray[which], mHeapArray[(which-1) / 2]);

		mHeapArray[which]->mHeapIndex = which;
		which = (which-1) / 2;
	}
	mHeapArray[which]->mHeapIndex = which;

	return 0;
}

int AI_heap::decrease_key(PathNode *node, PathNode * new_mPathPredecessor)
{
	int which = node->mHeapIndex;
	node->mPathPredecessor = new_mPathPredecessor;

	// buble it up in right place

	while ((which > 0) && (mHeapArray[which]->mFullDistance < mHeapArray[(which-1) / 2]->mFullDistance))
	{
		// swap them
		std::swap(mHeapArray[which], mHeapArray[(which-1) / 2]);

		mHeapArray[which]->mHeapIndex = which;
		which = (which-1) / 2;
	}
	mHeapArray[which]->mHeapIndex = which;

	return 0;
}

PathNode * AI_heap::extract_min()
{
	// extracting mHeapArray root
	PathNode *result = mHeapArray[0];
	result->mHeapIndex = -1;
	result->mClosed = true;
	
	int heapSize = mHeapArray.size();
	// LOGE << "AI " << " Extracting " << mHeapArray.size() << ". item from heap [" << result->mX << "," << result->mY << "]" << endl;

	mHeapArray[0] = mHeapArray[heapSize - 1];
	mHeapArray.pop_back();
	--heapSize;
	if (heapSize==0) return result;

	// buble it down in right place
	int which = 0;
	while (1)
	{
		int which_min=which;
		if (which*2+1 < heapSize && mHeapArray[which*2+1]->mFullDistance < mHeapArray[which_min]->mFullDistance) which_min = which*2+1;
		if (which*2+2 < heapSize && mHeapArray[which*2+2]->mFullDistance < mHeapArray[which_min]->mFullDistance) which_min = which*2+2;
		if (which!=which_min)
		{
				std::swap(mHeapArray[which], mHeapArray[which_min]);
				mHeapArray[which]->mHeapIndex = which;
		} else break;
		which=which_min;
	}
	mHeapArray[which]->mHeapIndex = which;
	return result;
}

bool AI_heap::notempty()
{
	if (! mHeapArray.empty())
		return true;
	return false;
}

// ------------------------------------- AI 2 ----------------------------------------


AI2::AI2(int playerID, AIManager* manager)
{
	assertL(manager);
	
	mPlayerID = playerID; 
	mAIManager = manager;
	mTank = 0;
	mPathEnd = GLVector3d(0, 0, 0);
	mPathCreated = false;
	// mDebug = 0;
	mSleepTime = 0;
	mMoving = false;
	pathfindingInit();

}

void AI2::update()
{
	Player &player = getAppServer().getGame().getPlayer(mPlayerID);
	
	//TODO: Here should be something like stop updating AI
	// if game isn't running or something like that
	
	if (getAppServer().getGame().getPlaySettings()->getStateOfGame() == WaitForPlayers)
	{
		player.ready();
	}
	
	if (player.getTank())
	{
		updateTank(player.getTank());
	}
	else
	{	
			//pick random tank
		int tank_num = rand() % (int)gParams.mTanks.size();
		getAppServer().getCommander().addCommand("/newtank " + itos( tank_num ), mPlayerID);
	
		// indication that mPathEnd is not valid
		// for magical reasons mPathEnd can not be initialized as wanted here
		mPathCreated = false;
		// clearing old pathfinfing data
		mPath.clear();

	}
}


void AI2::updateTank(Tank *tank)
{
	assertL(tank);

	mTank = tank;

// Random movement
	Tank::TankCommands &cmds = mTank->getCmds();
	GLVector3d position =  mTank->getPosition();
	GLVector3d PathEnd = GLVector3d(0, 0, 0);

	double border_limit = 100;

	std::list< PathNode * >::iterator it;

	if (! mPathCreated)
	{
		mPathCreated = true;
		// set mPathEnd to current position so tank is forced to find new destination
		mPathEnd = position;
	}
	if (mSleepTime != 0) {
		if (mSleepTime == 1) {

			GLVector3d scale;
			Terrain &terrain = getAppServer().getGame().getWorld().getLandscape().getTerrain();
			terrain.getScale(scale);

			// generating random position on map
			PathEnd.x = (rand()%((int)(scale.x - 2*border_limit)))+border_limit;
			PathEnd.y = (rand()%((int)(scale.y - 2*border_limit)))+border_limit;
			// mPathEnd.z = terrain.getHeight(mPathEnd.x, mPathEnd.y);

			if ( movetopoint(PathEnd) ) {
				mPathEnd = position; // if no path forced to look for new random dst
				++mSleepTime;		// try again with 1
			}
		}
		// mDebugging sleeping period
		cmds.forward  = false;
		cmds.backward = false;
		cmds.left     = false;
		cmds.right    = false;
		--mSleepTime;
	} else {
		if ( mPath.empty() )
		{
			GLVector3d MyDistnce = position - mPathEnd;
			if ( MyDistnce.length() < AI_UNINPORTANT_DISTANCE )
			{
				// have reached end of path
				LOGE << "AI " << mPlayerID << " reached destination and sleeping." << endl;
				mMoving = false;
				mSleepTime = AI_SLEEP_CONSTANT;
			} else
			{
				// not yet reached end of path
				simpleMove(mPathEnd);
			}
		} else
		{
			// skip node if near next one
			if ( mPath.size() > 1)
			{
				it = mPath.begin();
				if ((position - (*it)->mPosition).length() > (position - (*(++it))->mPosition).length())
				{
					LOGE << "AI " << mPlayerID << " skipping node. [" << mPath.front()->mX << "," << mPath.front()->mY << "]" << endl;
					mPath.pop_front();
				}
			}

			GLVector3d MyDistnce = position - mPath.front()->mPosition;
			if ( MyDistnce.length() < AI_UNINPORTANT_DISTANCE )
			{
				// have reached next node
				LOGE << "AI " << mPlayerID << " reached node. [" << mPath.front()->mX << "," << mPath.front()->mY << "]" << endl;

				mPath.pop_front();
			} else {
				simpleMove(mPath.front()->mPosition);
			}
		}
	}

// sometimes chage weapon	
	if (rand()%3000 == 0)
	{
		getAppServer().getCommander().addCommand("/nextweapon", mPlayerID);
	}

// Shooting
/*
	World &world = getAppServer().getGame().getWorld();


	int enemyEntityID = mAIManager->getNearestEnemy(mPlayerID,  tank->getPosition());

	if (enemyEntityID != INVALID_ENTITY_ID)
	{

		WorldEntity *enemyEntity = world.getWorldEntity(enemyEntityID);
		assertL(enemyEntity);

		mTarget = enemyEntity->getPosition();

		//modify target coordinates:
		mTarget += mShakeHand;

		tank->setYawPitchFrom3DTarget(mTarget, 1);
		
		if (rand()%10 == 0) tank->getCmds().fire = false;
		if (rand()%10 == 0)  tank->getCmds().fire = true;
	}
	else
	{
		// don't waste projectiles if no enemy
		tank->getCmds().fire = false;
	}

// modify the shaking hand
	mShakeHand.x += 2.0 - (rand()%1000)/250.0;
	mShakeHand.y += 2.0 - (rand()%1000)/250.0;
		if (rand()%10 == 0)  tank->getCmds().fire = true;
	}
	else
	{
		// don't waste projectiles if no enemy
		tank->getCmds().fire = false;
	}

// modify the shaking hand
	mShakeHand.x += 2.0 - (rand()%1000)/250.0;
	mShakeHand.y += 2.0 - (rand()%1000)/250.0;
		if (rand()%10 == 0)  tank->getCmds().fire = true;
	}
	else
	{
		// don't waste projectiles if no enemy
		tank->getCmds().fire = false;
	}

// modify the shaking hand
	mShakeHand.x += 2.0 - (rand()%1000)/250.0;
	mShakeHand.y += 2.0 - (rand()%1000)/250.0;
	mShakeHand.z += 1.0 - (rand()%1000)/500.0;
	
	if (mShakeHand.x >  10.0) mShakeHand.x =  10;
d()%1000)/500.0;
	
	if (mShakeHand.x >  10.0) mShakeHand.x =  10;
	if (mShakeHand.x < -10.0) mShakeHand.x = -10;
	if (mShakeHand.y >  10.0) mShakeHand.y =  10;
	if (mShakeHand.y < -10.0) mShakeHand.y = -10;
	if (mShakeHand.z >  5.0) mShakeHand.z =  5;
	if (mShakeHand.z < -5.0) mShakeHand.z = -5;
*/
	GLVector3d target = mPathEnd;
	tank->setYawPitchFrom3DTarget(target, 1);
	tank->getCmds().fire = true;


	
	mTank = 0;
}

int AI2::getNearestEnemy(double max_distance)
{
	GLVector3d position =  mTank->getPosition();
	GameServer &game = getAppServer().getGame();
	assertL(game.isPlayerIDValid(mPlayerID));

	int nearID = INVALID_ENTITY_ID;
	double nearDistance = max_distance;

	const std::vector<int>& playerIDVector =  game.getValidPlayersID();
	for (int i = 0; (unsigned int)i < playerIDVector.size(); i++)
	{
		// ignore our player and player without tank
		if (playerIDVector[i] == mPlayerID) continue;
		Player &playerToTest = game.getPlayer(playerIDVector[i]);
		if (!playerToTest.getTank()) continue;
		//TODO:
		// if (playerIsn'tEnemy()) continue;

		Tank *tank = playerToTest.getTank();
		if (!tank->isAlive()) continue; // ignore destroyed tanks
		
		GLVector3d dist3d = tank->getPosition() - position;
		
		double dist = dist3d.x*dist3d.x + dist3d.y*dist3d.y + dist3d.z*dist3d.z;
		
		if (dist < nearDistance)
		{
			nearID = tank->getID();
			nearDistance = dist;
		}
	}

	return nearID;
}


int AI2::getNearestBonus(int type ,double max_distance)
{
	GLVector3d position =  mTank->getPosition();
	World &world = getAppServer().getGame().getWorld();
	worldent_iterator worldentity_it=world.getWorldEntitiesBegin();
	worldent_iterator worldentity_end_it=world.getWorldEntitiesEnd();
	WorldEntity* worldentity = 0;
	
	int nearID = INVALID_ENTITY_ID;
	double nearDistance = max_distance;

	for ( ;worldentity_it != worldentity_end_it ; ++worldentity_it)
	{
		worldentity = world.getWorldEntity(worldentity_it);
		if ( worldentity->getClassType() == cBonusBox )
		{
			BonusBox * bonusbox = (BonusBox *)worldentity;
			if ( bonusbox->getBonus()->getID() == type ) // 6 is HP Bonus
			{
				if ( ( worldentity->getPosition() - position ).length() < nearDistance )
				{
					nearDistance = ( worldentity->getPosition() - position ).length();
					nearID = worldentity->getID();
				}
			}
		}
	}

	return nearID;
}

int AI2::getNearestLandEntity(const GLVector3d &position, double max_distance)
{
	Landscape &landscape = getAppServer().getGame().getWorld().getLandscape();
	landent_iterator landentity_it=landscape.getLandEntitiesBegin();
	landent_iterator landentity_end_it=landscape.getLandEntitiesEnd();
	LandscapeEntity* landentity = 0;
	
	int nearID = INVALID_ENTITY_ID;
	double nearDistance = max_distance;

	for ( ;landentity_it != landentity_end_it ; ++landentity_it)
	{
		landentity = landscape.getLandEntity(landentity_it);
		if ( ( landentity->getPosition() - position ).length() < nearDistance )
		{
			nearDistance = ( landentity->getPosition() - position ).length();
			nearID = landentity->getID();
		}
	}

	return nearID;
}

int AI2::update_neighbour(PathNode * ofwhich, PathNode * end, AI_heap &heap, const int addx, const int addy)
{
	int x,y;
	x = ofwhich->mX + addx;
	y = ofwhich->mY + addy;
	double temp = 0;

	// is neighbour in range?
	if (( x >= mPathNodesMaxx ) || ( x < 0 ) || ( y >= mPathNodesMaxy ) || ( y < 0 ))
		return 1;
	PathNode *neighbour = &(mPathfindingNodes[x][y]);
	// is it opened?
	if ( neighbour->mClosed ) {
		return 1;
	}
	// is mPassable ?
	if (! neighbour->mPassable ) {
		return 1;
	}
	// if neighbour is much highier than node (it is impossible or hard to climb)
	double xydistance = (neighbour->mPosition.x - ofwhich->mPosition.x)*(neighbour->mPosition.x - ofwhich->mPosition.x);
	xydistance += (neighbour->mPosition.y - ofwhich->mPosition.y)*(neighbour->mPosition.y - ofwhich->mPosition.y);
	if ( ((neighbour->mPosition.z - ofwhich->mPosition.z) > AI_INPASSABLE_PITCH*sqrt(xydistance)) || ((neighbour->mPosition.z - ofwhich->mPosition.z) < (-1)*AI_INPASSABLE_PITCH*sqrt(xydistance)) )
	{	
/*
		if  (sqrt(xydistance) > 0)
		{
		LOGE << "AI " << mPlayerID << " !!! INPASSABLE !!! pitch (" << (neighbour->position.z - ofwhich->position.z) / sqrt(xydistance) << ")" << endl;
		}
*/
		return 1;
	}

	// update him!
	if ( neighbour->mHeapIndex == -1 )
	{
		neighbour->mToDistance = ofwhich->mToDistance + (ofwhich->mPosition - neighbour->mPosition).length();
		neighbour->mFromDistance = (end->mPosition - neighbour->mPosition).length();
		neighbour->mFullDistance = neighbour->mToDistance + neighbour->mFromDistance;
		heap.insert(neighbour, ofwhich);
	} else
	{
		temp = ofwhich->mToDistance + (ofwhich->mPosition - neighbour->mPosition).length();
		if ( neighbour->mToDistance > temp )
		{
			neighbour->mToDistance = temp;
			neighbour->mFullDistance = neighbour->mToDistance + neighbour->mFromDistance;
			heap.decrease_key(neighbour, ofwhich);
		}
	}

	return 0;
}

int AI2::moveclosetopoint(GLVector3d &whereto)
{
	GLVector3d position =  mTank->getPosition();
	GLVector3d scale;
	Terrain &terrain = getAppServer().getGame().getWorld().getLandscape().getTerrain();
	terrain.getScale(scale);

	whereto.z = terrain.getHeight(whereto.x, whereto.y);

	// TODO find closest pathfinding node and navigate to it
	
	mPathEnd = whereto;

	if ( findPath(position, whereto, mPath)) {
		mPathEnd = position;
		return AI_INVALID_PATH;
	}

	return 0;
}

int AI2::movetopoint(GLVector3d &whereto)
{
	GLVector3d position =  mTank->getPosition();
	GLVector3d scale;
	Terrain &terrain = getAppServer().getGame().getWorld().getLandscape().getTerrain();
	terrain.getScale(scale);

	whereto.z = terrain.getHeight(whereto.x, whereto.y);

	mPathEnd = whereto;

	if ( findPath(position, whereto, mPath)) {
		mPathEnd = position;
		return AI_INVALID_PATH;
	}

	return 0;
}

int AI2::andmovetopoint(GLVector3d &whereto)
{
	GLVector3d position =  mTank->getPosition();
	GLVector3d scale;
	Terrain &terrain = getAppServer().getGame().getWorld().getLandscape().getTerrain();
	terrain.getScale(scale);

	whereto.z = terrain.getHeight(whereto.x, whereto.y);

	mPathEnd = whereto;

	if ( findPathadd(whereto, mPath)) {
		mPathEnd = position;
		return AI_INVALID_PATH;
	}

	return 0;
}

int AI2::findPathadd(GLVector3d &whereto, list< PathNode * > &mPath)
{
	// call findPath from end of current path to new end
	list< PathNode * > PathAdd;
	if ( mPath.size() != 0 ) {
		if ( findPath(mPath.back()->mPosition, whereto, PathAdd) == AI_INVALID_PATH) {
			// LOGE << "AI " << mPlayerID << " path invalid" << endl;
			return AI_INVALID_PATH;
		}
	} else
	{
		GLVector3d position =  mTank->getPosition();
		if ( findPath(position, whereto, PathAdd) == AI_INVALID_PATH) {
			// LOGE << "AI " << mPlayerID << " path invalid" << endl;
			return AI_INVALID_PATH;
		}
	}
	// merge path
	// LOGE << "AI " << mPlayerID << " path valid" << endl;
	mPath.splice(mPath.end(), PathAdd);
	return 0;
}

int AI2::findPath(GLVector3d &wherefrom, GLVector3d &whereto, list< PathNode * > &mPath)
{
	// create heap 
	AI_heap heap;
	mPath.clear();



	//  !!!  open all nodes  !!!
	//  by reseting (to -1) mHeapIndex (indicating that it is/was not in heap)

	double distance, firstdistance, lastdistance;
	int firstx = -1, firsty = -1, lastx = -1, lasty = -1;

	for (int i = 0; (i < mPathNodesMaxx) && (firstx == -1); ++i)
	{
		for (int j = 0; (j < mPathNodesMaxy) && (firstx == -1); ++j)
		{
			if ( mPathfindingNodes[i][j].mPassable )
			{
				lastx = firstx = i;
				lasty = firsty = j;
				firstdistance = (wherefrom - mPathfindingNodes[i][j].mPosition).length();
				lastdistance = (whereto - mPathfindingNodes[i][j].mPosition).length();
			}
		}
	}

	for (int i = 0; i < mPathNodesMaxx; ++i)
	{
		for (int j = 0; j < mPathNodesMaxy; ++j)
		{
			mPathfindingNodes[i][j].mClosed = false;
			mPathfindingNodes[i][j].mHeapIndex = -1;
			// (*nodes)[x][y].mHeapIndex = -1; // already set by apropriate functions
			if ( mPathfindingNodes[i][j].mPassable )
			{
				distance = (wherefrom - mPathfindingNodes[i][j].mPosition ).length();
				if ( distance <  firstdistance )
				{
					firstx = i;
					firsty = j;
					firstdistance = distance;
				}
				distance = (whereto - mPathfindingNodes[i][j].mPosition ).length();
				if ( distance <  lastdistance )
				{
					lastx = i;
					lasty = j;
					lastdistance = distance;
				}
			}
		}
	}

	// find first node and add it to the heap (heap is queue of opened nodes)
	// with zero distance
	PathNode * first = &(mPathfindingNodes[firstx][firsty]);
	PathNode * last = &(mPathfindingNodes[lastx][lasty]);


	// LOGE << "AI " << mPlayerID << " [" << first->mX << "," << first->mY << "]  =>> [" << last->mX << "," << last->mY << "]" << endl;

	// add first node to the heap (heap is queue of opened nodes)
	heap.insert(first, 0);


	// start searching loop (get lowest mFullDistance, is final -> end, adjust neighbours, close)
	PathNode *smallest = 0;

	while (heap.notempty())
	{
		smallest = heap.extract_min();
		if (smallest == last)
			break;

		// update all neighbours if shorter distance from smallest to them
		// mPathPredecessor is updated by heap insert
		
		// neighbour exists?
		update_neighbour(smallest, last, heap, -1, -1);
		update_neighbour(smallest, last, heap, -1, 0);
		update_neighbour(smallest, last, heap, -1, 1);
		update_neighbour(smallest, last, heap, 0, -1);
		// (0,0) not updated (guess why!?)
		update_neighbour(smallest, last, heap, 0, 1);
		update_neighbour(smallest, last, heap, 1, -1);
		update_neighbour(smallest, last, heap, 1, 0);
		update_neighbour(smallest, last, heap, 1, 1);


		// close smallest
		smallest->mHeapIndex = -1;
		// smallest->mClosed = true; // handled by heap
	}
	if (smallest == last)
	{
		// LOGE << "AI " << mPlayerID << " creating path!" << endl;
		// backtrack path, create newPath and return it
		// backtrack while NULL pointer -> last added have had no predecessor -> begining of path
		while ( smallest != 0 ) {
			mPath.push_front(smallest);
			smallest = smallest-> mPathPredecessor;
		}
		mMoving = true;
		return 0;
	} else
	{
		// LOGE << "AI " << mPlayerID << " path dont exist!" << endl;
	}

	mMoving = false;
	return AI_INVALID_PATH;
}

int AI2::pathfindingInit()
{
	LOGE << "AI " << mPlayerID << " Creating map nodes for pathfinding!" << endl;

	// Preprocess map for AI information
	GLVector3d position;
	GLVector3d scale;

	Terrain &terrain = getAppServer().getGame().getWorld().getLandscape().getTerrain();
	terrain.getScale(scale);
	PathNode new_node;
	std::vector< PathNode > empty;
	
	// limits on Node field size
	mPathNodesMaxx = (int)(scale.x / AI_NODE_DENSITY);
	mPathNodesMaxy = (int)(scale.y / AI_NODE_DENSITY);

	for (int x = 0; x <= mPathNodesMaxx; ++x)
	{
		mPathfindingNodes.push_back(empty);
		for (int y = 0; y <= mPathNodesMaxy; ++y)
		{
			// std::vector< std::vector< PathNode > > *pathfindingNodes;
			// create node
			new_node.mX = x;
			new_node.mY = y;
			new_node.mPosition.x = x*AI_NODE_DENSITY;
			new_node.mPosition.y = y*AI_NODE_DENSITY;
			new_node.mPosition.z = terrain.getHeight(x*AI_NODE_DENSITY, y*AI_NODE_DENSITY);
			new_node.mClosed = false;
			new_node.mHeapIndex = -1;
			new_node.mToDistance = new_node.mFromDistance = new_node.mFullDistance = 0;

			// test passability of terrain ( is obstacle (tree) somewhere near? )
			// and set mPassable member of new_node
			new_node.mPassable = false;
			if (getNearestLandEntity(new_node.mPosition, AI_NODE_DENSITY*AI_TREE_AVOIDANCE_CONST ) == INVALID_ENTITY_ID )
			{
				new_node.mPassable = true;
			}

			new_node.mHeapIndex = -1;

			mPathfindingNodes[x].push_back(new_node);
		}
	}

	return 0;
}

int AI2::simpleMove(GLVector3d whereto)
{
	Tank::TankCommands &cmds = mTank->getCmds();
	GLVector3d position =  mTank->getPosition();
	GLVector3d distance = whereto - position;
	double     angle = getXYangle(position, whereto);
	angle = mTank->getXYangle() - angle;
	if ( angle < 0 )
	{
		angle += 360;
	}

/*
	if (mDebug > 1000) {
		mDebug  = 0;
		LOGE << "AI " << mPlayerID << " pos: " << position.x << "," << position.y << "," << position.z << " heading:" << whereto.x << "," << whereto.y << "," << whereto.z << " "<< "angle: " << angle << endl;
	}
	++mDebug;
*/

	// MAGIC
	int magic = 0;

	if (angle > AI_UNINPORTANT_ANGLE)
	{
		if ( angle < 180 )
		{
			if ( angle < AI_SLOW_TURNING_ANGLE )
			{
				magic = 1;
			} else 
			{
				magic = 0;
			}
			cmds.right = true;
			cmds.left = false;
		} else
		{
			if ( angle > 360 - AI_SLOW_TURNING_ANGLE )
			{
				magic = 2;
			} else 
			{
				magic = 0;
			}
			cmds.right = false;
			cmds.left = true;
		}
	} else
	{
		cmds.right = false;
		cmds.left = false;
		magic = 1;

		if (mTank->getMaxSpeed() != 0)
		{
/*
			// based on turning
			if (((distance.length()*(mTank->getTurningTrackSpeedAdd()/mTank->getMaxSpeed())) > AI_MAGIC ) || ((!cmds.right) && (!cmds.left)))
			{
				magic = 1;
			} else {
				magic = 0;
			}
*/
			// based on distance
			if (distance.length() / mTank->getMaxSpeed() > AI_SLOW_APROACH_CONST)
			{
				magic = 3;
			}
		}
	}

	// 0 - 1/8 speed, 1 - 1/2 speed, 2 - 3/4 speed, 3 - full speed
	switch (magic)
	{
		case 0:
			if (((mTank->getSpeed().length() * AI_MAGIC_2) * 8) > mTank->getMaxSpeed() )
			{
				cmds.forward  = false;
			} else
			{
				cmds.forward  = true;
			}
			break;
		case 1:
			if (((mTank->getSpeed().length() * AI_MAGIC_2) * 2) > mTank->getMaxSpeed() )
			{
				cmds.forward  = false;
			} else
			{
				cmds.forward  = true;
			}
			break;
		case 2:
			if ((((mTank->getSpeed().length() * AI_MAGIC_2) * 4)/3) > mTank->getMaxSpeed() )
			{
				cmds.forward  = false;
			} else
			{
				cmds.forward  = true;
			}
			break;
		default:
			cmds.forward  = true;
			break;
	}

	return 0;
}

double getXYangle(GLVector3d first, GLVector3d second)
{
	GLVector3d orient = second - first;
	orient.z = 0;
	if (orient.x == 0 && orient.y == 0) return 0;
	orient.normalize();
	
	double rv = atan2(orient.y, orient.x)*180.0/M_PI;
	if (rv < 0) rv += 360;
	return rv;
}

