
extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}

#include "ai3.h"
#include "platform.h"


AI3::AI3(int playerID, AIManager* manager, int newtype)
{
	assertL(manager);

	type = newtype;

	mPlayerID = playerID; 
	mAIManager = manager;
	mTank = 0;
	mPathEnd = GLVector3d(0, 0, 0);
	mPathCreated = false;
	// mDebug = 0;
	mSleepTime = 0;
	mMoving = false;
	pathfindingInit();
	// LUA
	mLuaState = luaL_newstate(); // creating new lua state
	// load libs
	luaL_openlibs(mLuaState);

	switch ( type )
	{
		case 3:
		{
			if ( luaL_dofile(mLuaState, locateFile("ai3functions.lua").c_str()) == 1 )
			{
				LOGE << "AI " << mPlayerID << " Lua AI3 functions error occured! " << endl;
			}
		} break;
		case 4:
		{
			if ( luaL_dofile(mLuaState, locateFile("ai4functions.lua").c_str()) == 1 )
			{
				LOGE << "AI " << mPlayerID << " Lua AI4 functions error occured! " << endl;
			}
		} break;
		default:
		{
			if ( luaL_dofile(mLuaState, locateFile("ai3functions.lua").c_str()) == 1 )
			{
				LOGE << "AI " << mPlayerID << " Lua default functions error occured! " << endl;
			}
		} break;
	}

	// telling lua class pointer
	lua_getglobal(mLuaState, "setaipointer");
	if (! lua_isfunction(mLuaState, -1) )
	{
		lua_pop(mLuaState, 1);
		LOGE << "AI " << mPlayerID << " Lua function setaipointer is not loaded!" << endl;
	} else
	{
		lua_pushlightuserdata (mLuaState, this);
		lua_pcall(mLuaState, 1, 0, 0);
	}

	registerluafunctions(mLuaState);

}

AI3::~AI3() {
	lua_close(mLuaState);
}

void AI3::updateTank(Tank *tank)
{

	assertL(tank);

	mTank = tank;

	// World &world = getAppServer().getGame().getWorld();
	// Tank::TankCommands &cmds = mTank->getCmds();
	GLVector3d position =  mTank->getPosition();

/*
	if ( mMoving )
	{
		LOGE << "AI " << mPlayerID << " moving " << endl;
	} else
	{
		LOGE << "AI " << mPlayerID << " not! moving " << endl;
	}
*/

	if (! mPathCreated)
	{
		mPathCreated = true;
		// set mPathEnd to current position so tank is forced to find new destination
		mPathEnd = position;
	}

	// AI LOGIC
	if (mSleepTime  > 0) {
		--mSleepTime;
	} else {
		switch ( type )
		{
			case 3:
			{
				//(luaL_loadfile(L, filename) || lua_pcall(L, 0, LUA_MULTRET, 0))

				if ( luaL_dofile(mLuaState, locateFile("ai3.lua").c_str()) == 1 )
				{
					int stackSize = lua_gettop(mLuaState);
					const char* errmsg = lua_tolstring(mLuaState, stackSize, NULL);
					

					LOGE << "AI " << mPlayerID << " Lua AI3 error '" << errmsg << "' lua status: " << lua_status(mLuaState) << " occured! " << endl;
				}
			} break;
			case 4:
			{
				if ( luaL_dofile(mLuaState, locateFile("ai4.lua").c_str()) == 1 )
				{
					LOGE << "AI " << mPlayerID << " Lua AI4 error occured! " << endl;
				}
			} break;
			default:
			{
				if ( luaL_dofile(mLuaState, locateFile("ai3.lua").c_str()) == 1 )
				{
					LOGE << "AI " << mPlayerID << " Lua default error occured! " << endl;
				}
			} break;
		}
	}

// Movement by actual path choosen by AI logic
	moveByPath();
	if ( mMoving == false )
	{
		// Random movement
		Tank::TankCommands &cmds = mTank->getCmds();
		switch (getMapLimits())
		{
		case 0: // extra close to border
			cmds.forward  = false;
			cmds.backward = false;
			if (!cmds.left && !cmds.right)
			{
				if (rand()%2 == 1) cmds.left = true;
				else cmds.right = true;	
			}
			cmds.left = !cmds.right;
			break;
		
		case 1:
			cmds.forward  = true;
			cmds.backward = false;
			cmds.left     = false;
			cmds.right    = false;	
			break;
			
		case 2:
			if (rand()%100 == 0) mTank->getCmds().forward = true;
			if (rand()%500 == 0) mTank->getCmds().forward = false;
		
			if (rand()%500 == 0) mTank->getCmds().left = true;
			if (rand()%500 == 0) mTank->getCmds().left = false;
			
			if (rand()%500 == 0) mTank->getCmds().right = true;
			if (rand()%500 == 0) mTank->getCmds().right = false;
			break;
		}
	}

// sometimes chage weapon	
/*
	if (rand()%3000 == 0)
	{
		getAppServer().getCommander().addCommand("/nextweapon", mPlayerID);
	}
*/

	mTank = 0;
}

void AI3::moveByPath()
{

	GLVector3d position =  mTank->getPosition();
	std::list< PathNode * >::iterator it;

	if ( mPath.empty() )
	{
		GLVector3d MyDistnce = position - mPathEnd;
		if ( MyDistnce.length() < AI_UNINPORTANT_DISTANCE )
		{
			if (mMoving) {
				// have reached end of path
				// LOGE << "AI " << mPlayerID << " reached destination" << endl;
				mMoving = false;
			}
		} else
		{
			// not yet reached end of path
			simpleMove(mPathEnd);
		}
	} else
	{
		// skip node if near next one
		if ( mPath.size() > 1)
		{
			it = mPath.begin();
			if ((position - (*it)->mPosition).length() > (position - (*(++it))->mPosition).length())
			{
				// LOGE << "AI " << mPlayerID << " skipping node. [" << mPath.front()->mX << "," << mPath.front()->mY << "]" << endl;
				mPath.pop_front();
			}
		}
		GLVector3d MyDistnce = position - mPath.front()->mPosition;
		if ( MyDistnce.length() < AI_UNINPORTANT_DISTANCE )
		{
			// have reached next node
			// LOGE << "AI " << mPlayerID << " reached node. [" << mPath.front()->mX << "," << mPath.front()->mY << "] of " << mPath.size() << endl;
			mPath.pop_front();
		} else {
			simpleMove(mPath.front()->mPosition);
		}
	}

	return;
}

void AI3::printPath()
{
	std::list< PathNode * >::iterator it = mPath.begin();
	LOGE << "AI " << mPlayerID << "BEGIN" << endl;
	for (; it != mPath.end() ; ++it)
	{
		LOGE << "AI " << mPlayerID << " [" << (*it)->mX << "," << (*it)->mY << "]" << endl;
	}
	LOGE << "AI " << mPlayerID << "END" << endl;
	return;
}
