/* MAINTAINER: Bernard */
#include "aimanager.h"
#include "ai.h"
#include "ai2.h"
#include "ai3.h"
#include "application_server.h"
#include "game_server.h"
#include "commander.h"
#include "tank.h"
#include "settings.h"
#include "platform.h"
#include <fstream>

using namespace std;

AIManager::AIManager()
{
		//Load AI names from file
	string file = locateFile( g_settings.getString( SETTING_AI_NAMES_FILE ) );
	if ( !exist( file.c_str() ) )
	{
		LOGE << "File with names for AI players not found!" << endl;
		mAINames.push_back( "AI player" );
	}
	else
	{
		ifstream namesfile( file.c_str() );
		string line;
		while ( namesfile.good() )
		{
			getline( namesfile, line );
			mAINames.push_back( line );
		}

		if ( mAINames.size() == 0 )
		{
			LOGE << "File with names for AI players is empty!" << endl;
			mAINames.push_back( "AI player" );
		}
	}
}
	
AIManager::~AIManager()
{
	// Delete the all
	AIVector::iterator pAI = mAIVector.begin();
	while(pAI != mAIVector.end())
	{
		assertL(*pAI);
		
		delete (*pAI);
		pAI = mAIVector.erase(pAI);
	}	
}
	
void AIManager::removedPlayer(int playerID)
{
	// Find the AI...
	AIVector::iterator pAI;
	for(pAI = mAIVector.begin(); pAI != mAIVector.end(); ++pAI)
	{
		assertL(*pAI);
		// ...that control playerID
		if ((*pAI)->getControlledPlayerID() == playerID)
		{
			// and get rid of it
			delete (*pAI);
			mAIVector.erase(pAI);
			break;
		}
	}
}

void AIManager::update(int atime)
{
	GameServer &game = getAppServer().getGame();
	// Find the AI...
	AIVector::iterator pAI;
	for(pAI = mAIVector.begin(); pAI != mAIVector.end(); ++pAI)
	{
		assertL(*pAI);
		// ...that control playerID
		
		if (!game.isPlayerIDValid((*pAI)->getControlledPlayerID()))
		{
			LOGE << "Found AI which is controlling invalid player!!" << endl;
			// and get rid of it
			delete (*pAI);
			mAIVector.erase(pAI);
			break;
		}
		
		(*pAI)->update();
	}	
}

void AIManager::addAI(int type)
{
	// First create a player
	Player player;
	player.dontUpdateClient();

	int name_index = rand() % (int)mAINames.size();
	player.setName( mAINames[name_index] );
	
	int playerID = getAppServer().getGame().newPlayer(player);
	
	//TODO: Create AI is some clever way
	AI* ai = 0;
	switch (type) {
		case 0: {
			ai = new AI(playerID, this);
		} break;
		case 1: {
			ai = new AI2(playerID, this);
		} break;
		case 2: {
			ai = new AI3(playerID, this, 3);
		} break;
		case 3: {
			ai = new AI3(playerID, this, 4);
		} break;
		default: {
			ai = new AI(playerID, this);
		}
	}
	
	if (ai == 0) DIE("new AI failed");
	
	mAIVector.push_back(ai);
}


int AIManager::getNearestEnemy(int playerID, const GLVector3d &position, double max_distance)
{
	GameServer &game = getAppServer().getGame();
	assertL(game.isPlayerIDValid(playerID));

	int nearID = INVALID_ENTITY_ID;
	double nearDistance = max_distance;

	const std::vector<int>& playerIDVector =  game.getValidPlayersID();
	for (int i = 0; (unsigned int)i < playerIDVector.size(); i++)
	{
		// ignore our player and player without tank
		if (playerIDVector[i] == playerID) continue;
		Player &playerToTest = game.getPlayer(playerIDVector[i]);
		if (!playerToTest.getTank()) continue;
		//TODO:
		// if (playerIsn'tEnemy()) continue;

		Tank *tank = playerToTest.getTank();
		if (!tank->isAlive()) continue; // ignore destroyed tanks
		
		GLVector3d dist3d = tank->getPosition() - position;
		
		double dist = dist3d.x*dist3d.x + dist3d.y*dist3d.y + dist3d.z*dist3d.z;
		
		if (dist < nearDistance)
		{
			nearID = tank->getID();
			nearDistance = dist;
		}
	}

	return nearID;
}

int AIManager::getAICount()
{
	return 	(int)mAIVector.size();
}

