/* MAINTAINER: Bernard */
#ifndef _AI_MANAGER_H
#define _AI_MANAGER_H


#include "task.h"
#include "globals.h"
#include <vector>
#include <string>

class AI;

/**
 * \brief Manage all AI.
 * 
 * Means it is responsible for managing number of AI (creating new and
 * removing old) and calling update to AI.
 */
class AIManager : public Task
{
public:
	AIManager();
	
	// Delete all AI if there are still some left
	~AIManager();
	
// General public task	
	
	/**
	 * \brief Callback from game if player with ID exited the game.
	 * 
	 * The player may be cotrolled by AI so we need to check all AI
	 * and clear AI connected to this player
	 */
	void removedPlayer(int playerID);

	/**
	 * \brief Add new player to game and connect AI to it.
	 * 
	 * \param type not decided yet but probably something like this will be
	 *             here?
	 */
	void addAI(int type);

	/**
	 * \brief Call update to every AI
	 */
	virtual void update(int atime);


	/**
	 * \brief Give current number of AI in game.
	 */
	int getAICount(); 

// Interface for AI	 ? Or maybe it could be a part of AI - at least
// some of them.
	// some pathfinding?
	
	/**
	 * \brief Get neares enemy to some position.
	 *
	 * \return entiryID or INVALID_ENTITY_ID on error (like no enemy)
	 */
	int getNearestEnemy(int playerID, const GLVector3d &position, double max_distance = 1000000000);


	//int getNearestBonus()
	// getAIM()
	
	
protected:
	
	typedef std::vector<AI*> AIVector;
	
	/**
	 * \brief Vector containing all AI
	 */	
	AIVector  mAIVector;

	std::vector< std::string > mAINames;	///< \brief List of names for AI
};


#endif
