#include "luainterface.h"
#include "GLVector.h"
#include "log.h"
#include "tank.h"
#include "platform.h"

#include "ai.h"
#include "ai2.h"
#include "ai3.h"


#include <string>
#include <math.h>




int registerluafunctions(lua_State *L)
{

	// reporting messages from lua; params ( class AI3*, int x, int y)
	lua_register(L, "luaerrormsg", luaerrormsg);
	lua_register(L, "luaerrormsnd", luaerrormsgnd);
	lua_register(L, "luasay", luasay);

	// searching for path from current position to x,y; params ( class AI3*, int x, int y)
	lua_register(L, "luamoveto", luamoveto);
	lua_register(L, "luaandmoveto", luaandmoveto);
	lua_register(L, "luastopmoving", luastopmoving);// params (class AI3*)


	// others
	lua_register(L, "luafire", luafire);		// params (class AI3*)
	lua_register(L, "luastopfire", luastopfire);	// params (class AI3*)
	lua_register(L, "luasettarget", luasettarget);	// params (class AI3*, double x, double y, double z)
	lua_register(L, "luagetenemysposition", luagetenemysposition);	// params (int targetID)
	lua_register(L, "luaenemyisalive", luaenemyisalive);	// params (int targetID)
	lua_register(L, "luagetnearestenemy", luagetnearestenemy);	// params (class AI3*)
	lua_register(L, "luasettargetenemy", luasettargetenemy);	// params (class AI3*, int targetID)
	lua_register(L, "luaismoving", luaismoving);	// params (class AI3*)
	lua_register(L, "luasetweapon", luasetweapon);	// params (class AI3*, int weapon_type)

	// getting tank params
	lua_register(L, "luagetmaxarmour", luagetmaxarmour);	// params (class AI3*)
	lua_register(L, "luagetarmour", luagetarmour);	// params (class AI3*)
	lua_register(L, "luagetmaxhp", luagetmaxhp);	// params (class AI3*)
	lua_register(L, "luagethp", luagethp);	// params (class AI3*)
	lua_register(L, "luagetposition", luagetposition);	// params (class AI3*)
	lua_register(L, "luagetammo", luagetammo);	// params (class AI3*)
	lua_register(L, "luagetmaxammo", luagetmaxammo);	// params (class AI3*)


	// search for bonuses
	lua_register(L, "luagetnearestbonuspos", luagetnearestbonuspos);	// params (class AI3*)

	// weapons
	lua_register(L, "luaswitchweapon", luaswitchweapon);	// params (class AI3* ... type or what)
	lua_register(L, "luaswitchweaponrand", luaswitchweaponrand);	// params (class AI3*)

	// random  returns 0 if random in second arg numbers hits frst arg or less
	lua_register(L, "luarand", luarand);

	// system stuff
	lua_register(L, "lualocatefile", lualocatefile);

	// debug
	lua_register(L, "luaprintpath", luaprintpath); // params (class AI3*)

	return 0;
}

int luaprintpath(lua_State *L)
{
	LOGE << "AI " << "wooohaaa" << endl;
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	} else
	{
		AI3 *ai = (AI3 *)lua_touserdata(L, -1);
		ai->printPath();
	}
	return 0;
}

int luarand(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	} else if (n == 1)
	{
		int ofwhat = (int)lua_tonumber(L, -1);
		lua_pushnumber(L, rand() % ofwhat );
	} else
	{
		// AI3 *ai = (AI3 *)lua_touserdata(L, -3);
		int what = (int)lua_tonumber(L, -2);
		int ofwhat = (int)lua_tonumber(L, -1);
	
		if ( ( rand() % ofwhat ) <= what )
		{
			lua_pushnumber(L, 0);
		} else
		{
			lua_pushnumber(L, 1);
		}
	}
	return 1;
}

int luaswitchweapon(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 2 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -2);
	int type = (int)lua_tonumber(L, -1);

	ai->getmTank()->setWeaponServer( type );

	return 0;
}

int luaswitchweaponrand(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -2);

	ai->getmTank()->setWeaponServer( rand() % 4 );

	return 0;
}

int luagetnearestbonuspos(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 2 ) {
		lua_pushnumber(L, -1);		// ERROR!
		lua_pushnumber(L, -1);
		lua_pushnumber(L, -1);
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -2);
	int type = (int)lua_tonumber(L, -1);



	/* types
		0 - faster reloading
		1 - double damage
		2 - enable weapon 3 (fast shooting)
		3 - piercing bullets
		4 - better tank speed
		5 - armour
		6 - HP
		7 - basic ammo
		8 - sliz ammo	
	*/
	int worldEntityID = ai->getNearestBonus( type );

	if ( worldEntityID == INVALID_ENTITY_ID ) {
		lua_pushnumber(L, -1);		// ERROR!
		lua_pushnumber(L, -1);		// ERROR!
		lua_pushnumber(L, -1);		// ERROR!
		return 3;
	}

	GLVector3d pos = getAppServer().getGame().getWorld().getWorldEntity(worldEntityID)->getPosition();
						// OK!

	// LOGE << "AI " << " BONUS:" << pos.x << " " << pos.y << " " << pos.z << " " << endl;

	lua_pushnumber(L, pos.x);
	lua_pushnumber(L, pos.y);
	lua_pushnumber(L, pos.z);
	return 3;
}


int luagetposition(lua_State *L)
{
	// LOGE << "AI " << " luagetposition" << endl;
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	GLVector3d pos = ai->getmTank()->getPosition();

						// OK!
	lua_pushnumber(L, pos.x);
	lua_pushnumber(L, pos.y);
	lua_pushnumber(L, pos.z);
	return 3;
}

int luagetmaxammo(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	int maxprojectiles = gParams.mProjectiles[ ai->getmTank()->getWeapon() ].carry_max;

	lua_pushnumber(L, maxprojectiles);		// OK!
	return 1;
}

int luagetammo(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	int projectiles = ai->getmTank()->getProjectiles();

	lua_pushnumber(L, projectiles);		// OK!
	return 1;
}

int luagetmaxarmour(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	int maxhp = ai->getmTank()->getMaxArmour();

	lua_pushnumber(L, maxhp);		// OK!
	return 1;
}

int luagetarmour(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	int maxhp = ai->getmTank()->getArmour();

	lua_pushnumber(L, maxhp);		// OK!
	return 1;
}

int luagetmaxhp(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	int maxhp = ai->getmTank()->getMaxHP();

	lua_pushnumber(L, maxhp);		// OK!
	return 1;
}

int luagethp(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	int hp = ai->getmTank()->getHP();

	lua_pushnumber(L, hp);		// OK!
	return 1;
}

int luastopmoving(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	ai->getmPath()->clear();

	return 0;
}


int luamoveto(lua_State *L)
{
	// LOGE << "AI " << " luamoveto" << endl;
	int n = lua_gettop(L);
	if ( n < 3 ) {
		lua_pushnumber(L, 1);		// ERROR!
		return 1;
	}

	Terrain &terrain = getAppServer().getGame().getWorld().getLandscape().getTerrain();
	AI3 *ai = (AI3 *)lua_touserdata(L, -3);
	GLVector3d newPathEnd;

	newPathEnd.x = (double)lua_tonumber(L, -2);
	newPathEnd.y = (double)lua_tonumber(L, -1);
	newPathEnd.z = terrain.getHeight(newPathEnd.x, newPathEnd.y);

	if ( ai->movetopoint(newPathEnd) ) {
		lua_pushnumber(L, 1);		// ERROR!
	} else {
		lua_pushnumber(L, 0);		// OK!
	}

	return 1;
}

int luaandmoveto(lua_State *L)
{
	int n = lua_gettop(L);
	if ( n < 3 ) {
		lua_pushnumber(L, 1);		// ERROR!
		return 1;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -3);
	GLVector3d newPathEnd;

	newPathEnd.x = (double)lua_tonumber(L, -2);
	newPathEnd.y = (double)lua_tonumber(L, -1);

	if ( ai->andmovetopoint(newPathEnd) ) {
		lua_pushnumber(L, 1);		// ERROR!
	} else {
		lua_pushnumber(L, 0);		// OK!
	}

	return 1;
}

int luagetnearestenemy(lua_State *L)
{
	// LOGE << "AI " << " luagetnearestenemy" << endl;
	int n = lua_gettop(L);
	if ( n < 1 ) {
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}
	AI3 *ai = (AI3 *)lua_touserdata(L, -1);
	Tank * tank = ai->getmTank();
	int enemy = ai->getmAIManager()->getNearestEnemy(ai->getmPlayerID(),  tank->getPosition());
	if (enemy == -1)
	{
		lua_pushnumber(L, -1);		// ERROR!
		return 1;
	}


	// pushing C int onto lua stack
	int * userdata = (int *)lua_newuserdata(L, sizeof(int));
	(*userdata) = enemy;
	
	return 1;
}

int luaenemyisalive(lua_State *L)
{
	int n = lua_gettop(L);
	if (n < 1)
	{
		return 0;	// false
	}

	int userdata = *((int *)lua_touserdata(L, -1));
	if (userdata == INVALID_ENTITY_ID)
	{
		return 0;	// false
	}

	Tank *ent =  (Tank *)getAppServer().getGame().getWorld().getWorldEntity(userdata);
	if ( ent == NULL )
	{
		return 0;	// false
	}
	if ( ent->isAlive() )
	{
		lua_pushnumber(L, 1);	// true
		return 1;
	} else
	{
		return 0;	// false
	}
}

int luagetenemysposition(lua_State *L)
{
	// LOGE << "AI " << " luagetenemysposition" << endl;
	int n = lua_gettop(L);
	if (n < 1)
	{
		lua_pushnumber(L, -1);
		return 1;
	}

	int userdata = *((int *)lua_touserdata(L, -1));
	if (userdata == INVALID_ENTITY_ID)
	{
		lua_pushnumber(L, -1);
		return 1;
	}

	WorldEntity *ent =  getAppServer().getGame().getWorld().getWorldEntity(userdata);
	if ( ent == NULL )
	{
		lua_pushnumber(L, -1);
		return 1;
	}
	GLVector3d Target = ent->getPosition();
	
	lua_pushnumber(L, Target.x);
	lua_pushnumber(L, Target.y);
	lua_pushnumber(L, Target.z);

	return 3;
}

inline bool isNan(double x)
{
	return (x != x);
}

int shakeHandModify(GLVector3d * ShakeHand)
{
	// modify the shaking hand

	ShakeHand->x += 2.0 - (rand()%1000)/250.0;
	ShakeHand->y += 2.0 - (rand()%1000)/250.0;
	ShakeHand->z += 1.0 - (rand()%1000)/500.0;
	
	if (ShakeHand->x >  10.0) ShakeHand->x =  10;
	if (ShakeHand->x < -10.0) ShakeHand->x = -10;
	if (ShakeHand->y >  10.0) ShakeHand->y =  10;
	if (ShakeHand->y < -10.0) ShakeHand->y = -10;
	if (ShakeHand->z >  5.0) ShakeHand->z =  5;
	if (ShakeHand->z < -5.0) ShakeHand->z = -5;


	if ( isNan(ShakeHand->x) ) ShakeHand->x = 0;
	if ( isNan(ShakeHand->y) ) ShakeHand->y = 0;
	if ( isNan(ShakeHand->z) ) ShakeHand->z = 0;


	return 0;
}

int luasettargetenemy(lua_State *L)
{
	// LOGE << "AI " << " luasettargetenemy" << endl;
	int n = lua_gettop(L);
	if (n < 2)
	{
		return 0;
	}
	
	AI3 *ai = (AI3 *)lua_touserdata(L, -2);
	GLVector3d * ShakeHand = ai->getmShakeHand();
	GLVector3d * Target = ai->getmTarget();
	Tank * tank = ai->getmTank();

	int * userdata = (int *)lua_touserdata(L, -1);

	(*Target) = getAppServer().getGame().getWorld().getWorldEntity((*userdata))->getPosition();

	// alter position of enemy to position where he will be when bullets get there

	// count my speed
	GLVector3d myspeed = ai->getmTank()->getSpeed();
	
	// count his speed
	GLVector3d enemyspeed = getAppServer().getGame().getWorld().getWorldEntity((*userdata))->getSpeed();
	
	// count enemys speed in respect to me
	GLVector3d speed = enemyspeed - myspeed;

	// count distance
	GLVector3d distancevector = (*Target) - ai->getmTank()->getPosition();
	double distance = sqrt(distancevector.x*distancevector.x + distancevector.y*distancevector.y + distancevector.z*distancevector.z);

	// count time of bullet flight (to his position at the moment) (that distance he will be gone will create only uninportatnt time change)
	const double VELOCITY = 450;
	double time = distance / VELOCITY ;

	// add to target vector he will probably be gone in bullet flight time(*IMAGINE HERE A DEVILS LAUGH*)

	(*Target) += speed * time;
	(*Target) += (*ShakeHand);

	tank->setYawPitchFrom3DTarget((*Target), 1);

	shakeHandModify(ShakeHand);

	return 0;
}

int luasetweapon(lua_State *L)
{
	int n = lua_gettop(L);
	if (n < 2)
	{
		return 0;
	}
	
	AI3 *ai = (AI3 *)lua_touserdata(L, -2);
	int wep = (int)lua_tonumber(L, -1);
	ai->setweapon(wep);
	return 0;
}

int luasettarget(lua_State *L)
{
	// LOGE << "AI " << " luasettarget" << endl;
	int n = lua_gettop(L);
	if (n < 4)
	{
		return 0;
	}
	
	AI3 *ai = (AI3 *)lua_touserdata(L, -4);
	GLVector3d * ShakeHand = ai->getmShakeHand();
	GLVector3d * Target = ai->getmTarget();
	Tank * tank = ai->getmTank();

	Target->x = (double)lua_tonumber(L, -3);
	Target->y = (double)lua_tonumber(L, -2);
	Target->z = (double)lua_tonumber(L, -1);
	(*Target) += (*ShakeHand);

	// LOGE << "AI sh " << "x: " << ShakeHand->x << " y: " << ShakeHand->y << " z: " << ShakeHand->z << endl;
	// LOGE << "AI ta " << "x: " << Target->x << " y: " << Target->y << " z: " << Target->z << endl;
	tank->setYawPitchFrom3DTarget((*Target), 1);

	shakeHandModify(ShakeHand);

	return 0;
}

int luastopfire(lua_State *L)
{
	int n = lua_gettop(L);
	if (n < 1)
	{
		return 0;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -1);
	Tank::TankCommands &cmds = ai->getmTank()->getCmds();
	cmds.fire = false;
	return 0;
}


int luafire(lua_State *L)
{
	int n = lua_gettop(L);
	if (n < 1)
	{
		return 0;
	}
	
	AI3 *ai = (AI3 *)lua_touserdata(L, -1);
	Tank::TankCommands &cmds = ai->getmTank()->getCmds();
	cmds.fire = true;
	return 0;
}

int luaismoving(lua_State *L)
{
	int n = lua_gettop(L);
	if (n < 1)
	{
		return 0;
	}
	AI3 *ai = (AI3 *)lua_touserdata(L, -1);

	if ( ai->getmMoving() )
	{
		lua_pushnumber(L, 0);
	} else {
		lua_pushnumber(L, 1);
	}

	return 1;
}


int luasay(lua_State *L)
{
	// getting params number from lua
	int n = lua_gettop(L);

	string msg = "/chat ";
	msg += lua_tostring(L, -1);

	if (n > 0)
	{

		getAppServer().getCommander().addCommand( msg );
		// LOGE << "AI " << "LUA: player said: \"" << lua_tostring(L, -1) << "\"" << endl;
	} else
	{
		// getAppClient().getConsole().addMessage( _( "???" ) );
		// LOGE << "AI " << "LUA: player said: \"???\"" << endl;
	}

	// used when returning value to lua ... also c rertun must return number of returned values
	// lua_pushnumber(L, 0);
	return 0;
}

int luaerrormsgnd(lua_State *L)
{
	// getting params number from lua
	int n = lua_gettop(L);
	if (n < 1)
	{
		return 0;
	}

	AI3 *ai = (AI3 *)lua_touserdata(L, -2);
	// LOGE << "AI " << ai->getmPlayerID() << " LUA: " << lua_tostring(L, -1) << endl;

	if (n > 1)
	{
		LOGE << "AI " << ai->getmPlayerID() << " LUA: " << lua_tostring(L, -1) << endl;
	} else
	{
		LOGE << "AI " << "LUA: " << endl;
	}

	// used when returning value to lua ... also c rertun must return number of returned values
	// lua_pushnumber(L, 0);
	return 0;
}

int luaerrormsg(lua_State *L)
{
	// getting params number from lua
	int n = lua_gettop(L);

	if (n > 0)
	{
		LOGE << "AI " << "LUA: " << lua_tostring(L, -1) << endl;
	} else
	{
		LOGE << "AI " << "LUA: " << endl;
	}

	// used when returning value to lua ... also c rertun must return number of returned values
	// lua_pushnumber(L, 0);
	return 0;
}

int lualocatefile(lua_State *L)
{
	// getting params number from lua
	int n = lua_gettop(L);

	if (n == 0)
	{
		LOGE << "One argument of string expected" << endl;
		return 0;
	}

	string path = locateFile(lua_tostring(L, -1));
	// lua_pop(1); // remove the filename (maybe not)
	lua_pushstring(L, path.c_str()); // add path


	// used when returning value to lua ... also c rertun must return number of returned values
	// lua_pushnumber(L, 0);
	return 1;
}
