/* MAINTAINER: Bernard */

#define WANT_TRACE
#include "commander_server.h"
#include "log.h"
#include "application_server.h"
#include "inputconstants.h"
#include "bonus.h"
#include "timemanager.h"
#include "globals.h"
#include "tank.h"
#include "world.h"
#include <string>
#include "player.h"
#include "networkmanager_server.h"
#include "game_server.h"
#include "platform.h"
#include "aimanager.h"
#include "gameplaysettings.h"
#include "endconditions.h"

using namespace std;



CommanderServer::CommanderServer()
{

	// Admin
	mAdminCommandsHash["/gamespeed"]      =   command_game_speed;
	mAdminCommandsHash["/quit"]           =   command_quit;		
	mAdminCommandsHash["/addai"]          =   command_addai;
	mAdminCommandsHash["/newmap"]         =   command_newmap;
	mAdminCommandsHash["/kick"]           =   command_kick;
	mAdminCommandsHash["/fraglimit"]	  =   command_fraglimit;
	mAdminCommandsHash["/timelimit"]	  =   command_timelimit;
	mAdminCommandsHash["/restart"]		  =   command_restart;
	mAdminCommandsHash["/nextmap"]		  =   command_nextmap;
	mAdminCommandsHash["/maxplayers"]	  =   command_maxplayers;
	mAdminCommandsHash["/maxtimenohuman"] =   command_maxtimenohuman;
	
	// ?
	mCommandsHash["/pause"]          =   command_pause;
	
	// Normal
	mCommandsHash["/newtank"]        =   command_new_tank;
	mCommandsHash["/changename"]     =   command_change_name;
	mCommandsHash["/ready"]			 =	 command_ready;
	mCommandsHash["/suicide"]        =   command_suicide;
	mCommandsHash["/chat"]           =   command_chat;
	mCommandsHash["/weapon"]         =   command_weapon;
	mCommandsHash["/nextweapon"]     =   command_next_weapon;
	mCommandsHash["/previousweapon"] =   command_previous_weapon;
	mCommandsHash["/ready"]          =   command_ready;
	mCommandsHash["/wannabeadmin"]   =   command_wannabeadmin;
	mCommandsHash["/su"]             =   command_wannabeadmin;
		
}


Tank* CommanderServer::getTank(int playerID)
{
	if (getAppServer().getGame().isPlayerIDValid(playerID))
	{
		return getAppServer().getGame().getPlayer(playerID).getTank();
	}
	return NULL;	
}


Player* CommanderServer::getPlayer(int playerID)
{
	if (getAppServer().getGame().isPlayerIDValid(playerID))
	{
		return &getAppServer().getGame().getPlayer(playerID);
	}
	return NULL;	
}

bool CommanderServer::isAdmin(int playerID)
{
	// This is a console player :-)
	if (playerID == INVALID_PLAYER_ID) return true;
	
	if (getAppServer().getGame().isPlayerIDValid(playerID))
	{
		return getAppServer().getGame().getPlayer(playerID).isBigAdmin();
	}
	
	return false;	
}


void CommanderServer::processCommand(const string &command, int playerID)
{

	stringstream s(command);
	string cmd;
	
	LOGD << "Processing command " << command << endl;

	s >> cmd;


	// Find command in our table
	commads_hash::iterator it = mCommandsHash.find(cmd);
	if (it != mCommandsHash.end())
	{
		(it->second)(s, playerID);		
		return;
	}

	it = mAdminCommandsHash.find(cmd);
	if (it != mAdminCommandsHash.end())
	{
		if (!isAdmin(playerID))
		{
			if (getAppServer().getGame().isPlayerIDValid(playerID))
			{
				getAppServer().getNetworkManager().sendCommand("consolemessage You need to be admin to do that.", playerID);				
				return;
			}		
		}
		(it->second)(s, playerID);		
		return;
	}

	// this is last to catch invalid commands
	// notify the player who did this about invalid command
	if (getAppServer().getGame().isPlayerIDValid(playerID))
	{
		getAppServer().getNetworkManager().sendCommand("consolemessage Unknown command", playerID);				
	}
}


void CommanderServer::command_chat(std::stringstream &args, int playerID)
{

	// read firts ' ' that isn't part of the message
	args.get();
	
	string msg;
	getline(args, msg);
	if (msg.size() == 0) return; // ignore empty message
	
	stringstream message;
	message << "chatmessage " << NOT_TRANSLATE_CHAR;
	if (getAppServer().getGame().isPlayerIDValid(playerID))
	{
		Player &p = getAppServer().getGame().getPlayer(playerID);
		message << p.getName() << ": ";
	}
	else
	{
		message << "Operator: ";
	}
	message << msg;

	//TODO: Here should be code that send the message to everybody
	getAppServer().getNetworkManager().sendCommand(message.str());
	
	LOGI << "Chat message: " << args << endl;
}

void CommanderServer::command_quit(std::stringstream &args, int playerID)
{	
	LOGI << "Quit command" << endl;
	
	// Here should be something like test for permissions to shut down the server!
	getAppServer().quit();	
}

void CommanderServer::command_weapon(std::stringstream &args, int playerID)
{
	int weaponID;
	args >> weaponID;

	Tank *tank = getTank(playerID);
	if (tank)
	{
		tank->setWeaponServer(weaponID);
	}
	else
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage You don't have tank :-)", playerID);
		}
	}
}

void CommanderServer::command_next_weapon(std::stringstream &args, int playerID)
{
	Tank *tank = getTank(playerID);
	if (tank)
	{
		tank->setNextWeaponServer();
	}
}

void CommanderServer::command_previous_weapon(std::stringstream &args, int playerID)
{
	Tank *tank = getTank(playerID);
	if (tank)
	{
		tank->setPreviousWeaponServer();
	}
}

void CommanderServer::command_game_speed(std::stringstream &args, int playerID)
{
	double gameSpeed = 0;

	args >> gameSpeed;

	//if (gameSpeed != 0) 
	gTime.setGameSpeed(gameSpeed);
}


void CommanderServer::command_pause(std::stringstream &args, int playerID)
{
	gTime.pauseSwitch();
		
	stringstream message;
	message << "pause " << gTime.isPaused();

	// Send to All
	getAppServer().getNetworkManager().sendCommand(message.str());
}

void CommanderServer::command_new_tank(std::stringstream &args, int playerID)
{
	if (!getAppServer().getGame().isPlayerIDValid(playerID)) return;

	// Create new tank only if player doesn't have any
	Player &p = getAppServer().getGame().getPlayer(playerID);
	if (p.getTank() != NULL) 
	{
		LOGD << "player " << playerID << " allready has tank" << endl;
		getAppServer().getNetworkManager().sendCommand("consolemessage You allready have tank", playerID);		
		return;
	}

	int tank_type;

	args >> tank_type;
	if (args.fail()) 
	{	
		LOGE << "Invalid tank wanted" << endl;
		return;
	}

	GLVector3d pos;
	double angle;
	getAppServer().getGame().getSpawnPoint(pos, angle);

	// creating the tank
	getAppServer().getGame().getWorld().createTank( 
		pos, 
		GLQuaterniond(GLVector3d(0,0,1), angle).normalize(), 
		tank_type,
		playerID);
}


void CommanderServer::command_change_name(std::stringstream &args, int playerID)
{
	if (!getAppServer().getGame().isPlayerIDValid(playerID)) return;

	Player &p = getAppServer().getGame().getPlayer(playerID);

	// read firts ' ' that isn't part of the message
	args.get();
	
	// read players name
	string name;
	getline(args, name);
	if (name.size() == 0) 
	{
		getAppServer().getNetworkManager().sendCommand("consolemessage Usage: /changename new_name", playerID);						
		return; // ignore empty message
	}
	
	// set the name
	p.setName(name);
}

void CommanderServer::command_suicide(std::stringstream &args, int playerID)
{
	if (!getAppServer().getGame().isPlayerIDValid(playerID)) return;

	Tank *tank = getTank(playerID);
	Player *p  = getPlayer(playerID);
	if (tank && p)
	{
		tank->suicide();
	}
	else
	{
		getAppServer().getNetworkManager().sendCommand("consolemessage Unable to suicide if you have no tank.", playerID);						
	}
}

void CommanderServer::command_addai(std::stringstream &args, int playerID)
{
	int ais, aitype = 0;
	args >> ais;
	
	if ( args.fail() )
	{
		ais = 1;
	}
	else
	{
		args >> aitype;
		if ( args.fail() )
		{
			aitype = 0;
		}
	}
	
	while (ais-- > 0)
	{
		getAppServer().getGame().getAIManager().addAI(aitype);
	}
}



void CommanderServer::command_newmap(std::stringstream &args, int playerID)
{
	string mapName;
	args >> mapName;
	
	if (args.fail())
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Usage: /newmap map_file", playerID);
		}
		return;
	}
	
	// Verify if the map file at least exist
	string locatedMap = locateFile(mapName);
	if (!exist(locatedMap.c_str()))
	{
		LOGE << mapName << " not found" << endl;
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Map not found", playerID);				
		}
		return;	
	}
	
	// Verify if the file not only exist but also has some
	// structure of map file
	if ( !Params::isProbablyMapFile(locatedMap) )
	{
		LOGE << mapName << " isn't file with map" << endl;
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Probably isn't valid map file.", playerID);				
		}
		return;	
	}
		
	// We could also reload the map
	getAppServer().reloadToMap(mapName);
}

void CommanderServer::command_kick(std::stringstream &args, int playerID)
{
	int kickedID;
	args >> kickedID;
	
	if (args.fail() || !getAppServer().getGame().isPlayerIDValid(kickedID))
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Usage: /kick playerID", playerID);
		}
		return;
	}

	getAppServer().getGame().preparePlayerDeletion(kickedID);
}

void CommanderServer::command_ready(std::stringstream &args, int playerID)
{
	if (getAppServer().getGame().isPlayerIDValid(playerID))
	{
		getAppServer().getGame().getPlayer( playerID ).ready();
	}
	return;
}

void CommanderServer::command_wannabeadmin(std::stringstream &args, int playerID)
{
	if (!getAppServer().getGame().isPlayerIDValid(playerID)) return;	
	
	string password;
	args >> password;

	// Verify the password
	if (password != g_settings.getString(SETTING_SERVER_PASSWORD))
	{
		stringstream message;
		message << "chatmessage " << NOT_TRANSLATE_CHAR;
		Player &p = getAppServer().getGame().getPlayer(playerID);
		
		message << "!!!ALLARM!!! " << p.getName() << " IS TRYING TO HACK THE SERVER!!! "
		           "GO AND KILL THAT SON OF A BITCH!!!!";
		           
		getAppServer().getNetworkManager().sendCommand(message.str());
		return;
	}
	
	// access granted
	getAppServer().getGame().getPlayer(playerID).setBigAdmin(true);
	getAppServer().getNetworkManager().sendCommand("consolemessage :-)", playerID);
	
}

void CommanderServer::command_fraglimit(std::stringstream &args, int playerID)
{
	int limit;
	args >> limit;
	
	if ( args.fail() || limit < 0 )
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Usage: /fraglimit limit", playerID);
		}
		return;
	}

	if	( 
		getAppServer().getGame().getPlaySettings() && 
		getAppServer().getGame().getPlaySettings()->addEndCondition( new FragLimit( limit ) ) 
		)
	{
		//successfull
	}
	else
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("Map not ready for accepting this type of command", playerID);
		}
	}	
}

void CommanderServer::command_timelimit(std::stringstream &args, int playerID)
{
	double limit;
	args >> limit;
	
	if ( args.fail() || limit < 0 )
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Usage: /timelimit limit", playerID);
		}
		return;
	}

	if	( 
		getAppServer().getGame().getPlaySettings() && 
		getAppServer().getGame().getPlaySettings()->addEndCondition( new TimeLimit( limit ) ) 
		)
	{
		//successfull
	}
	else
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("Map not ready for accepting this type of command", playerID);
		}
	}
	
}

void CommanderServer::command_restart(std::stringstream &args, int playerID)
{
	if	( 
		getAppServer().getGame().getPlaySettings() && 
		getAppServer().getGame().getPlaySettings()->restart()
		)
	{
		//successfull
	}
	else
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("Map not ready for accepting this type of command", playerID);
		}
	}
	
}

void CommanderServer::command_nextmap(std::stringstream &args, int playerID)
{
	getAppServer().loadNextMap();
}

void CommanderServer::command_maxplayers(std::stringstream &args, int playerID)
{
	int limit;
	args >> limit;
	
	if ( args.fail() || limit < 1)
	{
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Usage: /maxplayers limit, where limit >= 1", playerID);
		}
		return;
	}

	g_settings.set(SETTING_MAX_PLAYERS, xtos(limit));
}

void CommanderServer::command_maxtimenohuman(std::stringstream &args, int playerID)
{
	int limit;
	args >> limit;
	
	if ( args.fail() || limit < 0)
	{
		LOGE << "Invalid args for command_maxtimenohuman" << endl;
		if (getAppServer().getGame().isPlayerIDValid(playerID))
		{
			getAppServer().getNetworkManager().sendCommand("consolemessage Usage: /maxtimenohuman limit, where limit >= 0", playerID);
		}
		return;
	}

	g_settings.set(SETTING_MAX_TIME_WITHOUT_HUMAN, xtos(limit));	
}
