#include "endconditions.h"
#include "player.h"
#include "game_server.h"
#include "application_server.h"
#include "globals.h"
#include "log.h"
#include "timemanager.h"

bool FragLimit::satisfied()
{
	if ( mLimit <= 0 )
		return false;

	const std::vector<int> & plids = getApp().getGameCommon().getValidPlayersID();
	for ( unsigned int i = 0; i < plids.size(); i++ )
	{
		int frags = getApp().getGameCommon().getPlayer( plids[i] ).getKills();

		if ( frags > mCurrent )
			mCurrent = frags;

		if ( frags >= mLimit )
			return true;
	}

	return false;
}

bool TimeLimit::satisfied()
{
		//mLimit is computed from double. So to avoid rounding error, timilimet must be at least 1 second :)
	if ( mLimit < 1000 || gTime.isPaused() )	
		return false;

	mLastTime = gTime.getTicks();
	if ( mLastTime >= mStartTime + mLimit )
	{
		return true;
	}
	else
	{
		return false;
	}
}

void TimeLimit::start()
{
	mStartTime = gTime.getTicks();
	mLastTime = mStartTime;
}

int TimeLimit::getCurrentValue()
{
	mLastTime = gTime.getTicks();
	return mLastTime - mStartTime;
}

