#ifndef _ENDCONDITIONS_H_
#define _ENDCONDITIONS_H_

#include "gameplaysettings.h"

/**	\brief Implements frag limit condition - satisfied if some player reaches specified number of kills
 */
class FragLimit : public MapEndCondition
{
public:
	/**	\brief Constructor - initializes limit name and frag limit.
	 *
	 *	\param limit Number of frags (!kills!)
	 */
	FragLimit( int limit ) : MapEndCondition( "fraglimit" ), mLimit( limit ), mCurrent( 0 ) {};

	virtual bool satisfied();

	virtual int getTargetValue()
	{
		return mLimit;
	}
	virtual int getCurrentValue()
	{
		return mCurrent;
	}
	virtual void start()
	{
		mCurrent = 0;
	}
private:
	int mLimit;
	int mCurrent;
};

/**	\brief Implements time limit condition. Satisfied when playing limit minutes (excluding pauses)
 */
class TimeLimit : public MapEndCondition
{
public:
	/**	\brief Constructor - initializes limit name and time limit (in minutes).
	 *
	 *	\param limit Minutes of play
	 */
	TimeLimit( double limit ) : 
		MapEndCondition( "timelimit" ), 
		mLimit( (int)( limit * 60 * 1000 ) ), 
		mLastTime( -1 ),
		mStartTime( -1 )
	{};

	virtual bool satisfied();

	virtual int getTargetValue()
	{
		return mLimit;
	}

	virtual int getCurrentValue();
	
	virtual void start();

	virtual void paused( int from, int to )
	{
		mStartTime+= ( to - from );
	}
private:
	int mLimit;
	int mLastTime;
	int mStartTime;
};

#endif
