/* MAINTAINER: Lukas */

#ifndef _EVENTS_CLIENT_H
#define _EVENTS_CLIENT_H

#include "event.h"
#include <string>
#include "netobjects.h"

/**
 * Command from client
 */
class EvCommand : public Event
{

public:

	EvCommand(std::string command, int playerID) { mCommand = command; mPlayerID = playerID; }
	virtual ~EvCommand() { };

	/**
	 * Calls commander that cares about rest
	 */
	virtual void process();

private:

	std::string mCommand;
	int mPlayerID;

};

/**
 * \brief Event that some buttost on client were pressed.
 *
 * Only change of state is sended
 */
class EvInputButton : public Event
{

public:
	
	EvInputButton(int button, bool value, double yaw, double pitch, int playerID) 
	{ 
		mButton   = button; 
		mValue    = value; 
		mYaw      = yaw;
		mPitch    = pitch;
		mPlayerID = playerID;
	}

	virtual void process();

private:
	int    mButton;
	bool   mValue;
	double mYaw;
	double mPitch;
	int    mPlayerID;

};

/**
 * \brief Message from client with actual yaw and pitch of
 *        client's tank. 
 *
 * The message is sended from client regullary, not every
 * change. In isn't importat for simulation to know exact 
 * angle of tank.
 */
class EvYawPitch : public Event
{

public:
	
	EvYawPitch(double yaw, double pitch, int playerID)
	{
		mYaw      = yaw;
		mPitch    = pitch;
		mPlayerID = playerID;
	}

	virtual void process();

private:
	double mYaw;
	double mPitch;
	int    mPlayerID;
};

/**
 * New player connected
 */
class EvPlayerConnected : public Event
{

public:

	EvPlayerConnected(std::string name, Client_var cl) { mPlayerName = name; mClient = cl; };
	virtual ~EvPlayerConnected() { };

	/**
	 * Try add player to the list and call callback function in network manager
	 */
	virtual void process();

private:

	std::string mPlayerName;
	Client_var mClient;

};

/**
 * Game info request
 */
class EvGameInfoRequest : public Event
{

public:

	EvGameInfoRequest(int playerID) { mPlayerID = playerID; };
	virtual ~EvGameInfoRequest() { };

	/**
	 * Try add player to the list and call callback function in network manager
	 */
	virtual void process();

private:

	int mPlayerID;

};

/**
 * Player disconnected
 */
class EvPlayerDisconnected : public Event
{

public:

	EvPlayerDisconnected(int playerID) { mPlayerID = playerID; };
	virtual ~EvPlayerDisconnected() { };

	/**
	 * Remove player
	 */
	virtual void process();

private:

	int mPlayerID;

};

#endif // _EVENTS_CLIENT_H
