
#include "bonus.h"
#include "params.h"
#include "application_server.h"
#include <sstream>
#include "timemanager.h"
#include "tank.h"
#include "bonusbox.h"
#include "world.h"
#include "log.h"
#include "player.h"
#include "game_server.h"
#include "networkmanager_server.h"

using namespace std;

void Bonus::picked(Tank &t)
{
	mInfluencedTank = &t;

	const Params::sBonusParams& p = getParams();

	// Do bonus efect
	
	t.enableWeapon(p.weapon);
	t.addProjectiles(p.ammo_type, p.ammo_addition);
	if ((t.mHP += p.HP_change) >= t.mMaxHP) t.mHP = t.mMaxHP;
	if ((t.mArmour += p.armour) >= t.mMaxArmour) t.mArmour = t.mMaxArmour;
	t.mMaxSpeed *= p.max_speed;
	t.mDamageMultiplication *= p.damage;

	applyReloadTime();

	//TODO: damage
	//TODO: invulnerability
	//TODO: invisibility

	// Amuse user - ned command to create a little bit of fun
	if (t.mPlayer)
	{
		stringstream message;
		message << "pickedbonus " << mID << endl;
		getAppServer().getNetworkManager().sendCommand(message.str(), t.mPlayer->getID());
	}

	// Keep bonuses with longer time to the tank
	if (p.duration > 0 || p.outlive_tank)
	{
		if (p.duration != 0)
		{
			//mDuration = gTime.getGameTicks() + p.duration;
			mDuration = (int)gTime.getSimulatedTime() + p.duration;
		}
		t.mBonuses.push_back(this);
	}
	else
	{
		delete this;
	}
}


bool Bonus::testEspired(int time)
{
	assertL(mInfluencedTank);

	if (mDuration < time && mDuration > 0)
	{
		const Params::sBonusParams& p = getParams();

		mInfluencedTank->mMaxSpeed /= p.max_speed;
		mInfluencedTank->mReloadTime /= p.reload_time;
		mInfluencedTank->mDamageMultiplication /= p.damage;

		// Amuse user - tell him bonus is over :(
		if (mInfluencedTank->mPlayer)
		{
			stringstream message;
			message << "expiredbonus " << mID << endl;
			getAppServer().getNetworkManager().sendCommand(message.str());
		}

		// Create bonus duplicant if required
		if (getParams().drop_on_expire) dropBonusBox();  
		
		// Delete us and leave in peace
		delete this;

		return true;
	}
	return false;
}


void Bonus::tankDestroyed(bool killed)
{
	// create diplicant if required and finish
	if (killed && getParams().outlive_tank) dropBonusBox();

	delete this;
}


void Bonus::dropBonusBox()
{
	//TODO: Create new bonus box and put bonus back to game
	//We should somehow modify the position so the bonus box will
	//move a little.
	
	assertL(mInfluencedTank);

	GLVector3d speed;
	speed.z =  8 + (double)(rand()%100)/10.0;
	speed.x = -7 + (double)(rand()%100)/14.0;
	speed.y = -7 + (double)(rand()%100)/14.0;

	GLVector3d position =  mInfluencedTank->mPosition;
	position += speed;

	BonusBox *bb = getAppServer().getGame().getWorld().createBonusBox(
		position, 
		GLQuaterniond( GLVector3d(1,0,0), rand()%360 ).normalize(),
		mID );
		

	if (bb)
	{
		bb->setSpeed(speed);
	}
	else
	{
		// NOTE THAT THIS IS SERIOS ERROR
		DIE("Unable to create bonusbox");
	}
}
