#include "log.h"
#include "physical_terrain.h"
#include "settings.h"

void PhysicalTerrain::init(void)
{	
	// HACK: this shouldn't be in settings-file in the future
	mBlockShift = g_settings.getInt(SETTING_SERVER_TMP_BLOCK_SHIFT);
	createBlocks();
}


// HACK: remove this #include and rendering code from server
#include "SDL_opengl.h"
void PhysicalTerrain::render(PhysicalTerrain::TVertex * vertices, int vertCount, int *indices, int indCount)
{	
	glBegin(GL_TRIANGLES);
	for (int i = 0; i < indCount; ++i)
		glVertex3dv(vertices[indices[i]]);
	glEnd();
}

void PhysicalTerrain::render(void)
{	
	for (int by = 0; by < getBlocksCountY(); ++by)
		for (int bx = 0; bx < getBlocksCountX(); ++bx)
		{
			PhysicalTerrain::TVertex *vert;
			int* ind;
			int vc, ic;
			getTriMeshData(bx, by, vert, vc, ind, ic);
			render(vert, vc, ind, ic);
		}
}



PhysicalTerrain::TerrainBlock::BlockDescriptor& PhysicalTerrain::getDescriptor(int bx, int by, PhysicalTerrain::TerrainBlock::BlockDescriptor & d)
{
	struct { int w, h; } hm;
	
	// get heightmap extents
	getExtents(hm.w, hm.h);
	
	d.tb = this;
	d.x = std::min(std::max(bx << getBlockShift(), 0), hm.w - 1);
	d.y = std::min(std::max(by << getBlockShift(), 0), hm.h - 1);
	d.w = (1 << mBlockShift) + 1;
	d.h = (1 << mBlockShift) + 1;	
	if (d.x + d.w > hm.w) 
		d.w = hm.w - d.x;
	if (d.y + d.h > hm.h) 
		d.h = hm.h - d.y;
	
	return d;
}

void PhysicalTerrain::createBlocks()
{
	if (mBlocks)
	{
		delete [] mBlocks;
		mBlocks = 0;
	}

	struct 
	{
		int w, h;
	} hm, ba;

	// get heightmap extents
	getExtents(hm.w, hm.h);

	// compute extents of block-array ( rounded up to block size )
	ba.w = ( (hm.w - 2 ) >> getBlockShift() ) + 1;
	ba.h = ( (hm.h - 2 ) >> getBlockShift() ) + 1;

	// allocate memory for block-array and set member variables width and height 
	mBlocks = new TerrainBlock [ba.h * ba.w];
	if (!mBlocks)
		DIE("Not enought memory for array of terrain blocks!");

	mBlocksW = ba.w;
	mBlocksH = ba.h;

	// create triangle mesh data of each block
	TerrainBlock * block = mBlocks;
	for (int by = 0; by < mBlocksH; ++by)
		for (int bx = 0; bx < mBlocksW; ++bx, block++)
		{
			TerrainBlock::BlockDescriptor bd;
			getDescriptor(bx, by, bd);
			block->createTriMeshData(bd);
		}
}

void PhysicalTerrain::TerrainBlock::createTriMeshData(const PhysicalTerrain::TerrainBlock::BlockDescriptor &bd)
{
	const int step_size = 1;
	int i;
	int vh = (bd.h - 1) / step_size + 1, vw = (bd.w - 1) / step_size + 1;
	int vb_size = vh * vw;
	int ib_size = 2 * 3 * (vh-1) * (vw - 1);
	
	if (mVertexBuffer && vb_size != mVertexBufferSize)
	{
		delete [] mVertexBuffer;
		mVertexBuffer = 0;
	}
	
	if (mIndexBuffer && ib_size != mIndexBufferSize){
		delete [] mIndexBuffer;
		mIndexBuffer = 0;
	}

	// allocate buffer for vertices and for indices	
	mVertexBuffer = new TVertex[vb_size];
	mVertexBufferSize = vb_size;

	mIndexBuffer = new int[ib_size];
	mIndexBufferSize = ib_size;

	
	GLVector3d scale;
	bd.tb->getRealScale(scale);

	// create vertices
	i = 0;
	for (int y = bd.y; y < bd.y + bd.h; y += step_size) 
	{
		for (int x = bd.x; x < bd.x + bd.w; x += step_size) 
		{
			assertL(i < vb_size && "Array of block's vertices out of bounds!");
			mVertexBuffer[i][0] = scale.x * x;
			mVertexBuffer[i][1] = scale.y * y;
			mVertexBuffer[i][2] = scale.z * bd.tb->getTrueHeight(x, y);
			++i;
		}
	}
	
	// create indices	
	i = 0;
	for (int y = 0; y < vh - 1; y ++) 
	{		
		for (int x = 0; x < vw - 1; x++)
		{
			assertL(3*(i+1)+2 < ib_size && "Array of block's indices out of bounds!");
			mIndexBuffer[3*i + 0] = y * vw + x;
			mIndexBuffer[3*i + 1] = y * vw + x + vw;
			mIndexBuffer[3*i + 2] = y * vw + x + 1;
			++i;

			mIndexBuffer[3*i + 0] = y * vw + x + 1;
			mIndexBuffer[3*i + 1] = y * vw + x + vw;
			mIndexBuffer[3*i + 2] = y * vw + x + vw + 1;
			++i;
		}
	}
}
