/* MAINTAINER: Petko */

#include "projectile.h"
#include "netobjects.h"
#include "history.h"
#include "player.h"
#include "log.h"
#include "application_server.h"
#include "networkmanager_server.h"
#include "globals.h"
#include "timemanager.h"

void Projectile::handleCollisionWEntityServer( int ID, const GLVector3d & coll_pos, const GLVector3d & coll_normal )
{
	mDeleteMe = true;
	hit( ID );

	// collisions are added for all players - confrontor will say for every 
	// player which collision is relevant
	getAppServer().getNetworkManager().sendCollision(Collision(getID(), ID,
		coll_pos, coll_normal, gTime.getSimulatedTime()));
}

void Projectile::handleCollisionWStaticServer( const GLVector3d & coll_pos, const GLVector3d & coll_normal )
{
	mDeleteMe = true;
	hit();

	// collisions are added for all players - confrontor will say for every
	// player which collision is relevant
	getAppServer().getNetworkManager().sendCollision(Collision(getID(), INVALID_ENTITY_ID,
		coll_pos, coll_normal, gTime.getSimulatedTime()));
}

void Projectile::packCreationData(NETGameState & data)
{
	// actual points at the end of newTanks
	int actual = data.newObj.newProjectiles.length();

	// add one tank creation data - actual points at the end
	data.newObj.newProjectiles.length(actual + 1);

	NETNewProjectile & npData = data.newObj.newProjectiles[actual];

	// DATA COPY
	
	// ID
	npData.id = getID();

	// Player
	npData.player = mOwner->getID();

	// Type
	npData.type = getSubType();

	// orientation
	npData.orientation.x = (float)getPresentPoint()->mOrientation.getV().x;
	npData.orientation.y = (float)getPresentPoint()->mOrientation.getV().y;
	npData.orientation.z = (float)getPresentPoint()->mOrientation.getV().z;
	npData.orientation.w = (float)getPresentPoint()->mOrientation.getW();

	// position
	npData.position.x = (float)getPresentPoint()->mPosition.x;
	npData.position.y = (float)getPresentPoint()->mPosition.y;
	npData.position.z = (float)getPresentPoint()->mPosition.z;
}

void Projectile::render()
{
	DIE("Access to render() on server");
}
