/* MAINTAINER: Petko */
#ifndef _SERVER_PHYSICS_DATA_H
#define _SERVER_PHYSICS_DATA_H

#include <ode/ode.h>
#include <list>
#include "GLVector.h"
#include "GLQuaternion.h"
#include "tasksimulationserver.h"

const int ODE_MAX_JOINTS_PER_ENTITY = 128;
const double WHEEL_RADIUS = 2;

class ServerPhysicsData
{
public:
	ServerPhysicsData(){};
	virtual ~ServerPhysicsData(){};

	virtual bool createODERigidAndColModel( int entity_id, GLVector3d position, GLQuaterniond orientation, TaskSimulationServer * sim );

	std::list< dBodyID  >	mBodies;	///< \brief List of all body IDs that forms one entity
	std::list< dJointID >	mJoints;	///< \brief List of all joint IDs that connect bodies from one entity
	dBodyID					mMainBody;	///< \brief Body that position corresponds to position of entity. This is also in mBodies
};

class TankPhysicsData:public ServerPhysicsData
{
public:
	TankPhysicsData() {}
	virtual ~TankPhysicsData() {}

	dBodyID mLeftTrack;
	dBodyID mRightTrack;

	dJointID mLeftJoints[2];
	dJointID mRightJoints[2];

	double mTotSpeed;
	double mLeftSpeed;
	double mRightSpeed;

	virtual bool createODERigidAndColModel( int entity_id, GLVector3d position, GLQuaterniond orientation, TaskSimulationServer * sim );
};

class LandscapeEntityPhysicsData: public ServerPhysicsData
{
public:
	LandscapeEntityPhysicsData() {}
	virtual ~LandscapeEntityPhysicsData() {}

	virtual bool createODERigidAndColModel( int entity_id, GLVector3d position, GLQuaterniond orientation, TaskSimulationServer * sim );
};
#endif // _SERVER_PHYSICS_DATA_H
