/* MAINTAINER: Lukas */

#include "tank.h"
#include "player.h"
#include "timemanager.h"
#include "log.h"
#include "application_server.h"
#include "history.h"
#include "bonus.h"
#include "netobjects.h"
#include "globals.h"
#include "gameplaysettings.h"
#include "game_server.h"
#include "gameplaysettings.h"
#include "world.h"
#include "serverphysicsdata.h"

bool Tank::init()
{
	return CommonEntity::init();
}

void Tank::render()
{
	TIN

	DIE("Cannot call this method on server!" );
}

void TanksHistoryPoint::transferToNET(NETGameState & data)
{
	// actual points at the end of tanks
	int actual = data.objects.tanks.length();

	// add one tank - actual points at the end
	data.objects.tanks.length(actual + 1);

	NETWETank & tData = data.objects.tanks[actual];

	// DATA COPY

	// BASIC History Point - calls history point data transfer
	HistoryPoint::transferToNETDirect(tData.basic);

	// TANK History Point - calls tank data transfer
	transferToNETDirect(tData);
}

void TanksHistoryPoint::transferToNETDirect(NETWETank & data)
{
	data.armour = mArmour;
	data.hp = mHP;
	data.projectiles = mProjectiles;
	data.weapon = mWeapon;

	data.pitch = (float)mPitch;
	data.yaw = (float)mYaw;
}

void Tank::packCreationData(NETGameState & data)
{
	TIN

	// actual points at the end of newTanks
	int actual = data.newObj.newTanks.length();

	// add one tank creation data - actual points at the end
	data.newObj.newTanks.length(actual + 1);

	NETNewTank & ntData = data.newObj.newTanks[actual];

	// DATA COPY
	
	// ID
	ntData.id = getID();

	// Player
	ntData.player = mPlayer->getID();

	// Type
	ntData.type = getSubType();

	if(getPresentPoint() != 0)
	{
		// orientation
		ntData.orientation.x = (float)getPresentPoint()->mOrientation.getV().x;
		ntData.orientation.y = (float)getPresentPoint()->mOrientation.getV().y;
		ntData.orientation.z = (float)getPresentPoint()->mOrientation.getV().z;
		ntData.orientation.w = (float)getPresentPoint()->mOrientation.getW();

		// position
		ntData.position.x = (float)getPresentPoint()->mPosition.x;
		ntData.position.y = (float)getPresentPoint()->mPosition.y;
		ntData.position.z = (float)getPresentPoint()->mPosition.z;
	}
	else
	{
		// confrontor doesn't let this happend

		LOGE << "No present point (worldentity id: " << getID() << ")";
		DIE();
	}
}

int Tank::canFire( const double simulation_time, const int sim_timestamp )
{
	if (mProjectile == INVALID_PROJECTILE_ID) return INVALID_PROJECTILE_ID;
	if (mProjectiles[mProjectile] <= 0) return INVALID_PROJECTILE_ID;

	int time_now = sim_timestamp;
	if ( time_now > mReloadTime  + mLastFireTime )
	{
		mLastFireTime = time_now;
		
		mProjectiles[mProjectile]--;
		return mProjectile;
	}

	return INVALID_PROJECTILE_ID;
}

void Tank::setWeaponServer( int type_id )
{
	if (type_id == INVALID_WEAPON_ID) return;
	if (type_id < 0 || (unsigned int)type_id >= mWeapons.size()) return; // FAILURE
	if (mWeapons[type_id] == false) return; // We don't have such a weapon

	mLastFireTime = gParams.mWeapons[type_id].change_in_time + (int)gTime.getSimulatedTime();

	if (mWeapon >= 0) // The weapon mey be -1 at the begining
		mLastFireTime += gParams.mWeapons[mWeapon].change_out_time;

	mWeapon     = type_id;
	mReloadTime = gParams.mWeapons[type_id].reload_time;
	mProjectile = gParams.mWeapons[type_id].projectile;

	//TODO: check all bonuses and let them modify mReloadTime
	bonusListType::iterator it = mBonuses.begin();
	for (;it != mBonuses.end();++it)
	{
		(*it)->applyReloadTime();
	}

	LOGT << "Weapon: " << gParams.mWeapons[mWeapon].name << " with "
	     << mProjectiles[mProjectile] << " projectiles" << endl;
}

void Tank::setNextWeaponServer()
{
	int next = mWeapon;
	int i = mWeapons.size();
	do
	{
		next++;
		if ((unsigned int)next >= mWeapons.size()) next = 0;
	}
	while (mWeapons[next] == false && (--i) >= 0);
	
	if (i < 0) return; // Failure

	setWeaponServer(next);
}

void Tank::setPreviousWeaponServer()
{
	int previous = mWeapon;
	int i = mWeapons.size();
	do
	{
		previous--;
		if (previous < 0) previous =  mWeapons.size() - 1;
	}
	while (mWeapons[previous] == false && (--i) >= 0);
	
	if (i < 0) return; // Failure

	setWeaponServer(previous);

}

void Tank::fallenServer()
{
	GamePlaySettings * gps = getAppServer().getGame().getPlaySettings();
	assertL( gps );
	assertL( mPlayer );

	gps->fall( mPlayer );
}

void Tank::checkDestroy( Player * player )
{
	if ( mHP <= 0 && mDestroyTime < 0 )
	{
		GamePlaySettings * gps = getAppServer().getGame().getPlaySettings();
		assertL( gps );

		if ( ( mPlayer && player == mPlayer ) || mSuicide )  //if player shot himself, just substract one his kill
		{
			gps->suicide( mPlayer );
		}
		else
		{
			gps->killed( player, mPlayer );
		}
		
		mDestroyTime = (int)gTime.getSimulatedTime();
		//mDeleteMe = true;

		// tell every bonus that tank was destroyed
		bonusListType::iterator it = mBonuses.begin();
		while (it != mBonuses.end())
		{
			(*it)->tankDestroyed(true);
			it = mBonuses.erase(it);
		}
	}
}

void Tank::handleCollisionWEntityServer( int ID, const GLVector3d &coll_pos, const GLVector3d &coll_normal )
{
	assertL( getAppServer().getGame().getPlaySettings() );
	getAppServer().getGame().getPlaySettings()->collide( mPlayer, getApp().getGameCommon().getWorld().getWorldEntity( ID ) );
}

void Tank::killedCallbackClient()
{
	DIE( "Cannot call this method on server!" );
}

void Tank::tankControllServer( const double ode_to_real )
{
		//tank is 'dead' -> it cannot make any movement adjustments
		//TODO: toto mozno patri skor do Tank::checkDestroy(), ale je otazka, ci niekto zo siete potom este niekedy nebude chciet nastavovat komandy....
	if ( getHP() <= 0 )
	{
		mCmds.reset();
	}

	TankPhysicsData * tpd = dynamic_cast< TankPhysicsData* > (getPhysicsData());

	tpd->mLeftSpeed= tpd->mTotSpeed;
	tpd->mRightSpeed= tpd->mTotSpeed;

/*
#ifdef _DEBUG
	if ( getCmds().forward && getCmds().backward )
		LOGW << "Tank with id " << getID() << "has set commands for front and back simultanously!" << endl;
	if ( getCmds().left && getCmds().right )
		LOGW << "Tank with id " << getID() << "has set commands for left and right simultanously!" << endl;
#endif
*/

	double max_speed = getMaxSpeed() * ode_to_real;
		//TODO: cosi s tou konstantou
	double dv = 3 * (getEngineForce()/getWeight()) * ode_to_real;
	double tchng = getTurningTrackSpeedAdd();

	if ( getCmds().forward )
	{
		tpd->mTotSpeed += dv;
		if ( tpd->mTotSpeed > max_speed )
			tpd->mTotSpeed = max_speed;
	}
	if ( getCmds().backward )
	{
		tpd->mTotSpeed -= dv;
		if ( tpd->mTotSpeed < -max_speed )
			tpd->mTotSpeed = -max_speed;
	}

	if ( !getCmds().forward && !getCmds().backward )
	{
		int sign = 0;
		if ( tpd->mTotSpeed > 0 ) sign = 1;
		if ( tpd->mTotSpeed < 0 ) sign = -1;
		if ( abs( tpd->mTotSpeed ) < DBL_TOLER ) sign = 0;
		tpd->mTotSpeed -= (dv / 2) * sign;
	}

	if ( getCmds().left && !getCmds().right )
	{
		tpd->mLeftSpeed= tpd->mTotSpeed - tchng * MOVEMENT_OPPTRACK_MULTIPLE;
		tpd->mRightSpeed= tpd->mTotSpeed + tchng;
	}
	if ( getCmds().right && !getCmds().left )
	{
		tpd->mLeftSpeed= tpd->mTotSpeed + tchng;
		tpd->mRightSpeed= tpd->mTotSpeed - tchng * MOVEMENT_OPPTRACK_MULTIPLE;
	}
}

