
#include "mapcycle.h"
#include "log.h"
#include "platform.h"
#include "settings.h"


void MapCycle::init()
{
	if ( g_settings.getString(SETTING_GAMEPLAY_MAPCYCLE_FILE) != "")
	{
		string fn = locateFile( g_settings.getString( SETTING_GAMEPLAY_MAPCYCLE_FILE ) );
		if ( !exist( fn.c_str() ) )
		{
			LOGW << "Mapcycle file " << fn << " not found." << endl;
			DIE();
		}

		ifstream mc( fn.c_str() );
		string line;
		while ( mc.good() )
		{
			getline( mc, line );  // I don't like this because
			// some forgotten space at the beginning or at the end will
			// ruin the list
			
			// ignore jokes
			if (line == "")
				continue;
			
			if ( Params::isProbablyMapFile(locateFile(line)) )
			{
				mMapCycle.push_back( line );
			}
			else
			{
				LOGW << "Map with filename " << line << " not found." << endl;
			}
		}
	}
	
/*
	if ( mMapCycle.empty() )
	{
		LOGW << "Mapcycle file " << fn << " contains no maps." << endl;
		return;
	}
*/
	
	mActual = "";
	mNextMapIt = mMapCycle.begin();
	mNext = g_settings.getString( SETTING_SERVER_MAP );
	if (mNext != "")
	{
		if (!exist( locateFile(mNext).c_str() ))
		{
			LOGE << "Map " << mNext << " not found." << endl;
			DIE("Map must be valid if specified.");
		}
	}

	if (mNext == "" && mMapCycle.empty())
	{
		DIE( "Not map file specified. Please provide it on the command line or create proper mapcycle file" );		
	}
}

const std::string & MapCycle::getNextMapFile()
{
	if (mNext != "")
	{
		mActual = mNext;	
		mNext = "";
	}
	else
	{
		if ( mNextMapIt != mMapCycle.end() )
		{
	
			mActual = *mNextMapIt;
			++mNextMapIt;

			if ( mNextMapIt == mMapCycle.end() )
			{
				mNextMapIt = mMapCycle.begin();
			}
		}
		// else  the only hope is that mActual is a valid map
	}

	// So we have new file in mActual (but not sure if valid)
	// If not valid the load will not end good :-)
	Params::loadMapBasicInfo(locateFile(mActual), mActualMapInfo);

	return mActual;
}

const std::string & MapCycle::getActualMapFile() const
{
	return mActual;	
}

void MapCycle::setNextMapToLoadNext( const std::string & map )
{
	if (!exist( locateFile(map).c_str() ))
	{
		LOGW << "Map " << mNext << " not found. - Ignored" << endl;
		return;
	}
	
	mNext = map;
}

const Params::sMap& MapCycle::getActualMapInfo() const
{
	return mActualMapInfo;
}
