#ifndef _MAP_CYCLE_H
#define _MAP_CYCLE_H

#include <list>
#include <string>
#include "params.h"

/**	\brief Manages cycle of maps, gives "suggestion" to next map
 * 
 * Also take care of currently loaded map info.
 */
class MapCycle
{
public:
	/**	\brief Reads mapcycle file and fills list of maps
	 */
	void init();

	/**	\brief "No map" return value
	 */
	static std::string NO_MAP;

	/**	\brief Gets next map in cycle and moves iterator
	 *
	 *	\return map file or NO_MAP
	 */
	const std::string & getNextMapFile();

	/**	\brief Returns last map returned by getNextMapFile() or set by setLastMapFile() - result depends on what was called later
	 *
	 *	\return Last map or empty string
	 */
	const std::string & getActualMapFile() const;

	/**	\brief Sets map which will be loaded in next reload
	 *
	 * 	It is possible to "skip" the queue and "plug" new map there
	 *	\param map Map file to remember as last map
	 */
	void setNextMapToLoadNext( const std::string & map );
	
	/**
	 * \brief Gives currently loaded map info.
	 * 
	 * Note that if getActualMapFile() return "" you will find
	 * only some randome garbage in this structure.
	 */
	const Params::sMap& getActualMapInfo() const;
	
private:
	std::list< std::string > mMapCycle;				///< \brief List with maps from mapcycle file
	std::list< std::string >::iterator mNextMapIt;	///< \brief Iterator that points to map that will be returned next (or end)
	std::string mActual;							///< \brief Last map returned getNextMapFile() or set by setLastMapFile()
	std::string mNext;								///< \brief next file to load - this may overlap mNextMapIt
	Params::sMap mActualMapInfo;                    ///< \brief info about last map. Valid since getNextMapFile() called.
};
#endif
