/* MAINTAINER: Lukas */

#include "basicconfrontor.h"
#include "application_server.h"
#include "playernetdata_server.h"
#include "game_server.h"
#include "worldentity.h"
#include "timemanager.h"
#include "world.h"

bool BasicConfrontor::isObjectRelevant(WorldEntity * we)
{
	assertL(we);
	assertL(mPlayerID != INVALID_PLAYER_ID);
	assertL(getAppServer().getGame().getPlayer(mPlayerID).canUpdateClient());

	KnownWE * kwe = 
		(dynamic_cast<PlayerNetDataServer &>(getAppServer().getGame().getPlayer(mPlayerID).netData())).
		getKnownWorldEntities().getKnownObject(we->getID());

	if(kwe != 0)
	{
		// --- can predict on client test ---
		if(kwe->isPredictable())
		{
			return false;
		}

		// get world entity's net point from net data...
		HistoryPoint * netPoint = kwe->getNetPoint();

		assertL(netPoint);

		// ...and confront with present point
		// --- equals test ---
		assertL(we->getPresentPoint());
		return ! netPoint->equals(*(we->getPresentPoint()));
	}
	else
		if(we->getPresentPoint())
			// new object
			return true;
		else
			// ignore object without present point
			return false;
}

bool BasicConfrontor::isCollisionRelevant(const Collision & coll)
{
	const int COLLISION_RELEVANT_TIME = 50; // number of engine loops

	assertL(mPlayerID != INVALID_PLAYER_ID);

	// ignore old collisions
	if(gTime.getSimulatedTime() - COLLISION_RELEVANT_TIME >
			coll.getCreationTime())
		return false;

	return true;
}
