/* MAINTAINER: Lukas */

#ifndef _CONFRONTOR_H
#define _CONFRONTOR_H

#include "worldentity.h"

class Collision;

/**
 * Base class of confrontor
 *
 * Confrontor determines whether is some objects relevat or irrelevant for player
 */
class Confrontor
{
public:
	virtual ~Confrontor() { };

	/**
	 * Determines whether is object relevat or irrelevant for player
	 *
	 * \param we tested world entity
	 *
	 * \return true if world entity is relevant false otherwise
	 */
	// not const becouse we->getID() isn't const
	virtual bool isObjectRelevant(WorldEntity * we) = 0;

	/**
	 * Determines whether is collision relevat or irrelevant for player
	 *
	 * \param coll tested collision
	 *
	 * \return true if world entity is relevant false otherwise
	 */
	virtual bool isCollisionRelevant(const Collision & coll) = 0;

	/**
	 * Init method called before every player
	 */
	virtual void init(int playerID) = 0;

	/**
	 * Clean method called before every player
	 */
	virtual void clean() = 0;

protected:
	Confrontor() { };

};

#endif // _CONFRONTOR_H
