/* Maintainer: Petko */
#ifndef _IORTRANSFER_SERVER_H_
#define _IORTRANSFER_SERVER_H_

#include "tcpiptools.h"
#include "thread.h"
#include <string>

/**	\brief Holds UDP server used to transfer IOR
 *
 */
class IORTransferServer : public Thread
{
public:
	/**	\brief Fills data that are transmitted to clients and defines port to listen on
	 *
	 *	\param Port UDP port to listen on
	 *	\param IOR IOR of the server
	 *	\param HelloString Welcome string that will be send to the clients
	 */
	IORTransferServer(	unsigned short Port, const std::string & IOR, 
						const std::string & HelloString );
	virtual ~IORTransferServer(){};

	/**	\brief Sets the map description that will be sent to clients
	 *	\note Thread safe
	 *
	 *	\param MapDesc Map description
	 */
	void setMapDescription( const std::string & MapDesc );

	/**	\brief Sets the number of players that will be sent to clients
	 *	\note Thread safe
	 *
	 *	\param PlCount Number of players
	 */
	void setPlayerCount( int PlCount );

	/**	\brief UDP server 'main'
	 */
	virtual int run();

private:
	TcpIpTools::BJS_SOCKET_TYPE mSocket;
	std::string mIOR;
	std::string mVersion;
	std::string mHelloString;
	std::string mMapDescription;
	int			mPlCount;
	unsigned short mPort;

	std::string handleRequest( const char * data );
	std::string addHeader( const std::string & data );

	Mutex		mSetMutex;

	IORTransferServer( const IORTransferServer & its );
};

#endif	//_IORTRANSFER_SERVER_H_

