/* MAINTAINER: Lukas */

#include "netobjects_server.h"
#include "application_server.h"
#include "events_server.h"
#include "game_server.h"
#include "networkmanager_server.h"

// Methods corresponding to IDL attributes and operations
void Connector_i::connect(const char* playerName, Client_ptr cl)
{
	EvPlayerConnected * ev = 
		new EvPlayerConnected(playerName, Client::_duplicate(cl));

	// push to EventQueue (threadsafe)
	getAppServer().getGame().getEventQueue().push(ev);
}

Server_i::Server_i(int playerID)
{
	assertL(playerID != INVALID_PLAYER_ID);

	mPlayerID = playerID;
	mActive = true;
};

void Server_i::disconnect()
{
	if(! mActive)
		return;

	EvPlayerDisconnected * ev = 
		new EvPlayerDisconnected(mPlayerID);

	// push to EventQueue (threadsafe)
	getAppServer().getGame().getEventQueue().push(ev);
}

void Server_i::sendCommand(const char* command)
{
	if(! mActive)
		return;

	// this creates new event from string stored in IDL
	// (const char *) is there because IDL is const
	// new string is created
	EvCommand * ev = 
		new EvCommand(command, mPlayerID);

	// push to EventQueue (threadsafe)
	getAppServer().getGame().getEventQueue().push(ev);
}

void Server_i::sendInputButton(CORBA::Long button, CORBA::Boolean value, CORBA::Double yaw, CORBA::Double pitch)
{
	if(! mActive)
		return;

	EvInputButton * ev = 
		new EvInputButton(button, value, yaw, pitch, mPlayerID);

	// push to EventQueue (threadsafe)
	getAppServer().getGame().getEventQueue().push(ev);
}

void Server_i::sendYawPitch(CORBA::Double yaw, CORBA::Double pitch)
{
	if(! mActive)
		return;

	EvYawPitch * ev = 
		new EvYawPitch(yaw, pitch, mPlayerID);

	// push to EventQueue (threadsafe)
	getAppServer().getGame().getEventQueue().push(ev);
}

void Server_i::gameInfoRequest()
{
	if(! mActive)
		return;

	EvGameInfoRequest * ev = 
		new EvGameInfoRequest(mPlayerID);

	// push to EventQueue (threadsafe)
	getAppServer().getGame().getEventQueue().push(ev);
}
