/* MAINTAINER: Lukas */

#ifndef _NETOBJECTS_SERVER_H
#define _NETOBJECTS_SERVER_H

#include "netobjects.h"
#include "log.h"

/**
 * Implements defined IDL operations
 */
class Connector_i: public POA_Connector,
				   public PortableServer::RefCountServantBase
{
public:
	Connector_i() { };
	virtual ~Connector_i() { };
	/**
	 * Recieves player name and pointer to the client's object
	 *
	 * \param playerName player name
	 * \param cl pointer to the clients object
	 */
	void connect(const char* playerName, Client_ptr cl);
}; 

class Server_i: public POA_Server,
				public PortableServer::RefCountServantBase
{
public:
	/**
	 * Creates new server network object for client
	 *
	 * \param playerID palyer that calls functions on this object
	 * \param gim game info manager
	 */
	Server_i(int playerID);
	virtual ~Server_i() { };

	/**
	 * Close connection between client and server
	 */
	void disconnect();

	/**
	 * Recieves command and creates new event for this command
	 *
	 * \param command string that contains command
	 */
	void sendCommand(const char* command);

	/*
	 * Send keyboard input + cannon position
	 *
	 * \param button affected button
	 * \param value button value
	 * \param yaw cannon yaw
	 * \param pitch cannon pitch
	 */
	void sendInputButton(CORBA::Long button, CORBA::Boolean value, CORBA::Double yaw, CORBA::Double pitch);
	
	/*
	 * Send cannon position
	 *
	 * \param yaw cannon yaw
	 * \param pitch cannon pitch
	 */
	void sendYawPitch(CORBA::Double yaw, CORBA::Double pitch);

	/**
	 * Deactivate object
	 */
	// I hope this is atomic(no incorrect data) if accessed(mActive) from multiple threads
	void deactivate() { mActive = false; };

	/**
	 * Creates event that sends game info to client
	 */
	void gameInfoRequest();

private:
	int mPlayerID;
	bool mActive;
};

#endif // _NETOBJECTS_SERVER_H
