/* MAINTAINER: Lukas */

#ifndef _NETWORKMANAGER_SERVER_H
#define _NETWORKMANAGER_SERVER_H

#include "netobjects.h"
#include "packermanager.h"
#include "basicconfrontor.h"
#include "settings.h"
#include "collision_bjs.h"
#include "game.h"
#include "orbmanager.h"

/**
 * Manages all network related stuff
 */
class NetworkManagerServer
{

public:
	NetworkManagerServer() : mPackerManager(new BasicConfrontor()) { };
	virtual ~NetworkManagerServer() { };

	/**
	 * Initializes ORB and PackerManager
	 */
	void init();

	/**
	 * Cleanups ORB and PackerManager
	 */
	void clean();

	// TODO LUKAS toto predelat na job
	// network manager bude mit jeden  vlastni thread
	/**
	 * Callback for client if connection has been processed
	 *
	 * \param playerID player's id
	 * \param cl client's corba object
	 * \param msg additional message for client default is OK
	 */
	void addPlayerCallback(Client_var client, int playerID, string msg = "OK");

	// TODO LUKAS toto predelat na job
	void gameInfoCallback(const GameInfo & gi, int playerID);

	/**
	 * Collect data from game state
	 */
	void collectDataForClients();

	// TODO LUKAS predelat ze se bude kazdemu klientovi posilat ruzne
	// v teto metode se vyhodnoti zda uz nekomu poslat a pokud ano tak vytvori job
	/**
	 * Sends all prepared data to clients
	 */
	void sendDataToClients();

	/**
	 * Return number of clients
	 *
	 * \retunr number of clients
	 */
	int numberOfClients();

	/**
	 * Sends command to all players
	 *
	 * \param command string that contains command
	 */
	void sendCommand(string command);

	/**
	 * Sends command to player
	 *
	 * \param command string that contains command
	 * \param playerID deliver command to this palyer
	 */
	void sendCommand(string command, int playerID);

	/**
	 * Send collision to all clients
	 *
	 * \param sc collision to send
	 */
	void sendCollision(const Collision & sc);

	/**
	 * Returns PackerManager
	 *
	 * \return PackerManager
	 */
	PackerManager & getPackerManager() { return mPackerManager; };

	/**
	 * Returns OrbManager
	 *
	 * \return OrbManager
	 */
	OrbManager & getOrbManager() { return mOrbManager; };

	// TODO LUKAS debug smazat ----
	
	struct debug
	{
		char * data;
		int length;
		int id;
	};

	void sendDebugData(char * data, int length, int id)
	{
		debug d;
		d.data = new char[1024];

		for(int i = 0; i < 1024 && i < length; ++i)
			d.data[i] = data[i];
		
		d.length = length;
		d.id = id;

		mDebugData.push_back(d);
	};

	typedef list<debug> dd;

	dd mDebugData;

	// TODO LUKAS debug smazat ----

private:
	// no copy
	NetworkManagerServer(NetworkManagerServer &) : mPackerManager(new BasicConfrontor()) { };

	/**
	 * Makes game state point that will be send to client
	 *
	 * \param playerID get data from this player
	 */
	void makeSyncPoint(int playerID, NETGameState & data);

	// TODO comment
	void determinePings();
	
	/**
	 * PackerManager
	 */
	PackerManager mPackerManager;

	/**
	 * OrbManager
	 */
	OrbManager mOrbManager;
};

#endif // _NETWORKMANAGER_SERVER_H
