/* MAINTAINER: Lukas */

#include "orbmanager.h"
#include "log.h"
#include "corbaexceptionmacros.h"
#include "settings.h"

void OrbManager::startOrb(int port)
{
	// reset synchronizer
	mOrbStartSync.reset();

	// TODO LUKAS nacist default
	if((port <= 0) || (port > 65535))
	{
		LOGE << "Invalid CORBA port number: " << port << std::endl;
		DIE();
	}

	mOrbThread.setCorbaPort(port);
	mOrbThread.start();
}

void OrbManager::stopOrb()
{
	if(g_settings.getBool(SETTING_NETWORK_CORBA_SHUTDOWN))
	{

		// wait for start - if no start in progress -> no waiting 
		waitForOrbStart();
		
		if(! CORBA::is_nil(mOrb))
		{
			try
			{
				mOrb->shutdown(false);
			}
			catch(CORBA::SystemException& e)
			{
				SYSEXCEPTION_FATALERROR("Corba shutdown failed")
			}
			catch(CORBA::Exception& e)
			{
				EXCEPTION_FATALERROR("Corba shutdown failed")
			}
			catch(omniORB::fatalException& e)
			{
				OMNIORB_FATAL_EXCEPTION("Corba shutdown failed")	
			}
			catch(...)
			{
				UNKNOWN_ERROR("Corba shutdown failed")
			}
		}
		else
			DIE();

		// wait for orb shutdown
		mOrbThread.wait();
	}
}

void OrbManager::setServerIOR(CORBA::String_var serverIOR)
{
	mOrbVarsLock.lock();
	mServerIOR = serverIOR;
	mOrbVarsLock.unlock();
}

std::string OrbManager::getServerIOR()
{ 
	mOrbVarsLock.lock();
	std::string s = mServerIOR;
	mOrbVarsLock.unlock();

	return  s;
}

void OrbManager::setOrb(CORBA::ORB_var orb)
{
	mOrbVarsLock.lock();
	mOrb = orb;
	mOrbVarsLock.unlock();
}

CORBA::ORB_var OrbManager::getOrb()
{
	mOrbVarsLock.lock();
	CORBA::ORB_var orb = mOrb;
	mOrbVarsLock.unlock();

	return orb;
}

void OrbManager::waitForOrbStart()
{
	// no starting no waiting :)
	mOrbStartSync.waitForCompleting();
}

void OrbManager::orbStartedCallback(bool successfuly)
{
	// !!! must be called from different thread then waitForOrb() ;)

	mOrbVarsLock.lock();
	mOrbRunning = successfuly;
	mOrbVarsLock.unlock();

	mOrbStartSync.setCompletion();
}

bool OrbManager::orbRunning()
{
	if(! mOrbStartSync.isCompleted())
		// orb starting now
		return false;

	// was start successful?
	mOrbVarsLock.lock();
	bool res = mOrbRunning;
	mOrbVarsLock.unlock();

	return res;
}
