/* MAINTAINER: Lukas */

#ifndef _ORBMANAGER_H
#define _ORBMANAGER_H

#include "orbthread.h"
#include "netobjects.h"
#include "synchronizer.h"

class OrbManager
{
friend class OrbThread;

public:
	OrbManager() : mOrbStartSync(true) // no orb start now - we must set true(action is done)
	{
		mOrbRunning = false;
	}

	/**
	 * Initializes and starts ORB
	 */
	void startOrb(int port);

	/**
	 * Cleanups ORB
	 */
	void stopOrb();

	/**
	 * Gets Orb
	 *
	 * \return Orb
	 */
	CORBA::ORB_var getOrb();
	
	/**
	 * Gets IOR
	 *
	 * \return string that contains IOR
	 */
	std::string getServerIOR();

	/**
	 * This function waits for orb start
	 */
	void waitForOrbStart();

	/**
	 * Determines if orb starts successfuly
	 *
	 * \return true if orb starts successfuly, false otherwise
	 */
	bool orbRunning();

protected:

	/**
	 * Sets Orb that is commanded by the OrbManager
	 *
	 * \param orb pointer to Orb
	 */
	void setOrb(CORBA::ORB_var orb);

	/**
	 * Sets server IOR that is send to clients
	 *
	 * \param serverIOR pointer to corba string that contains IOR
	 */
	void setServerIOR(CORBA::String_var serverIOR);

	/**
	 * Helper function for orb thread indication
	 * Sets result of orb start
	 *
	 * \param successful start
	 */
	void orbStartedCallback(bool successfuly);

private:

	/**
	 * Mutex that locks OrbVars
	 */
	Mutex mOrbVarsLock;

	// OrbVars ---
	
	/**
	 * Pointer to ORB that handles communication
	 */
	CORBA::ORB_var mOrb;

	/**
	 * Server IOR that is send to clients
	 */
	std::string mServerIOR;

	/**
	 * Orb running
	 */
	bool mOrbRunning;

	// OrbVars ---

	/**
	 * This is thread that runs Orb
	 */
	OrbThread mOrbThread;

	/**
	 * Orb start synchronizer
	 */
	Synchronizer mOrbStartSync;
};

#endif // _ORBMANAGER_H
