/* MAINTAINER: Lukas */

#include "orbthread.h"
#include "application_server.h"
#include "netobjects_server.h"
#include "log.h"
#include "networkmanager_server.h"
#include "corbaexceptionmacros.h"
#include "conversions.h"

int OrbThread::run()
{
	TIN

	CORBA::ORB_var orb;

	LOGD << "Starting orb..." << endl;

	try
	{
		const int configLength = 9;

		int count = configLength;
		char * config[configLength];
		
		config[0] = "nic.exe";
		// BiDirectional settings
		config[1] = "-ORBacceptBiDirectionalGIOP";
		config[2] = "1";
		config[3] = "-ORBserverTransportRule";
		config[4] = "* unix,tcp,bidir";
		
		// port settings
		config[5] = "-ORBendPoint";
		char portBuffer[100];
		strcpy(portBuffer, "giop:tcp::");
		strcat(portBuffer, itos(mCorbaPort).c_str());
		config[6] = portBuffer;

		// trace settings
		config[7] = "-ORBtraceLevel";
		char traceBuffer[100];
		strcpy(traceBuffer, itos(g_settings.getInt(SETTING_NETWORK_CORBA_TRACE_LEVEL)).c_str());
		config[8] = traceBuffer;
	
		// Initialise the ORB.
		orb = CORBA::ORB_init(count, config);
		assertL(! CORBA::is_nil(orb));

		// Obtain a reference to the root POA.
		CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
		assertL(! CORBA::is_nil(obj));
		PortableServer::POA_var rootpoa = PortableServer::POA::_narrow(obj);
		assertL(! CORBA::is_nil(rootpoa));

		// Obtain a POAManager.
		PortableServer::POAManager_var pman = rootpoa->the_POAManager();
		assertL(! CORBA::is_nil(pman));

		// Create a POA with the Bidirectional policy
		CORBA::PolicyList pl;
		pl.length(1);
		CORBA::Any a;
		a <<= BiDirPolicy::BOTH;
		pl[0] = orb->create_policy(BiDirPolicy::BIDIRECTIONAL_POLICY_TYPE, a);

		PortableServer::POA_var poa = rootpoa->create_POA("bidir", pman, pl);
		assertL(! CORBA::is_nil(poa));

		// We allocate the objects on the heap.  Since these are reference
		// counted objects, they will be deleted by the POA when they are no
		// longer needed.
		Connector_i* myConnector_i = new Connector_i();

		// Activate the objects.  This tells the POA that the objects are
		// ready to accept requests.
		poa->activate_object(myConnector_i);

		// Obtain a reference to each object and output the stringified
		
		// IOR
		CORBA::Object_var ref = myConnector_i->_this();
		CORBA::String_var serverIOR = orb->object_to_string(ref);
		myConnector_i->_remove_ref();

		// set server IOR
		getAppServer().getNetworkManager().getOrbManager().setServerIOR(serverIOR);

		// Tell the POA to start accepting requests on its objects.
		pman->activate();

		// set orb
		getAppServer().getNetworkManager().getOrbManager().setOrb(orb);

		// orb "started" successfuly
		getAppServer().getNetworkManager().getOrbManager().orbStartedCallback(true);

		// run orb
		orb->run();
	}
	catch(CORBA::SystemException& e)
	{
		// orb started unsuccessfuly
		getAppServer().getNetworkManager().getOrbManager().orbStartedCallback(false);

		SYSEXCEPTION_FATALERROR("Corba start failed")
	}
	catch(CORBA::Exception& e)
	{
		// orb started unsuccessfuly
		getAppServer().getNetworkManager().getOrbManager().orbStartedCallback(false);

		EXCEPTION_FATALERROR("Corba start failed")
	}
	catch(omniORB::fatalException& e)
	{
		// orb started unsuccessfuly
		getAppServer().getNetworkManager().getOrbManager().orbStartedCallback(false);

		OMNIORB_FATAL_EXCEPTION("Corba start failed")
	}
	catch(...)
	{
		// orb started unsuccessfuly
		getAppServer().getNetworkManager().getOrbManager().orbStartedCallback(false);

		UNKNOWN_ERROR("Corba start failed")
	}

	try
	{
		orb->destroy();
	}
	catch(CORBA::SystemException& e)
	{
		SYSEXCEPTION_FATALERROR("Corba shutdown failed")
	}
	catch(CORBA::Exception& e)
	{
		EXCEPTION_FATALERROR("Corba shutdown failed")
	}
	catch(omniORB::fatalException& e)
	{
		OMNIORB_FATAL_EXCEPTION("Corba shutdown failed")	
	}
	catch(...)
	{
		UNKNOWN_ERROR("Corba shutdown failed")
	}

	return 0;
}
