/* MAINTAINER: Lukas */

#include "packermanager.h"
#include "application_server.h"
#include "game_server.h"

PackerManager::PackerManager(Confrontor * c)
{ 
	mPlayerID = INVALID_PLAYER_ID;

	assertL(c);
	mConfrontor = c;
}

PackerManager::~PackerManager()
{
	delete mConfrontor;
}

void PackerManager::pack(int playerID, NETGameState & gameState)
{
	assertL(mSContainer);
	assertL(mConfrontor);
	assertL(playerID != INVALID_PLAYER_ID);

	mConfrontor->init(playerID);

	tSWorkerList::iterator end = mSWorkerList.end();

	mPlayerID = playerID;

	// for all registered workers call process
	for(tSWorkerList::iterator runer = mSWorkerList.begin();
		runer != end; ++runer)
	{
		(dynamic_cast<PackerWorker *>(*runer))->pack(gameState);
	}

	// clean up
	mPlayerID = INVALID_PLAYER_ID;

	mConfrontor->clean();
}

PlayerNetDataServer & PackerManager::getNetData()
{
	assertL(mPlayerID != INVALID_PLAYER_ID);
	
	return 
		dynamic_cast<PlayerNetDataServer &>(getAppServer().getGame().getPlayer(mPlayerID).netData());
}
