/* MAINTAINER: Lukas */

#ifndef _PACKERMANAGER_H
#define _PACKERMANAGER_H

#include "packercontainer.h"
#include "globals.h"
#include "log.h"
#include "confrontor.h"
#include "playernetdata_server.h"

/**
 * Runs all packers and that unpacks whole game state
 */
class PackerManager : public SerializeManager
{
public:
	PackerManager(Confrontor * c);
	virtual ~PackerManager();

	/**
	 * Create and register all packers
	 */
	virtual void init() { mSContainer = new PackerContainer(); };

	/**
	 * Packs WHOLE data to the given game state
	 *
	 * Call all workers that are registred(are in container).
	 *
	 * \param gameState net data where will be packed
	 * \param changesOnly pack only changes
	 */
	void pack(int playerID, NETGameState & gameState);

	/**
	 * Returns actual player id for which is packed data.
	 * \b This function is valid only in pack function! (When packers are working.) \b
	 *
	 * \return processed player ID
	 */
	int getPlayerID() { assertL(mPlayerID == INVALID_PLAYER_ID); return mPlayerID; };

	/**
	 * Returns confrontor object.
	 * \b This function is valid only in pack function! (When packers are working.) \b
	 *
	 * \return confrontor object
	 */
	Confrontor & getConfrontor() { assertL(mConfrontor); return *mConfrontor; };

	/**
	 * Returns actual player's net data
	 * \b This function is valid only in pack function! (When packers are working.) \b
	 *
	 * \return net data object
	 */
	PlayerNetDataServer & getNetData();

private:
	Confrontor * mConfrontor;

	int mPlayerID; ///< ID of set player
};

#endif // _PACKERMANAGER_H
