/* MAINTAINER: Lukas */
#include "packers.h"
#include "application_server.h"
#include "world.h"
#include "history.h"
#include "worldentity.h"
#include "game_server.h"
#include "networkmanager_server.h"
#include "packermanager.h"
#include "gameplaysettings.h"

using namespace std;

void HistoryPacker::pack(NETGameState &gameState)
{
	World & w = getAppServer().getGame().getWorld();

	worldent_iterator itEnd = w.getWorldEntitiesEnd();

	// get and proccess world entities
	for(worldent_iterator it = w.getWorldEntitiesBegin();
			it != itEnd; ++it)
	{
		WorldEntity * ent = *it;

		assertL(ent);

		// is object relevant for client?
		if(getPackerManager().getConfrontor().isObjectRelevant(ent))
		{
			if(getPackerManager().getNetData().getKnownWorldEntities().isObjectKnown(ent->getID()))
			{
				// if known then send
				if(ent->getPresentPoint())
				{
					// pack data to gamestate
					ent->getPresentPoint()->transferToNET(gameState);

					// update object in known entities
					if(KnownWE * kwe = getPackerManager().getNetData().getKnownWorldEntities().
						getKnownObject(ent->getID()))
					{
						kwe->setNetPoint(ent->getPresentPoint()->copy());
						kwe->setPredictable(true);
					}
				}
				else
					DIE();
			}
		}
	}
}

void CommandPacker::pack(NETGameState &gameState)
{
	// get list
	list<string> commandList = getPackerManager().getNetData().getCommands();

	// process all
	for(list<string>::iterator commandIt = commandList.begin();
			commandIt != commandList.end(); ++commandIt)
	{
		// pack data to gamestate

		// actual points at the end of commands
		int actual = gameState.commands.length();

		// add one command - actual points at the end
		gameState.commands.length(actual + 1);

		// pack command to net data
		gameState.commands[actual] = 
			commandIt->c_str();
	}

	// clear all send commands
	getPackerManager().getNetData().clearCommands();
}

void CollisionsPacker::pack(NETGameState &gameState)
{
	list<Collision>::const_iterator itEnd = getPackerManager().getNetData().getCollisions().end();

	for(list<Collision>::const_iterator it = 
		getPackerManager().getNetData().getCollisions().begin(); it != itEnd; ++it)
	{
		// is collison relevant for client?
		if(getPackerManager().getConfrontor().isCollisionRelevant(*it))
		{
			// pack data to gamestate
			
			// actual points at the end of collisions
			int actual = gameState.collisions.length();

			// add one collision - actual points at the end
			gameState.collisions.length(actual + 1);

			// pack collisions to net data
			gameState.collisions[actual].entity1ID = it->getWe1ID();
			gameState.collisions[actual].entity2ID = it->getWe2ID();
			
			gameState.collisions[actual].position.x = static_cast<float>(it->getPosition().x);
			gameState.collisions[actual].position.y = static_cast<float>(it->getPosition().y);
			gameState.collisions[actual].position.z = static_cast<float>(it->getPosition().z);
			
			gameState.collisions[actual].normal.x = static_cast<float>(it->getNormal().x);
			gameState.collisions[actual].normal.y = static_cast<float>(it->getNormal().y);
			gameState.collisions[actual].normal.z = static_cast<float>(it->getNormal().z);
		}
	}

	// erase all send collisions
	getPackerManager().getNetData().clearCollisions();
}

void NewObjectsPacker::pack(NETGameState &gameState)
{
	World & w = getAppServer().getGame().getWorld();

	worldent_iterator itEnd = w.getWorldEntitiesEnd();

	for(worldent_iterator it = w.getWorldEntitiesBegin();
			it != itEnd; ++it)
	{
		WorldEntity * ent = *it;

		assertL(ent);

		// is object relevant for client?
		if(getPackerManager().getConfrontor().isObjectRelevant(ent))
		{
			if(! getPackerManager().getNetData().getKnownWorldEntities().isObjectKnown(ent->getID()))
			{
				// if not known then new -> send
				if(ent->getPresentPoint())
				{
					// pack world entity data for create
					ent->packCreationData(gameState);

					// add new object to known entities
					getPackerManager().getNetData().getKnownWorldEntities().
						add(ent->getID(), ent->getPresentPoint()->copy());
				}
				else
					DIE();
			}
		}
	}
}

void DeletedObjectsPacker::pack(NETGameState &gameState)
{
	KnownWorldEntities::KnownObjectsIt knEnd = getPackerManager().getNetData()
		.getKnownWorldEntities().ItEnd();

	list<int> deleteList;

	for(KnownWorldEntities::KnownObjectsIt knIte = getPackerManager().getNetData()
		.getKnownWorldEntities().ItBegin(); knIte != knEnd; ++knIte)
	{
		// not in world entity list -> deleted -> send
		if(getAppServer().getGame().getWorld().getWorldEntity((*knIte)->getID()) == 0)
		{
			// pack data to gamestate

			// actual points at the end of deleteObj
			int actual = gameState.deletedObjs.length();

			// add one obj id - actual points at the end
			gameState.deletedObjs.length(actual + 1);

			// pack id to net data
			gameState.deletedObjs[actual] = (*knIte)->getID();

			deleteList.push_back((*knIte)->getID());
		}
	}

	// remove from known entities
	while(! deleteList.empty())
	{
		getPackerManager().getNetData().getKnownWorldEntities().
			remove(deleteList.front());

		deleteList.pop_front();
	}
}

void PlayersPacker::pack(NETGameState &gameState)
{
	// new and send players list
	vector<int>::const_iterator itEnd = getAppServer().getGame().getValidPlayersID().end();

	for(vector<int>::const_iterator it = getAppServer().getGame().getValidPlayersID().begin();
			it != itEnd; ++it)
	{
		if(getPackerManager().getNetData().getKnownPlayers().isObjectKnown(*it))
		{
			if(! 
				getPackerManager().getNetData().getKnownPlayers().getKnownObject(*it)->isSame(
					getAppServer().getGame().getPlayer(*it))
				)
			// known and different then last -> send
			{
				// pack data to gamestate

				// actual points at the end of players
				int actual = gameState.players.length();

				// add one obj id - actual points at the end
				gameState.players.length(actual + 1);

				Player & p = getAppServer().getGame().getPlayer(*it);

				// pack player to net data
				gameState.players[actual].pID = p.getID();
				gameState.players[actual].deaths = p.getDeaths();
				gameState.players[actual].kills = p.getKills();
				gameState.players[actual].score = p.getScore();
				gameState.players[actual].team = p.getTeam();
				gameState.players[actual].name = p.getName().c_str();
				gameState.players[actual].ready = p.isReady();

				// update player in known players

				if(KnownPlayer * kp =
					getPackerManager().getNetData().getKnownPlayers().getKnownObject(*it))
				{
					kp->update(p.getKills(),
						p.getDeaths(), p.getScore(), p.getTeam(), p.getName(), p.isReady());
				}
			}
		}
	}
}

void NewPlayersPacker::pack(NETGameState &gameState)
{
	vector<int>::const_iterator itEnd = getAppServer().getGame().getValidPlayersID().end();

	for(vector<int>::const_iterator it = getAppServer().getGame().getValidPlayersID().begin();
			it != itEnd; ++it)
	{
		if(! getPackerManager().getNetData().getKnownPlayers().isObjectKnown(*it))
		// if not known then new -> send
		{
			// pack data to gamestate

			// actual points at the end of newPlayers
			int actual = gameState.newPlayers.length();

			// add one obj id - actual points at the end
			gameState.newPlayers.length(actual + 1);

			Player & p = getAppServer().getGame().getPlayer(*it);

			// pack new player to net data
			gameState.newPlayers[actual].pID = p.getID();
			gameState.newPlayers[actual].name = p.getName().c_str();

			// add new player to known entities

			getPackerManager().getNetData().getKnownPlayers().add(*it, p.getName());
		}
	}
}

void DeletedPlayersPacker::pack(NETGameState &gameState)
{
	KnownPlayers::KnownObjectsIt knEnd = getPackerManager().getNetData().
		getKnownPlayers().ItEnd();

	list<int> deleteList;

	for(KnownPlayers::KnownObjectsIt knIte = getPackerManager().getNetData().
		getKnownPlayers().ItBegin(); knIte != knEnd; ++knIte)
	{
		// not valid -> deleted
		if(! getAppServer().getGame().isPlayerIDValid((*knIte)->getID()))
		{
			// pack data to gamestate

			// actual points at the end of deletedPlayers
			int actual = gameState.deletedPlayers.length();

			// add one obj id - actual points at the end
			gameState.deletedPlayers.length(actual + 1);

			// pack id to net data
			gameState.deletedPlayers[actual] = (*knIte)->getID();

			deleteList.push_back((*knIte)->getID());
		}
	}

	// remove from known entities
	while(! deleteList.empty())
	{
		getPackerManager().getNetData().getKnownPlayers().
			remove(deleteList.front());
	
		deleteList.pop_front();
	}
}

void GameStateNumberPacker::pack(NETGameState &gameState)
{
	gameState.gsNumber = getPackerManager().getNetData().getGameStateNumber();
	
	// increment game state number
	getPackerManager().getNetData().incGameStateNumber();
}

void StateOfGamePacker::pack(NETGameState &gameState)
{
	if(getAppServer().getGame().getPlaySettings())
	{
		eStateOfGame sog;

		if( (sog = getAppServer().getGame().getPlaySettings()->getStateOfGame())
			!= getPackerManager().getNetData().getStateOfGame())
		{
			// Over network is enum send as int
			gameState.stateOfGame = sog;

			getPackerManager().getNetData().setStateOfGame(sog);
		}
		else
			gameState.stateOfGame = NETNoChange;
	}
}

void ConditionStatusPacker::pack(NETGameState &gameState)
{
	if(getPackerManager().getNetData().getGameStateNumber() %
		PlayerNetDataServer::UpdateIntervals::ConditionStatusUpdateInterval
		!= 0)
		return;

	if(getAppServer().getGame().getPlaySettings())
	{
		const std::list< ConditionStatus > & csList = 
			getAppServer().getGame().getPlaySettings()->getCondStatuses();

		std::list< ConditionStatus >::const_iterator itEnd = csList.end();

		for(std::list< ConditionStatus >::const_iterator it = csList.begin();
				it != itEnd; ++it)
		{
			// pack data to gamestate

			// actual points at the end of conditionStatuses
			int actual = gameState.conditionStatuses.length();

			// add one conditionStatus - actual points at the end
			gameState.conditionStatuses.length(actual + 1);

			// pack conditionStatus to net data
			gameState.conditionStatuses[actual].name = it->name.c_str();
			gameState.conditionStatuses[actual].targetValue = it->targetvalue;
			gameState.conditionStatuses[actual].currentValue = it->currentvalue;
		}

		// TODO pokud je list prazdny tak tam nasypat nejake empty
	}
}

// HERE PLACE YOU PACKER CLASS IMPLEMENTATION(PROCESS METHOD)
