/* MAINTAINER: Lukas */

#ifndef _PACKERS_H
#define _PACKERS_H

#include "packerworker.h"

/**
 * Packer for HistoryPoint
 */
class HistoryPacker : public PackerWorker
{
public:
	HistoryPacker() { };
	virtual ~HistoryPacker() { };

	/**
	 * Pack history points to the given gamestate
	 *
	 * If changesOnly is false then calls transfer agent for 
	 * all worldentities in game.
	 * If changesOnly is set true then compares data that was send
	 * before with current data. If they are different then send
	 * current data. Otherwise do nothing.
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for Commands
 */
class CommandPacker : public PackerWorker
{
public:
	CommandPacker() { };
	virtual ~CommandPacker() { };

	/**
	 * Pack commands to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for Collisions
 */
class CollisionsPacker : public PackerWorker
{
public:
	CollisionsPacker() { };
	virtual ~CollisionsPacker() { };

	/**
	 * Pack collisions to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for new objects
 */
class NewObjectsPacker : public PackerWorker
{
public:
	NewObjectsPacker() { };
	virtual ~NewObjectsPacker() { };

	/**
	 * Pack new objects to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for deleted objects
 */
class DeletedObjectsPacker : public PackerWorker
{
public:
	DeletedObjectsPacker() { };
	virtual ~DeletedObjectsPacker() { };

	/**
	 * Pack deleted objects to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for changed players
 */
class PlayersPacker : public PackerWorker
{
public:
	PlayersPacker() { };
	virtual ~PlayersPacker() { };

	/**
	 * Pack changed to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for new players
 */
class NewPlayersPacker : public PackerWorker
{
public:
	NewPlayersPacker() { };
	virtual ~NewPlayersPacker() { };

	/**
	 * Pack new players to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for deleted players
 */
class DeletedPlayersPacker : public PackerWorker
{
public:
	DeletedPlayersPacker() { };
	virtual ~DeletedPlayersPacker() { };

	/**
	 * Pack deleted players to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for game state number
 */
class GameStateNumberPacker : public PackerWorker
{
public:
	GameStateNumberPacker() { };
	virtual ~GameStateNumberPacker() { };

	/**
	 * Pack game state number to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for state of game
 */
class StateOfGamePacker : public PackerWorker
{
public:
	StateOfGamePacker() { };
	virtual ~StateOfGamePacker() { };

	/**
	 * Pack game state number to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

/**
 * Packer for condition statuses
 */
class ConditionStatusPacker : public PackerWorker
{
public:
	ConditionStatusPacker() { };
	virtual ~ConditionStatusPacker() { };

	/**
	 * Pack condition statuses to the given gamestate
	 *
	 * \param gameState net data where will be packed
	 */
	virtual void pack(NETGameState & gameState);
};

// HERE PLACE YOU PACKER CLASS

#endif // _PACKERS_H
