/* MAINTAINER: Petko */
#include "tasksimulationserver.h"
#include "settings.h"

//#define WANT_TRACE

#include "timemanager.h"
#include "log.h"
#include "application_server.h"
#include "terrain.h"
#include "physical_terrain.h"
#include "serverphysicsdata.h"
#include "tank.h"
#include "globals.h"
#include "world.h"
#include "projectile.h"
#include "game_server.h"

#undef TESTING_ODE

#ifdef TESTING_ODE
#include "networkmanager_server.h"

struct _info
{
	GLVector3d pos;
	GLQuaterniond ori;
};
bool sent=false;
#endif

//TODO: Remove this var
int run = 0, id= 0;

//TODO: Toto by MOZNO mohlo byt v konfigu, ale mozno aj nie...
const double	ODE_GLOBAL_ERP = 0.8;
const double	ODE_GLOBAL_CFM = 1.0e-6;
const int		ODE_MAX_CONTACTS	= 64;
const int		ODE_MAX_TOTAL_CONTACTS = 4096;
const int		ODE_TERRAIN_GEOMS_ID = 0x1234;
const int		ODE_LANDSCAPE_GEOMS_ID = 0x4321;
const double	ODE_MAX_OBJECT_SIZE = 20;		///< \brief The maximum expected size of the objects in the world - good value => better performance
const int		BONUS_FREE_ZONE_WIDTH = 10;		///< \brief The width of zone at the border of the map, where don't fall bonusboxes
const int		BONUS_FALL_HEIGTH = 20;			///< \brief The heigth over the terrain, from which the bonusboxes will be falling down
const int		AREA_DMG_LINEAR_PERCENTAGE = 40;///< \brief Per cents of area damage radius in that the damage falls only linear
const double	AREA_DMG_LINEAR_CONSTANT = 0.25;///< \brief Linear constant that describes how fast fall damage in linear case. 1 means damage goes directly to 0 in radius distance, 0 means no fall.
const double	AREA_DMG_DMG2FORCE_MULTIP = 0.4;///< \brief The damage that is given to object (in area dmg) is multiplied by this and its result is force that is applied to entity
const double	MOVEMENT_OPPTRACK_MULTIPLE =0.2;///< \brief Ratio of speed of inner and outer track differences

const double	ODE_TANK_MU = 20;
const double	ODE_TANK_SLIP = 0.16;
const double	ODE_TANK_SOFT_ERP = 0.5;
const double	ODE_TANK_SOFT_CFM = 0.3;

const double	ODE_CONT_MU = 10;
const double	ODE_CONT_BOUNCE = 0.5;
const double	ODE_CONT_BOUNCE_VEL = 0.0;
const double	ODE_CONT_SOFT_CFM = 0.01;

TaskSimulationServer::TaskSimulationServer()
{
	mSimulatedTime = 0;

	initODE();
}

TaskSimulationServer::~TaskSimulationServer()
{
	destroyODE();

	LOGD << "Number of simulation runs: "<< run << endl ;
}

void TaskSimulationServer::tick( double time_change, int time_stamp )
{
#ifdef TESTING_ODE
	if (sent) return;
#endif

	initPlayers();

	initSimData();

	fireControll( time_stamp );

	updateSpeeds();

	
#ifdef TESTING_ODE

	if ( run % 1 == 0 && !sent)
		gAppServer->getNetworkManager().sendDebugData( 0, 0, 1024 );	

	World * wrld = &(gAppServer->getGame().getWorld());
	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	for ( ; item != end ; ++item )
	{
		(*item)->updateServer( time_stamp );
		if ( (*item)->getClassType() == cTank  && !sent )
		{
			Tank * tnk = (Tank*)(*item);

			struct _info in;
			TankPhysicsData * tpd = (TankPhysicsData*)(tnk->getPhysicsData());

			in.pos = dBodyGetPosition( tpd->mMainBody );
			in.ori = dBodyGetQuaternion( tpd->mMainBody );
			gAppServer->getNetworkManager().sendDebugData( (char*)&in, sizeof in, 256 );

			in.pos = dBodyGetPosition( tpd->mLeftTrack );
			in.ori = dBodyGetQuaternion( tpd->mLeftTrack );
			gAppServer->getNetworkManager().sendDebugData( (char*)&in, sizeof in, 256 );

			in.pos = dBodyGetPosition( tpd->mRightTrack );
			in.ori = dBodyGetQuaternion( tpd->mRightTrack );
			gAppServer->getNetworkManager().sendDebugData( (char*)&in, sizeof in, 256 );

			dVector3 jp;
			dJointGetHinge2Anchor( tpd->mRightJoints[0], jp );
			GLVector3d js = jp;
			gAppServer->getNetworkManager().sendDebugData( (char*)&js, sizeof js, 1 );

			dJointGetHinge2Anchor( tpd->mRightJoints[1], jp );
			js = jp;
			gAppServer->getNetworkManager().sendDebugData( (char*)&js, sizeof js, 1 );

			//sent = true;

			//gTime.pauseSwitch();
		}
	}
#endif

		//collision detection
	dSpaceCollide( mODESpace, this, nearCallback );
		
		//needs information about contacts - for wheels to turn
	tankControll();

		//simulation step - think about changing it to dWorldStep
	dWorldQuickStep( mODEWorld, mStepSize );
	//dWorldStep( mODEWorld, mStepSize );

		//destroy all contact joints
	dJointGroupEmpty( mODEContacts );

		//check projectile collisions as ray - vs - everything
	projectileAdvCollisionDet();

		//apply damage done by projectiles
	applyDamage();

		//change entity properties according to that from ODE
#ifdef TESTING_ODE
	if (!sent) updateSimChanges();
#else
	updateSimChanges();
#endif

		//cleanup objects, that are too far or ready for deletion
	cleanupObjects();

		//destroys players that are marked for deletion
	cleanupPlayers();

		//call updateServer() on every object
	updateObjects( time_stamp );

		//drops bonuses
	dropBonusBoxes( time_change );

	updateHistory( time_stamp );

	run++;
}

void TaskSimulationServer::nearCallback( void *data, dGeomID o1, dGeomID o2 )
{
	assertL( o1 != o2 );

	TaskSimulationServer * simtask = (TaskSimulationServer*)data;

	if ( dGeomIsSpace( o1 ) || dGeomIsSpace( o2 ) )
	{
		dSpaceCollide2( o1, o2, data, nearCallback );
		if ( (dSpaceID)o1 == simtask->mODEOtherSpace )
			dSpaceCollide( (dSpaceID)o1, data, nearCallback );
		if ( (dSpaceID)o2 == simtask->mODEOtherSpace )
			dSpaceCollide( (dSpaceID)o2, data, nearCallback );
		return;
	}
	
	dBodyID bid1 = dGeomGetBody( o1 );
	dBodyID bid2 = dGeomGetBody( o2 );

		//ignoring "collisions" between static objects
	if ( bid1 == 0 && bid2 == 0 )
		return;

	if 
	( 
		bid1 && bid2 && 
		dAreConnectedExcluding( bid1, bid2, dJointTypeContact ) 
	)
		return;


	// ignore collisions between tracks and anything except static environment
	if	
	( 
		bid1 && ( ((ODEBodyInfo*)dBodyGetData( bid1 ) )->type == bTrack ) &&
		bid2 != 0
	)
		return;
	if	
	( 
		bid2 && ( ((ODEBodyInfo*)dBodyGetData( bid2 ) )->type == bTrack ) &&
		bid1 != 0
	)
		return;

	WorldEntity * ent1 = 0, *ent2 = 0;
	if ( bid1 )
	{
		ent1 = gApp->getGameCommon().getWorld().getWorldEntity( ((ODEBodyInfo*)dBodyGetData( bid1 ) )->entID );
		assertL( ent1 );
	}
	if ( bid2 )
	{
		ent2 = gApp->getGameCommon().getWorld().getWorldEntity( ((ODEBodyInfo*)dBodyGetData( bid2 ) )->entID );
		assertL( ent2 );
	}

	//ignore collisions between projectiles
	if ( ent1 && ent2 && ( ent1->getClassType() == cProjectile ) && ( ent2->getClassType() == cProjectile ) )
		return;

	//DEBUG:
	if ( (ent1 && ( ent1->getClassType() == cProjectile )) || ( ent2 && ( ent2->getClassType() == cProjectile ) ) )
		return;

	dContact contact[ODE_MAX_CONTACTS];
	int n = dCollide (o1,o2,ODE_MAX_CONTACTS,&contact[0].geom,sizeof(dContact));
	
	if (n > 0) 
	{
		if		//This checks if this is contact between static environment and tracks
		( 
			 ( !bid1 && bid2 && (((ODEBodyInfo*)dBodyGetData( bid2 ) )->type == bTrack )) ||
			 ( !bid2 && bid1 && (((ODEBodyInfo*)dBodyGetData( bid1 ) )->type == bTrack ))
		)
		{
			//set note that this wheel is set on terrain
			if ( bid1 )
				((ODEBodyInfo*)dBodyGetData( bid1 ))->spec.trackTerrContact= true;
			else
				((ODEBodyInfo*)dBodyGetData( bid2 ))->spec.trackTerrContact= true;

			for ( int i = 0; i < n; i++ ) 
			{
				contact[i].surface.mode = dContactSlip1 | dContactSlip2 |
					dContactSoftERP | dContactSoftCFM | dContactApprox1;

				contact[i].surface.mu = ODE_TANK_MU;
				contact[i].surface.slip1 = ODE_TANK_SLIP;
				contact[i].surface.slip2 = ODE_TANK_SLIP;
				contact[i].surface.soft_erp = ODE_TANK_SOFT_ERP;
				contact[i].surface.soft_cfm = ODE_TANK_SOFT_CFM;

				//////////////////////////////////////

				GLVector3d cdir;

				ODEBodyInfo * obi;
				
				bool tercnt = false;
	
				WorldEntity * we = 0;

				if ( bid1 == 0 || bid2 == 0 )
				{
					tercnt = true;
			
					if ( bid1 != 0 )
						obi = (ODEBodyInfo *) dBodyGetData( bid1 );
					else
						obi = (ODEBodyInfo *) dBodyGetData( bid2 );
				
					we = getApp().getGameCommon().getWorld().getWorldEntity( obi->entID );
					assertL( we->getClassType() == cTank );

					cdir= we->getOrientation().rotateVector( GLVector3d( 1,0,0) ); 
				}

				if ( tercnt )
				{
					contact[i].surface.mode |= dContactFDir1 | dContactMotion1;
					contact[i].fdir1[0] = cdir.x;
					contact[i].fdir1[1] = cdir.y;
					contact[i].fdir1[2] = cdir.z;

					TankPhysicsData * tpd = dynamic_cast< TankPhysicsData* >( we->getPhysicsData() );

					//left or right track
					if ( tpd->mLeftTrack == bid1 || tpd->mLeftTrack == bid2 )
					{
						contact[i].surface.motion1 = tpd->mLeftSpeed;
					}
					else
					{
						contact[i].surface.motion1 = tpd->mRightSpeed;
					}
				}

				//////////////////////////////////////

				dJointID c = dJointCreateContact (simtask->mODEWorld, simtask->mODEContacts, &contact[i]);
				dJointAttach 
				( 
					c, bid1, bid2
				);
#ifdef TESTING_ODE
				if ( run%1 == 0 )
				{
					GLVector3d contpos( contact[i].geom.pos );
					gAppServer->getNetworkManager().sendDebugData( (char*)&contpos, sizeof contpos, 1 );
				}
#endif
			}
		}
		else
		{
			for ( int i = 0; i < n; i++ ) 
			{
				contact[i].surface.mode = dContactBounce | dContactSoftCFM;
				contact[i].surface.mu = ODE_CONT_MU;
				contact[i].surface.bounce = ODE_CONT_BOUNCE;
				contact[i].surface.bounce_vel = ODE_CONT_BOUNCE_VEL;
				contact[i].surface.soft_cfm = ODE_CONT_SOFT_CFM;

				dJointID c = dJointCreateContact (simtask->mODEWorld, simtask->mODEContacts, &contact[i]);
				dJointAttach 
				( 
					c, //dGeomGetBody(contact[i].geom.g1),dGeomGetBody(contact[i].geom.g2)
					bid1, bid2
				);

#ifdef TESTING_ODE
				if ( run%10 == 0 )
				{
					GLVector3d contpos( contact[i].geom.pos );
					gAppServer->getNetworkManager().sendDebugData( (char*)&contpos, sizeof contpos, 1 );
				}
#endif
			}

			/* calling entity collision handlers */

			//1) One of colliding bodies is terrain
			if ( !ent1 || !ent2 )
			{
				(ent1 ? ent1 : ent2)->handleCollisionWStaticServer( contact[0].geom.pos, contact[0].geom.normal );
			}
			//2) One of colliding bodies is projectile
			if ( ent1 && ent2 )
			{
				if ( ent1->getClassType() == cProjectile )
					ent2->handleCollisionWProjectileServer( ent1->getID(), contact[0].geom.pos, contact[0].geom.normal );

				if ( ent2->getClassType() == cProjectile )
					ent1->handleCollisionWProjectileServer( ent2->getID(), contact[0].geom.pos, contact[0].geom.normal );
			}
			//3) Other case ( this includes 2), too )
			if ( ent1 && ent2 )
			{
				ent1->handleCollisionWEntityServer( ent2->getID(), contact[0].geom.pos, contact[0].geom.normal );
				ent2->handleCollisionWEntityServer( ent1->getID(), contact[0].geom.pos, contact[0].geom.normal );
			}

				//entities have contacts => force => unpredictable
			if ( ent1 )
				ent1->setUnpredictableFlag();
			if ( ent2 )
				ent2->setUnpredictableFlag();

		}
	}
}

bool TaskSimulationServer::initODE()
{
	mODEWorld = dWorldCreate();
	const double gravity_const = getApp().getGameCommon().getMapParams().gravity_constant;
		//g_settings.getDouble(SETTING_SIMULATION_GRAVITY_CONST);

	dWorldSetGravity( mODEWorld, 0, 0, -gravity_const );
	dWorldSetCFM( mODEWorld, ODE_GLOBAL_CFM );
	dWorldSetERP( mODEWorld, ODE_GLOBAL_ERP );
	//dWorldSetAutoDisableFlag( mODEWorld, 1 );
	dWorldSetContactSurfaceLayer( mODEWorld, 0.1 );
	
	/*
	PhysicalTerrain * terrain = 
		dynamic_cast<PhysicalTerrain*>(gApp->getGameCommon().getWorld().getLandscape().getTerrain().getTerrainBase());

	int xsz, ysz;
	terrain->getExtents(xsz, ysz );
	GLVector3d extents( xsz, ysz, 0 );
	extents.z = terrain->getScaledMaxHeight();
	GLVector3d center(0,0,0);
	center += extents / 2;

	int x_blocks = terrain->getBlocksCountX();
	int y_blocks = terrain->getBlocksCountY();
	*/
	mODESpace = dSimpleSpaceCreate( 0 );		//TODO: otestovat vykon v porovani s quadtree
	mODETerrainSpace = dHashSpaceCreate( mODESpace );
	mODEOtherSpace = dHashSpaceCreate( mODESpace );

	/*
	double blsz = ((double)max( extents.x, extents.y )) / ((double)min( x_blocks, y_blocks ));

	dHashSpaceSetLevels( mODETerrainSpace, 0.01, blsz < ODE_MAX_OBJECT_SIZE ? ODE_MAX_OBJECT_SIZE : blsz );
	*/

	mODEContacts = dJointGroupCreate( ODE_MAX_TOTAL_CONTACTS );

	ODEAddTerrain();

	ODEAddLandscapeEntities();

	return true;
}

void TaskSimulationServer::destroyODE()
{
	PhysicalTerrain * terrain = 
		dynamic_cast<PhysicalTerrain*>(gApp->getGameCommon().getWorld().getLandscape().getTerrain().getTerrainBase());

	int x_blocks = terrain->getBlocksCountX();
	int y_blocks = terrain->getBlocksCountY();

	dJointGroupDestroy( mODEContacts );

	for ( int i = 0; i < x_blocks; i++ )
		for ( int j = 0; j < y_blocks; j++ )
		{
			dGeomTriMeshClearTCCache( mTerrainGeoms[i][j] );
			dGeomTriMeshDataDestroy( mTerrainMeshes[i][j] );
		}

	delete[] mTerrainMeshes;

	dSpaceDestroy( mODESpace );

	delete[] mTerrainGeoms;

	dWorldDestroy( mODEWorld );
}

void TaskSimulationServer::ODEAddTerrain()
{

	PhysicalTerrain * terrain = 
		dynamic_cast<PhysicalTerrain*>(gApp->getGameCommon().getWorld().getLandscape().getTerrain().getTerrainBase());

	int x_blocks = terrain->getBlocksCountX();
	int y_blocks = terrain->getBlocksCountY();

	mTerrainGeoms = new vector< dGeomID >[x_blocks];
	mTerrainMeshes= new vector< dTriMeshDataID >[x_blocks];

	for ( int i = 0; i < x_blocks; i++ )
	{
		for ( int j = 0; j < y_blocks; j++ )
		{
			int vc, ic;
			double (*vertexy)[3];
			int * indexy;
			
			terrain->getTriMeshData( i,j, vertexy,vc, indexy, ic);

			for ( int k = 0; k < ic/3; k++ )
			{
				int (*inds)[3] = (int (*)[3])indexy;
				int h = inds[k][2];
				inds[k][2] = inds[k][1];
				inds[k][1] = h;
			}
			
			dTriMeshDataID trimesh = dGeomTriMeshDataCreate();
			dGeomTriMeshDataBuildDouble(trimesh, vertexy, 3 * sizeof(double), vc, indexy, ic, 3 * sizeof(int) );
			mTerrainMeshes[i].push_back( trimesh );

			dGeomID gid = dCreateTriMesh( mODETerrainSpace, trimesh, 0, 0, 0 );
			dGeomSetData( gid, (void*)((int)ODE_TERRAIN_GEOMS_ID) );

			dGeomTriMeshEnableTC( gid, dBoxClass, 1 );
			dGeomTriMeshEnableTC( gid, dSphereClass, 1 );

			mTerrainGeoms[i].push_back( gid );
		}
	}
}

void TaskSimulationServer::ODEAddLandscapeEntities()
{
	Landscape & lscape = gApp->getGameCommon().getWorld().getLandscape();
	landent_iterator it, ite = lscape.getLandEntitiesEnd();

	for  ( it = lscape.getLandEntitiesBegin(); it != ite; ++it )
	{
		LandscapeEntity * le = *it;

		ServerPhysicsData * spd = new LandscapeEntityPhysicsData;
		spd->createODERigidAndColModel( le->getID(), le->getPosition(), le->getOrientation(), this );
		le->setPhysicsData( spd );
	}
}

void TaskSimulationServer::addedWorldObject(WorldEntity *we)
{
	assertL( we->getPhysicsData() == 0 );		//if anything is in physics structure... is'a a bit strange

	ServerPhysicsData * spd;
	if ( we->getClassType()==cTank )
		spd = new TankPhysicsData;
	else
		spd = new ServerPhysicsData;
	spd->createODERigidAndColModel( we->getID(), we->getPosition(), we->getOrientation(), this );

	we->setPhysicsData( spd );

		//if this is bonus box, add it to the list of available bonuses
	if ( we->getClassType() == cBonusBox )
	{
		mBonuses.push_back( we->getID() );

		//TODO: na tie boxy by mala byt nejaka funkcia mozno, co by to davala spolu s globalnym obmedzenim
			//if there are too many bonus boxes, we delete the oldest one
		while ( (int)mBonuses.size() > getAppServer().getGame().getMapParams().max_bonus_boxes )
		{
			int to_del_id = mBonuses.front();
			mBonuses.pop_front();
			getApp().getGameCommon().getWorld().deleteBonusBox( to_del_id, true );
			//LOGD << "Removed old bonus" <<endl;
		}
		
	}

	if ( we->getClassType() == cProjectile )
	{
		mProjectiles.push_front( we->getID() );
	}
	
}

void TaskSimulationServer::deletedWorldObject(WorldEntity *we)
{
	ServerPhysicsData * spd = we->getPhysicsData();
	assertL( spd );

	std::list< dBodyID >::iterator it = spd->mBodies.begin();
	while ( it != spd->mBodies.end() )
	{
		ODEBodyInfo * obi = (ODEBodyInfo*)dBodyGetData( *it );

		std::list< dGeomID >::iterator geit;
		std::list< dGeomID >::iterator geite = obi->geoms.end();
		
		for ( geit = obi->geoms.begin(); geit != geite; ++geit )
		{
			//POZOR: Pri pouziti trimesov tu musi byt zrusenie dat!!!
			dGeomDestroy( *geit );
		}

		dBodyDestroy( *it );

		delete obi;

		++it;
	}
	
	std::list< dJointID >::iterator it2 = spd->mJoints.begin();
	while ( it2 != spd->mJoints.end() )
	{
		dJointDestroy( *it2 );
		++it2;
	}

	delete spd;

		//if this is bonus box, remove it from list of available bonuses
	if ( we->getClassType() == cBonusBox )
		mBonuses.remove( we->getID() );

	if ( we->getClassType() == cProjectile )
		mProjectiles.remove( we->getID() );
}

void TaskSimulationServer::tankControll()
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	double ode_to_real = mTimeDelta / (mStepSize * 1000);

	for ( ; item != end ; ++item )
	{
		if ( (*item)->getClassType() == cTank )
		{
			Tank * tnk;
			tnk = dynamic_cast< Tank* > (*item);

			tnk->tankControllServer( ode_to_real );
		}
	}
}

void TaskSimulationServer::fireControll( const int time_stamp )
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	PhysicalTerrain * terrain = 
		dynamic_cast<PhysicalTerrain*>( getAppServer().getGame().getWorld().getLandscape().getTerrain().getTerrainBase() );

	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	std::list< struct proj_info *> to_create;

	for ( ; item != end ; ++item )
	{
		if ( (*item)->getClassType() == cTank )
		{
			Tank * tnk;
			tnk = dynamic_cast< Tank* > (*item);

				//tank is 'dead' -> it cannot make any movement adjustments
				//TODO: toto mozno patri skor do Tank::checkDestroy(), ale je otazka, ci niekto zo siete potom este niekedy nebude chciet nastavovat komandy....
			if ( tnk->getHP() <= 0 )
			{
				tnk->getCmds().reset();
			}

			GLVector3d pos;
			GLQuaterniond ori;
			tnk->getShotOriAndStart( ori, pos );

			GLVector3d t_ext = terrain->getScale();
			
			if ( pos.x > 0 && pos.y > 0 && pos.x < t_ext.x && pos.y < t_ext.y && pos.z <= terrain->getHeight( pos.x, pos.y ) )
			{		//the shot starts under terrain...
				continue;	
			}

			int proj_id = -1;
			if ( tnk->getCmds().fire && (( proj_id = tnk->canFire( mTimeDelta, time_stamp ) ) != -1 ) )
			{	
				struct proj_info *pi = new struct proj_info;

				pi->pos = pos;
				pi->ori = ori;
				pi->type_id = proj_id;
				pi->player_id = tnk->getPlayerID();

				to_create.push_back( pi );
			}
		}
	}
	
	struct proj_info * pr_i = 0;
	while ( !to_create.empty() )
	{
		pr_i = to_create.front();
		to_create.pop_front();

		wrld->createProjectile( pr_i->pos, pr_i->ori, pr_i->type_id, pr_i->player_id );

		delete pr_i;
	}
}

void TaskSimulationServer::projectileAdvCollisionDet()
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	std::list< int >::iterator item	= mProjectiles.begin();
	std::list< int >::iterator end	= mProjectiles.end();

	dSpaceID rayspace = dHashSpaceCreate( 0 );

	int proj_num = 0;

	for ( ; item != end ; ++item )
	{
		assertL( wrld->getWorldEntity( *item )->getClassType() == cProjectile );	//nothing else than projectile can be here

		Projectile * proj = dynamic_cast< Projectile* >( wrld->getWorldEntity( *item ) );

		if ( proj->getHit() )
			continue;

		GLVector3d ray_beg = proj->getPosition();
		GLVector3d ray_end = dBodyGetPosition( proj->getPhysicsData()->mMainBody );
		GLVector3d ray_dif = ray_end - ray_beg;

		dGeomID ray = dCreateRay( rayspace, (ray_dif).length() );
		dGeomRaySet( ray, ray_beg.x, ray_beg.y, ray_beg.z, ray_dif.x, ray_dif.y, ray_dif.z );

		//void * entity_id = reinterpret_cast< void *> ( proj->getID() );

		dGeomSetData( ray, proj );
		proj_num++;
	}

	if ( proj_num )
	{
		list< ODE_proj_replace_info > replace_list;

		dSpaceCollide2( (dGeomID)mODESpace, (dGeomID)rayspace, &replace_list, TaskSimulationServer::projectileNearCallback );

		//move projectiles that has collided to their collision point
		while ( !replace_list.empty() )
		{
			const ODE_proj_replace_info & opri = replace_list.front();
			dBodySetPosition( opri.bid, opri.pos.x, opri.pos.y, opri.pos.z );
			replace_list.pop_front();
		}
	}

	dSpaceDestroy( rayspace );
}

void TaskSimulationServer::projectileNearCallback( void *data, dGeomID o1, dGeomID o2 )
{
	assertL( o1 != o2 );

	list< ODE_proj_replace_info > *replace_list = (list< ODE_proj_replace_info >*) data;

	if ( dGeomIsSpace( o1 ) || dGeomIsSpace( o2 ) )
	{
		dSpaceCollide2( o1, o2, data, projectileNearCallback );
		return;
	}
	
	dBodyID bid1 = dGeomGetBody( o1 );
	dBodyID bid2 = dGeomGetBody( o2 );

	// ignore collisions with wheels
	if ( bid1 && ( ((ODEBodyInfo*)dBodyGetData( bid1 ) )->type == bTrack ) )
		return;
	if ( bid2 && ( ((ODEBodyInfo*)dBodyGetData( bid2 ) )->type == bTrack ) )
		return;

	WorldEntity * ent1 = 0, *ent2 = 0;
	if ( bid1 )
	{
		ent1 = gApp->getGameCommon().getWorld().getWorldEntity( ((ODEBodyInfo*)dBodyGetData( bid1 ) )->entID );
		assertL( ent1 );
	}
	if ( bid2 )
	{
		ent2 = gApp->getGameCommon().getWorld().getWorldEntity( ((ODEBodyInfo*)dBodyGetData( bid2 ) )->entID );
		assertL( ent2 );
	}

	//ignore collisions with projectiles
	if ( (ent1 && ( ent1->getClassType() == cProjectile )) || ( ent2 && ( ent2->getClassType() == cProjectile ) ) )
		return;

	assertL( dGeomGetClass( o1 ) == dRayClass || dGeomGetClass( o2 ) == dRayClass );

	dContact contact[1];
	int n = dCollide( o1, o2, 1, &contact[0].geom, sizeof(dContact) );
	
	if (n > 0) // n == 0
	{
		dGeomID ray;
		dBodyID bdy;
		WorldEntity * ent = 0;
		if ( dGeomGetClass( o1 ) == dRayClass )
		{
			ray = o1;
			bdy = bid2;
			ent = ent2;
		}
		else
		{
			ray = o2;
			bdy = bid1;
			ent = ent1;
		}

		//int projID = reinterpret_cast< int >( dGeomGetData( ray ) );
		//Projectile* proj = dynamic_cast< Projectile* >(gApp->getGameCommon().getWorld().getWorldEntity( projID ));
		Projectile * proj = static_cast< Projectile* >( dGeomGetData( ray ) );
		assertL( proj );
		int projID = proj->getID();

		//move the projectile back to the point of collision 
		assertL( proj->getPhysicsData()->mBodies.size() == 1 );	//projectile cannot consist of more than 1 body

		//save information where to move the projectile - to the point of collision
		ODE_proj_replace_info opri;
		opri.bid = proj->getPhysicsData()->mMainBody;
		opri.pos = contact[0].geom.pos;
		replace_list->push_back( opri );

		if ( bdy == 0 )	//collision with terrain
			proj->handleCollisionWStaticServer( contact[0].geom.pos, contact[0].geom.normal );
		else
		{
			ent->handleCollisionWProjectileServer( projID, contact[0].geom.pos, contact[0].geom.normal );
			ent->handleCollisionWEntityServer( projID, contact[0].geom.pos, contact[0].geom.normal );
			proj->handleCollisionWEntityServer( ent->getID(), contact[0].geom.pos, contact[0].geom.normal );
		}
	}
}

void TaskSimulationServer::applyDamage()
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	std::list< int >::iterator item	= mProjectiles.begin();
	std::list< int >::iterator end	= mProjectiles.end();

	for ( ; item != end ; ++item )
	{
		assertL( wrld->getWorldEntity( *item )->getClassType() == cProjectile );	//nothing else than projectile can be here

		Projectile * proj = dynamic_cast< Projectile* >( wrld->getWorldEntity( *item ) );

		if ( proj->getHit() )
		{
			const Params::sProjectileParams & pprm = gParams.projectile( proj->getSubType() );

			if ( dblEqual( 0, pprm.radius ) )
			{	//no area damage
				int entID = proj->getEntity();
				if ( entID <= 0 )		//hit to static or invalid object
					continue;

				WorldEntity * we = gAppServer->getGame().getWorld().getWorldEntity( entID );
				we->doDamage( pprm.getDamage(), pprm.piercing );
				we->checkDestroy( proj->getOwner() );
			}
			else
			{	//area damage
				worldent_iterator other_item = wrld->getWorldEntitiesBegin();
				worldent_iterator other_end = wrld->getWorldEntitiesEnd();

				for ( ; other_item != other_end; ++other_item )
				{
					WorldEntity * we = *other_item;
					if ( proj->getID() == we->getID() )	//cannot hit itself
						continue;

					double distance = ( proj->getPosition() - we->getPosition() ).length();
					if ( distance > pprm.radius )	//object is far
						continue;

					double maxdmg = proj->getDamage();
					double damage = 0;
					if ( distance < pprm.radius * AREA_DMG_LINEAR_PERCENTAGE / 100 )	//object is in area with linear fall of damage
					{
						damage= distance*( -maxdmg / pprm.radius ) + maxdmg;
						assertL( damage < maxdmg && damage > 0 );	
					}
					else
					{
							//distance of start of quadratic fall of damage
						double quad_start_distance = pprm.radius / 100 * AREA_DMG_LINEAR_PERCENTAGE;
						
							//the damage is computed as A*(distance-radius)^2, 
							//where A is quad_const=M/(quad_start_distance-radius)^2
							//and M is the damage at the end of linear fall part ending
						double quad_const = ( quad_start_distance * ( -maxdmg / pprm.radius ) + maxdmg ) / 
											 ((quad_start_distance - pprm.radius)*(quad_start_distance - pprm.radius));

						damage= quad_const * ( distance - pprm.radius ) * ( distance - pprm.radius );
					}
					
					LOGD << "Giving damage:" << damage << endl;
					we->doDamage( (int)damage, pprm.piercing );
					we->checkDestroy( proj->getOwner() );

					GLVector3d forcevec = ( we->getPosition() - proj->getPosition() ).unit();
					forcevec*= damage * AREA_DMG_DMG2FORCE_MULTIP;
					
					LOGD << "Applying force" << forcevec << endl;

					dBodyAddForce( we->getPhysicsData()->mMainBody, forcevec.x, forcevec.y, forcevec.z );
					
					we->setUnpredictableFlag();
				}
			}
		}
	}
}

void TaskSimulationServer::updateSpeeds()
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	for ( ; item != end ; ++item )
	{
		if ( (*item)->getSpeedChange() )
		{
			ServerPhysicsData * spd = (*item)->getPhysicsData();
			std::list< dBodyID >::iterator body_it = spd->mBodies.begin();
			while ( body_it != spd->mBodies.end() )
			{
				GLVector3d speed = (*item)->getSpeed();
				dBodySetLinearVel( *body_it, speed.x, speed.y, speed.z );

				++body_it;
			}
			(*item)->resetSpeedChange();
		}
		//LOGD << "speed: " << (*item)->getSpeed().length() << endl;
	}
}

void TaskSimulationServer::cleanupObjects()
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	Skybox * s_box = wrld->getLandscape().getSkybox();

	worldent_list to_delete;

	for ( ; item != end ; ++item )
	{
		if ( (*item)->getDeleteFlag() )
		{
			to_delete.push_back( (*item) );
		}
		else
		{
			if ( ! s_box->isPointInsideSkyBox( (*item)->getPosition() ) )
			{
				to_delete.push_back( (*item) );
				(*item)->fallenServer();
			}
		}
	}

	item = to_delete.begin();
	end	 = to_delete.end();

	for ( ; item != end ; ++item )
	{
		wrld->deleteObjectGeneric( (*item)->getID() );
	}
}

void TaskSimulationServer::cleanupPlayers()
{
	GameServer * game = &getAppServer().getGame();
	playerVectorType::iterator it = game->getValidPlayersVectorBegin();
	playerVectorType::iterator ite = game->getValidPlayersVectorEnd();

	std::list< int > toRemove;

	for ( ; it != ite; ++it )
	{
		Player * pl = *it;

		assertL( pl );

		if ( pl->getDeleteMe() )
			toRemove.push_back( pl->getID() );
	}

	while ( !toRemove.empty() )
	{
		game->removePlayer( toRemove.front() );
		toRemove.pop_front();
	}
}

void TaskSimulationServer::updateObjects( const int time_stamp )
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	for ( ; item != end ; ++item )
	{
		(*item)->updateServer( time_stamp );
	}
}

void TaskSimulationServer::initPlayers()
{
	GameServer * game = &getAppServer().getGame();
	playerVectorType::iterator it = game->getValidPlayersVectorBegin();
	playerVectorType::iterator ite = game->getValidPlayersVectorEnd();

	for ( ; it != ite; ++it )
	{
		Player * pl = *it;

		assertL( pl );

		if ( ! pl->getDeleteMe() )
			continue;

		Tank * tnk = pl->getTank();

		if ( tnk )
		{
			Tank::TankCommands & tcmds = tnk->getCmds();
			tcmds.backward = false;
			tcmds.forward = false;
			tcmds.left = false;
			tcmds.right = false;
			tcmds.fire = false;
		}
	}
}

void TaskSimulationServer::initSimData()
{
	World* wrld	= &(gAppServer->getGame().getWorld());

	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	for ( ; item != end ; ++item )
	{
		WorldEntity * en = *item;
		std::list<dBodyID>::iterator it = en->getPhysicsData()->mBodies.begin();
		std::list<dBodyID>::iterator ite= en->getPhysicsData()->mBodies.end();
		while ( it != ite )
		{
			ODEBodyInfo * obi = (ODEBodyInfo*)dBodyGetData( *it );
			if ( obi->type == bTrack )
				obi->spec.trackTerrContact = false;
			
			++it;
		}
	}
}

void TaskSimulationServer::updateSimChanges()
{
	World* wrld	= &(gAppServer->getGame().getWorld());
	
	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	for ( ; item != end ; ++item )
	{
		dBodyID bid = *((*item)->getPhysicsData()->mBodies.begin());

		(*item)->setSpeedSilent( dBodyGetLinearVel( bid ) );
		(*item)->setPosition( dBodyGetPosition( bid ) );
		(*item)->setOrientation( dBodyGetQuaternion( bid ) );
		(*item)->setAngVelocity( dBodyGetAngularVel( bid ) );
	}
}

void TaskSimulationServer::dropBonusBoxes( double time_elapsed )
{
	// if bonus_count >= bonus_limit then return;

	GameServer & game = gAppServer->getGame();
	World & wrld = game.getWorld();

	const int ticks_in_10_minutes = (int)(10*60*1000 / time_elapsed);
	const double prob_dividor = double( RAND_MAX ) / ticks_in_10_minutes;

	const int b_count = (int)gParams.mBonus.size();
	for ( int i = 0; i < b_count; i++ )
	{
		int b_rate = game.getBonusesIn10Minutes( gParams.mBonus[i].id );
		int b_prob = (int)( (double)rand() / prob_dividor ) ;

		if ( b_prob < b_rate )
		{
			GLVector3d terr_sz = wrld.getLandscape().getTerrain().getScale();
			int xpos = (int)((double)rand() / 
				( double( RAND_MAX ) / ( terr_sz.x - BONUS_FREE_ZONE_WIDTH * 2 ) )) + BONUS_FREE_ZONE_WIDTH;
			int ypos = (int)((double)rand() / 
				( double( RAND_MAX ) / ( terr_sz.y - BONUS_FREE_ZONE_WIDTH * 2 ) )) + BONUS_FREE_ZONE_WIDTH;
			int zpos = (int)(wrld.getLandscape().getTerrain().getHeight( xpos, ypos )) + BONUS_FALL_HEIGTH;

			//LOGD << "Bonusbox created - bonus ID = " << gParams.mBonus[i].id << "at pos: (" << xpos << ", " << ypos 
			//	<< ", " << zpos << ")" << endl;

			wrld.createBonusBox
				( GLVector3d( xpos, ypos, zpos ), GLQuaterniond( GLVector3d(1,0,0), 0 ), gParams.mBonus[i].id );
		}
	}

}

void TaskSimulationServer::updateHistory( int time_stamp )
{
	World* wrld	= &(gAppServer->getGame().getWorld());
	
	worldent_iterator item	= wrld->getWorldEntitiesBegin();
	worldent_iterator end	= wrld->getWorldEntitiesEnd();

	for ( ; item != end ; ++item )
	{
		(*item)->createPresentPoint( time_stamp );
		(*item)->shiftHistory();
	}
}

bool TaskSimulationServer::isInHalfSpace( const double a, const double b, const double c, const double d, const GLVector3d& point, const bool sharp ) const
{
	if ( sharp )
		return ((a*point.x + b*point.y + c*point.z + d) > DBL_TOLER);
	else
		return ((a*point.x + b*point.y + c*point.z + d) >= 0);
}

void TaskSimulationServer::update(int dtime, int atime)
{
	TIN;
		
	int timeOverflow = gTime.getGameTicks() - mSimulatedTime;
	int max_ticks = 3; 
	
	while (timeOverflow >= mTimeDelta && --max_ticks > 0)
	{
	
		int time_st = gTime.getTicks();

		//TODO: tu musi byt nejake vyrabanie timestampov, nielen nejaky cas v doubloch
		// A preci - je to jen nejaku cas v intech :)
		gTime.setSimulatedTime(mSimulatedTime);	
		
		tick( mTimeDelta, mSimulatedTime );

		mSimulatedTime += mTimeDelta;
		timeOverflow   -= mTimeDelta;
		
		mPerfMeasureTime += gTime.getTicks() - time_st;
		mPerfMeasureRuns++;

			
			//performance shaping
		if ( mPerfMeasureRuns % 100 == 0 )
		{
			double utilization = mPerfMeasureTime / mPerfMeasureRuns;
			
			double game_multiplicator = gTime.getGameSpeed();

			if ( !dblEqual(mTimeDelta * game_multiplicator, 0) )
			{
				if ( mPerfMeasureTime / (mTimeDelta * game_multiplicator ) > mCPUMaxUtilization )
				{
					if ( mCurrentMultiple < mMaxStepMultiple )
					{
						mCurrentMultiple++;
						LOGI << "simulation takes: " << utilization << " ms" << endl;
						LOGI << "Increasing mCurrentMultiple to " << mCurrentMultiple << endl;
						mTimeDelta	= mTimeDeltaOrig * mCurrentMultiple;
						mStepSize	= mStepSizeOrig * mCurrentMultiple;
					}
				}
				else
				{
					if ( mPerfMeasureTime / (mTimeDelta * game_multiplicator ) < mCPUMaxUtilization / 2 )
					{
						if ( mCurrentMultiple > 1 )
						{
							mCurrentMultiple--;
							LOGI << "simulation takes: " << utilization << " ms" << endl;
							LOGI << "Decreasing mCurrentMultiple to " << mCurrentMultiple << endl;
							mTimeDelta	= mTimeDeltaOrig * mCurrentMultiple;
							mStepSize	= mStepSizeOrig * mCurrentMultiple;
						}
					}
				}
			}

			mPerfMeasureTime = 0;
			mPerfMeasureRuns = 0;
		}
				 
	}
	
	// We have some performance problems, so slow down
	// the time
	if (max_ticks == 0)
	{
		gTime.setGameTicks(mSimulatedTime);
	}
}

void TaskSimulationServer::init()
{
	mSimulatedTime  = gTime.getGameTicks();

	mTimeDeltaOrig	= g_settings.getInt(	SETTING_TICK_DELAY );
	mStepSizeOrig	= g_settings.getDouble(	SETTING_SIMULATION_ODE_STEPSIZE );

	mCurrentMultiple= 1;
	mMaxStepMultiple= g_settings.getInt(	SETTING_SIMULATION_MAX_STEP_MULTIPLY );

	mStepSize		= mStepSizeOrig;
	mTimeDelta		= mTimeDeltaOrig;

	mCPUMaxUtilization = g_settings.getInt(	SETTING_SIMULATION_MAX_CPU_PERCENTAGE );

	mBasicTnkWeight	= g_settings.getInt(	SETTING_SIMULATION_BASIC_TANK_WEIGHT );
	mPerfMeasureRuns= 0;
	mPerfMeasureTime= 0;

	gTime.setSimulatedTime(0);
}
